@@ -107,7 +107,6 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
   s.push_back(mp("-rand float",        "Set random perturbation factor"));
   s.push_back(mp("-bgm file",          "Load background mesh from file"));
   s.push_back(mp("-check",             "Perform various consistency checks on mesh"));
-  s.push_back(mp("-mpass int",         "Do several passes on the mesh for complex background fields"));
   s.push_back(mp("-ignorePartBound",   "Ignore partitions boundaries"));
 #if defined(HAVE_FLTK)
   s.push_back(mp("Post-processing options:", ""));
@@ -118,6 +117,7 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
   s.push_back(mp("-listen",            "Always listen to incoming connection requests"));
   s.push_back(mp("-minterpreter string", "Name of Octave interpreter"));
   s.push_back(mp("-pyinterpreter string", "Name of Python interpreter"));
+  s.push_back(mp("-run",               "Run ONELAB solver(s)"));
   s.push_back(mp("Display options:", ""));
   s.push_back(mp("-n",                 "Hide all meshes and post-processing views on startup"));
   s.push_back(mp("-nodb",              "Disable double buffering"));
@@ -309,6 +309,13 @@ void GetOptions(int argc, char *argv[])
         CTX::instance()->batch = -99;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "run")) {
+        // same as '-', but will run local Gmsh client (if no other clients are
+        // requested, e.g. by opening a '.pro' or '.py' file)
+        CTX::instance()->batch = -99;
+        CTX::instance()->launchSolverAtStartup = -2;
+        i++;
+      }
       else if(!strcmp(argv[i] + 1, "onelab")) {
         i++;
         if(argv[i] && argv[i + 1] && argv[i + 1][0] != '-'){
@@ -553,12 +560,32 @@ void GetOptions(int argc, char *argv[])
         i++;
 	if (i + 1 < argc && argv[i][0] != '-') {
           gmsh_yysymbols[argv[i]].value = std::vector<double>(1, atof(argv[i + 1]));
-          Msg::GetCommandLineNumbers()[argv[i]] = atof(argv[i + 1]);
+          Msg::GetCommandLineNumbers()[argv[i]] = std::vector<double>(1, atof(argv[i + 1]));
           i += 2;
 	}
         else
           Msg::Error("Missing name and/or value for number definition");
       }
+      else if (!strcmp(argv[i]+1, "setlist") ||
+               !strcmp(argv[i]+1, "setlistofnumbers")) {
+        i++;
+	if (i + 1 < argc && argv[i][0] != '-') {
+          std::string n(argv[i]);
+          std::vector<double> v;
+          int s = atoi(argv[i + 1]), j = 0;
+          i += 2;
+          while(j < s && i < argc){
+            v.push_back(atof(argv[i]));
+            i++; j++;
+          }
+          if(j < s)
+            Msg::Error("Missing values in list (got %d instead of %d)", j, s);
+          gmsh_yysymbols[n].value = v;
+          Msg::GetCommandLineNumbers()[n] = v;
+	}
+        else
+          Msg::Error("Missing name and/or value for definition of list of numbers");
+      }
 #endif
       else if(!strcmp(argv[i] + 1, "option")) {
         i++;
@@ -888,12 +915,18 @@ void GetOptions(int argc, char *argv[])
 #if !defined(HAVE_ONELAB2)
         CTX::instance()->solver.listen = 1;
 #else
-        CTX::instance()->batch = 1;
         if(argv[i]) {
-          CTX::instance()->onelab.listen_port = atoi(argv[i++]);
+          if(strstr(argv[i], "/") || strstr(argv[i], "\\") || !strstr(argv[i], ":")) // UNIX
+            CTX::instance()->onelab.unixSock = std::string(argv[i]);
+          else if(argv[i][0] == 'u') // UDT
+            CTX::instance()->onelab.udtSock = std::string(argv[i++]);
+          else if(argv[i][0] == 't') // TCP
+            CTX::instance()->onelab.tcpSock = std::string(argv[i++]);
+          else // TCP
+            CTX::instance()->onelab.tcpSock = std::string(argv[i]);
         }
-        else
-          CTX::instance()->onelab.listen_port = 1148;
+        else // TCP
+          CTX::instance()->onelab.tcpSock = "127.0.0.1:1148";
 #endif
       }
       else if(!strcmp(argv[i] + 1, "minterpreter")) {
