@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -23,12 +23,14 @@ struct contextMeshOptions {
   int points, lines, triangles, quadrangles, tetrahedra, hexahedra, prisms, pyramids;
   int surfacesEdges, surfacesFaces, volumesEdges, volumesFaces, numSubEdges;
   int pointsNum, linesNum, surfacesNum, volumesNum, qualityType, labelType;
-  int optimize, optimizeNetgen, optimizeLloyd, smoothCrossField, refineSteps, remove4triangles;
+  int optimize, optimizeNetgen, optimizeLloyd, smoothCrossField, refineSteps;
+  int remove4triangles;
   double normals, tangents, explode, angleSmoothNormals, allowSwapEdgeAngle;
   double mshFileVersion, mshFilePartitioned, pointSize, lineWidth;
   double qualityInf, qualitySup, radiusInf, radiusSup;
   double scalingFactor, lcFactor, randFactor, lcIntegrationPrecision;
-  double lcMin, lcMax, toleranceEdgeLength, anisoMax, smoothRatio;
+  double lcMin, lcMax, toleranceEdgeLength, toleranceInitialDelaunay;
+  double anisoMax, smoothRatio;
   int lcFromPoints, lcFromCurvature, lcExtendFromBoundary;
   int dual, voronoi, drawSkinOnly, colorCarousel, labelSampling;
   int fileFormat, nbSmoothing, algo2d, algo3d, algoSubdivide;
@@ -70,9 +72,12 @@ struct contextGeometryOptions {
   double tolerance, snap[3], transform[3][3], offset[3];
   int occFixDegenerated, occFixSmallEdges, occFixSmallFaces;
   int occSewFaces, occConnectFaces;
+  double occScaling;
   int copyMeshingMethod, copyDisplayAttributes, exactExtrusion;
   int matchGeomAndMesh;
-  int hideCompounds, orientedPhysicals;
+  int hideCompounds, orientedPhysicals, doubleClickedEntityTag;
+  std::string doubleClickedPointCommand, doubleClickedLineCommand;
+  std::string doubleClickedSurfaceCommand, doubleClickedVolumeCommand;
 };
 
 class CTX {
@@ -88,7 +93,8 @@ class CTX {
   std::string bgmFileName, outputFileName, defaultFileName, tmpFileName;
   std::string sessionFileName, optionsFileName, errorFileName;
   std::string meshStatReportFileName;
-  std::string argv0;
+  // filename of the executable, with full path
+  std::string exeFileName;
   // the home directory
   std::string homeDir;
   // file history
@@ -97,6 +103,8 @@ class CTX {
   int createAppendMeshStatReport;
   // should we launch a solver at startup?
   int launchSolverAtStartup ;
+  // solver to use with ONELAB2
+  int solverToRun ;
   // save session/option file on exit?
   int sessionSave, optionsSave;
   // ask confirmation when overwriting files?
@@ -105,6 +113,8 @@ class CTX {
   std::string display;
   // FLTK theme
   std::string guiTheme;
+  // FLTK color scheme
+  int guiColorScheme;
   // print messages on to the terminal?
   int terminal;
   // number of graphical windows/tiles
@@ -123,6 +133,8 @@ class CTX {
   int fileChooserPosition[2], extraPosition[2], extraSize[2];
   // use the system menu bar on Mac OS X?
   int systemMenuBar;
+  // use high-resolution opengl graphics (retina Macs)
+  int highResolutionGraphics;
   // batch mode (-4: lua session, -3: server daemon, -2: check coherence, -1: write
   // geo, 0: full gfx, 1: 1D mesh, 2: 2D mesh, 3: 3D mesh, 4: adapt
   // mesh, 5: refine mesh)
@@ -187,8 +199,11 @@ class CTX {
   // font name, FLTK enum and size for opengl graphics
   std::string glFont, glFontTitle, glFontEngine;
   int glFontEnum, glFontEnumTitle, glFontSize, glFontSizeTitle;
+  // font size of messages
+  int msgFontSize;
   // point/line widths
   double pointSize, lineWidth;
+  double highResolutionPointSizeFactor;
   // light options
   int light[6];
   double lightPosition[6][4], shine, shineExponent;
@@ -232,18 +247,26 @@ class CTX {
   struct{
     int draw, link, horizontalScales;
     int smooth, animCycle, animStep, combineTime, combineRemoveOrig;
-    int fileFormat, plugins, forceNodeData;
+    int fileFormat, plugins, forceNodeData, forceElementData;
     double animDelay;
+    std::string doubleClickedGraphPointCommand;
+    double doubleClickedGraphPointX, doubleClickedGraphPointY;
+    int doubleClickedView;
   }post;
   // solver options
   struct{
     int plugins, listen;
     double timeout;
-    std::string socketName;
+    std::string socketName, pythonInterpreter, octaveInterpreter;
     std::string name[NUM_SOLVERS], executable[NUM_SOLVERS], remoteLogin[NUM_SOLVERS];
     int autoSaveDatabase, autoArchiveOutputFiles, autoMesh, autoMergeFile;
     int autoShowViews, autoShowLastStep, autoCheck, showInvisibleParameters;
   }solver;
+  // onelab options
+  struct{
+    std::string unixSock, tcpSock, udtSock;
+    bool unixConnected, tcpConnected, udtConnected;
+  }onelab;
   // print options
   struct{
     int fileFormat, epsQuality, epsCompress, epsPS3Shading;
