@@ -130,6 +130,15 @@ StringXString GeneralOptions_String[] = {
 } ;
 
 StringXString GeometryOptions_String[] = {
+  { F|O, "DoubleClickedPointCommand" , opt_geometry_double_clicked_point_command, "" ,
+    "Command parsed when double-clicking on a point" },
+  { F|O, "DoubleClickedLineCommand" , opt_geometry_double_clicked_line_command, "" ,
+    "Command parsed when double-clicking on a line" },
+  { F|O, "DoubleClickedSurfaceCommand" , opt_geometry_double_clicked_surface_command, "" ,
+    "Command parsed when double-clicking on a surface" },
+  { F|O, "DoubleClickedVolumeCommand" , opt_geometry_double_clicked_volume_command, "" ,
+    "Command parsed when double-clicking on a volume" },
+
   { 0, 0 , 0 , "" , 0 }
 } ;
 
@@ -221,17 +230,19 @@ StringXString SolverOptions_String[] = {
 } ;
 
 StringXString PostProcessingOptions_String[] = {
-  { F|O, "GraphPointCommand" , opt_post_graph_point_command, "" ,
+  { F|O, "DoubleClickedGraphPointCommand" , opt_post_double_clicked_graph_point_command, "" ,
     "Command parsed when double-clicking on a graph data point "
     "(e.g. Merge Sprintf('file_%g.pos', PostProcessing.GraphPointX);)" },
 
+  { F|O, "GraphPointCommand" , opt_post_double_clicked_graph_point_command, "" ,
+    "Synonym for `DoubleClickedGraphPointCommand'" },
+
   { 0, 0 , 0 , "" , 0 }
 } ;
 
 StringXString ViewOptions_String[] = {
   { F|O, "Attributes" , opt_view_attributes , "" ,
     "Optional string attributes" },
-
   { F|O, "AxesFormatX" , opt_view_axes_format0 , "%.3g" ,
     "Number format for X-axis (in standard C form)" },
   { F|O, "AxesFormatY" , opt_view_axes_format1 , "%.3g" ,
@@ -245,6 +256,9 @@ StringXString ViewOptions_String[] = {
   { F|O, "AxesLabelZ" , opt_view_axes_label2 , "" ,
     "Z-axis label" },
 
+  { F|O, "DoubleClickedCommand" , opt_view_double_clicked_command , "" ,
+    "Command parsed when double-clicking on the view" },
+
   { F,   "FileName" , opt_view_filename , "" ,
     "Default post-processing view file name" },
   { F|O, "Format" , opt_view_format , "%.3g" ,
@@ -259,6 +273,8 @@ StringXString ViewOptions_String[] = {
   { F|O, "GeneralizedRaiseZ" , opt_view_gen_raise2 , "v2" ,
     "Generalized elevation of the view along Z-axis (in model coordinates, "
     "using formula possibly containing x, y, z, s[tep], t[ime], v0, ... v8)" },
+  { F|O, "Group" , opt_view_group , "" ,
+    "Group to which this view belongs" },
 
   { F,   "Name" , opt_view_name , "" ,
     "Default post-processing view name" },
@@ -768,7 +784,8 @@ StringXNumber GeneralOptions_Number[] = {
 
 StringXNumber GeometryOptions_Number[] = {
   { F|O, "AutoCoherence" , opt_geometry_auto_coherence , 1. ,
-    "Should all duplicate entities be automatically removed? (if ==2, also remove degenerate entities)" },
+    "Should all duplicate entities be automatically removed? (If AutoCoherence == 2, "
+    "also remove degenerate entities)" },
 
   { F,   "Clip" , opt_geometry_clip , 0.,
     "Enable clipping planes? (Plane[i]=2^i, i=0,...,5)" },
@@ -777,6 +794,9 @@ StringXNumber GeometryOptions_Number[] = {
   { F|O, "CopyDisplayAttributes" , opt_geometry_copy_display_attributes, 0. ,
     "Copy display attributes (visibiliy, color) when duplicating geometrical entities?" },
 
+  { F, "DoubleClickedEntityTag" , opt_geometry_double_clicked_entity_tag, 0. ,
+    "Tag of last double-clicked geometrical entity" },
+
   { F|O, "ExactExtrusion" , opt_geometry_exact_extrusion, 1. ,
     "Use exact extrusion formula in interpolations (set to 0 to allow "
     "geometrical transformations of extruded entities)" },
@@ -1278,6 +1298,15 @@ StringXNumber PostProcessingOptions_Number[] = {
   { F|O, "CombineRemoveOriginal" , opt_post_combine_remove_orig , 1. ,
     "Remove original views after a Combine operation" },
 
+  { F, "DoubleClickedGraphPointX" , opt_post_double_clicked_graph_point_x , 0. ,
+    "Abscissa of last double-clicked graph point" },
+  { F, "DoubleClickedGraphPointY" , opt_post_double_clicked_graph_point_y , 0. ,
+    "Ordinate of last double-clicked graph point" },
+  { F, "DoubleClickedView" , opt_post_double_clicked_view, 0. ,
+    "Index of last double-clicked view" },
+
+  { F|O, "ForceElementData" , opt_post_force_element_data , 0. ,
+    "Try to force saving datasets as ElementData" },
   { F|O, "ForceNodeData" , opt_post_force_node_data , 0. ,
     "Try to force saving datasets as NodeData" },
   { F|O, "Format" , opt_post_file_format , 10. ,
@@ -1285,10 +1314,10 @@ StringXNumber PostProcessingOptions_Number[] = {
     "view, 2=parsed view, 3=STL triangulation, 4=raw text, 5=Gmsh mesh, 6=MED file, "
     "10=automatic)" },
 
-  { F|O, "GraphPointX" , opt_post_graph_point_x , 0. ,
-    "Abscissa of last selected graph point" },
-  { F|O, "GraphPointY" , opt_post_graph_point_y , 0. ,
-    "Ordinate of last selected graph point" },
+  { F, "GraphPointX" , opt_post_double_clicked_graph_point_x , 0. ,
+    "Synonym for `DoubleClickedGraphPointX'" },
+  { F, "GraphPointY" , opt_post_double_clicked_graph_point_y , 0. ,
+    "Synonym for `DoubleClickedGraphPointY'" },
 
   { F|O, "HorizontalScales" , opt_post_horizontal_scales , 1. ,
     "Display value scales horizontally" },
@@ -1357,6 +1386,8 @@ StringXNumber ViewOptions_Number[] = {
     "Center glyphs (arrows, numbers, etc.)? (0=left, 1=centered, 2=right)" },
   { F,   "Clip" , opt_view_clip , 0.,
     "Enable clipping planes? (Plane[i]=2^i, i=0,...,5)" },
+  { F|O, "Closed" , opt_view_closed , 0,
+    "Close the subtree containing this view" },
   { F|O, "ColormapAlpha" , opt_view_colormap_alpha , 1.0 ,
     "Colormap alpha channel value (used only if != 1)" },
   { F|O, "ColormapAlphaPower" , opt_view_colormap_alpha_power , 0.0 ,
@@ -1699,7 +1730,7 @@ StringXColor GeneralOptions_Color[] = {
     {245, 245, 245, 255}, {255, 255, 255, 255}, {245, 245, 245, 255}, {50, 50, 50, 255},
     "Background color" },
   { F|O, "BackgroundGradient" , opt_general_color_background_gradient ,
-    {165, 165, 165, 255}, {208, 215, 255, 255}, {165, 165, 165, 255}, {50, 50, 50, 255},
+    {185, 185, 185, 255}, {208, 215, 255, 255}, {185, 185, 185, 255}, {50, 50, 50, 255},
     "Background gradient color" },
   { F|O, "Foreground" , opt_general_color_foreground ,
     {85, 85, 85, 255}, {85, 85, 85, 255}, {85, 85, 85, 255}, {170, 170, 170, 255},
