@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -21,12 +21,24 @@ int GmshSetOption(const std::string &category, const std::string &name,
                   double value, int index=0);
 int GmshSetOption(const std::string &category, const std::string &name,
                   unsigned int value, int index=0);
+void GmshSetStringOption(const std::string &category, const std::string &name,
+                         std::string value, int index=0);
+void GmshSetNumberOption(const std::string &category, const std::string &name,
+                         double value, int index=0);
+void GmshSetColorOption(const std::string &category, const std::string &name,
+                        unsigned int value, int index=0);
 int GmshGetOption(const std::string &category, const std::string &name,
                   std::string &value, int index=0);
 int GmshGetOption(const std::string &category, const std::string &name,
                   double &value, int index=0);
 int GmshGetOption(const std::string &category, const std::string &name,
                   unsigned int &value, int index=0);
+std::string GmshGetStringOption(const std::string &category, const std::string &name,
+                                int index=0);
+double GmshGetNumberOption(const std::string &category, const std::string &name,
+                           int index=0);
+unsigned int GmshGetColorOption(const std::string &category, const std::string &name,
+                                int index=0);
 int GmshRestoreDefaultOptions();
 int GmshOpenProject(const std::string &fileName);
 int GmshClearProject();
