@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -18,6 +18,8 @@ double Cpu();
 double TotalRam();
 long GetMemoryUsage();
 int GetProcessId();
+std::string GetExecutableFileName();
+std::string GetAbsolutePath(const std::string &fileName);
 std::string GetHostName();
 int UnlinkFile(const std::string &fileName);
 int StatFile(const std::string &fileName);
@@ -25,6 +27,8 @@ int KillProcess(int pid);
 int CreateSingleDir(const std::string &dirName);
 void CreatePath(const std::string &fullPath);
 int SystemCall(const std::string &command, bool blocking=false);
+int SystemCallExe(const std::string &exe, const std::string &argsOrCommand,
+                  bool blocking=false);
 std::string GetCurrentWorkdir();
 void RedirectIOToConsole();
 FILE *Fopen(const char* f, const char *mode);
