@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -27,6 +27,7 @@
 
 #if defined(HAVE_PARSER)
 #include "Parser.h"
+#include "FunctionManager.h"
 #endif
 
 #if defined(HAVE_MESH)
@@ -42,7 +43,11 @@
 #if defined(HAVE_FLTK)
 #include <FL/fl_ask.H>
 #include "FlGui.h"
+#if defined(HAVE_ONELAB2)
+#include "onelab2Group.h"
+#else
 #include "onelabGroup.h"
+#endif
 #include "graphicWindow.h"
 #include "drawContext.h"
 #endif
@@ -186,9 +191,9 @@ int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
     return 0;
   }
 
-#if defined(HAVE_FLTK) && defined(HAVE_POST)
-  int numViewsBefore = PView::list.size();
-#endif
+  //#if defined(HAVE_FLTK) && defined(HAVE_POST)
+  // int numViewsBefore = PView::list.size();
+  //#endif
 
   std::string old_yyname = gmsh_yyname;
   gmshFILE old_yyin = gmsh_yyin;
@@ -227,8 +232,11 @@ int ParseFile(const std::string &fileName, bool close, bool warnIfMissing)
   gmsh_yyviewindex = old_yyviewindex;
 
 #if defined(HAVE_FLTK) && defined(HAVE_POST)
-  if(FlGui::available())
-    FlGui::instance()->updateViews(numViewsBefore != (int)PView::list.size(), false);
+  if(FlGui::available()){
+    // this is not enough if the script creates and deletes views
+    //FlGui::instance()->updateViews(numViewsBefore != (int)PView::list.size(), false);
+    FlGui::instance()->updateViews(true, false);
+  }
 #endif
 
   return 1;
@@ -250,15 +258,21 @@ static bool doSystemUncompress(std::string fileName, std::string noExt)
   return false;
 }
 
-void ParseString(const std::string &str)
+void ParseString(const std::string &str, bool inCurrentModelDir)
 {
   if(str.empty()) return;
-  std::string fileName = CTX::instance()->homeDir + CTX::instance()->tmpFileName;
+  std::string fileName;
+  if(inCurrentModelDir)
+    fileName = FixRelativePath(GModel::current()->getFileName(),
+                               CTX::instance()->tmpFileName);
+  else
+    fileName = CTX::instance()->homeDir + CTX::instance()->tmpFileName;
   FILE *fp = Fopen(fileName.c_str(), "w");
   if(fp){
     fprintf(fp, "%s\n", str.c_str());
     fclose(fp);
     GModel::readGEO(fileName);
+    UnlinkFile(fileName);
   }
 }
 
@@ -426,13 +440,16 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
     GModel::current()->setName(split[1] + ".geo");
     GModel::current()->setFileName(split[0] + split[1] + ".geo");
     CTX::instance()->launchSolverAtStartup = num;
+    CTX::instance()->solverToRun = num; // used in ONELAB2
     return 1;
   }
   else if(ext == ".py" || ext == ".PY" ||
+          ext == ".m" || ext == ".M" ||
           ext == ".exe" || ext == ".EXE"){
     int num = defineSolver(split[1]);
     opt_solver_executable(num, GMSH_SET, fileName);
     CTX::instance()->launchSolverAtStartup = num;
+    CTX::instance()->solverToRun = num; // used in ONELAB2
     return 1;
   }
 #endif
@@ -457,10 +474,17 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
       if(status > 1) status = PView::readMSH(fileName);
 #endif
 #if defined(HAVE_MESH)
+      /*
+      This was introduced in r6039 by Koen to snap high order vertices on the
+      geometry. But it introduces subtle bugs when reading high-order
+      post-processing views ; we should have an explicit command to do this, and
+      not modify the mesh without warning just by reading a file --CG.
+
       if(CTX::instance()->mesh.order > 1)
         SetOrderN(GModel::current(), CTX::instance()->mesh.order,
                   CTX::instance()->mesh.secondOrderLinear,
                   CTX::instance()->mesh.secondOrderIncomplete);
+      */
 #endif
     }
 #if defined(HAVE_POST)
@@ -471,6 +495,9 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
 #endif
     else {
       status = GModel::readGEO(fileName);
+#ifdef HAVE_ONELAB2
+      Msg::SetOnelabString("Gmsh/1ModelName", fileName, false);
+#endif
     }
   }
 
@@ -500,7 +527,6 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
   // merge the associated option file if there is one
   if(!StatFile(fileName + ".opt"))
     MergeFile(fileName + ".opt");
-
   return status;
 }
 
@@ -636,44 +662,46 @@ void OpenProject(const std::string &fileName, bool setWindowTitle)
     // reuse it
     GModel::current()->destroy();
     GModel::current()->getGEOInternals()->destroy();
-    // don't clear the parser variables if we just launched gmsh with the
-    // -string, -setstring or -setnumber command line options
-#if defined(HAVE_PARSER)
-    std::string c = Msg::GetCommandLineArgs();
-    if(c.find("-string") == std::string::npos &&
-       c.find("-setstring") == std::string::npos &&
-       c.find("-setnumber") == std::string::npos){
-      gmsh_yysymbols.clear();
-      gmsh_yystringsymbols.clear();
-    }
-#endif
   }
   else{
-    // if the current model is not empty make it invisible, clear the parser
-    // variables and add a new model
-#if defined(HAVE_PARSER)
-    gmsh_yysymbols.clear();
-    gmsh_yystringsymbols.clear();
-#endif
+    // if the current model is not empty make it invisible and add a new model
     new GModel();
     GModel::current(GModel::list.size() - 1);
   }
 
+  // clear parser variables, but keep -setnumber/-setstrings command line
+  // definitions
+#if defined(HAVE_PARSER)
+  gmsh_yysymbols.clear();
+  gmsh_yystringsymbols.clear();
+  std::map<std::string, std::vector<double> > cln(Msg::GetCommandLineNumbers());
+  for(std::map<std::string, std::vector<double> >::iterator it = cln.begin(); it != cln.end(); it++)
+    gmsh_yysymbols[it->first].value = it->second;
+  std::map<std::string, std::string> cls(Msg::GetCommandLineStrings());
+  for(std::map<std::string, std::string>::iterator it = cls.begin(); it != cls.end(); it++)
+    gmsh_yystringsymbols[it->first] = it->second;
+  FunctionManager::Instance()->clear();
+#endif
+
   // temporary hack until we fill the current GModel on the fly during parsing
   ResetTemporaryBoundingBox();
 
   // merge the file
-  if(MergeFile(fileName, false, setWindowTitle)) {
-    if(fileName != CTX::instance()->recentFiles.front())
-      CTX::instance()->recentFiles.insert
-        (CTX::instance()->recentFiles.begin(), fileName);
-    if(CTX::instance()->recentFiles.size() > 5)
-      CTX::instance()->recentFiles.resize(5);
-#if defined(HAVE_FLTK)
-    if(FlGui::available())
-      FlGui::instance()->graph[0]->fillRecentHistoryMenu();
-#endif
+  MergeFile(fileName, false, setWindowTitle);
+
+  // fill recent opened file list
+  std::vector<std::string> tmp = CTX::instance()->recentFiles;
+  CTX::instance()->recentFiles.clear();
+  CTX::instance()->recentFiles.push_back(fileName);
+  for(unsigned int i = 0; i < tmp.size(); i++){
+    if(tmp[i] != fileName)
+      CTX::instance()->recentFiles.push_back(tmp[i]);
   }
+  CTX::instance()->recentFiles.resize(10);
+#if defined(HAVE_FLTK)
+  if(FlGui::available())
+    FlGui::instance()->graph[0]->fillRecentHistoryMenu();
+#endif
 
   // close the files that might have been left open by ParseFile
   if(openedFiles.size()){
