@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -36,6 +36,7 @@ std::string opt_general_axes_format2(OPT_ARGS_STR);
 std::string opt_general_background_image_filename(OPT_ARGS_STR);
 std::string opt_general_display(OPT_ARGS_STR);
 std::string opt_general_filename(OPT_ARGS_STR);
+std::string opt_general_executable_filename(OPT_ARGS_STR);
 std::string opt_general_default_filename(OPT_ARGS_STR);
 std::string opt_general_tmp_filename(OPT_ARGS_STR);
 std::string opt_general_error_filename(OPT_ARGS_STR);
@@ -46,12 +47,21 @@ std::string opt_general_recent_file1(OPT_ARGS_STR);
 std::string opt_general_recent_file2(OPT_ARGS_STR);
 std::string opt_general_recent_file3(OPT_ARGS_STR);
 std::string opt_general_recent_file4(OPT_ARGS_STR);
+std::string opt_general_recent_file5(OPT_ARGS_STR);
+std::string opt_general_recent_file6(OPT_ARGS_STR);
+std::string opt_general_recent_file7(OPT_ARGS_STR);
+std::string opt_general_recent_file8(OPT_ARGS_STR);
+std::string opt_general_recent_file9(OPT_ARGS_STR);
 std::string opt_general_editor(OPT_ARGS_STR);
 std::string opt_general_watch_file_pattern(OPT_ARGS_STR);
 std::string opt_general_gui_theme(OPT_ARGS_STR);
 std::string opt_general_graphics_font(OPT_ARGS_STR);
 std::string opt_general_graphics_font_title(OPT_ARGS_STR);
 std::string opt_general_graphics_font_engine(OPT_ARGS_STR);
+std::string opt_geometry_double_clicked_point_command(OPT_ARGS_STR);
+std::string opt_geometry_double_clicked_line_command(OPT_ARGS_STR);
+std::string opt_geometry_double_clicked_surface_command(OPT_ARGS_STR);
+std::string opt_geometry_double_clicked_volume_command(OPT_ARGS_STR);
 std::string opt_solver_socket_name(OPT_ARGS_STR);
 std::string opt_solver_name(OPT_ARGS_STR);
 std::string opt_solver_name0(OPT_ARGS_STR);
@@ -86,9 +96,14 @@ std::string opt_solver_remote_login6(OPT_ARGS_STR);
 std::string opt_solver_remote_login7(OPT_ARGS_STR);
 std::string opt_solver_remote_login8(OPT_ARGS_STR);
 std::string opt_solver_remote_login9(OPT_ARGS_STR);
+std::string opt_solver_octave_interpreter(OPT_ARGS_STR);
+std::string opt_solver_python_interpreter(OPT_ARGS_STR);
+std::string opt_post_double_clicked_graph_point_command(OPT_ARGS_STR);
 std::string opt_view_name(OPT_ARGS_STR);
 std::string opt_view_format(OPT_ARGS_STR);
 std::string opt_view_filename(OPT_ARGS_STR);
+std::string opt_view_double_clicked_command(OPT_ARGS_STR);
+std::string opt_view_group(OPT_ARGS_STR);
 std::string opt_view_axes_label0(OPT_ARGS_STR);
 std::string opt_view_axes_label1(OPT_ARGS_STR);
 std::string opt_view_axes_label2(OPT_ARGS_STR);
@@ -115,6 +130,7 @@ std::string opt_print_parameter_command(OPT_ARGS_STR);
 
 double opt_general_initial_context(OPT_ARGS_NUM);
 double opt_general_fontsize(OPT_ARGS_NUM);
+double opt_general_gui_color_scheme(OPT_ARGS_NUM);
 double opt_general_graphics_fontsize(OPT_ARGS_NUM);
 double opt_general_graphics_fontsize_title(OPT_ARGS_NUM);
 double opt_general_graphics_position0(OPT_ARGS_NUM);
@@ -130,6 +146,7 @@ double opt_general_polygon_offset_factor(OPT_ARGS_NUM);
 double opt_general_polygon_offset_units(OPT_ARGS_NUM);
 double opt_general_system_menu_bar(OPT_ARGS_NUM);
 double opt_general_message_size(OPT_ARGS_NUM);
+double opt_general_message_fontsize(OPT_ARGS_NUM);
 double opt_general_detached_menu(OPT_ARGS_NUM);
 double opt_general_menu_size0(OPT_ARGS_NUM);
 double opt_general_menu_size1(OPT_ARGS_NUM);
@@ -159,6 +176,8 @@ double opt_general_manip_position0(OPT_ARGS_NUM);
 double opt_general_manip_position1(OPT_ARGS_NUM);
 double opt_general_hot_position0(OPT_ARGS_NUM);
 double opt_general_hot_position1(OPT_ARGS_NUM);
+double opt_general_high_resolution_graphics(OPT_ARGS_NUM);
+double opt_general_high_resolution_point_size_factor(OPT_ARGS_NUM);
 double opt_general_session_save(OPT_ARGS_NUM);
 double opt_general_options_save(OPT_ARGS_NUM);
 double opt_general_rotation0(OPT_ARGS_NUM);
@@ -353,6 +372,7 @@ double opt_geometry_occ_fix_small_edges(OPT_ARGS_NUM);
 double opt_geometry_occ_fix_small_faces(OPT_ARGS_NUM);
 double opt_geometry_occ_sew_faces(OPT_ARGS_NUM);
 double opt_geometry_occ_connect_faces(OPT_ARGS_NUM);
+double opt_geometry_occ_scaling(OPT_ARGS_NUM);
 double opt_geometry_old_circle(OPT_ARGS_NUM);
 double opt_geometry_old_newreg(OPT_ARGS_NUM);
 double opt_geometry_old_ruled_surface(OPT_ARGS_NUM);
@@ -366,6 +386,7 @@ double opt_geometry_snap2(OPT_ARGS_NUM);
 double opt_geometry_clip(OPT_ARGS_NUM);
 double opt_geometry_copy_meshing_method(OPT_ARGS_NUM);
 double opt_geometry_copy_display_attributes(OPT_ARGS_NUM);
+double opt_geometry_double_clicked_entity_tag(OPT_ARGS_NUM);
 double opt_geometry_exact_extrusion(OPT_ARGS_NUM);
 double opt_geometry_match_geom_and_mesh(OPT_ARGS_NUM);
 double opt_mesh_label_sampling(OPT_ARGS_NUM);
@@ -381,6 +402,7 @@ double opt_mesh_scaling_factor(OPT_ARGS_NUM);
 double opt_mesh_lc_min(OPT_ARGS_NUM);
 double opt_mesh_lc_max(OPT_ARGS_NUM);
 double opt_mesh_tolerance_edge_length(OPT_ARGS_NUM);
+double opt_mesh_tolerance_initial_delaunay(OPT_ARGS_NUM);
 double opt_mesh_lc_factor(OPT_ARGS_NUM);
 double opt_mesh_lc_from_curvature(OPT_ARGS_NUM);
 double opt_mesh_lc_from_points(OPT_ARGS_NUM);
@@ -527,11 +549,18 @@ double opt_post_plugins(OPT_ARGS_NUM);
 double opt_post_nb_views(OPT_ARGS_NUM);
 double opt_post_file_format(OPT_ARGS_NUM);
 double opt_post_force_node_data(OPT_ARGS_NUM);
+double opt_post_force_element_data(OPT_ARGS_NUM);
+double opt_post_double_clicked_graph_point_x(OPT_ARGS_NUM);
+double opt_post_double_clicked_graph_point_y(OPT_ARGS_NUM);
+double opt_post_double_clicked_view(OPT_ARGS_NUM);
 double opt_view_nb_timestep(OPT_ARGS_NUM);
 double opt_view_nb_non_empty_timestep(OPT_ARGS_NUM);
 double opt_view_timestep(OPT_ARGS_NUM);
+double opt_view_time(OPT_ARGS_NUM);
 double opt_view_min(OPT_ARGS_NUM);
 double opt_view_max(OPT_ARGS_NUM);
+double opt_view_min_visible(OPT_ARGS_NUM);
+double opt_view_max_visible(OPT_ARGS_NUM);
 double opt_view_custom_min(OPT_ARGS_NUM);
 double opt_view_custom_max(OPT_ARGS_NUM);
 double opt_view_custom_abscissa_min(OPT_ARGS_NUM);
@@ -649,6 +678,7 @@ double opt_view_component_map5(OPT_ARGS_NUM);
 double opt_view_component_map6(OPT_ARGS_NUM);
 double opt_view_component_map7(OPT_ARGS_NUM);
 double opt_view_component_map8(OPT_ARGS_NUM);
+double opt_view_closed(OPT_ARGS_NUM);
 double opt_print_file_format(OPT_ARGS_NUM);
 double opt_print_eps_compress(OPT_ARGS_NUM);
 double opt_print_eps_ps3shading(OPT_ARGS_NUM);
@@ -784,7 +814,7 @@ typedef struct {
   // the defaults are stored in individual bytes so that we can initialize
   // them statically independently of the machine endianness. They will be
   // packed into unsigned ints at runtime
-  unsigned char def1[4], def2[4], def3[4];
+  unsigned char def1[4], def2[4], def3[4], def4[4];
   const char *help;
 } StringXColor;
 
