@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -91,17 +91,21 @@ std::string FixRelativePath(const std::string &reference, const std::string &in)
 
 std::vector<std::string> SplitFileName(const std::string &fileName)
 {
-  // returns [path, baseName, extension]
-  int idot = (int)fileName.find_last_of('.');
-  int islash = (int)fileName.find_last_of("/\\");
-  if(idot == (int)std::string::npos) idot = -1;
-  if(islash == (int)std::string::npos) islash = -1;
-  std::vector<std::string> s(3);
-  if(idot > 0)
-    s[2] = fileName.substr(idot);
-  if(islash > 0)
-    s[0] = fileName.substr(0, islash + 1);
-  s[1] = fileName.substr(s[0].size(), fileName.size() - s[0].size() - s[2].size());
+  // JFR DO NOT CHANGE TO std::vector<std::string> s(3), it segfaults while
+  // destructor si called
+  std::vector<std::string> s; s.resize(3);
+  if(fileName.size()){
+    // returns [path, baseName, extension]
+    int idot = (int)fileName.find_last_of('.');
+    int islash = (int)fileName.find_last_of("/\\");
+    if(idot == (int)std::string::npos) idot = -1;
+    if(islash == (int)std::string::npos) islash = -1;
+    if(idot > 0)
+      s[2] = fileName.substr(idot);
+    if(islash > 0)
+      s[0] = fileName.substr(0, islash + 1);
+    s[1] = fileName.substr(s[0].size(), fileName.size() - s[0].size() - s[2].size());
+  }
   return s;
 }
 
