@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -12,9 +12,15 @@
 // onelab servers
 
 namespace onelabUtils {
+#ifdef HAVE_ONELAB2
+  std::vector<std::string> getCommandLine(const std::string &client);
+  std::string getMshFileName();
+  void guessModelName(const std::string &client);
+#else
   std::vector<std::string> getCommandLine(onelab::client *c);
   std::string getMshFileName(onelab::client *c);
   void guessModelName(onelab::client *c);
+#endif
   void initializeLoop(const std::string &level);
   bool incrementLoop(const std::string &level);
   std::vector<double> getRange(onelab::number &p);
