@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -29,13 +29,19 @@ typedef unsigned long intptr_t;
 #include "clippingWindow.h"
 #include "manipWindow.h"
 #include "contextWindow.h"
+#ifdef HAVE_ONELAB2
+#include "onelab2Group.h"
+#include "OnelabWindow.h"
+#else
 #include "onelabGroup.h"
+#endif
 #include "helpWindow.h"
 #include "colorbarWindow.h"
 #include "fileDialogs.h"
 #include "GmshDefines.h"
 #include "GmshMessage.h"
 #include "GModel.h"
+#include "OS.h"
 #include "MElement.h"
 #include "PView.h"
 #include "Field.h"
@@ -70,6 +76,18 @@ static int globalShortcut(int event)
   return FlGui::instance()->testGlobalShortcuts(event);
 }
 
+int FlGui::lock()
+{
+  _in_main_thread++;
+  return Fl::lock();
+}
+
+void FlGui::unlock()
+{
+  Fl::unlock();
+  _in_main_thread--;
+}
+
 static void simple_right_box_draw(int x, int y, int w, int h, Fl_Color c)
 {
   fl_color(c); fl_rectf(x, y, w, h);
@@ -80,7 +98,6 @@ static void simple_top_box_draw(int x, int y, int w, int h, Fl_Color c)
 {
   fl_color(c); fl_rectf(x, y, w, h);
   fl_color(FL_DARK2); fl_line(x, y, x + w, y);
-  //fl_color(FL_LIGHT2); fl_line(x, y+1, x + w, y+1);
 }
 
 // Icons for the satus bar
@@ -226,6 +243,54 @@ static void fatal_error_handler(const char *fmt, ...)
   Msg::Fatal("%s (FLTK internal error)", str);
 }
 
+void FlGui::applyColorScheme()
+{
+  static int first = true;
+  int N = 4 + FL_NUM_GRAY;
+  static std::vector<unsigned char> r(N, 0), g(N, 0), b(N, 0);
+
+  if(first){
+    // store default (OS-dependent) interface colors:
+    Fl::get_system_colors();
+    Fl::get_color(FL_BACKGROUND_COLOR, r[0], g[0], b[0]);
+    Fl::get_color(FL_BACKGROUND2_COLOR, r[1], g[1], b[1]);
+    Fl::get_color(FL_FOREGROUND_COLOR, r[2], g[2], b[2]);
+    Fl::get_color(FL_SELECTION_COLOR, r[3], g[3], b[3]);
+    for (int i = 0; i < FL_NUM_GRAY; i++) {
+      Fl::get_color(fl_gray_ramp(i), r[4 + i], g[4 + i], b[4 + i]);
+    }
+  }
+
+  if(CTX::instance()->guiColorScheme == 1){ // dark mode
+    Fl::set_color(FL_BACKGROUND_COLOR, 50, 50, 50);
+    Fl::set_color(FL_BACKGROUND2_COLOR, 120, 120, 120);
+    Fl::set_color(FL_FOREGROUND_COLOR, 240, 240, 240);
+    for (int i = 0; i < FL_NUM_GRAY; i++) {
+      double min = 0., max = 135.;
+      int d = (int)(min + i * (max - min) / (FL_NUM_GRAY - 1.));
+      Fl::set_color(fl_gray_ramp(i), d, d, d);
+    }
+    if(available()) Fl::reload_scheme();
+    Fl::set_color(FL_SELECTION_COLOR, 200, 200, 200);
+    if(available()) updateViews(true, true);
+  }
+  else if(!first && available() && CTX::instance()->guiColorScheme == 0){
+    // retore default colors (only if not calling the routine from the
+    // constructor)
+    Fl::set_color(FL_BACKGROUND_COLOR, r[0], g[0], b[0]);
+    Fl::set_color(FL_BACKGROUND2_COLOR, r[1], g[1], b[1]);
+    Fl::set_color(FL_FOREGROUND_COLOR, r[2], g[2], b[2]);
+    for (int i = 0; i < FL_NUM_GRAY; i++) {
+      Fl::set_color(fl_gray_ramp(i), r[4 + i], g[4 + i], b[4 + i]);
+    }
+    Fl::reload_scheme();
+    Fl::set_color(FL_SELECTION_COLOR, r[3], g[3], b[3]);
+    updateViews(true, true);
+  }
+
+  first = false;
+}
+
 FlGui::FlGui(int argc, char **argv)
 {
   Fl::error = error_handler;
@@ -235,12 +300,9 @@ FlGui::FlGui(int argc, char **argv)
   if(CTX::instance()->display.size())
     Fl::display(CTX::instance()->display.c_str());
 
-#if 0 // dark scheme... not bad, but needs work
-  Fl::background(110, 110, 110);
-  Fl::background2(140, 140, 140);
-  Fl::foreground(230, 230, 230);
-  //Fl::set_color(FL_SELECTION_COLOR, 50, 50, 0);
-#endif
+  // apply color scheme before widget creation (noop if default color scheme is
+  // selected), so that there's no color "flashing"
+  applyColorScheme();
 
   // add new box types (dx dy dw dh)
   Fl::set_boxtype(GMSH_SIMPLE_RIGHT_BOX, simple_right_box_draw, 0, 0, 1, 0);
@@ -266,6 +328,11 @@ FlGui::FlGui(int argc, char **argv)
   Fl_Tooltip::size(FL_NORMAL_SIZE);
   Fl_Tooltip::delay(0.5);
 
+  // use retina resolution if available
+#if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3) && (FL_PATCH_VERSION >= 4)
+  Fl::use_high_res_GL(CTX::instance()->highResolutionGraphics);
+#endif
+
   // register image formats not in core fltk library (jpeg/png)
   fl_register_images();
 
@@ -323,6 +390,10 @@ FlGui::FlGui(int argc, char **argv)
   graph[0]->getWindow()->show(argc >0 ? 1 : 0, argv);
   if(graph[0]->getMenuWindow()) graph[0]->getMenuWindow()->show();
 
+  // re-apply color scheme (necessary for some reason to get the selection color
+  // right)
+  applyColorScheme();
+
   // graphic window should have the initial focus (so we can e.g. directly loop
   // through time steps with the keyboard)
   Fl::focus(graph[0]->gl[0]);
@@ -352,12 +423,15 @@ FlGui::FlGui(int argc, char **argv)
   fullscreen->mode(mode);
   fullscreen->end();
   fullscreen->fullscreen();
-#if not defined (__APPLE__)
+#if !defined (__APPLE__)
   fullscreen->icon(graph[0]->getWindow()->icon());
 #endif
 
   // create all other windows
   options = new optionWindow(CTX::instance()->deltaFontSize);
+#if defined(HAVE_ONELAB2)
+  onelab2 = new onelabWindow();
+#endif
   fields = new fieldWindow(CTX::instance()->deltaFontSize);
   plugins = new pluginWindow(CTX::instance()->deltaFontSize);
   stats = new statisticsWindow(CTX::instance()->deltaFontSize);
@@ -372,6 +446,8 @@ FlGui::FlGui(int argc, char **argv)
   // init solver plugin stuff
   callForSolverPlugin(-1);
 
+  _in_main_thread = 0;
+
   // draw
   for(unsigned int i = 0; i < graph.size(); i++)
     for(unsigned int j = 0; j < graph[i]->gl.size(); j++)
@@ -387,6 +463,11 @@ FlGui *FlGui::instance(int argc, char **argv)
     _instance = new FlGui(argc, argv);
     // set all options in the new GUI
     InitOptionsGUI(0);
+#ifdef HAVE_ONELAB2
+    // Enable multi-thread support by locking from the main thread
+    Fl::lock();
+#endif
+
     // say welcome!
     Msg::StatusBar(false, "Gmsh %s", GetGmshVersion());
     // log the following for bug reports
@@ -397,6 +478,7 @@ FlGui *FlGui::instance(int argc, char **argv)
     Msg::Info("Build date     : %s", GetGmshBuildDate());
     Msg::Info("Build host     : %s", GetGmshBuildHost());
     Msg::Info("Packager       : %s", GetGmshPackager());
+    Msg::Info("Executable     : %s", CTX::instance()->exeFileName.c_str());
     Msg::Info("Home directory : %s", CTX::instance()->homeDir.c_str());
     Msg::Info("Launch date    : %s", Msg::GetLaunchDate().c_str());
     Msg::Info("Command line   : %s", Msg::GetCommandLineArgs().c_str());
@@ -692,6 +774,15 @@ int FlGui::testGlobalShortcuts(int event)
     status = 2;
   }
   else if(Fl::test_shortcut(FL_ALT + 't')) {
+    for(unsigned int i = 0; i < PView::list.size(); i++)
+      if(opt_view_visible(i, GMSH_GET, 0)){
+        int t = opt_view_intervals_type(i, GMSH_GET, 0) + 1;
+        if(t == 4) t = 1; // skip numeric display
+        opt_view_intervals_type(i, GMSH_SET | GMSH_GUI, t);
+      }
+    status = 2;
+  }
+  else if(Fl::test_shortcut(FL_ALT + FL_SHIFT + 't')) {
     for(unsigned int i = 0; i < PView::list.size(); i++)
       if(opt_view_visible(i, GMSH_GET, 0))
         opt_view_intervals_type
@@ -891,7 +982,7 @@ char FlGui::selectEntity(int type)
 {
   return getCurrentOpenglWindow()->selectEntity
     (type, selectedVertices, selectedEdges, selectedFaces, selectedRegions,
-     selectedElements);
+     selectedElements, selectedPoints, selectedViews);
 }
 
 void FlGui::setStatus(const std::string &msg, bool opengl)
@@ -936,10 +1027,16 @@ void FlGui::setStatus(const std::string &msg, bool opengl)
 void FlGui::setLastStatus(int col)
 {
   for(unsigned int i = 0; i < graph.size(); i++){
-    if(col >= 0 && graph[0]->getMessageHeight() < FL_NORMAL_SIZE)
-      graph[i]->getProgress()->labelcolor(col);
-    else
+    if(col >= 0 && graph[0]->getMessageHeight() < FL_NORMAL_SIZE){
+      if(CTX::instance()->guiColorScheme) // dark
+        graph[i]->getProgress()->color(col);
+      else
+        graph[i]->getProgress()->labelcolor(col);
+    }
+    else{
+      graph[i]->getProgress()->color(FL_BACKGROUND_COLOR);
       graph[i]->getProgress()->labelcolor(FL_FOREGROUND_COLOR);
+    }
   }
   setStatus(_lastStatus);
 }
@@ -957,12 +1054,6 @@ void FlGui::setProgress(const std::string &msg, double val, double min, double m
   setStatus(msg);
 }
 
-void FlGui::setProgressColor(int col)
-{
-  for(unsigned int i = 0; i < FlGui::instance()->graph.size(); i++)
-    FlGui::instance()->graph[i]->getProgress()->labelcolor(col);
-}
-
 void FlGui::storeCurrentWindowsInfo()
 {
   CTX::instance()->glPosition[0] = graph[0]->getWindow()->x();
