@@ -22,6 +22,7 @@
 class graphicWindow;
 class openglWindow;
 class optionWindow;
+class onelabWindow;
 class fieldWindow;
 class pluginWindow;
 class statisticsWindow;
@@ -40,6 +41,7 @@ class GEdge;
 class GFace;
 class GRegion;
 class MElement;
+class PView;
 
 class FlGui{
  private:
@@ -54,9 +56,11 @@ class FlGui{
   std::vector<GRegion*> selectedRegions;
   std::vector<MElement*> selectedElements;
   std::vector<SPoint2> selectedPoints;
+  std::vector<PView*> selectedViews;
  public:
   std::vector<graphicWindow*> graph;
   optionWindow *options;
+  onelabWindow *onelab2;
   fieldWindow *fields;
   pluginWindow *plugins;
   statisticsWindow *stats;
