@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -11,6 +11,9 @@
 #include "Options.h"
 #include "PluginManager.h"
 #include "GModel.h"
+#if defined(HAVE_ONELAB2) && defined(__linux__)
+#include <X11/Xlib.h>
+#endif
 
 int main(int argc, char *argv[])
 {
@@ -38,5 +41,8 @@ int main(int argc, char *argv[])
   }
 
   // Interactive Gmsh with FLTK GUI
+#if defined(HAVE_ONELAB2) && defined(__linux__)
+  XInitThreads();
+#endif
   return GmshFLTK(argc, argv);
 }
