@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -8,7 +8,8 @@
 #include <string>
 #include <iostream>
 #include <stdio.h>
-#include <Navigator.h>
+#include <algorithm>
+#include "Navigator.h"
 #include "Trackball.h"
 #include "Context.h"
 #include "drawContext.h"
