@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -35,12 +35,15 @@ class openglWindow : public Fl_Gl_Window {
   bool _select(int type, bool multiple, bool mesh, int x, int y, int w, int h,
                std::vector<GVertex*> &vertices, std::vector<GEdge*> &edges,
                std::vector<GFace*> &faces, std::vector<GRegion*> &regions,
-               std::vector<MElement*> &elements);
+               std::vector<MElement*> &elements, std::vector<SPoint2> &points,
+               std::vector<PView*> &views);
  protected:
   void draw();
   int handle(int);
  public:
-  time_t  rawtime,  prev_rawtime;
+  int pixel_w();
+  int pixel_h();
+  time_t rawtime,  prev_rawtime;
   double response_frequency;
   bool addPointMode, lassoMode, selectionMode;
   int endSelection, undoSelection, invertSelection, quitSelection;
@@ -51,15 +54,14 @@ class openglWindow : public Fl_Gl_Window {
   char selectEntity(int type,
                     std::vector<GVertex*> &vertices, std::vector<GEdge*> &edges,
                     std::vector<GFace*> &faces, std::vector<GRegion*> &regions,
-                    std::vector<MElement*> &elements);
+                    std::vector<MElement*> &elements, std::vector<SPoint2> &points,
+                    std::vector<PView*> &views);
   static openglWindow *getLastHandled(){ return _lastHandled; }
   static void setLastHandled(openglWindow *w){ _lastHandled = w; }
   void drawTooltip(const std::string &text);
-  ///
   double frequency;
   void moveWithGamepad();
   Navigator *Nautilus;
 };
 
 #endif
-
