@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -25,8 +25,7 @@ static void solver_remove_cb(Fl_Widget *w, void *data)
 
   onelab::server::citer it = onelab::server::instance()->findClient(name);
   if(it != onelab::server::instance()->lastClient()){
-    onelab::client *c = it->second;
-    delete c;
+    delete *it;
   }
   FlGui::instance()->onelab->rebuildSolverList();
 }
