@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -326,11 +326,13 @@ static void _rebuild_list_browser()
       FlGui::instance()->visibility->browser->select(i + 1);
   }
 
+#if 0
   // activate/deactivate delete button
   if(type == VisibilityList::PhysicalEntities)
     FlGui::instance()->visibility->push[0]->activate();
   else
     FlGui::instance()->visibility->push[0]->deactivate();
+#endif
 }
 
 static void visibility_browser_apply_cb(Fl_Widget *w, void *data)
@@ -352,16 +354,15 @@ static void visibility_browser_apply_cb(Fl_Widget *w, void *data)
     for(int i = 0; i < VisibilityList::instance()->getNumEntities(); i++)
       if(FlGui::instance()->visibility->browser->selected(i + 1))
         VisibilityList::instance()->setVisibility(i, 1, recursive, allmodels);
-    // then refresh the browser to account for recursive selections
+    FlGui::instance()->visibility->browser->deselect();
     for(int i = 0; i < VisibilityList::instance()->getNumEntities(); i++)
       if(VisibilityList::instance()->getVisibility(i))
-        FlGui::instance()->visibility->browser->select(i + 1,true);
-      else
-        FlGui::instance()->visibility->browser->select(i + 1,false);
+        FlGui::instance()->visibility->browser->select(i + 1, 1);
     drawContext::global()->draw();
   }
 }
 
+#if 0
 static void visibility_delete_cb(Fl_Widget *w, void *data)
 {
   bool all = true;
@@ -384,6 +385,7 @@ static void visibility_delete_cb(Fl_Widget *w, void *data)
   }
   visibility_cb(NULL, (void*)"redraw_only");
 }
+#endif
 
 static void visibility_sort_cb(Fl_Widget *w, void *data)
 {
@@ -445,6 +447,8 @@ static void visibility_sort_cb(Fl_Widget *w, void *data)
 class listBrowser : public Fl_Browser{
   int handle(int event)
   {
+    void *l = selection();
+
     switch(event){
     case FL_SHORTCUT:
     case FL_KEYBOARD:
@@ -456,8 +460,25 @@ class listBrowser : public Fl_Browser{
       else if(Fl::test_shortcut(FL_Enter) ||
               Fl::test_shortcut(FL_KP_Enter)){
         visibility_browser_apply_cb(NULL, NULL);
+        Fl_Browser_::select(l);
         return 1;
       }
+      else if(Fl::test_shortcut(FL_Up)){
+        if(l && (l = item_prev(l))){
+          select_only(l, 1);
+          visibility_browser_apply_cb(NULL, NULL);
+          Fl_Browser_::select(l);
+          return 1;
+        }
+      }
+      else if(Fl::test_shortcut(FL_Down)){
+        if(l && (l = item_next(l))){
+          select_only(l, 1);
+          visibility_browser_apply_cb(NULL, NULL);
+          Fl_Browser_::select(l);
+          return 1;
+        }
+      }
     }
     return Fl_Browser::handle(event);
   }
@@ -1199,6 +1220,7 @@ visibilityWindow::visibilityWindow(int deltaFontSize)
       browser->type(FL_MULTI_BROWSER);
       browser->textsize(FL_NORMAL_SIZE - 1);
       browser->column_widths(cols);
+      browser->scrollbar_size(std::max(10, FL_NORMAL_SIZE - 2)); // thinner scrollbars
 
       gg->end();
       Fl_Group::current()->resizable(gg);
@@ -1216,9 +1238,12 @@ visibilityWindow::visibilityWindow(int deltaFontSize)
     browser_type->menu(browser_type_table);
     browser_type->value(2); // physicals
 
+    // "Delete" is out of place in a Visibility window - it's a destructive operation!
+#if 0
     push[0] = new Fl_Button
       (width - 2 * CC - 3 * WB, height - 2 * BH - 3 * WB, CC, BH, "Delete");
     push[0]->callback(visibility_delete_cb);
+#endif
 
     Fl_Return_Button *b1 = new Fl_Return_Button
       (width - 1 * CC - 2 * WB, height - 2 * BH - 3 * WB, CC, BH, "Apply");
@@ -1235,6 +1260,7 @@ visibilityWindow::visibilityWindow(int deltaFontSize)
     tree->labelsize(FL_NORMAL_SIZE - 1);
     tree->selectmode(FL_TREE_SELECT_MULTI);
     tree->connectorstyle(FL_TREE_CONNECTOR_SOLID);
+    tree->scrollbar_size(std::max(10, FL_NORMAL_SIZE - 2)); // thinner scrollbars
     tree->hide();
 
     tree_create = new Fl_Button
@@ -1402,6 +1428,7 @@ visibilityWindow::visibilityWindow(int deltaFontSize)
     per_window = new Fl_Multi_Browser
       (2 * WB, 2 * WB + BH, brw, height - 6 * WB - 3 * BH);
     per_window->callback(visibility_per_window_cb, (void*)"item");
+    per_window->scrollbar_size(std::max(10, FL_NORMAL_SIZE - 2)); // thinner scrollbars
 
     Fl_Button *b1 = new Fl_Button
       (width - 1 * CC - 2 * WB, height - 2 * BH - 3 * WB, CC, BH, "Reset all");
