@@ -34,7 +34,7 @@ class GEntity {
 
   // gives the number of the master entity in periodic mesh, gives _tag
   // if non-periodic
-  int _meshMaster;
+  GEntity* _meshMaster;
 
   // the visibility and the selection flag
   char _visible, _selection;
@@ -52,6 +52,9 @@ class GEntity {
   // the mesh vertices uniquely owned by the entity
   std::vector<MVertex*> mesh_vertices;
 
+  // corresponding principal vertices
+  std::map<GVertex*,GVertex*> vertexCounterparts;
+
   // the physical entitites (if any) that contain this entity
   std::vector<int> physicals;
 
@@ -167,7 +170,7 @@ class GEntity {
       return name[type];
   }
 
-  GEntity(GModel *m, int t);
+  GEntity(GModel *m,int t);
 
   virtual ~GEntity(){}
 
@@ -240,7 +243,7 @@ class GEntity {
 
   // get the native pointer of the particular representation
   virtual void *getNativePtr() const { return 0; }
-  
+
   // get the native id (int) of the particular representation
   virtual int getNativeInt() const { return 0; }
 
@@ -261,9 +264,10 @@ class GEntity {
     return physicals;
   }
 
-  // returns the tag of the entity that its master entity (for mesh)
-  int meshMaster() const;
-  void setMeshMaster(int m);
+  // returns the master entity (for mesh)
+  GEntity* meshMaster() const;
+  void setMeshMaster(GEntity*);
+  void setMeshMaster(GEntity*,const std::vector<double>&);
 
   // get the bounding box
   virtual SBoundingBox3d bounds() const { return SBoundingBox3d(); }
@@ -332,7 +336,7 @@ class GEntity {
   GRegion *cast2Region();
 
   // periodic data
-  double periodicTransformation[4][4];
+  std::vector<double> affineTransform;
   std::map<MVertex*,MVertex*> correspondingVertices;
 
 };
