@@ -34,8 +34,7 @@ struct surface_params
 class GRegion;
 
 // A model face.
-class GFace : public GEntity
-{
+class GFace : public GEntity{
  protected:
   // edge loops might replace what follows (list of all the edges of
   // the face + directions)
@@ -56,10 +55,13 @@ class GFace : public GEntity
   std::list<GEdgeLoop> edgeLoops;
 
   // periodic counterparts of edges
-  std::map<int,int> edgeCounterparts;
+  std::map<GEdge*,std::pair<GEdge*,int> > edgeCounterparts;
 
-  // encoding of an explicit affine transformation for period meshing
-  std::vector<double> affineTransform;
+  // specify mesh master with transformation, deduce edgeCounterparts
+  void setMeshMaster(GFace* master,const std::vector<double>&);
+
+  // specify mesh master and edgeCounterparts, deduce transformation
+  void setMeshMaster(GFace* master,const std::map<int,int>&);
 
   // an array with additional vertices that are supposed to exist in
   // the final mesh of the model face. This can be used for boundary
