@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -20,7 +20,6 @@
 #include "Geo.h"
 #include "GmshDefines.h"
 
-
 GVertex *GeoFactory::addVertex(GModel *gm, double x, double y, double z, double lc)
 {
   int num =  gm->getMaxElementaryNumber(0) + 1;
@@ -164,7 +163,6 @@ GFace *GeoFactory::addPlanarFace(GModel *gm, std::vector< std::vector<GEdge *> >
 
 GRegion* GeoFactory::addVolume (GModel *gm, std::vector<std::vector<GFace *> > faces)
 {
-
   //create surface loop
   int nLoops = faces.size();
   std::vector<SurfaceLoop *> vecLoops;
@@ -1169,8 +1167,8 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
          && IsEqualG(BarycenterFace1.X()+FaceTranslationVector[0],BarycenterFace2.X())
          && IsEqualG(BarycenterFace1.Y()+FaceTranslationVector[1],BarycenterFace2.Y())
          && IsEqualG(BarycenterFace1.Z()+FaceTranslationVector[2],BarycenterFace2.Z())){
-        numFaceMaster = gm->getOCCInternals()->getGTagOfOCCFaceByNativePtr(gm,aFace1);
-        numFaceSlave  = gm->getOCCInternals()->getGTagOfOCCFaceByNativePtr(gm,aFace2);
+        numFaceMaster = gm->getOCCInternals()->getOCCFaceByNativePtr(gm,aFace1)->tag();
+        numFaceSlave  = gm->getOCCInternals()->getOCCFaceByNativePtr(gm,aFace2)->tag();
         //Msg::Info("Face %d (slave) is most likely Face %d (master) translated by (%.2e,%.2e,%.2e)!",
         //           numFaceSlave,numFaceMaster,FaceTranslationVector[0],FaceTranslationVector[1],
         //           FaceTranslationVector[2]);
@@ -1183,7 +1181,7 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
         for (TopExp_Explorer aEdgeExplorer1(aFace1,TopAbs_EDGE); aEdgeExplorer1.More();
              aEdgeExplorer1.Next()) {
           TopoDS_Edge aEdge1 = TopoDS::Edge(aEdgeExplorer1.Current());
-          int numEdgeMaster  = gm->getOCCInternals()->getGTagOfOCCEdgeByNativePtr(gm,aEdge1);
+          int numEdgeMaster  = gm->getOCCInternals()->getOCCEdgeByNativePtr(gm,aEdge1)->tag();
           EdgeListMaster[i1] = numEdgeMaster;
           //i2=0;
           for (TopExp_Explorer aEdgeExplorer2(aFace2,TopAbs_EDGE); aEdgeExplorer2.More();
@@ -1201,7 +1199,7 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
                && IsEqualG(BarycenterEdge1.X()+FaceTranslationVector[0],BarycenterEdge2.X())
                && IsEqualG(BarycenterEdge1.Y()+FaceTranslationVector[1],BarycenterEdge2.Y())
                && IsEqualG(BarycenterEdge1.Z()+FaceTranslationVector[2],BarycenterEdge2.Z())){
-              int numEdgeSlave   = gm->getOCCInternals()->getGTagOfOCCEdgeByNativePtr(gm,aEdge2);
+              int numEdgeSlave   = gm->getOCCInternals()->getOCCEdgeByNativePtr(gm,aEdge2)->tag();
               EdgeListSlave[i1]  = numEdgeSlave;
             }
           }
@@ -1222,167 +1220,40 @@ void OCCFactory::setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTransla
 
 /* setPeriodicPairOfFaces: set periodic given a Slave/Master pair of
    numFace-Edgelist */
-void OCCFactory::setPeriodicPairOfFaces(GModel *gm, int numFaceMaster, std::vector<int> EdgeListMaster,
-                                        int numFaceSlave, std::vector<int> EdgeListSlave)
+void OCCFactory::setPeriodicPairOfFaces(GModel *gm, int numFaceMaster,
+                                        std::vector<int> EdgeListMaster,
+                                        int numFaceSlave,
+                                        std::vector<int> EdgeListSlave)
 {
-  int NEdges=EdgeListMaster.size();
   if (EdgeListMaster.size() != EdgeListSlave.size()){
     Msg::Error("Slave/Master faces don't have the same number of edges!");
   }
-  else{
+  else {
+
+    std::map<int,int> edgeCounterparts;
+    std::vector<int>::iterator siter = EdgeListSlave.begin();
+    std::vector<int>::iterator miter = EdgeListMaster.begin();
+
+    for (;siter!=EdgeListSlave.end();++siter,++miter) {
+      edgeCounterparts[*siter] = *miter;
+    }
+
     Surface *s_slave = FindSurface(abs(numFaceSlave));
     if(s_slave){
-      GModel::current()->getGEOInternals()->periodicFaces[numFaceSlave] = numFaceMaster;
-      for (int i = 0; i < NEdges; i++){
-        GModel::current()->getGEOInternals()->periodicEdges[EdgeListSlave[i]] = EdgeListMaster[i];
-        s_slave->edgeCounterparts[EdgeListSlave[i]] = EdgeListMaster[i];
-      }
+      s_slave->master = numFaceMaster;
+      s_slave->edgeCounterparts = edgeCounterparts;
     }
     else{
       GFace *gf = GModel::current()->getFaceByTag(abs(numFaceSlave));
-      if(gf){
-        gf->setMeshMaster(numFaceMaster);
-        for (int i = 0; i < NEdges; i++){
-          gf->edgeCounterparts[EdgeListSlave[i]] = EdgeListMaster[i];
-          GEdge *ges = GModel::current()->getEdgeByTag(abs(EdgeListSlave[i]));
-          ges->setMeshMaster(EdgeListMaster[i]);
-        }
+      if (gf) {
+        GFace *master = GModel::current()->getFaceByTag(abs(numFaceMaster));
+        gf->setMeshMaster(master,edgeCounterparts);
       }
       else Msg::Error("Slave surface %d not found", numFaceSlave);
     }
   }
 }
 
-/* setPhysicalNumToEntitiesInBox allows to set a physical number to all entities
-   of a given type (0:vertex, 1:edge, 2:face, 3:volume) lying inside a 3D box
-   defined by 2 points */
-void OCCFactory::setPhysicalNumToEntitiesInBox(GModel *gm, int EntityType, int PhysicalGroupNumber,
-                                               std::vector<double> p1, std::vector<double> p2)
-{
-  std::vector<int> ListOfGVerticeTagsInbox;
-  std::vector<int> ListOfGEdgesTagsInbox;
-  std::vector<int> ListOfGFacesTagsInbox;
-  std::vector<int> ListOfGRegionsTagsInbox;
-
-  Standard_Real aXmin, aYmin, aZmin, aXmax, aYmax, aZmax;
-  TopoDS_Shape shape = gm->_occ_internals->getShape();
-  switch (EntityType) {
-  case 0 :
-    for(TopExp_Explorer aVertexExplorer(shape, TopAbs_VERTEX); aVertexExplorer.More(); aVertexExplorer.Next()){
-      TopoDS_Vertex aVertex = TopoDS::Vertex(aVertexExplorer.Current());
-      Bnd_Box VertexBB;
-      BRepBndLib::Add(aVertex,VertexBB);
-      VertexBB.Get(aXmin, aYmin, aZmin, aXmax, aYmax, aZmax);
-      if(aXmin>p1[0]
-         && aYmin>p1[1]
-         && aZmin>p1[2]
-         && aXmax<p2[0]
-         && aYmax<p2[1]
-         && aZmax<p2[2]){
-        int GVertexTag = gm->getOCCInternals()->getGTagOfOCCVertexByNativePtr(gm,aVertex);
-        //Msg::Info("This volume %d (xmin,ymin,zmin)=(%lf,%lf,%lf)  (xmax,ymax,zmax)=(%lf,%lf,%lf)",GVertexTag);
-        ListOfGVerticeTagsInbox.push_back(GVertexTag);
-      }
-    }
-    Msg::Info("These edges have OCC bounding boxes inside the given box!");
-    for (std::vector<int>::iterator it = ListOfGVerticeTagsInbox.begin() ;
-         it != ListOfGVerticeTagsInbox.end(); ++it){
-      Msg::Info("- %d",*it);
-      GVertex *gv = gm->getVertexByTag(*it);
-      if(gv){
-        gv->addPhysicalEntity(PhysicalGroupNumber);
-      }
-    }
-    break;
-  case 1 :
-    for(TopExp_Explorer aEdgeExplorer(shape, TopAbs_EDGE); aEdgeExplorer.More(); aEdgeExplorer.Next()){
-      TopoDS_Edge aEdge = TopoDS::Edge(aEdgeExplorer.Current());
-      Bnd_Box EdgeBB;
-      BRepBndLib::Add(aEdge,EdgeBB);
-      EdgeBB.Get(aXmin, aYmin, aZmin, aXmax, aYmax, aZmax);
-
-      if(aXmin>p1[0]
-         && aYmin>p1[1]
-         && aZmin>p1[2]
-         && aXmax<p2[0]
-         && aYmax<p2[1]
-         && aZmax<p2[2]){
-        int GEdgeTag = gm->getOCCInternals()->getGTagOfOCCEdgeByNativePtr(gm,aEdge);
-        //Msg::Info("This edge %d (xmin,ymin,zmin)=(%lf,%lf,%lf)  (xmax,ymax,zmax)=(%lf,%lf,%lf)",GEdgeTag);
-        ListOfGEdgesTagsInbox.push_back(GEdgeTag);
-      }
-    }
-    Msg::Info("These edges have OCC bounding boxes inside the given box!");
-    for (std::vector<int>::iterator it = ListOfGEdgesTagsInbox.begin() ; it != ListOfGEdgesTagsInbox.end(); ++it){
-      Msg::Info("- %d",*it);
-      GEdge *ge = gm->getEdgeByTag(*it);
-      if(ge){
-        ge->addPhysicalEntity(PhysicalGroupNumber);
-      }
-    }
-    break;
-  case 2 :
-    for(TopExp_Explorer aFaceExplorer(shape, TopAbs_FACE); aFaceExplorer.More(); aFaceExplorer.Next()){
-      TopoDS_Face aFace = TopoDS::Face(aFaceExplorer.Current());
-      Bnd_Box FaceBB;
-      BRepBndLib::Add(aFace,FaceBB);
-      FaceBB.Get(aXmin, aYmin, aZmin, aXmax, aYmax, aZmax);
-
-      if(aXmin>p1[0]
-         && aYmin>p1[1]
-         && aZmin>p1[2]
-         && aXmax<p2[0]
-         && aYmax<p2[1]
-         && aZmax<p2[2]){
-        int GFaceTag = gm->getOCCInternals()->getGTagOfOCCFaceByNativePtr(gm,aFace);
-        //Msg::Info("This face %d (xmin,ymin,zmin)=(%lf,%lf,%lf)  (xmax,ymax,zmax)=(%lf,%lf,%lf)",GFaceTag);
-        ListOfGFacesTagsInbox.push_back(GFaceTag);
-      }
-    }
-    Msg::Info("This faces have OCC bounding boxes inside the given box!");
-    for (std::vector<int>::iterator it = ListOfGFacesTagsInbox.begin() ;
-         it != ListOfGFacesTagsInbox.end(); ++it){
-      Msg::Info("- %d",*it);
-      GFace *gf = gm->getFaceByTag(*it);
-      if(gf){
-        gf->addPhysicalEntity(PhysicalGroupNumber);
-      }
-    }
-    break;
-  case 3 :
-    for(TopExp_Explorer aSolidExplorer(shape, TopAbs_SOLID); aSolidExplorer.More();
-        aSolidExplorer.Next()){
-      TopoDS_Solid aSolid = TopoDS::Solid(aSolidExplorer.Current());
-      Bnd_Box SolidBB;
-      BRepBndLib::Add(aSolid,SolidBB);
-      SolidBB.Get(aXmin, aYmin, aZmin, aXmax, aYmax, aZmax);
-      if(aXmin>p1[0]
-         && aYmin>p1[1]
-         && aZmin>p1[2]
-         && aXmax<p2[0]
-         && aYmax<p2[1]
-         && aZmax<p2[2]
-         ){
-        int GSolidTag = gm->getOCCInternals()->getGTagOfOCCSolidByNativePtr(gm,aSolid);
-        //Msg::Info("This volume %d (xmin,ymin,zmin)=(%lf,%lf,%lf)  (xmax,ymax,zmax)=(%lf,%lf,%lf)",GSolidTag);
-        ListOfGRegionsTagsInbox.push_back(GSolidTag);
-      }
-    }
-    Msg::Info("These edges have OCC bounding boxes inside the given box!");
-    for (std::vector<int>::iterator it = ListOfGRegionsTagsInbox.begin() ;
-         it != ListOfGRegionsTagsInbox.end(); ++it){
-      Msg::Info("- %d",*it);
-      GRegion *gr = gm->getRegionByTag(*it);
-      if(gr){
-        gr->addPhysicalEntity(PhysicalGroupNumber);
-      }
-    }
-    break;
-  default:
-    Msg::Error("First argument of setPhysicalNumToEntitiesInBox should be 0,1,2 or 3 (Vertice, Edge, Surface or Volume)");
-  }
-}
-
 void OCCFactory::fillet(GModel *gm, std::vector<int> edges, double radius)
 {
   try{
