@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -210,15 +210,7 @@ class GModelFactory {
     Msg::Error("setPeriodicPairOfFaces not implemented yet");
   }
   ;
-  virtual void setPhysicalNumToEntitiesInBox(GModel *gm, int EntityType,
-                                             int PhysicalGroupNumber,
-                                             std::vector<double> p1,std::vector<double> p2)
-  {
-    Msg::Error("setPhysicalNumToEntitiesInBox not implemented yet");
-  }
-
   virtual void healGeometry(GModel *gm, double tolerance = -1.) = 0;
-
 };
 
 class GeoFactory : public GModelFactory {
@@ -287,9 +279,6 @@ class OCCFactory : public GModelFactory {
   void setPeriodicAllFaces(GModel *gm, std::vector<double> FaceTranslationVector);
   void setPeriodicPairOfFaces(GModel *gm, int numFaceMaster, std::vector<int> EdgeListMaster,
                               int numFaceSlave, std::vector<int> EdgeListSlave);
-  void setPhysicalNumToEntitiesInBox(GModel *gm, int EntityType, int PhysicalGroupNumber,
-                                     std::vector<double> p1, std::vector<double> p2);
-
   void fillet(GModel *gm, std::vector<int> edges, double radius);
   void healGeometry(GModel *gm, double tolerance = -1.);
 };
