@@ -332,31 +332,44 @@ int GModel::importGEOInternals()
     }
   }
 
-  // create periodic mesh relationships
-  for(std::map<int,int>::iterator it = _geo_internals->periodicEdges.begin();
-      it != _geo_internals->periodicEdges.end(); ++it){
-    GEdge *ge = getEdgeByTag(abs(it->first));
-    if (ge){
-      int MASTER = it->second * (it->first > 0 ? 1 : -1);
-      ge->setMeshMaster(MASTER);
-    }
+  std::map<int,GEO_Internals::MasterEdge>::iterator peIter =
+    _geo_internals->periodicEdges.begin();
+  for (;peIter!=_geo_internals->periodicEdges.end();++peIter) {
+    int iTarget = peIter->first;
+    GEO_Internals::MasterEdge& me = peIter->second;
+    int iSource = me.tag;
+    GEdge* target = getEdgeByTag(iTarget);
+    GEdge* source = getEdgeByTag(abs(iSource));
+    if (!target)
+      Msg::Error("Unknown target line for periodic connection from %d to %d",
+                 iTarget, iSource);
+    if (!source)
+      Msg::Error("Unknown source line for periodic connection from %d to %d",
+                 iTarget, iSource);
+    if (me.affineTransform.size()==16)
+      target->setMeshMaster(source,me.affineTransform);
+    else
+      target->setMeshMaster(source,me.tag > 0 ? 1 : -1);
   }
-  for(std::map<int,int>::iterator it = _geo_internals->periodicFaces.begin();
-      it != _geo_internals->periodicFaces.end(); ++it){
-    GFace *gf = getFaceByTag(abs(it->first));
-    if (gf)gf->setMeshMaster(it->second * (it->first > 0 ? 1 : -1));
-  }
-  for(eiter it = firstEdge() ; it != lastEdge() ; ++it){
-    int meshMaster = (*it)->meshMaster();
-    if (meshMaster != (*it)->tag()){
-      GEdge *ge_master = getEdgeByTag(abs(meshMaster));
-      if(ge_master)
-        (*it)->getBeginVertex()->setMeshMaster((meshMaster > 0) ? ge_master->getBeginVertex()->tag() :
-                                               ge_master->getEndVertex()->tag());
-      if(ge_master)
-        (*it)->getEndVertex()->setMeshMaster((meshMaster < 0) ? ge_master->getBeginVertex()->tag() :
-                                             ge_master->getEndVertex()->tag());
-    }
+
+  std::map<int,GEO_Internals::MasterFace>::iterator pfIter =
+    _geo_internals->periodicFaces.begin();
+  for (;pfIter!=_geo_internals->periodicFaces.end();++pfIter) {
+    int iTarget = pfIter->first;
+    GEO_Internals::MasterFace& mf = pfIter->second;
+    int iSource = mf.tag;
+    GFace* target = getFaceByTag(iTarget);
+    GFace* source = getFaceByTag(iSource);
+    if (!target)
+      Msg::Error("Unknown target surface for periodic connection from %d to %d",
+                 iTarget, iSource);
+    if (!source)
+      Msg::Error("Unknown source surface for periodic connection from %d to %d",
+                 iTarget, iSource);
+    if (mf.affineTransform.size()==16)
+      target->setMeshMaster(source,mf.affineTransform);
+    else
+      target->setMeshMaster(source,mf.edgeCounterparts);
   }
 
   Msg::Debug("Gmsh model (GModel) imported:");
@@ -364,7 +377,6 @@ int GModel::importGEOInternals()
   Msg::Debug("%d Edges", edges.size());
   Msg::Debug("%d Faces", faces.size());
   Msg::Debug("%d Regions", regions.size());
-
   return 1;
 }
 
@@ -483,6 +495,10 @@ static bool skipVertex(GVertex *gv)
 int GModel::writeGEO(const std::string &name, bool printLabels, bool onlyPhysicals)
 {
   FILE *fp = Fopen(name.c_str(), "w");
+  if(!fp){
+    Msg::Error("Could not open file '%s'", name.c_str());
+    return 0;
+  }
 
   std::map<double, std::string> meshSizeParameters;
   int cpt = 0;
@@ -533,6 +549,6 @@ int GModel::writeGEO(const std::string &name, bool printLabels, bool onlyPhysica
   if(getFields()->getBackgroundField() > 0)
     fprintf(fp, "Background Field = %i;\n", getFields()->getBackgroundField());
 
-  if(fp) fclose(fp);
+  fclose(fp);
   return 1;
 }
