@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -22,6 +22,8 @@
 #include "Partition_Spliter.hxx"
 #endif
 
+#include <BRepBuilderAPI_Transform.hxx>
+
 void addSimpleShapes(TopoDS_Shape theShape, TopTools_ListOfShape &theList);
 
 void OCC_Internals::buildLists()
@@ -35,7 +37,8 @@ void OCC_Internals::buildLists()
   addShapeToLists(shape);
 }
 
-void  OCC_Internals::buildShapeFromGModel(GModel* gm){
+void OCC_Internals::buildShapeFromGModel(GModel* gm)
+{
   somap.Clear();
   shmap.Clear();
   fmap.Clear();
@@ -45,13 +48,13 @@ void  OCC_Internals::buildShapeFromGModel(GModel* gm){
   for (GModel::riter it = gm->firstRegion(); it != gm->lastRegion() ; ++it){
     if ((*it)->getNativeType() == GEntity::OpenCascadeModel){
       OCCRegion *occ = static_cast<OCCRegion*> (*it);
-      if (occ)addShapeToLists (occ->getTopoDS_Shape());
+      if(occ) addShapeToLists(occ->getTopoDS_Shape());
     }
   }
   for (GModel::fiter it = gm->firstFace(); it != gm->lastFace() ; ++it){
     if ((*it)->getNativeType() == GEntity::OpenCascadeModel){
       OCCFace *occ = static_cast<OCCFace*> (*it);
-      if(occ)addShapeToLists (occ->getTopoDS_Face ());
+      if(occ) addShapeToLists(occ->getTopoDS_Face());
     }
   }
   BRep_Builder B;
@@ -240,9 +243,17 @@ void OCC_Internals::addShapeToLists(TopoDS_Shape _shape)
 
 void OCC_Internals::healGeometry(double tolerance, bool fixdegenerated,
                                  bool fixsmalledges, bool fixspotstripfaces,
-                                 bool sewfaces, bool makesolids, bool connect)
+                                 bool sewfaces, bool makesolids, int connect,
+                                 double scaling)
 {
-  
+  if(scaling != 1.0){
+    Msg::Info("Scaling geometry by factor %g", scaling);
+    gp_Trsf t;
+    t.SetScaleFactor(scaling);
+    BRepBuilderAPI_Transform trsf(shape, t);
+    shape = trsf.Shape();
+  }
+
   if(!fixdegenerated && !fixsmalledges && !fixspotstripfaces &&
      !sewfaces && !makesolids && !connect) return;
 
@@ -528,8 +539,8 @@ void OCC_Internals::healGeometry(double tolerance, bool fixdegenerated,
     }
   }
 
-  if(connect){
 #if defined(HAVE_SALOME)
+  if(connect == 2){
     Msg::Info("- cutting and connecting faces with Salome's Partition_Spliter");
     TopExp_Explorer e2;
     Partition_Spliter ps;
@@ -542,14 +553,16 @@ void OCC_Internals::healGeometry(double tolerance, bool fixdegenerated,
     catch(Standard_Failure &err){
       Msg::Error("%s", err.GetMessageString());
     }
-#else
+  }
+  else
+#endif
+  if(connect){
     Msg::Info("- cutting and connecting faces with OCC_Connect");
     OCC_Connect connect(1);
     for(TopExp_Explorer p(shape, TopAbs_SOLID); p.More(); p.Next())
       connect.Add(p.Current());
     connect.Connect();
     shape = connect;
-#endif
   }
 
   double newsurfacecont = 0;
@@ -591,7 +604,8 @@ void OCC_Internals::loadBREP(const char *fn)
                CTX::instance()->geom.occFixSmallFaces,
                CTX::instance()->geom.occSewFaces,
                false,
-               CTX::instance()->geom.occConnectFaces);
+               CTX::instance()->geom.occConnectFaces,
+               CTX::instance()->geom.occScaling);
   BRepTools::Clean(shape);
   buildLists();
 }
@@ -623,7 +637,8 @@ void OCC_Internals::loadSTEP(const char *fn)
                CTX::instance()->geom.occFixSmallFaces,
                CTX::instance()->geom.occSewFaces,
                false,
-               CTX::instance()->geom.occConnectFaces);
+               CTX::instance()->geom.occConnectFaces,
+               CTX::instance()->geom.occScaling);
   BRepTools::Clean(shape);
   buildLists();
 }
@@ -650,7 +665,8 @@ void OCC_Internals::loadIGES(const char *fn)
                CTX::instance()->geom.occFixSmallFaces,
                CTX::instance()->geom.occSewFaces,
                false,
-               CTX::instance()->geom.occConnectFaces);
+               CTX::instance()->geom.occConnectFaces,
+               CTX::instance()->geom.occScaling);
   BRepTools::Clean(shape);
   buildLists();
 }
@@ -737,35 +753,6 @@ GRegion* OCC_Internals::addRegionToModel(GModel *model, TopoDS_Solid region)
   //  return getOCCRegionByNativePtr(model, region);
 }
 
-/* I needed getGTagOfOCC*ByNativePtr whithin setPhysicalNumToEntitiesInBox */
-int OCC_Internals::getGTagOfOCCVertexByNativePtr(GModel *model, TopoDS_Vertex toFind)
-{
-  if(gvNumCache.IsBound(toFind))
-    return (int)gvNumCache.Find(toFind);
-  return 0;
-}
-
-int OCC_Internals::getGTagOfOCCFaceByNativePtr(GModel *model, TopoDS_Face toFind)
-{
-  if(gfNumCache.IsBound(toFind))
-    return (int)gfNumCache.Find(toFind);
-  return 0;
-}
-
-int OCC_Internals::getGTagOfOCCEdgeByNativePtr(GModel *model, TopoDS_Edge toFind)
-{
-  if(geNumCache.IsBound(toFind))
-    return (int)geNumCache.Find(toFind);
-  return 0;
-}
-
-int OCC_Internals::getGTagOfOCCSolidByNativePtr(GModel *model, TopoDS_Solid toFind)
-{
-  if(grNumCache.IsBound(toFind))
-    return (int)grNumCache.Find(toFind);
-  return 0;
-}
-
 void OCC_Internals::buildGModel(GModel *model)
 {
   // building geom vertices
@@ -1079,26 +1066,27 @@ int GModel::readOCCIGES(const std::string &fn)
 
 int GModel::writeOCCBREP(const std::string &fn)
 {
-  _occ_internals->buildShapeFromGModel(this);
-  
   if(!_occ_internals){
     Msg::Error("No OpenCASCADE model found");
     return 0;
   }
-  else
+  else{
+    _occ_internals->buildShapeFromGModel(this);
     _occ_internals->writeBREP(fn.c_str());
+  }
   return 1;
 }
 
 int GModel::writeOCCSTEP(const std::string &fn)
 {
-  _occ_internals->buildShapeFromGModel(this);
   if(!_occ_internals){
     Msg::Error("No OpenCASCADE model found");
     return 0;
   }
-  else
+  else{
+    _occ_internals->buildShapeFromGModel(this);
     _occ_internals->writeSTEP(fn.c_str());
+  }
   return 1;
 }
 
