@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -17,14 +17,14 @@
 #include "SBoundingBox3d.h"
 #include "OS.h"
 #include "MVertex.h"
-#include "MVertexPositionSet.h"
+#include "MVertexRTree.h"
 
 #if defined(HAVE_SOLVER)
 #include "linearSystem.h"
 #endif
 
 #if defined(HAVE_ANN)
-#include <ANN/ANN.h>
+#include "ANN/ANN.h"
 #endif
 
 static void SphereBB(void *a, double*mmin, double*mmax)
@@ -61,27 +61,29 @@ static int SphereInEle(void *a, double*c)
 static void printNodes(std::set<MVertex *> myNodes)
 {
   FILE * xyz = Fopen("myNodes.pos","w");
-  fprintf(xyz,"View \"\"{\n");
-  for(std::set<MVertex *>::iterator itv = myNodes.begin(); itv !=myNodes.end(); ++itv){
-    MVertex *v = *itv;
-    fprintf(xyz,"SP(%g,%g,%g){%d};\n", v->x(), v->y(), v->z(), v->getNum());
+  if(xyz){
+    fprintf(xyz,"View \"\"{\n");
+    for(std::set<MVertex *>::iterator itv = myNodes.begin(); itv !=myNodes.end(); ++itv){
+      MVertex *v = *itv;
+      fprintf(xyz,"SP(%g,%g,%g){%d};\n", v->x(), v->y(), v->z(), v->getNum());
+    }
+    fprintf(xyz,"};\n");
+    fclose(xyz);
   }
-  fprintf(xyz,"};\n");
-  fclose(xyz);
 }
 
 static void exportParametrizedMesh(fullMatrix<double> &UV, int nbNodes)
 {
-  FILE *f = Fopen ("UV.pos", "w");
-  fprintf(f,"View  \" uv \" {\n");
-
-  Msg::Info("*** RBF exporting 'UV.pos' ");
-  for(int id = 0; id < nbNodes; id++){
-    fprintf(f,"SP(%g,%g,%g){%d};\n", UV(id,0), UV(id,1), 0.0, id);
+  FILE *f = Fopen("UV.pos", "w");
+  if(f){
+    fprintf(f,"View  \" uv \" {\n");
+    Msg::Info("*** RBF exporting 'UV.pos' ");
+    for(int id = 0; id < nbNodes; id++){
+      fprintf(f,"SP(%g,%g,%g){%d};\n", UV(id,0), UV(id,1), 0.0, id);
+    }
+    fprintf(f,"};\n");
+    fclose(f);
   }
-  fprintf(f,"};\n");
-
-  fclose(f);
 }
 
 GRbf::GRbf(double sizeBox, int variableEps, int rbfFun,
@@ -107,22 +109,21 @@ GRbf::GRbf(double sizeBox, int variableEps, int rbfFun,
   }
 
   // then create Mvertex position
-  std::vector<MVertex*> vertices( allNodes.begin(), allNodes.end() );
-  MVertexPositionSet pos(vertices);
+  std::vector<MVertex*> vertices(allNodes.begin(), allNodes.end());
+  MVertexRTree pos(tol);
+  pos.insert(vertices);
   for(unsigned int i = 0; i < vertices.size(); i++){
     MVertex *v = vertices[i];
-    pos.find(v->x(), v->y(), v->z(), tol);
+    if(!pos.find(v->x(), v->y(), v->z()))
+      myNodes.insert(v); // keep only no duplicate vertices
     allCenters(i,0) = v->x()/sBox;
     allCenters(i,1) = v->y()/sBox;
     allCenters(i,2) = v->z()/sBox;
     _mapAllV.insert(std::make_pair(v, i));
   }
-  // keep only no duplicate vertices
-  for(unsigned int i = 0; i < vertices.size(); i++)
-    if(vertices[i]->getIndex()) myNodes.insert(vertices[i]);
 
   // initialize with  points
-  nbNodes= myNodes.size();
+  nbNodes = myNodes.size();
   centers.resize(nbNodes,3);
   normals.resize(nbNodes,3);
   int index = 0;
