@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -13,6 +13,7 @@
 
 GVertex::GVertex(GModel *m, int tag, double ms) : GEntity(m, tag), meshSize(ms)
 {
+  resetMeshAttributes();
 }
 
 GVertex::~GVertex()
@@ -30,6 +31,11 @@ void GVertex::deleteMesh()
   model()->destroyMeshCaches();
 }
 
+void GVertex::resetMeshAttributes()
+{
+  meshSize = MAX_LC;
+}
+
 void GVertex::setPosition(GPoint &p)
 {
   Msg::Error("Cannot set position of this kind of vertex");
@@ -55,6 +61,7 @@ SPoint2 GVertex::reparamOnFace(const GFace *gf, int) const
 std::string GVertex::getAdditionalInfoString()
 {
   std::ostringstream sstream;
+  sstream.precision(12);
   sstream << "{" << x() << "," << y() << "," << z() << "}";
   double lc = prescribedMeshSizeAtVertex();
   if(lc < MAX_LC) sstream << " (mesh size: " << lc << ")";
