@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -70,10 +70,10 @@ class MPolyhedron : public MElement {
   {
     return (num < (int)_vertices.size()) ? _vertices[num] : _innerVertices[num - _vertices.size()];
   }
-  virtual int getNumEdges() { return _edges.size(); }
+  virtual int getNumEdges()const { return _edges.size(); }
   virtual MEdge getEdge(int num) const{ return _edges[num]; }
-  virtual int getNumEdgesRep() { return _edges.size(); }
-  virtual void getEdgeRep(int num, double *x, double *y, double *z, SVector3 *n)
+  virtual int getNumEdgesRep(bool curved) { return _edges.size(); }
+  virtual void getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
   {
     MEdge e(getEdge(num));
     for(unsigned int i = 0; i < _faces.size(); i++)
@@ -89,8 +89,8 @@ class MPolyhedron : public MElement {
   }
   virtual int getNumFaces() { return _faces.size(); }
   virtual MFace getFace(int num) { return _faces[num]; }
-  virtual int getNumFacesRep() { return _faces.size(); }
-  virtual void getFaceRep(int num, double *x, double *y, double *z, SVector3 *n)
+  virtual int getNumFacesRep(bool curved) { return _faces.size(); }
+  virtual void getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
   {
     _getFaceRep(_faces[num].getVertex(0), _faces[num].getVertex(1),
                 _faces[num].getVertex(2), x, y, z, n);
@@ -121,9 +121,9 @@ class MPolyhedron : public MElement {
       vol += _parts[i]->getVolume();
     return vol;
   }
-  virtual const nodalBasis* getFunctionSpace(int order=-1) const
+  virtual const nodalBasis* getFunctionSpace(int order=-1, bool serendip=false) const
   {
-    return (_orig ? _orig->getFunctionSpace(order) : 0);
+    return (_orig ? _orig->getFunctionSpace(order, serendip) : 0);
   }
   virtual const JacobianBasis* getJacobianFuncSpace(int order=-1) const
   {
@@ -145,7 +145,7 @@ class MPolyhedron : public MElement {
   {
     return (_orig ? _orig->getNumShapeFunctions() : 0);
   }
-  virtual int getNumPrimaryShapeFunctions()
+  virtual int getNumPrimaryShapeFunctions() const
   {
     return (_orig ? _orig->getNumPrimaryShapeFunctions() : 0);
   }
@@ -227,10 +227,10 @@ class MPolygon : public MElement {
   {
     return (num < (int)_vertices.size()) ? _vertices[num] : _innerVertices[num - _vertices.size()];
   }
-  virtual int getNumEdges() { return _edges.size(); }
+  virtual int getNumEdges()const { return _edges.size(); }
   virtual MEdge getEdge(int num) const{ return _edges[num]; }
-  virtual int getNumEdgesRep() { return getNumEdges(); }
-  virtual void getEdgeRep(int num, double *x, double *y, double *z, SVector3 *n)
+  virtual int getNumEdgesRep(bool curved) { return getNumEdges(); }
+  virtual void getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
   {
     MEdge e(getEdge(num));
     _getEdgeRep(e.getVertex(0), e.getVertex(1), x, y, z, n, 0);
@@ -243,8 +243,8 @@ class MPolygon : public MElement {
   }
   virtual int getNumFaces() { return 1; }
   virtual MFace getFace(int num) { return MFace(_vertices); }
-  virtual int getNumFacesRep() { return _parts.size(); }
-  virtual void getFaceRep(int num, double *x, double *y, double *z, SVector3 *n)
+  virtual int getNumFacesRep(bool curved) { return _parts.size(); }
+  virtual void getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
   {
     _getFaceRep(_parts[num]->getVertex(0), _parts[num]->getVertex(1),
                 _parts[num]->getVertex(2), x, y, z, n);
@@ -271,9 +271,9 @@ class MPolygon : public MElement {
   virtual int getNumChildren() const { return _parts.size(); }
   virtual MElement *getChild(int i) const { return _parts[i]; }
   virtual bool ownsParent() const { return _owner; }
-  virtual const nodalBasis* getFunctionSpace(int order=-1) const
+  virtual const nodalBasis* getFunctionSpace(int order=-1, bool serendip=false) const
   {
-    return (_orig ? _orig->getFunctionSpace(order) : 0);
+    return (_orig ? _orig->getFunctionSpace(order, serendip) : 0);
   }
   virtual const JacobianBasis* getJacobianFuncSpace(int order=-1) const
   {
@@ -295,7 +295,7 @@ class MPolygon : public MElement {
   {
     return (_orig ? _orig->getNumShapeFunctions() : 0);
   }
-  virtual int getNumPrimaryShapeFunctions()
+  virtual int getNumPrimaryShapeFunctions() const
   {
     return (_orig ? _orig->getNumPrimaryShapeFunctions() : 0);
   }
@@ -341,9 +341,9 @@ class MLineChild : public MLine {
       delete _orig;
   }
   virtual int getTypeForMSH() const { return MSH_LIN_C; }
-  virtual const nodalBasis* getFunctionSpace(int order=-1) const
+  virtual const nodalBasis* getFunctionSpace(int order=-1, bool serendip=false) const
   {
-    if(_orig) return _orig->getFunctionSpace(order);
+    if(_orig) return _orig->getFunctionSpace(order, serendip);
     return 0;
   }
   virtual const JacobianBasis* getJacobianFuncSpace(int order=-1) const
