@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -7,10 +7,12 @@
 #include "Numeric.h"
 #include "Context.h"
 #include "BasisFactory.h"
-#include "nodalBasis.h"
 #include "polynomialBasis.h"
 #include "MQuadrangle.h"
+
+#if defined(HAVE_MESH)
 #include "qualityMeasures.h"
+#endif
 
 int MHexahedron::getVolumeSign()
 {
@@ -38,34 +40,10 @@ void MHexahedron::getIntegrationPoints(int pOrder, int *npts, IntPt **pts)
 
 double MHexahedron::angleShapeMeasure()
 {
-
 #if defined(HAVE_MESH)
-   double angleMax = 0.0;
-   double angleMin = M_PI;
-   double zeta = 0.0;
-   for (int i=0; i<getNumFaces(); i++){
-     std::vector<MVertex*> vv;
-     vv.push_back(getFace(i).getVertex(0));
-     vv.push_back(getFace(i).getVertex(1));
-     vv.push_back(getFace(i).getVertex(2));
-     vv.push_back(getFace(i).getVertex(3));
-     // MVertex *v0 = new MVertex(0, 0, 0); vv.push_back(v0);
-     // MVertex *v1 = new MVertex(1., 0, 0);vv.push_back(v1);
-     // MVertex *v2 = new MVertex(2., 1., 0);vv.push_back(v2);
-     // MVertex *v3 = new MVertex(1, 1., 0);vv.push_back(v3);
-     for (int j=0; j<4; j++){
-       SVector3 a(vv[(j+2)%4]->x()-vv[(j+1)%4]->x(),vv[(j+2)%4]->y()-vv[(j+1)%4]->y(),vv[(j+2)%4]->z()-vv[(j+1)%4]->z()  );
-       SVector3 b(vv[(j+1)%4]->x()-vv[(j)%4]->x(),  vv[(j+1)%4]->y()-vv[(j)%4]->y(),  vv[(j+1)%4]->z()-vv[(j)%4]->z()  );
-       double angle = acos( dot(a,b)/(norm(a)*norm(b))); //*180/M_PI;
-       angleMax = std::max(angleMax, angle);
-       angleMin = std::min(angleMin, angle);
-     }
-     //printf("angle max =%g min =%g \n", angleMax*180/M_PI, angleMin*180/M_PI);
-   }
-   zeta = 1.-std::max((angleMax-0.5*M_PI)/(0.5*M_PI),(0.5*M_PI-angleMin)/(0.5*M_PI));
-   return zeta;
+  return qmHexahedron::angles(this);
 #else
-   return 1.;
+   return 0.;
 #endif
 
 }
@@ -74,7 +52,8 @@ double MHexahedron::getInnerRadius()
   //Only for vertically aligned elements (not inclined)
   double innerRadius=std::numeric_limits<double>::max();
   for (int i=0; i<getNumFaces(); i++){
-    MQuadrangle quad(getFace(i).getVertex(0), getFace(i).getVertex(1), getFace(i).getVertex(2), getFace(i).getVertex(3));
+    MQuadrangle quad(getFace(i).getVertex(0), getFace(i).getVertex(1),
+                     getFace(i).getVertex(2), getFace(i).getVertex(3));
     innerRadius=std::min(innerRadius,quad.getInnerRadius());
   }
   return innerRadius;
@@ -163,17 +142,23 @@ static void _myGetEdgeRep(MHexahedron *hex, int num, double *x, double *y, doubl
   n[0] = n[1] = 1 ;
 }
 
-void MHexahedron20::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n) {
+void MHexahedron20::getEdgeRep(bool curved, int num,
+                               double *x, double *y, double *z, SVector3 *n)
+{
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MHexahedron::getEdgeRep(false, num, x, y, z, n);
 }
 
-void MHexahedron27::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n) {
+void MHexahedron27::getEdgeRep(bool curved, int num,
+                               double *x, double *y, double *z, SVector3 *n)
+{
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MHexahedron::getEdgeRep(false, num, x, y, z, n);
 }
 
-void MHexahedronN::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n) {
+void MHexahedronN::getEdgeRep(bool curved, int num,
+                              double *x, double *y, double *z, SVector3 *n)
+{
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MHexahedron::getEdgeRep(false, num, x, y, z, n);
 }
@@ -368,19 +353,22 @@ void _myGetFaceRep(MHexahedron *hex, int num, double *x, double *y, double *z,
 }
 
 
-void MHexahedron20::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MHexahedron20::getFaceRep(bool curved, int num,
+                               double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MHexahedron::getFaceRep(false, num, x, y, z, n);
 }
 
-void MHexahedron27::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MHexahedron27::getFaceRep(bool curved, int num,
+                               double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MHexahedron::getFaceRep(false, num, x, y, z, n);
 }
 
-void MHexahedronN::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MHexahedronN::getFaceRep(bool curved, int num,
+                              double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MHexahedron::getFaceRep(false, num, x, y, z, n);
@@ -388,16 +376,19 @@ void MHexahedronN::getFaceRep(bool curved, int num, double *x, double *y, double
 
 int MHexahedron20::getNumFacesRep(bool curved)
 {
-  return curved ? 6 * (CTX::instance()->mesh.numSubEdges * CTX::instance()->mesh.numSubEdges * 2) : 12;
+  return curved ? 6 * (CTX::instance()->mesh.numSubEdges *
+                       CTX::instance()->mesh.numSubEdges * 2) : 12;
 }
 
 
 int MHexahedron27::getNumFacesRep(bool curved)
 {
-  return curved ? 6 * (CTX::instance()->mesh.numSubEdges * CTX::instance()->mesh.numSubEdges * 2) : 12;
+  return curved ? 6 * (CTX::instance()->mesh.numSubEdges *
+                       CTX::instance()->mesh.numSubEdges * 2) : 12;
 }
 
 int MHexahedronN::getNumFacesRep(bool curved)
 {
-  return curved ? 6 * (CTX::instance()->mesh.numSubEdges * CTX::instance()->mesh.numSubEdges * 2) : 12;
+  return curved ? 6 * (CTX::instance()->mesh.numSubEdges *
+                       CTX::instance()->mesh.numSubEdges * 2) : 12;
 }
