@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -65,7 +65,7 @@ class MHexahedron : public MElement {
     static const int map[8] = {2, 3, 7, 6, 0, 1, 5, 4};
     return getVertex(map[num]);
   }
-  virtual int getNumEdges(){ return 12; }
+  virtual int getNumEdges()const{ return 12; }
   virtual MEdge getEdge(int num) const
   {
     return MEdge(_v[edges_hexa(num, 0)], _v[edges_hexa(num, 1)]);
@@ -267,6 +267,12 @@ class MHexahedron20 : public MHexahedron {
                                 19, 14, 11, 15, 13, 8, 16, 17, 10};
     return getVertex(map[num]);
   }
+  virtual MVertex *getVertexVTK(int num)
+  {
+    static const int map[20] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 13,
+                                9, 16, 18, 19, 17, 10, 12, 14, 15};
+    return getVertex(map[num]);
+  }
   virtual int getNumEdgeVertices() const { return 12; }
   virtual int getNumEdgesRep(bool curved);
   virtual void getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n);
@@ -372,6 +378,12 @@ class MHexahedron27 : public MHexahedron {
                                 23, 21, 1, 9, 11, 19, 16, 4, 12, 14, 22, 10, 13};
     return getVertex(map[num]);
   }
+  virtual MVertex *getVertexVTK(int num)
+  {
+    static const int map[27] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 13, 9, 16, 18, 19,
+                                17, 10, 12, 14,15, 22, 23, 21, 24, 20, 25, 26};
+    return getVertex(map[num]);
+  }
   virtual int getNumEdgeVertices() const { return 12; }
   virtual int getNumFaceVertices() const { return 6; }
   virtual int getNumVolumeVertices() const { return 1; }
@@ -486,14 +498,14 @@ class MHexahedronN : public MHexahedron {
   virtual int getNumEdgeVertices() const { return 12 * (_order - 1); }
   virtual int getNumFaceVertices() const
   {
-    if (ElementType::SerendipityFromTag(getTypeForMSH()) > 0)
+    if (getIsAssimilatedSerendipity())
       return 0;
     else
       return 6 * (_order - 1)*(_order - 1);
   }
   virtual int getNumVolumeVertices() const
   {
-    if (ElementType::SerendipityFromTag(getTypeForMSH()) > 0)
+    if (getIsAssimilatedSerendipity())
       return 0;
     else
       return (_order - 1) * (_order - 1) * (_order - 1);
@@ -508,7 +520,7 @@ class MHexahedronN : public MHexahedron {
   }
   virtual void getFaceVertices(const int num, std::vector<MVertex*> &v) const
   {
-    if (ElementType::SerendipityFromTag(getTypeForMSH()) > 0) {
+    if (getIsAssimilatedSerendipity()) {
       v.resize(4 * _order);
     }
     else {
