@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -47,7 +47,7 @@ class MLine : public MElement {
   {
     ithVertex = _v[0] == vertex ? 0 : 1;
   }
-  virtual int getNumEdges(){ return 1; }
+  virtual int getNumEdges()const{ return 1; }
   virtual MEdge getEdge(int num) const{ return MEdge(_v[0], _v[1]); }
   virtual int getNumEdgesRep(bool curved){ return 1; }
   virtual void getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
@@ -206,6 +206,13 @@ class MLineN : public MLine {
     Msg::Error("no tag matches a line with %d vertices", 8+_vs.size());
     return 0;
   }
+  virtual void reverse()
+  {
+    MVertex *tmp = _v[0]; _v[0] = _v[1]; _v[1] = tmp;
+    std::vector<MVertex*> inv;
+    inv.insert(inv.begin(), _vs.rbegin(), _vs.rend());
+    _vs = inv;
+  }
   virtual void getNode(int num, double &u, double &v, double &w) const
   {
     num < 2 ? MLine::getNode(num, u, v, w) : MElement::getNode(num, u, v, w);
