@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -8,6 +8,10 @@
 #include "BasisFactory.h"
 #include "Context.h"
 
+#if defined(HAVE_MESH)
+#include "qualityMeasures.h"
+#endif
+
 int MPrism::getVolumeSign()
 {
   double mat[3][3];
@@ -122,8 +126,8 @@ void MPrism::getFaceInfo(const MFace &face, int &ithFace, int &sign, int &rot) c
 }
 
 static void _myGetEdgeRep(MPrism *pri, int num, double *x, double *y, double *z,
-                          SVector3 *n, int numSubEdges) {
-
+                          SVector3 *n, int numSubEdges)
+{
   //const int numSubEdges = CTX::instance()->mesh.numSubEdges;
   static double pp[6][3] = {
     {0,0,-1},{1,0,-1},{0,1,-1},
@@ -159,36 +163,43 @@ static void _myGetEdgeRep(MPrism *pri, int num, double *x, double *y, double *z,
   n[0] = n[1] = 1 ;
 }
 
-
-void MPrism15::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n) {
+void MPrism15::getEdgeRep(bool curved, int num,
+                          double *x, double *y, double *z, SVector3 *n)
+{
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MPrism::getEdgeRep(false, num, x, y, z, n);
 }
 
-void MPrism18::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n) {
+void MPrism18::getEdgeRep(bool curved, int num,
+                          double *x, double *y, double *z, SVector3 *n)
+{
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MPrism::getEdgeRep(false, num, x, y, z, n);
 }
 
-void MPrismN::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n) {
+void MPrismN::getEdgeRep(bool curved, int num,
+                         double *x, double *y, double *z, SVector3 *n)
+{
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MPrism::getEdgeRep(false, num, x, y, z, n);
 }
 
-int MPrism15::getNumEdgesRep(bool curved) {
+int MPrism15::getNumEdgesRep(bool curved)
+{
   return curved ? 9 * CTX::instance()->mesh.numSubEdges : 9;
 }
 
-int MPrism18::getNumEdgesRep(bool curved) {
+int MPrism18::getNumEdgesRep(bool curved)
+{
   return curved ? 9 * CTX::instance()->mesh.numSubEdges : 9;
 }
 
-int MPrismN::getNumEdgesRep(bool curved) {
+int MPrismN::getNumEdgesRep(bool curved)
+{
   return curved ? 9 * CTX::instance()->mesh.numSubEdges : 9;
 }
 
-
-void _myGetFaceRep(MPrism *pri, int num, double *x, double *y, double *z,
+static void _myGetFaceRep(MPrism *pri, int num, double *x, double *y, double *z,
                           SVector3 *n, int numSubEdges)
 {
   static double pp[6][3] = {
@@ -348,7 +359,8 @@ void _myGetFaceRep(MPrism *pri, int num, double *x, double *y, double *z,
       pri->pnt(U3, V3, W3, pnt3);
     }
 
-  } else {
+  }
+  else {
     int ix = 0, iy = 0;
     int nbt = 0;
     for (int i = 0; i < numSubEdges; i++){
@@ -406,55 +418,60 @@ void _myGetFaceRep(MPrism *pri, int num, double *x, double *y, double *z,
   n[2] = n[0];
 }
 
-void MPrism15::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MPrism15::getFaceRep(bool curved, int num,
+                          double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MPrism::getFaceRep(false, num, x, y, z, n);
 }
 
-void MPrism18::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MPrism18::getFaceRep(bool curved, int num,
+                          double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MPrism::getFaceRep(false, num, x, y, z, n);
 }
 
-void MPrismN::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MPrismN::getFaceRep(bool curved, int num,
+                         double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MPrism::getFaceRep(false, num, x, y, z, n);
 }
 
-int MPrism15::getNumFacesRep(bool curved) {
-  return curved ? 4 * (CTX::instance()->mesh.numSubEdges * CTX::instance()->mesh.numSubEdges * 2) : 8;
-}
-
-int MPrism18::getNumFacesRep(bool curved) {
-  return curved ? 4 * (CTX::instance()->mesh.numSubEdges * CTX::instance()->mesh.numSubEdges * 2) : 8;
-}
-
-int MPrismN::getNumFacesRep(bool curved) {
-  return curved ? 4 * (CTX::instance()->mesh.numSubEdges * CTX::instance()->mesh.numSubEdges * 2) : 8;
-}
-
-namespace {
-
-void _addEdgeNodes(int num, bool reverse, int order, const std::vector<MVertex*> &vs,
-                   int &ind, std::vector<MVertex*> &v)
+int MPrism15::getNumFacesRep(bool curved)
 {
+  return curved ? 4 * (CTX::instance()->mesh.numSubEdges *
+                       CTX::instance()->mesh.numSubEdges * 2) : 8;
+}
 
+int MPrism18::getNumFacesRep(bool curved)
+{
+  return curved ? 4 * (CTX::instance()->mesh.numSubEdges *
+                       CTX::instance()->mesh.numSubEdges * 2) : 8;
+}
+
+int MPrismN::getNumFacesRep(bool curved)
+{
+  return curved ? 4 * (CTX::instance()->mesh.numSubEdges *
+                       CTX::instance()->mesh.numSubEdges * 2) : 8;
+}
+
+static void _addEdgeNodes(int num, bool reverse, int order,
+                          const std::vector<MVertex*> &vs,
+                          int &ind, std::vector<MVertex*> &v)
+{
   const int nNode = order-1, startNode = num*nNode, endNode = startNode+nNode-1;
 
   if (reverse)
     for (int i=endNode; i>=startNode; i--, ind++) v[ind] = vs[i];
   else
     for (int i=startNode; i<=endNode; i++, ind++) v[ind] = vs[i];
-
 }
 
-void _addFaceNodes(int num, int order, const std::vector<MVertex*> &vs,
-                   int &ind, std::vector<MVertex*> &v)
+static void _addFaceNodes(int num, int order, const std::vector<MVertex*> &vs,
+                          int &ind, std::vector<MVertex*> &v)
 {
-
   const int nNodeEd = order-1, nNodeTri = (order-2)*(order-1)/2;
 
   int startNode, endNode;
@@ -469,9 +486,6 @@ void _addFaceNodes(int num, int order, const std::vector<MVertex*> &vs,
   }
 
   for (int i=startNode; i<endNode; i++, ind++) v[ind] = vs[i];
-
-}
-
 }
 
 // To be tested
@@ -510,42 +524,18 @@ void MPrismN::getFaceVertices(const int num, std::vector<MVertex*> &v) const
   int ind = nEdge;
 
   // Add edge nodes
-  for (int iE=0; iE<nEdge; iE++) _addEdgeNodes(edge[num][iE],reverse[num][iE],_order,_vs,ind,v);
+  for (int iE=0; iE<nEdge; iE++)
+    _addEdgeNodes(edge[num][iE],reverse[num][iE],_order,_vs,ind,v);
 
   // Add face nodes
   _addFaceNodes(num,_order,_vs,ind,v);
-
 }
 
-static double scaled_jacobian(MVertex* a,MVertex* b,MVertex* c,MVertex* d){
-  double val;
-  double l1,l2,l3;
-  SVector3 vec1,vec2,vec3;
-
-  vec1 = SVector3(b->x()-a->x(),b->y()-a->y(),b->z()-a->z());
-  vec2 = SVector3(c->x()-a->x(),c->y()-a->y(),c->z()-a->z());
-  vec3 = SVector3(d->x()-a->x(),d->y()-a->y(),d->z()-a->z());
-
-  l1 = vec1.norm();
-  l2 = vec2.norm();
-  l3 = vec3.norm();
-
-  val = dot(vec1,crossprod(vec2,vec3));
-  return fabs(val)/(l1*l2*l3);
+double MPrism::gammaShapeMeasure()
+{
+#if defined(HAVE_MESH)
+  return qmPrism::minNCJ(this);
+#else
+  return 0.;
+#endif
 }
-
-double MPrism::gammaShapeMeasure() {
-  MVertex *a = getVertex(0),*b= getVertex(1),*c= getVertex(2);
-  MVertex *d = getVertex(3),*e= getVertex(4),*f= getVertex(5);
-  const double j [6] = {
-    scaled_jacobian(a,b,c,d),
-    scaled_jacobian(b,a,c,e),
-    scaled_jacobian(c,a,b,f),
-    scaled_jacobian(d,a,e,f),
-    scaled_jacobian(e,b,d,f),
-    scaled_jacobian(f,c,d,e)};  
-  const double result = *std::min_element(j,j+6);
-  printf("%12.5E\n",result);
-  return result;
-}
-
