@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -66,7 +66,7 @@ class MPrism : public MElement {
   virtual MVertex *getVertex(int num){ return _v[num]; }
   virtual const MVertex *getVertex(int num)const{ return _v[num]; }
   virtual void setVertex(int num,   MVertex *v){ _v[num] = v; }
-  virtual int getNumEdges(){ return 9; }
+  virtual int getNumEdges()const{ return 9; }
   virtual MEdge getEdge(int num) const
   {
     return MEdge(_v[edges_prism(num, 0)], _v[edges_prism(num, 1)]);
@@ -413,14 +413,14 @@ class MPrismN : public MPrism {
   virtual int getNumEdgeVertices() const { return 9*(_order-1); }
   virtual int getNumFaceVertices() const
   {
-    if (ElementType::SerendipityFromTag(getTypeForMSH()) > 0)
+    if (getIsAssimilatedSerendipity())
       return 0;
     else
       {int n = _order-1; return (n-1 + 3*n) * n;}
   }
   virtual int getNumVolumeVertices() const
   {
-    if (ElementType::SerendipityFromTag(getTypeForMSH()) > 0)
+    if (getIsAssimilatedSerendipity())
       return 0;
     else
       {int n = _order-1; return n * (n * (n+1) / 2);}
