@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -33,28 +33,23 @@ void MPyramid::getIntegrationPoints(int pOrder, int *npts, IntPt **pts)
   *pts = getGQPyrPts(pOrder);
 }
 
-const JacobianBasis* MPyramid::getJacobianFuncSpace(int o) const
+const JacobianBasis* MPyramid::getJacobianFuncSpace(int order) const
 {
-  // FIXME add other order and see MPyramid::getFunctionSpace for 'design'
-  int order = (o == -1) ? getPolynomialOrder() : o;
+  if (order == -1) return BasisFactory::getJacobianBasis(getTypeForMSH());
 
-  switch (order) {
-    case 1: return BasisFactory::getJacobianBasis(MSH_PYR_5);
-    case 2: return BasisFactory::getJacobianBasis(MSH_PYR_14);
-    case 3: return BasisFactory::getJacobianBasis(MSH_PYR_30);
-    default: Msg::Error("Order %d pyramid function space not implemented", order); break;
-  }
-
-  return 0;
+  int tag = ElementType::getTag(TYPE_PYR, order);
+  return tag ? BasisFactory::getJacobianBasis(tag) : NULL;
 }
 
 MPyramidN::~MPyramidN() {}
 
-int MPyramidN::getNumEdgesRep(bool curved) {
+int MPyramidN::getNumEdgesRep(bool curved)
+{
   return curved ? 8 * CTX::instance()->mesh.numSubEdges : 8;
 }
 
-void MPyramidN::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MPyramidN::getEdgeRep(bool curved, int num,
+                           double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) {
     int numSubEdges = CTX::instance()->mesh.numSubEdges;
@@ -90,7 +85,8 @@ void MPyramidN::getEdgeRep(bool curved, int num, double *x, double *y, double *z
 }
 
 
-int MPyramidN::getNumFacesRep(bool curved) {
+int MPyramidN::getNumFacesRep(bool curved)
+{
   return curved ? 6 * SQU(CTX::instance()->mesh.numSubEdges) : 6;
 }
 
@@ -330,7 +326,8 @@ static void _myGetFaceRep(MPyramid *pyr, int num, double *x, double *y, double *
   }
 }
 
-void MPyramidN::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MPyramidN::getFaceRep(bool curved, int num,
+                           double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MPyramid::getFaceRep(false, num, x, y, z, n);
