@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -67,7 +67,7 @@ double MTetrahedron::gammaShapeMeasure()
 {
 #if defined(HAVE_MESH)
   double vol;
-  return qmTet(this, QMTET_2, &vol);
+  return qmTetrahedron::qm(this, qmTetrahedron::QMTET_GAMMA, &vol);
 #else
   return 0.;
 #endif
@@ -77,7 +77,7 @@ double MTetrahedron::etaShapeMeasure()
 {
 #if defined(HAVE_MESH)
   double vol;
-  return qmTet(this, QMTET_3, &vol);
+  return qmTetrahedron::qm(this, qmTetrahedron::QMTET_ETA, &vol);
 #else
   return 0.;
 #endif
@@ -148,13 +148,15 @@ static void _myGetEdgeRep(MTetrahedron *tet, int num, double *x, double *y, doub
   n[0] = n[1] = tet->getFace(f[iEdge]).normal();
 }
 
-void MTetrahedron10::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MTetrahedron10::getEdgeRep(bool curved, int num,
+                                double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MTetrahedron::getEdgeRep(false, num, x, y, z, n);
 }
 
-void MTetrahedronN::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MTetrahedronN::getEdgeRep(bool curved, int num,
+                               double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MTetrahedron::getEdgeRep(false, num, x, y, z, n);
@@ -247,13 +249,15 @@ static void _myGetFaceRep(MTetrahedron *tet, int num, double *x, double *y, doub
   n[2] = n[0];
 }
 
-void MTetrahedronN::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MTetrahedronN::getFaceRep(bool curved, int num,
+                               double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MTetrahedron::getFaceRep(false, num, x, y, z, n);
 }
 
-void MTetrahedron10::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MTetrahedron10::getFaceRep(bool curved, int num,
+                                double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MTetrahedron::getFaceRep(false, num, x, y, z, n);
