@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -14,24 +14,6 @@
 #include "GmshMessage.h"
 #include "StringUtils.h"
 
-double MVertexLessThanLexicographic::tolerance = 1.e-6;
-
-bool MVertexLessThanLexicographic::operator()(const MVertex *v1, const MVertex *v2) const
-{
-  if(v1->x() - v2->x() >  tolerance) return true;
-  if(v1->x() - v2->x() < -tolerance) return false;
-  if(v1->y() - v2->y() >  tolerance) return true;
-  if(v1->y() - v2->y() < -tolerance) return false;
-  if(v1->z() - v2->z() >  tolerance) return true;
-  return false;
-}
-
-bool MVertexLessThanNum::operator()(const MVertex *v1, const MVertex *v2) const
-{
-  if(v1->getNum() < v2->getNum()) return true;
-  return false;
-}
-
 double angle3Vertices(const MVertex *p1, const MVertex *p2, const MVertex *p3)
 {
   SVector3 a(p1->x() - p2->x(), p1->y() - p2->y(), p1->z() - p2->z());
@@ -349,13 +331,24 @@ void MVertex::writeSU2(FILE *fp, int dim, double scalingFactor)
             z() * scalingFactor, _index - 1);
 }
 
-std::set<MVertex*, MVertexLessThanLexicographic>::iterator
-MVertex::linearSearch(std::set<MVertex*, MVertexLessThanLexicographic> &pos)
+bool MVertexLessThanNum::operator()(const MVertex *v1, const MVertex *v2) const
 {
-  for(std::set<MVertex*, MVertexLessThanLexicographic>::iterator it = pos.begin();
-      it != pos.end(); ++it)
-    if(distance(*it) < MVertexLessThanLexicographic::tolerance) return it;
-  return pos.end();
+  if(v1->getNum() < v2->getNum()) return true;
+  return false;
+}
+
+double MVertexLessThanLexicographic::tolerance = 1.e-6;
+
+bool MVertexLessThanLexicographic::operator()(const MVertex *v1, const MVertex *v2) const
+{
+  // you should not use this unless you know what you are doing; to create
+  // unique vertices, use MVertexRTree
+  if(v1->x() - v2->x() >  tolerance) return true;
+  if(v1->x() - v2->x() < -tolerance) return false;
+  if(v1->y() - v2->y() >  tolerance) return true;
+  if(v1->y() - v2->y() < -tolerance) return false;
+  if(v1->z() - v2->z() >  tolerance) return true;
+  return false;
 }
 
 static void getAllParameters(MVertex *v, GFace *gf, std::vector<SPoint2> &params)
@@ -379,11 +372,11 @@ static void getAllParameters(MVertex *v, GFace *gf, std::vector<SPoint2> &params
           params.push_back((*it)->reparamOnFace(gf, range.low(),-1));
           params.push_back((*it)->reparamOnFace(gf, range.low(), 1));
         }
-        else if (gv == (*it)->getEndVertex()){
+        if (gv == (*it)->getEndVertex()){
           params.push_back((*it)->reparamOnFace(gf, range.high(),-1));
           params.push_back((*it)->reparamOnFace(gf, range.high(), 1));
         }
-        else{
+        if (gv != (*it)->getBeginVertex() && gv != (*it)->getEndVertex()){
           Msg::Warning("Strange!");
         }
         seam = true;
@@ -419,49 +412,49 @@ bool reparamMeshEdgeOnFace(MVertex *v1, MVertex *v2, GFace *gf,
   if (p1.size() == 1 && p2.size() == 1){
     param1 = p1[0];
     param2 = p2[0];
-    return true;
   }
-  else if (p1.size() == 1 && p2.size() == 2){
-    double d1 =
-      (p1[0].x() - p2[0].x()) * (p1[0].x() - p2[0].x()) +
-      (p1[0].y() - p2[0].y()) * (p1[0].y() - p2[0].y());
-    double d2 =
-      (p1[0].x() - p2[1].x()) * (p1[0].x() - p2[1].x()) +
-      (p1[0].y() - p2[1].y()) * (p1[0].y() - p2[1].y());
-    param1 = p1[0];
-    param2 = d2 < d1 ? p2[1] : p2[0];
-    return true;
-  }
-  else if (p2.size() == 1 && p1.size() == 2){
-    double d1 =
-      (p2[0].x() - p1[0].x()) * (p2[0].x() - p1[0].x()) +
-      (p2[0].y() - p1[0].y()) * (p2[0].y() - p1[0].y());
-    double d2 =
-      (p2[0].x() - p1[1].x()) * (p2[0].x() - p1[1].x()) +
-      (p2[0].y() - p1[1].y()) * (p2[0].y() - p1[1].y());
-    param1 = d2 < d1 ? p1[1] : p1[0];
-    param2 = p2[0];
-    return true;
-  }
-  else if(p1.size() > 1 && p2.size() > 1){
-    param1 = p1[0];
-    param2 = p2[0];
-
-    printf("NO WAY : TWO VERTICES ON THE SEAM, CANNOT CHOOSE\n");
-
-    // shout, both vertices are on seams
-    return false;
+  else if(p1.size() >= 1 && p2.size() >= 1){
+    int imin = 0;
+    int jmin = 0;
+    {
+      double d =
+	(p2[0].x() - p1[0].x()) * (p2[0].x() - p1[0].x()) +
+	(p2[0].y() - p1[0].y()) * (p2[0].y() - p1[0].y());
+      for (unsigned int i=0;i<p2.size();i++){
+	double d1 =
+	  (p2[i].x() - p1[0].x()) * (p2[i].x() - p1[0].x()) +
+	  (p2[i].y() - p1[0].y()) * (p2[i].y() - p1[0].y());
+	if (d1 < d){
+	  imin = i;
+	  d = d1;
+	}
+      }
+    }
+    {
+      double d =
+	(p2[0].x() - p1[0].x()) * (p2[0].x() - p1[0].x()) +
+	(p2[0].y() - p1[0].y()) * (p2[0].y() - p1[0].y());
+      for (unsigned int i=0;i<p1.size();i++){
+	double d1 =
+	  (p2[0].x() - p1[i].x()) * (p2[0].x() - p1[i].x()) +
+	  (p2[0].y() - p1[i].y()) * (p2[0].y() - p1[i].y());
+	if (d1 < d){
+	  jmin = i;
+	  d = d1;
+	}
+      }
+    }
+    param1 = p1[jmin];
+    param2 = p2[imin];
   }
   else{
     // brute force!
     param1 = gf->parFromPoint(SPoint3(v1->x(), v1->y(), v1->z()));
     param2 = gf->parFromPoint(SPoint3(v2->x(), v2->y(), v2->z()));
-    return true;
   }
+  return true;
 }
 
-
-
 bool reparamMeshVertexOnFace(MVertex *v, const GFace *gf, SPoint2 &param,
                              bool onSurface)
 {
