@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -28,7 +28,11 @@
 #include <BRepMesh_FastDiscret.hxx>
 
 #if (OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR >= 6)
-#include <BOPInt_Context.hxx>
+#  if (OCC_VERSION_MINOR < 8)
+#  include <BOPInt_Context.hxx>
+#  else
+#  include <IntTools_Context.hxx>
+#  endif
 #include <BOPTools_AlgoTools2D.hxx>
 #include <BOPTools_AlgoTools.hxx>
 #else
@@ -66,6 +70,12 @@ void OCCFace::setup()
       if(!e){
 	Msg::Error("Unknown edge in face %d", tag());
       }
+      else if(edge.Orientation() == TopAbs_INTERNAL){
+        Msg::Info("Adding embedded edge %d", e->tag());
+        embedded_edges.push_back(e);
+        OCCEdge *occe = (OCCEdge*)e;
+        occe->setTrimmed(this);
+      }
       else{
         l_wire.push_back(e);
         Msg::Debug("Edge %d (%d --> %d) ori %d", e->tag(),
@@ -119,6 +129,30 @@ void OCCFace::setup()
   //          (*it)->getBeginVertex()->tag(), (*it)->getEndVertex()->tag(),
   //          (*it)->isSeam(this));
   // }
+
+  /*
+  for(exp2.Init(s, TopAbs_VERTEX); exp2.More(); exp2.Next()){
+    TopoDS_Vertex vertex = TopoDS::Vertex(exp2.Current());
+    GVertex *v = model()->getOCCInternals()->getOCCVertexByNativePtr(model(), vertex);
+    if(!v){
+      Msg::Error("Unknown vertex in face %d", tag());
+    }
+    else if(vertex.Orientation() == TopAbs_INTERNAL){
+      Msg::Info("Adding embedded vertex %d", v->tag());
+      embedded_vertices.push_back(v);
+    }
+  }
+  */
+}
+
+SBoundingBox3d OCCFace::bounds() const
+{
+  Bnd_Box b;
+  BRepBndLib::Add(s, b);
+  double xmin, ymin, zmin, xmax, ymax, zmax;
+  b.Get(xmin, ymin, zmin, xmax, ymax, zmax);
+  SBoundingBox3d bbox(xmin, ymin, zmin, xmax, ymax, zmax);
+  return bbox;
 }
 
 Range<double> OCCFace::parBounds(int i) const
@@ -455,7 +489,7 @@ bool OCCFace::buildSTLTriangulation(bool force)
 void OCCFace::replaceEdgesInternal(std::list<GEdge*> &new_edges)
 {
 
-#if (OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR >= 6)
+#if (OCC_VERSION_MAJOR == 6) && (OCC_VERSION_MINOR >= 6) && (OCC_VERSION_MINOR < 8)
   Handle(BOPInt_Context) myContext = new BOPInt_Context;
 #elif defined(OCC_VERSION_HEX) && OCC_VERSION_HEX >= 0x060503
   Handle(IntTools_Context) myContext = new IntTools_Context;
