@@ -22,7 +22,9 @@ gmshEdge::gmshEdge(GModel *m, Curve *edge, GVertex *v1, GVertex *v2)
 
 bool gmshEdge::degenerate(int dim) const
 {
-  if (c->beg == c->end && c->Typ ==  MSH_SEGM_LINE){
+  if (c->beg == c->end &&
+      c->Typ ==  MSH_SEGM_LINE &&
+      List_Nbr(c->Control_Points) == 0){
     Msg::Info("Model Edge %d is degenerate", tag());
     return true;
   }
@@ -407,7 +409,7 @@ void gmshEdge::writeGEO(FILE *fp)
       fprintf(fp, ", %d", v->Num);
     else
       fprintf(fp, "{%d", v->Num);
-    if(i % 6 == 7)
+    if(i % 8 == 7)
       fprintf(fp, "\n");
   }
   fprintf(fp, "};\n");
