@@ -21,12 +21,12 @@
 #endif
 
 gmshFace::gmshFace(GModel *m, Surface *face)
-  : GFace(m, face->Num), s(face)
+  : GFace(m, face->Num), s(face), isSphere(false), radius(0.)
 {
   resetMeshAttributes();
 
-  edgeCounterparts = s->edgeCounterparts;
-  affineTransform = s->affineTransform;
+  // edgeCounterparts = s->edgeCounterparts;
+  // affineTransform = s->affineTransform;
 
   std::vector<GEdge*> eds;
   std::vector<int> nums;
