@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -30,10 +30,10 @@ static void drawEntityLabel(drawContext *ctx, GEntity *e,
       strcat(str, tmp);
     }
   }
-  glRasterPos3d(x + offset / ctx->s[0],
-                y + offset / ctx->s[1],
-                z + offset / ctx->s[2]);
-  ctx->drawString(str);
+  ctx->drawString(str,
+                  x + offset / ctx->s[0],
+                  y + offset / ctx->s[1],
+                  z + offset / ctx->s[2]);
 }
 
 class drawGVertex {
@@ -53,14 +53,21 @@ class drawGVertex {
       glPushName(v->tag());
     }
 
+    double ps = CTX::instance()->geom.pointSize;
+    double sps = CTX::instance()->geom.selectedPointSize;
+    if(_ctx->isHighResolution()){
+      ps *= CTX::instance()->highResolutionPointSizeFactor;
+      sps *= CTX::instance()->highResolutionPointSizeFactor;
+    }
+
     if(v->getSelection()) {
-      glPointSize((float)CTX::instance()->geom.selectedPointSize);
+      glPointSize((float)sps);
       gl2psPointSize((float)(CTX::instance()->geom.selectedPointSize *
                              CTX::instance()->print.epsPointSizeFactor));
       glColor4ubv((GLubyte *) & CTX::instance()->color.geom.selection);
     }
     else {
-      glPointSize((float)CTX::instance()->geom.pointSize);
+      glPointSize((float)ps);
       gl2psPointSize((float)(CTX::instance()->geom.pointSize *
                              CTX::instance()->print.epsPointSizeFactor));
       glColor4ubv((GLubyte *) & CTX::instance()->color.geom.point);
@@ -80,11 +87,9 @@ class drawGVertex {
     if(CTX::instance()->geom.points) {
       if(CTX::instance()->geom.pointType > 0) {
         if(v->getSelection())
-          _ctx->drawSphere(CTX::instance()->geom.selectedPointSize, x, y, z,
-                           CTX::instance()->geom.light);
+          _ctx->drawSphere(sps, x, y, z, CTX::instance()->geom.light);
         else
-          _ctx->drawSphere(CTX::instance()->geom.pointSize, x, y, z,
-                           CTX::instance()->geom.light);
+          _ctx->drawSphere(ps, x, y, z, CTX::instance()->geom.light);
       }
       else {
         glBegin(GL_POINTS);
@@ -94,7 +99,7 @@ class drawGVertex {
     }
 
     if(CTX::instance()->geom.pointsNum) {
-      double offset = (0.5 * CTX::instance()->geom.pointSize +
+      double offset = (0.5 * ps +
                        0.1 * CTX::instance()->glFontSize) * _ctx->pixel_equiv_x;
       drawEntityLabel(_ctx, v, x, y, z, offset);
     }
@@ -466,7 +471,6 @@ class drawGRegion {
   void operator () (GRegion *r)
   {
     if(!r->getVisibility()) return;
-    //    if(r->geomType() == GEntity::DiscreteVolume) return;
 
     bool select = (_ctx->render_mode == drawContext::GMSH_SELECT &&
                    r->model() == GModel::current());
