@@ -11,13 +11,21 @@
 #include "Context.h"
 #include "gl2ps.h"
 #include "SVector3.h"
+#include "GModel.h"
 
-void drawContext::drawString(const std::string &s, const std::string &font_name,
-                             int font_enum, int font_size, int align)
+void drawContext::drawString(const std::string &s, double x, double y, double z,
+                             const std::string &font_name, int font_enum,
+                             int font_size, int align)
 {
   if(s.empty()) return;
   if(CTX::instance()->printing && !CTX::instance()->print.text) return;
 
+  if(s.size() > 8 && s.substr(0, 7) == "file://"){
+    drawImage(s.substr(7), x, y, z, align);
+    return;
+  }
+
+  glRasterPos3d(x, y, z);
   GLboolean valid;
   glGetBooleanv(GL_CURRENT_RASTER_POSITION_VALID, &valid);
   if(valid == GL_FALSE) return; // the primitive is culled
@@ -88,30 +96,31 @@ void drawContext::drawString(const std::string &s, const std::string &font_name,
   }
 }
 
-void drawContext::drawString(const std::string &s)
+void drawContext::drawString(const std::string &s, double x, double y, double z)
 {
-  drawString(s, CTX::instance()->glFont, CTX::instance()->glFontEnum,
+  drawString(s, x, y, z, CTX::instance()->glFont, CTX::instance()->glFontEnum,
              CTX::instance()->glFontSize, 0);
 }
 
-void drawContext::drawStringCenter(const std::string &s)
+void drawContext::drawStringCenter(const std::string &s, double x, double y, double z)
 {
-  drawString(s, CTX::instance()->glFont, CTX::instance()->glFontEnum,
+  drawString(s, x, y, z, CTX::instance()->glFont, CTX::instance()->glFontEnum,
              CTX::instance()->glFontSize, 1);
 }
 
-void drawContext::drawStringRight(const std::string &s)
+void drawContext::drawStringRight(const std::string &s, double x, double y, double z)
 {
-  drawString(s, CTX::instance()->glFont, CTX::instance()->glFontEnum,
+  drawString(s, x, y, z, CTX::instance()->glFont, CTX::instance()->glFontEnum,
              CTX::instance()->glFontSize, 2);
 }
 
-void drawContext::drawString(const std::string &s, double style)
+void drawContext::drawString(const std::string &s, double x, double y, double z,
+                             double style)
 {
   unsigned int bits = (unsigned int)style;
 
   if(!bits){ // use defaults
-    drawString(s);
+    drawString(s, x, y, z);
   }
   else{
     int size = (bits & 0xff);
@@ -120,7 +129,118 @@ void drawContext::drawString(const std::string &s, double style)
     int font_enum = drawContext::global()->getFontEnum(font);
     std::string font_name = drawContext::global()->getFontName(font);
     if(!size) size = CTX::instance()->glFontSize;
-    drawString(s, font_name, font_enum, size, align);
+    drawString(s, x, y, z, font_name, font_enum, size, align);
+  }
+}
+
+void drawContext::drawImage(const std::string &name, double x, double y, double z,
+                            int align)
+{
+  // format can be "@wxh" or "@wxh,wx,wy,wz,hx,hy,hz", where w and h are the
+  // width and height (in model coordinates for T3 or in pixels for T2) of the
+  // image, wx,wy,wz is the direction of the bottom edge of the image and
+  // hx,hy,hz is the direction of the left edge of the image.
+  size_t p = name.find_last_of("@");
+  std::string file = name, format;
+  if(p != std::string::npos){
+    format = name.substr(p + 1);
+    file = name.substr(0, p);
+  }
+  double w = 0., h = 0., wx = 1., wy = 0., wz = 0., hx = 0., hy = 1., hz = 0.;
+  bool billboard = false;
+  if(format.size()){
+    bool ok;
+    if(format.find(",") != std::string::npos)
+      ok = (sscanf(format.c_str(), "%lfx%lf,%lf,%lf,%lf,%lf,%lf,%lf",
+                   &w, &h, &wx, &wy, &wz, &hx, &hy, &hz) == 8);
+    else
+      ok = (sscanf(format.c_str(), "%lfx%lf", &w, &h) == 2);
+    if(!ok)
+      Msg::Warning("Bad image format: use `@wxh' or `@wxh,wx,wy,wz,hx,hy,hz'");
+    if(format.find("#") != std::string::npos)
+      billboard = true; // texture will always face camera
+  }
+
+  imgtex *img;
+  if(!_imageTextures.count(file)){
+    img = &_imageTextures[file];
+    file = FixRelativePath(GModel::current()->getFileName(), file);
+    if(!generateTextureForImage(file, 1, img->tex, img->w, img->h)){
+      Msg::Error("Problem generating image texture");
+      return;
+    }
+  }
+  else{
+    img = &_imageTextures[file];
+  }
+  if(!img->tex){
+    Msg::Error("No texture for image");
+    return;
+  }
+
+  if(w == 0 && h == 0){
+    w = img->w;
+    h = img->h;
+  }
+  else if(h == 0){
+    h = w * img->h / img->w;
+  }
+  else if(w == 0){
+    w = h * img->w / img->h;
+  }
+
+  GLboolean valid = GL_TRUE;
+  GLint matrixMode = 0;
+  if(billboard){
+    glRasterPos3d(x, y, z);
+    GLfloat pos[4];
+    glGetFloatv(GL_CURRENT_RASTER_POSITION, pos);
+    glGetIntegerv(GL_MATRIX_MODE, &matrixMode);
+    glMatrixMode(GL_PROJECTION);
+    glPushMatrix();
+    glLoadIdentity();
+    glMatrixMode(GL_MODELVIEW);
+    glPushMatrix();
+    glLoadIdentity();
+    double fact = isHighResolution() ? 2. : 1.;
+    glOrtho((double)viewport[0], (double)viewport[2] * fact,
+            (double)viewport[1], (double)viewport[3] * fact, -1, 1);
+    x = pos[0]; y = pos[1]; z = 0;
+    w *= fact * s[0] / pixel_equiv_x;
+    h *= fact * s[1] / pixel_equiv_y;
+    glGetBooleanv(GL_CURRENT_RASTER_POSITION_VALID, &valid);
+  }
+  if(valid == GL_TRUE){
+    switch(align){
+    case 1: x -= w/2.;            break; // bottom center
+    case 2: x -= w;               break; // bottom right
+    case 3:            y -= h;    break; // top left
+    case 4: x -= w/2.; y -= h;    break; // top center
+    case 5: x -= w;    y -= h;    break; // top right
+    case 6:            y -= h/2.; break; // center left
+    case 7: x -= w/2.; y -= h/2.; break; // center center
+    case 8: x -= w;    y -= h/2.; break; // center right
+    default: break;
+    }
+    glEnable(GL_BLEND);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    glEnable(GL_TEXTURE_2D);
+    glBindTexture(GL_TEXTURE_2D, img->tex);
+    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+    glBegin(GL_QUADS);
+    glTexCoord2f(1.0f, 1.0f); glVertex3d(x+wx*w, y+wy*w, z+wz*w);
+    glTexCoord2f(1.0f, 0.0f); glVertex3d(x+wx*w+hx*h, y+wy*w+hy*h, z+wz*w+hz*h);
+    glTexCoord2f(0.0f, 0.0f); glVertex3d(x+hx*h, y+hy*h, z+hz*h);
+    glTexCoord2f(0.0f, 1.0f); glVertex3d(x, y, z);
+    glEnd();
+    glDisable(GL_TEXTURE_2D);
+    glDisable(GL_BLEND);
+  }
+  if(billboard){
+    glPopMatrix();
+    glMatrixMode(GL_PROJECTION);
+    glPopMatrix();
+    glMatrixMode(matrixMode);
   }
 }
 
@@ -549,16 +669,14 @@ void drawContext::drawBox(double xmin, double ymin, double zmin,
   if(labels){
     char label[256];
     double offset = 0.3 * CTX::instance()->glFontSize * pixel_equiv_x;
-    glRasterPos3d(xmin + offset / s[0],
-                  ymin + offset / s[1],
-                  zmin + offset / s[2]);
     sprintf(label, "(%g,%g,%g)", xmin, ymin, zmin);
-    drawString(label);
-    glRasterPos3d(xmax + offset / s[0],
-                  ymax + offset / s[1],
-                  zmax + offset / s[2]);
+    drawString(label, xmin + offset / s[0],
+               ymin + offset / s[1],
+               zmin + offset / s[2]);
     sprintf(label, "(%g,%g,%g)", xmax, ymax, zmax);
-    drawString(label);
+    drawString(label, xmax + offset / s[0],
+               ymax + offset / s[1],
+               zmax + offset / s[2]);
   }
 }
 
