@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -31,429 +31,10 @@
 #include "linearSystemPETSc.h"
 #endif
 
-// computes the characteristic length of the mesh at a vertex in order
-// to have the geometry captured with accuracy. A parameter called
-// CTX::instance()->mesh.minCircPoints tells the minimum number of points per
-// radius of curvature
-
 #if defined(HAVE_ANN)
 static int _NBANN = 2;
 #endif
 
-SMetric3 buildMetricTangentToCurve(SVector3 &t, double l_t, double l_n)
-{
-  if (l_t == 0.0) return SMetric3(1.e-22);
-  SVector3 a;
-  if (fabs(t(0)) <= fabs(t(1)) && fabs(t(0)) <= fabs(t(2))){
-    a = SVector3(1,0,0);
-  }
-  else if (fabs(t(1)) <= fabs(t(0)) && fabs(t(1)) <= fabs(t(2))){
-    a = SVector3(0,1,0);
-  }
-  else{
-    a = SVector3(0,0,1);
-  }
-  SVector3 b = crossprod (t,a);
-  SVector3 c = crossprod (b,t);
-  b.normalize();
-  c.normalize();
-  t.normalize();
-  SMetric3 Metric (1./(l_t*l_t),1./(l_n*l_n),1./(l_n*l_n),t,b,c);
-  //  printf("bmttc %g %g %g %g %g\n",l_t,l_n,Metric(0,0),Metric(0,1),Metric(1,1));
-  return Metric;
-}
-
-SMetric3 buildMetricTangentToSurface(SVector3 &t1, SVector3 &t2,
-                                     double l_t1, double l_t2, double l_n)
-{
-  t1.normalize();
-  t2.normalize();
-  SVector3 n = crossprod (t1,t2);
-  n.normalize();
-
-  l_t1 = std::max(l_t1, CTX::instance()->mesh.lcMin);
-  l_t2 = std::max(l_t2, CTX::instance()->mesh.lcMin);
-  l_t1 = std::min(l_t1, CTX::instance()->mesh.lcMax);
-  l_t2 = std::min(l_t2, CTX::instance()->mesh.lcMax);
-  SMetric3 Metric (1./(l_t1*l_t1),1./(l_t2*l_t2),1./(l_n*l_n),t1,t2,n);
-  return Metric;
-}
-
-SMetric3 max_edge_curvature_metric(const GVertex *gv)
-{
-  SMetric3 val (1.e-12);
-  std::list<GEdge*> l_edges = gv->edges();
-  for (std::list<GEdge*>::const_iterator ite = l_edges.begin();
-       ite != l_edges.end(); ++ite){
-    GEdge *_myGEdge = *ite;
-    Range<double> range = _myGEdge->parBounds(0);
-    SMetric3 cc;
-    if (gv == _myGEdge->getBeginVertex()) {
-      SVector3 t = _myGEdge->firstDer(range.low());
-      t.normalize();
-      double l_t = ((2 * M_PI) /( fabs(_myGEdge->curvature(range.low()))
-				*  CTX::instance()->mesh.minCircPoints ));
-      double l_n = 1.e12;
-      cc = buildMetricTangentToCurve(t,l_t,l_n);
-    }
-    else {
-      SVector3 t = _myGEdge->firstDer(range.high());
-      t.normalize();
-      double l_t = ((2 * M_PI) /( fabs(_myGEdge->curvature(range.high()))
-				  *  CTX::instance()->mesh.minCircPoints ));
-      double l_n = 1.e12;
-      cc = buildMetricTangentToCurve(t,l_t,l_n);
-    }
-    val = intersection(val,cc);
-  }
-  return val;
-}
-
-SMetric3 max_edge_curvature_metric(const GEdge *ge, double u)
-{
-  SVector3 t =  ge->firstDer(u);
-  t.normalize();
-  double l_t = ((2 * M_PI) /( fabs(ge->curvature(u))
-			      *  CTX::instance()->mesh.minCircPoints ));
-  double l_n = 1.e12;
-  return buildMetricTangentToCurve(t,l_t,l_n);
-}
-
-static double max_edge_curvature(const GVertex *gv)
-{
-  double val = 0;
-  std::list<GEdge*> l_edges = gv->edges();
-  for (std::list<GEdge*>::const_iterator ite = l_edges.begin();
-       ite != l_edges.end(); ++ite){
-    GEdge *_myGEdge = *ite;
-    Range<double> range = _myGEdge->parBounds(0);
-    double cc;
-    if (gv == _myGEdge->getBeginVertex()) cc = _myGEdge->curvature(range.low());
-    else cc = _myGEdge->curvature(range.high());
-    val = std::max(val, cc);
-  }
-  return val;
-}
-
-static double max_surf_curvature(const GEdge *ge, double u)
-{
-  double val = 0;
-  std::list<GFace *> faces = ge->faces();
-  std::list<GFace *>::iterator it = faces.begin();
-  while(it != faces.end()){
-    if ((*it)->geomType() != GEntity::CompoundSurface &&
-        (*it)->geomType() != GEntity::DiscreteSurface){
-      SPoint2 par = ge->reparamOnFace((*it), u, 1);
-      double cc = (*it)->curvature(par);
-      val = std::max(cc, val);
-    }
-    ++it;
-  }
-  return val;
-}
-
-
-// static double max_surf_curvature_vertex(const GVertex *gv)
-// {
-//   double val = 0;
-//   std::list<GEdge*> l_edges = gv->edges();
-//   for (std::list<GEdge*>::const_iterator ite = l_edges.begin();
-//        ite != l_edges.end(); ++ite){
-//     GEdge *_myGEdge = *ite;
-//     Range<double> bounds = _myGEdge->parBounds(0);
-//     if (gv == _myGEdge->getBeginVertex())
-//       val = std::max(val, max_surf_curvature(_myGEdge, bounds.low()));
-//     else
-//       val = std::max(val, max_surf_curvature(_myGEdge, bounds.high()));
-//   }
-//   return val;
-// }
-
-
-SMetric3 metric_based_on_surface_curvature(const GFace *gf, double u, double v,
-					   bool surface_isotropic,
-					   double d_normal ,
-					   double d_tangent_max)
-{
-  if (gf->geomType() == GEntity::Plane)return SMetric3(1.e-12);
-  double cmax, cmin;
-  SVector3 dirMax,dirMin;
-  cmax = gf->curvatures(SPoint2(u, v),&dirMax, &dirMin, &cmax,&cmin);
-  if (cmin == 0)cmin =1.e-12;
-  if (cmax == 0)cmax =1.e-12;
-  double lambda1 =  ((2 * M_PI) /( fabs(cmin) *  CTX::instance()->mesh.minCircPoints ) );
-  double lambda2 =  ((2 * M_PI) /( fabs(cmax) *  CTX::instance()->mesh.minCircPoints ) );
-  SVector3 Z = crossprod(dirMax,dirMin);
-  if (surface_isotropic)  lambda2 = lambda1 = std::min(lambda2,lambda1);
-  dirMin.normalize();
-  dirMax.normalize();
-  Z.normalize();
-  lambda1 = std::max(lambda1, CTX::instance()->mesh.lcMin);
-  lambda2 = std::max(lambda2, CTX::instance()->mesh.lcMin);
-  lambda1 = std::min(lambda1, CTX::instance()->mesh.lcMax);
-  lambda2 = std::min(lambda2, CTX::instance()->mesh.lcMax);
-  double lambda3 = std::min(d_normal, CTX::instance()->mesh.lcMax);
-  lambda3 = std::max(lambda3, CTX::instance()->mesh.lcMin);
-  lambda1 = std::min(lambda1, d_tangent_max);
-  lambda2 = std::min(lambda2, d_tangent_max);
-
-  SMetric3 curvMetric (1./(lambda1*lambda1),1./(lambda2*lambda2),
-		       1./(lambda3*lambda3),
-                       dirMin, dirMax, Z );
-  return curvMetric;
-}
-
-static SMetric3 metric_based_on_surface_curvature(const GEdge *ge, double u, bool iso_surf)
-{
-  const GEdgeCompound* ptrCompoundEdge = dynamic_cast<const GEdgeCompound*>(ge);
-  if (ptrCompoundEdge){
-    double cmax, cmin;
-    SVector3 dirMax,dirMin;
-    cmax = ptrCompoundEdge->curvatures(u,&dirMax, &dirMin, &cmax,&cmin);
-    if (cmin == 0)cmin =1.e-12;
-    if (cmax == 0)cmax =1.e-12;
-    double lambda2 =  ((2 * M_PI) /( fabs(cmax) *  CTX::instance()->mesh.minCircPoints ) );
-    double lambda1 =  ((2 * M_PI) /( fabs(cmin) *  CTX::instance()->mesh.minCircPoints ) );
-    SVector3 Z = crossprod(dirMax,dirMin);
-
-    lambda1 = std::max(lambda1, CTX::instance()->mesh.lcMin);
-    lambda2 = std::max(lambda2, CTX::instance()->mesh.lcMin);
-    lambda1 = std::min(lambda1, CTX::instance()->mesh.lcMax);
-    lambda2 = std::min(lambda2, CTX::instance()->mesh.lcMax);
-
-    SMetric3 curvMetric (1. / (lambda1 * lambda1), 1. / (lambda2 * lambda2),
-                         1.e-12, dirMin, dirMax, Z);
-    return curvMetric;
-  }
-  else{
-    SMetric3 mesh_size(1.e-12);
-    std::list<GFace *> faces = ge->faces();
-    std::list<GFace *>::iterator it = faces.begin();
-    // we choose the metric eigenvectors to be the ones
-    // related to the edge ...
-    SMetric3 curvMetric = max_edge_curvature_metric(ge, u);
-    while(it != faces.end()){
-      if (((*it)->geomType() != GEntity::CompoundSurface) &&
-          ((*it)->geomType() != GEntity::DiscreteSurface)){
-        SPoint2 par = ge->reparamOnFace((*it), u, 1);
-        SMetric3 m = metric_based_on_surface_curvature (*it, par.x(), par.y(), iso_surf);
-        curvMetric = intersection_conserveM1(curvMetric,m);
-      }
-      ++it;
-    }
-
-    return curvMetric;
-  }
-}
-
-static SMetric3 metric_based_on_surface_curvature(const GVertex *gv, bool iso_surf)
-{
-  SMetric3 mesh_size(1.e-15);
-  std::list<GEdge*> l_edges = gv->edges();
-  for (std::list<GEdge*>::const_iterator ite = l_edges.begin();
-       ite != l_edges.end(); ++ite){
-    GEdge *_myGEdge = *ite;
-    Range<double> bounds = _myGEdge->parBounds(0);
-
-    // ES: Added extra if condition to use the code below only with compund curves
-    // This is because we want to call the function
-    // metric_based_on_surface_curvature(const GEdge *ge, double u) for the case when
-    // ge is a compound edge
-    if (_myGEdge->geomType() == GEntity::CompoundCurve){
-      if (gv == _myGEdge->getBeginVertex())
-        mesh_size = intersection
-          (mesh_size,
-           metric_based_on_surface_curvature(_myGEdge, bounds.low(), iso_surf));
-      else
-        mesh_size = intersection
-          (mesh_size,
-           metric_based_on_surface_curvature(_myGEdge, bounds.high(), iso_surf));
-    }
-  }
-  return mesh_size;
-}
-
-// the mesh vertex is classified on a model vertex.  we compute the
-// maximum of the curvature of model faces surrounding this point if
-// it is classified on a model edge, we do the same for all model
-// faces surrounding it if it is on a model face, we compute the
-// curvature at this location
-
-static double LC_MVertex_CURV(GEntity *ge, double U, double V)
-{
-  double Crv = 0;
-  switch(ge->dim()){
-  case 0:
-    Crv = max_edge_curvature((const GVertex *)ge);
-    //Crv = std::max(max_surf_curvature_vertex((const GVertex *)ge), Crv);
-    // Crv = max_surf_curvature((const GVertex *)ge);
-    break;
-  case 1:
-    {
-      GEdge *ged = (GEdge *)ge;
-      Crv = ged->curvature(U);
-      Crv = std::max(Crv, max_surf_curvature(ged, U));
-      // Crv = max_surf_curvature(ged, U);
-    }
-    break;
-  case 2:
-    {
-      GFace *gf = (GFace *)ge;
-      Crv = gf->curvature(SPoint2(U, V));
-    }
-    break;
-  }
-  double lc = Crv > 0 ? 2 * M_PI / Crv / CTX::instance()->mesh.minCircPoints : MAX_LC;
-  return lc;
-}
-
-SMetric3 LC_MVertex_CURV_ANISO(GEntity *ge, double U, double V)
-{
-  bool iso_surf = CTX::instance()->mesh.lcFromCurvature == 2;
-
-  switch(ge->dim()){
-  case 0: return metric_based_on_surface_curvature((const GVertex *)ge, iso_surf);
-  case 1: return metric_based_on_surface_curvature((const GEdge *)ge, U, iso_surf);
-  case 2: return metric_based_on_surface_curvature((const GFace *)ge, U, V, iso_surf);
-  }
-  Msg::Error("Curvature control impossible to compute for a volume!");
-  return SMetric3();
-}
-
-// compute the mesh size at a given vertex due to prescribed sizes at
-// mesh vertices
-static double LC_MVertex_PNTS(GEntity *ge, double U, double V)
-{
-  switch(ge->dim()){
-  case 0:
-    {
-      GVertex *gv = (GVertex *)ge;
-      double lc = gv->prescribedMeshSizeAtVertex();
-      // FIXME we might want to remove this to make all lc treatment consistent
-      if(lc >= MAX_LC) return CTX::instance()->lc / 10.;
-      return lc;
-    }
-  case 1:
-    {
-      GEdge *ged = (GEdge *)ge;
-      GVertex *v1 = ged->getBeginVertex();
-      GVertex *v2 = ged->getEndVertex();
-      if (v1 && v2){
-        Range<double> range = ged->parBounds(0);
-        double a = (U - range.low()) / (range.high() - range.low());
-        double lc = (1 - a) * v1->prescribedMeshSizeAtVertex() +
-          (a) * v2->prescribedMeshSizeAtVertex() ;
-        // FIXME we might want to remove this to make all lc treatment consistent
-        if(lc >= MAX_LC) return CTX::instance()->lc / 10.;
-        return lc;
-      }
-      else
-        return MAX_LC;
-    }
-  default:
-    return MAX_LC;
-  }
-}
-
-// This is the only function that is used by the meshers
-double BGM_MeshSize(GEntity *ge, double U, double V,
-                    double X, double Y, double Z)
-{
-  // default lc (mesh size == size of the model)
-  double l1 = CTX::instance()->lc;
-
-  // lc from points
-  double l2 = MAX_LC;
-  if(CTX::instance()->mesh.lcFromPoints && ge->dim() < 2)
-    l2 = LC_MVertex_PNTS(ge, U, V);
-
-  // lc from curvature
-  double l3 = MAX_LC;
-  if(CTX::instance()->mesh.lcFromCurvature && ge->dim() < 3)
-    l3 = LC_MVertex_CURV(ge, U, V);
-
-  // lc from fields
-  double l4 = MAX_LC;
-  FieldManager *fields = ge->model()->getFields();
-  if(fields->getBackgroundField() > 0){
-    Field *f = fields->get(fields->getBackgroundField());
-    if(f) l4 = (*f)(X, Y, Z, ge);
-  }
-
-  // take the minimum, then constrain by lcMin and lcMax
-  double lc = std::min(std::min(std::min(l1, l2), l3), l4);
-  lc = std::max(lc, CTX::instance()->mesh.lcMin);
-  lc = std::min(lc, CTX::instance()->mesh.lcMax);
-
-  if(lc <= 0.){
-    Msg::Error("Wrong mesh element size lc = %g (lcmin = %g, lcmax = %g)",
-               lc, CTX::instance()->mesh.lcMin, CTX::instance()->mesh.lcMax);
-    lc = l1;
-  }
-
-  //Msg::Debug("BGM X,Y,Z=%g,%g,%g L4=%g L3=%g L2=%g L1=%g LC=%g LFINAL=%g DIM =%d ",
-  //X, Y, Z, l4, l3, l2, l1, lc, lc * CTX::instance()->mesh.lcFactor, ge->dim());
-
-  //Emi fix
-  //if (lc == l1) lc /= 10.;
-
-  return lc * CTX::instance()->mesh.lcFactor;
-}
-
-
-// anisotropic version of the background field
-SMetric3 BGM_MeshMetric(GEntity *ge,
-                        double U, double V,
-                        double X, double Y, double Z)
-{
-
-  // Metrics based on element size
-  // Element size  = min. between default lc and lc from point (if applicable), constrained by lcMin and lcMax
-  double lc = CTX::instance()->lc;
-  if(CTX::instance()->mesh.lcFromPoints && ge->dim() < 2) lc = std::min(lc, LC_MVertex_PNTS(ge, U, V));
-  lc = std::max(lc, CTX::instance()->mesh.lcMin);
-  lc = std::min(lc, CTX::instance()->mesh.lcMax);
-  if(lc <= 0.){
-    Msg::Error("Wrong mesh element size lc = %g (lcmin = %g, lcmax = %g)",
-               lc, CTX::instance()->mesh.lcMin, CTX::instance()->mesh.lcMax);
-    lc = CTX::instance()->lc;
-  }
-  SMetric3 m0(1./(lc*lc));
-
-  // Intersect with metrics from fields if applicable
-  FieldManager *fields = ge->model()->getFields();
-  SMetric3 m1 = m0;
-  if(fields->getBackgroundField() > 0){
-    Field *f = fields->get(fields->getBackgroundField());
-    if(f) {
-      SMetric3 l4;
-      if (!f->isotropic()) (*f)(X, Y, Z, l4, ge);
-      else {
-        const double L = (*f)(X, Y, Z, ge);
-        l4 = SMetric3(1/(L*L));
-      }
-      m1 = intersection(l4, m0);
-    }
-  }
-
-  // Intersect with metrics from curvature if applicable
-  SMetric3 m = (CTX::instance()->mesh.lcFromCurvature && ge->dim() < 3) ?
-      intersection(m1, LC_MVertex_CURV_ANISO(ge, U, V)) : m1;
-
-  return m;
-
-}
-
-bool Extend1dMeshIn2dSurfaces()
-{
-  return CTX::instance()->mesh.lcExtendFromBoundary ? true : false;
-}
-
-bool Extend2dMeshIn3dVolumes()
-{
-  return CTX::instance()->mesh.lcExtendFromBoundary ? true : false;
-}
-
 void backgroundMesh::set(GFace *gf)
 {
   if (_current) delete _current;
@@ -509,7 +90,8 @@ backgroundMesh::backgroundMesh(GFace *_gf, bool cfd)
       else newv = it->second;
       news[j] = newv;
     }
-    _triangles.push_back(new MTriangle(news[0],news[1],news[2]));
+    MTriangle *T2D = new MTriangle(news[0],news[1],news[2]);
+    _triangles.push_back(T2D);
   }
 
 #if defined(HAVE_ANN)
@@ -569,8 +151,9 @@ backgroundMesh::~backgroundMesh()
 }
 
 static void propagateValuesOnFace(GFace *_gf,
-    std::map<MVertex*,double> &dirichlet,
-    bool in_parametric_plane = false)
+                                  std::map<MVertex*,double> &dirichlet,
+				  simpleFunction<double> *ONE,
+				  bool in_parametric_plane = false)
 {
 #if defined(HAVE_SOLVER)
   linearSystem<double> *_lsys = 0;
@@ -594,7 +177,6 @@ static void propagateValuesOnFace(GFace *_gf,
     myAssembler.fixVertex(itv->first, 0, 1, itv->second);
   }
 
-
   // Number vertices
   std::set<MVertex*> vs;
   for (unsigned int k = 0; k < _gf->triangles.size(); k++)
@@ -616,8 +198,7 @@ static void propagateValuesOnFace(GFace *_gf,
     myAssembler.numberVertex(*it, 0, 1);
 
   // Assemble
-  simpleFunction<double> ONE(1.0);
-  laplaceTerm l(0, 1, &ONE);
+  laplaceTerm l(0, 1, ONE);
   for (unsigned int k = 0; k < _gf->triangles.size(); k++){
     MTriangle *t = _gf->triangles[k];
     SElement se(t);
@@ -673,7 +254,8 @@ void backgroundMesh::propagate1dMesh(GFace *_gf)
     }
   }
 
-  propagateValuesOnFace(_gf, sizes);
+  simpleFunction<double> ONE(1.0);
+  propagateValuesOnFace(_gf, sizes,&ONE);
 
   std::map<MVertex*,MVertex*>::iterator itv2 = _2Dto3D.begin();
   for ( ; itv2 != _2Dto3D.end(); ++itv2){
@@ -765,16 +347,100 @@ void backgroundMesh::propagateCrossFieldByDistance(GFace *_gf)
 #endif
 }
 
-inline double myAngle (const SVector3 &a, const SVector3 &b, const SVector3 &d){
+inline double myAngle (const SVector3 &a, const SVector3 &b, const SVector3 &d)
+{
   double cosTheta = dot(a,b);
   double sinTheta = dot(crossprod(a,b),d);
   return atan2 (sinTheta,cosTheta);
 }
 
+// smoothness = h * (|grad (cos 4 a)| + |grad (sin 4 a)|)
+// smoothness is of order 1 if not smooth
+// smoothness is of order h/L if smooth
+// h --> mesh size
+// L --> domain size
+double backgroundMesh::getSmoothness(MElement *e)
+{
+  MVertex *v0 = _3Dto2D[e->getVertex(0)];
+  MVertex *v1 = _3Dto2D[e->getVertex(1)];
+  MVertex *v2 = _3Dto2D[e->getVertex(2)];
+  std::map<MVertex*,double> :: const_iterator i0 = _angles.find (v0);
+  std::map<MVertex*,double> :: const_iterator i1 = _angles.find (v1);
+  std::map<MVertex*,double> :: const_iterator i2 = _angles.find (v2);
+  double a[3] = {cos(4*i0->second),cos(4*i1->second),cos(4*i2->second)};
+  double b[3] = {sin(4*i0->second),sin(4*i1->second),sin(4*i2->second)};
+  //      printf("coucou\n");
+  double f[3];
+  e->interpolateGrad(a,0,0,0,f);
+  const double gradcos = sqrt (f[0]*f[0]+f[1]*f[1]+f[2]*f[2]);
+  e->interpolateGrad(b,0,0,0,f);
+  //const double gradsin = sqrt (f[0]*f[0]+f[1]*f[1]+f[2]*f[2]);
+  const double h = e->maxEdge();
+  return (gradcos /*+ gradsin*/) * h;
+}
+
+double backgroundMesh::getSmoothness(double u, double v, double w)
+{
+  MElement *e = _octree->find(u, v, w, 2, true);
+  if (!e) return -1.0;
+  MVertex *v0 = e->getVertex(0);
+  MVertex *v1 = e->getVertex(1);
+  MVertex *v2 = e->getVertex(2);
+  std::map<MVertex*,double> :: const_iterator i0 = _angles.find (v0);
+  std::map<MVertex*,double> :: const_iterator i1 = _angles.find (v1);
+  std::map<MVertex*,double> :: const_iterator i2 = _angles.find (v2);
+  double a[3] = {cos(4*i0->second),cos(4*i1->second),cos(4*i2->second)};
+  double b[3] = {sin(4*i0->second),sin(4*i1->second),sin(4*i2->second)};
+  //      printf("coucou\n");
+  double f[3];
+  e->interpolateGrad(a,0,0,0,f);
+  const double gradcos = sqrt (f[0]*f[0]+f[1]*f[1]+f[2]*f[2]);
+  e->interpolateGrad(b,0,0,0,f);
+  //const double gradsin = sqrt (f[0]*f[0]+f[1]*f[1]+f[2]*f[2]);
+  const double h = e->maxEdge();
+  return (gradcos /*+ gradsin*/) * h;
+}
 
 void backgroundMesh::propagateCrossField(GFace *_gf)
 {
+  propagateCrossFieldHJ (_gf);
+  // solve the non liear problem
+  constantPerElement<double> C;
+  int ITER = 0;
+  //  int NSMOOTH = _gf->triangles.size();
+  while(0){
+    //    int NSMOOTH_NOW = 0;
+    for (unsigned int i = 0; i < _gf->triangles.size(); i++){
+      double smoothness = getSmoothness (_gf->triangles[i]);
+      double val = smoothness < .5 ? 1.0 : 1.e-3 ;//exp(-absf/10);
+      C.set(_gf->triangles[i],val);
+    }
+    //    if (NSMOOTH_NOW == NSMOOTH) break;
+    //    NSMOOTH = NSMOOTH_NOW;
+    //    break;
+    _angles.clear();
+    propagateCrossField (_gf,&C);
+    if (++ITER > 0)break;
+  }
+  //  printf("converged in %d iterations\n", ITER);
+#if 0 // debug print
+  char name[256];
+  sprintf(name, "cross-%d-%d.pos", _gf->tag(), ITER);
+  print(name, 0, 1);
+  sprintf(name, "smooth-%d-%d.pos", _gf->tag(), ITER);
+  print(name, _gf, 2);
+#endif
+}
 
+void backgroundMesh::propagateCrossFieldHJ(GFace *_gf)
+{
+  simpleFunction<double> ONE(1.0);
+  propagateCrossField (_gf, &ONE);
+
+}
+
+void backgroundMesh::propagateCrossField(GFace *_gf, simpleFunction<double> *ONE)
+{
   std::map<MVertex*,double> _cosines4,_sines4;
 
   std::list<GEdge*> e;
@@ -816,8 +482,8 @@ void backgroundMesh::propagateCrossField(GFace *_gf)
     }
   }
 
-  propagateValuesOnFace(_gf,_cosines4,false);
-  propagateValuesOnFace(_gf,_sines4,false);
+  propagateValuesOnFace(_gf,_cosines4,ONE,false);
+  propagateValuesOnFace(_gf,_sines4,ONE,false);
 
   std::map<MVertex*,MVertex*>::iterator itv2 = _2Dto3D.begin();
   for ( ; itv2 != _2Dto3D.end(); ++itv2){
@@ -863,7 +529,7 @@ void backgroundMesh::updateSizes(GFace *_gf)
     }
   }
   const double _beta = 1.3;
-  for (int i=0;i<0;i++){
+  for (int i=0;i<3;i++){
     std::set<MEdge,Less_Edge>::iterator it = edges.begin();
     for ( ; it != edges.end(); ++it){
       MVertex *v0 = it->getVertex(0);
@@ -980,39 +646,57 @@ double backgroundMesh::getAngle(double u, double v, double w) const
 }
 
 void backgroundMesh::print(const std::string &filename, GFace *gf,
-    const std::map<MVertex*,double> &_whatToPrint) const
+                           const std::map<MVertex*,double> &_whatToPrint, int smooth)
 {
-  FILE *f = Fopen (filename.c_str(),"w");
-  fprintf(f,"View \"Background Mesh\"{\n");
-  for(unsigned int i=0;i<_triangles.size();i++){
-    MVertex *v1 = _triangles[i]->getVertex(0);
-    MVertex *v2 = _triangles[i]->getVertex(1);
-    MVertex *v3 = _triangles[i]->getVertex(2);
-    std::map<MVertex*,double>::const_iterator itv1 = _whatToPrint.find(v1);
-    std::map<MVertex*,double>::const_iterator itv2 = _whatToPrint.find(v2);
-    std::map<MVertex*,double>::const_iterator itv3 = _whatToPrint.find(v3);
-    if (!gf){
+  FILE *f = Fopen(filename.c_str(), "w");
+  if(!f){
+    Msg::Error("Could not open file '%s'", filename.c_str());
+    return;
+  }
+  fprintf(f, "View \"Background Mesh\"{\n");
+  if (smooth){
+    for(unsigned int i = 0; i < gf->triangles.size(); i++){
+      MVertex *v1 = gf->triangles[i]->getVertex(0);
+      MVertex *v2 = gf->triangles[i]->getVertex(1);
+      MVertex *v3 = gf->triangles[i]->getVertex(2);
+      double x = getSmoothness (gf->triangles[i]);
       fprintf(f,"ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%g,%g,%g};\n",
-          v1->x(),v1->y(),v1->z(),
-          v2->x(),v2->y(),v2->z(),
-          v3->x(),v3->y(),v3->z(),itv1->second,itv2->second,itv3->second);
-    }
-    else {
-
-      GPoint p1 = gf->point(SPoint2(v1->x(),v1->y()));
-      GPoint p2 = gf->point(SPoint2(v2->x(),v2->y()));
-      GPoint p3 = gf->point(SPoint2(v3->x(),v3->y()));
-      fprintf(f,"ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%g,%g,%g};\n",
-          p1.x(),p1.y(),p1.z(),
-          p2.x(),p2.y(),p2.z(),
-          p3.x(),p3.y(),p3.z(),itv1->second,itv2->second,itv3->second);
+	      v1->x(), v1->y(), v1->z(),
+	      v2->x(), v2->y(), v2->z(),
+	      v3->x(), v3->y(), v3->z(), x, x, x);
     }
   }
-  fprintf(f,"};\n");
+  else {
+    for(unsigned int i = 0; i < _triangles.size(); i++){
+      MVertex *v1 = _triangles[i]->getVertex(0);
+      MVertex *v2 = _triangles[i]->getVertex(1);
+      MVertex *v3 = _triangles[i]->getVertex(2);
+      std::map<MVertex*,double>::const_iterator itv1 = _whatToPrint.find(v1);
+      std::map<MVertex*,double>::const_iterator itv2 = _whatToPrint.find(v2);
+      std::map<MVertex*,double>::const_iterator itv3 = _whatToPrint.find(v3);
+      if (!gf){
+	fprintf(f,"ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%g,%g,%g};\n",
+		v1->x(), v1->y(), v1->z(),
+		v2->x(), v2->y(), v2->z(),
+		v3->x(), v3->y(), v3->z(), itv1->second, itv2->second, itv3->second);
+      }
+      else {
+	GPoint p1 = gf->point(SPoint2(v1->x(),v1->y()));
+	GPoint p2 = gf->point(SPoint2(v2->x(),v2->y()));
+	GPoint p3 = gf->point(SPoint2(v3->x(),v3->y()));
+	fprintf(f,"ST(%g,%g,%g,%g,%g,%g,%g,%g,%g) {%g,%g,%g};\n",
+		p1.x(), p1.y(), p1.z(),
+		p2.x(), p2.y(), p2.z(),
+		p3.x(), p3.y(), p3.z(), itv1->second, itv2->second, itv3->second);
+      }
+    }
+  }
+  fprintf(f, "};\n");
   fclose(f);
 }
 
 MElementOctree* backgroundMesh::get_octree(){
+
   return _octree;
 }
 
