@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -33,7 +33,7 @@
 #include "discreteFace.h"
 #include "GEdgeCompound.h"
 #include "GFaceCompound.h"
-#include "BackgroundMesh.h"
+#include "BackgroundMeshTools.h"
 #include "meshGFace.h"
 #include "meshGEdge.h"
 #include "MQuadrangle.h"
@@ -44,8 +44,7 @@
 #include "meshGRegion.h"
 
 #if defined(HAVE_ANN)
-#include <ANN/ANN.h>
-
+#include "ANN/ANN.h"
 
 static void erase(std::vector<MLine*>& lines, MLine* l)
 {
@@ -296,21 +295,18 @@ static void cutTriangle(MTriangle *tri,
 Centerline::Centerline(std::string fileName): kdtree(0), kdtreeR(0)
 {
   recombine = (CTX::instance()->mesh.recombineAll) || (CTX::instance()->mesh.recombine3DAll);
-
-  printf("centerline filename =%s \n", fileName.c_str());
-  importFile(fileName);
-  buildKdTree();
   nbPoints = 25;
   hLayer = 0.3;
   hSecondLayer = 0.3;
   nbElemLayer = 3;
   nbElemSecondLayer = 0;
-
-  update_needed = false;
   is_cut = 0;
   is_closed = 0;
   is_extruded = 0;
 
+  importFile(fileName);
+  buildKdTree();
+  update_needed = false;
 }
 
 Centerline::Centerline(): kdtree(0), kdtreeR(0)
@@ -578,8 +574,9 @@ void Centerline::computeRadii()
 
 void Centerline::buildKdTree()
 {
-  FILE * f = Fopen("myPOINTS.pos","w");
-  fprintf(f, "View \"\"{\n");
+  FILE *f = Fopen("myPOINTS.pos","w");
+
+  if(f) fprintf(f, "View \"\"{\n");
 
   int nbPL = 3;  //10 points per line
   //int nbNodes  = (lines.size()+1) + (nbPL*lines.size());
@@ -589,35 +586,37 @@ void Centerline::buildKdTree()
   int ind = 0;
   std::map<MVertex*, int>::iterator itp = colorp.begin();
   while (itp != colorp.end()){
-     MVertex *v = itp->first;
-     nodes[ind][0] = v->x();
-     nodes[ind][1] = v->y();
-     nodes[ind][2] = v->z();
-     itp++; ind++;
+    MVertex *v = itp->first;
+    nodes[ind][0] = v->x();
+    nodes[ind][1] = v->y();
+    nodes[ind][2] = v->z();
+    itp++; ind++;
   }
   for(unsigned int k = 0; k < lines.size(); ++k){
-   MVertex *v0 = lines[k]->getVertex(0);
-   MVertex *v1 = lines[k]->getVertex(1);
-   SVector3 P0(v0->x(),v0->y(), v0->z());
-   SVector3 P1(v1->x(),v1->y(), v1->z());
-   for (int j= 1; j < nbPL+1; j++){
-     double inc = (double)j/(double)(nbPL+1);
-     SVector3 Pj = P0+inc*(P1-P0);
-     nodes[ind][0] = Pj.x();
-     nodes[ind][1] = Pj.y();
-     nodes[ind][2] = Pj.z();
-     ind++;
-   }
- }
+    MVertex *v0 = lines[k]->getVertex(0);
+    MVertex *v1 = lines[k]->getVertex(1);
+    SVector3 P0(v0->x(),v0->y(), v0->z());
+    SVector3 P1(v1->x(),v1->y(), v1->z());
+    for (int j= 1; j < nbPL+1; j++){
+      double inc = (double)j/(double)(nbPL+1);
+      SVector3 Pj = P0+inc*(P1-P0);
+      nodes[ind][0] = Pj.x();
+      nodes[ind][1] = Pj.y();
+      nodes[ind][2] = Pj.z();
+      ind++;
+    }
+  }
 
- kdtree = new ANNkd_tree(nodes, nbNodes, 3);
+  kdtree = new ANNkd_tree(nodes, nbNodes, 3);
 
- for(int i = 0; i < nbNodes; ++i){
-   fprintf(f, "SP(%g,%g,%g){%g};\n",
-	   nodes[i][0], nodes[i][1],nodes[i][2],1.0);
- }
- fprintf(f,"};\n");
- fclose(f);
+  if(f){
+    for(int i = 0; i < nbNodes; ++i){
+      fprintf(f, "SP(%g,%g,%g){%g};\n",
+              nodes[i][0], nodes[i][1],nodes[i][2],1.0);
+    }
+    fprintf(f,"};\n");
+    fclose(f);
+  }
 }
 
 void Centerline::createSplitCompounds()
@@ -1009,8 +1008,10 @@ bool Centerline::cutByDisk(SVector3 &PT, SVector3 &NORM, double &maxRad)
       double rdist = -V1/(V2-V1);
       if (inters && rdist > EPS && rdist < 1.-EPS){
 	SVector3 PZ = P1+rdist*(P2-P1);
-	MVertex *newv = new MVertex (PZ.x(), PZ.y(), PZ.z());
-	if (inDisk) cutEdges.insert(std::make_pair(me,newv));
+	if (inDisk){
+          MVertex *newv = new MVertex (PZ.x(), PZ.y(), PZ.z());
+          cutEdges.insert(std::make_pair(me,newv));
+        }
       }
       else if (inters && rdist <= EPS && inDisk )
 	cutVertices.push_back(me.getVertex(0));
@@ -1034,9 +1035,10 @@ bool Centerline::cutByDisk(SVector3 &PT, SVector3 &NORM, double &maxRad)
       // char name[256];
       // sprintf(name, "myCUT-%d.pos", step);
       // FILE * f2 = Fopen(name,"w");
-      // fprintf(f2, "View \"\"{\n");
-      // std::set<MEdge,Less_Edge>::iterator itp =  newCut.begin();
-      // while (itp != newCut.end()){
+      // if(f2){
+      //   fprintf(f2, "View \"\"{\n");
+      //   std::set<MEdge,Less_Edge>::iterator itp =  newCut.begin();
+      //   while (itp != newCut.end()){
       // 	MEdge l = *itp;
       // 	fprintf(f2, "SL(%g,%g,%g,%g,%g,%g){%g,%g};\n",
       // 		  l.getVertex(0)->x(), l.getVertex(0)->y(), l.getVertex(0)->z(),
@@ -1046,6 +1048,7 @@ bool Centerline::cutByDisk(SVector3 &PT, SVector3 &NORM, double &maxRad)
       // 	}
       // 	fprintf(f2,"};\n");
       // 	fclose(f2);
+      // }
     }
   }
 
@@ -1283,46 +1286,51 @@ void Centerline::computeCrossField(double x,double y,double z,
 void Centerline::printSplit() const
 {
   FILE * f = Fopen("mySPLIT.pos","w");
-  fprintf(f, "View \"\"{\n");
-  for(unsigned int i = 0; i < edges.size(); ++i){
-    std::vector<MLine*> lines = edges[i].lines;
-    for(unsigned int k = 0; k < lines.size(); ++k){
-      MLine *l = lines[k];
-      fprintf(f, "SL(%g,%g,%g,%g,%g,%g){%g,%g};\n",
-	      l->getVertex(0)->x(), l->getVertex(0)->y(), l->getVertex(0)->z(),
-	      l->getVertex(1)->x(), l->getVertex(1)->y(), l->getVertex(1)->z(),
-	      (double)edges[i].tag, (double)edges[i].tag);
+  if(f){
+    fprintf(f, "View \"\"{\n");
+    for(unsigned int i = 0; i < edges.size(); ++i){
+      std::vector<MLine*> lines = edges[i].lines;
+      for(unsigned int k = 0; k < lines.size(); ++k){
+        MLine *l = lines[k];
+        fprintf(f, "SL(%g,%g,%g,%g,%g,%g){%g,%g};\n",
+                l->getVertex(0)->x(), l->getVertex(0)->y(), l->getVertex(0)->z(),
+                l->getVertex(1)->x(), l->getVertex(1)->y(), l->getVertex(1)->z(),
+                (double)edges[i].tag, (double)edges[i].tag);
+      }
     }
+    fprintf(f,"};\n");
+    fclose(f);
   }
-  fprintf(f,"};\n");
-  fclose(f);
 
   // FILE * f3 = Fopen("myJUNCTIONS.pos","w");
-  // fprintf(f3, "View \"\"{\n");
-  //  std::set<MVertex*>::const_iterator itj = junctions.begin();
-  //  while (itj != junctions.end()){
-  //    MVertex *v =  *itj;
-  //    fprintf(f3, "SP(%g,%g,%g){%g};\n",
+  // if(f3){
+  //   fprintf(f3, "View \"\"{\n");
+  //    std::set<MVertex*>::const_iterator itj = junctions.begin();
+  //    while (itj != junctions.end()){
+  //      MVertex *v =  *itj;
+  //      fprintf(f3, "SP(%g,%g,%g){%g};\n",
   // 	     v->x(),  v->y(), v->z(),
   // 	     (double)v->getNum());
-  //    itj++;
+  //      itj++;
+  //   }
+  //   fprintf(f3,"};\n");
+  //   fclose(f3);
   // }
-  // fprintf(f3,"};\n");
-  // fclose(f3);
 
   FILE * f4 = Fopen("myRADII.pos","w");
-  fprintf(f4, "View \"\"{\n");
-  for(unsigned int i = 0; i < lines.size(); ++i){
-    MLine *l = lines[i];
-    std::map<MLine*,double>::const_iterator itc =  radiusl.find(l);
-    fprintf(f4, "SL(%g,%g,%g,%g,%g,%g){%g,%g};\n",
- 	    l->getVertex(0)->x(), l->getVertex(0)->y(), l->getVertex(0)->z(),
- 	    l->getVertex(1)->x(), l->getVertex(1)->y(), l->getVertex(1)->z(),
- 	    itc->second,itc->second);
+  if(f4){
+    fprintf(f4, "View \"\"{\n");
+    for(unsigned int i = 0; i < lines.size(); ++i){
+      MLine *l = lines[i];
+      std::map<MLine*,double>::const_iterator itc =  radiusl.find(l);
+      fprintf(f4, "SL(%g,%g,%g,%g,%g,%g){%g,%g};\n",
+              l->getVertex(0)->x(), l->getVertex(0)->y(), l->getVertex(0)->z(),
+              l->getVertex(1)->x(), l->getVertex(1)->y(), l->getVertex(1)->z(),
+              itc->second,itc->second);
+    }
+    fprintf(f4,"};\n");
+    fclose(f4);
   }
-  fprintf(f4,"};\n");
-  fclose(f4);
-
 }
 
 #endif
