@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -14,6 +14,7 @@
 #include <string>
 #include <string.h>
 #include <sstream>
+#include <algorithm>
 #include "GmshConfig.h"
 #include "Context.h"
 #include "Field.h"
@@ -22,7 +23,7 @@
 #include "GmshMessage.h"
 #include "Numeric.h"
 #include "mathEvaluator.h"
-#include "BackgroundMesh.h"
+#include "BackgroundMeshTools.h"
 #include "CenterlineField.h"
 #include "STensor3.h"
 #include "meshMetric.h"
@@ -485,7 +486,7 @@ class CylinderField : public Field
   }
 };
 
-class SphereField : public Field
+class BallField : public Field
 {
   double v_in, v_out;
   double xc,yc,zc;
@@ -494,35 +495,33 @@ class SphereField : public Field
  public:
   std::string getDescription()
   {
-    return "The value of this field is VIn inside a sphere, VOut outside. "
-      "The sphere is given by\n\n"
+    return "The value of this field is VIn inside a spherical ball, VOut outside. "
+      "The ball is defined by\n\n"
       "  ||dX||^2 < R^2 &&\n"
       "  dX = (X - XC)^2 + (Y-YC)^2 + (Z-ZC)^2";
   }
-  SphereField()
+  BallField()
   {
     v_in = v_out = xc = yc = zc = R = 0;
 
     options["VIn"] = new FieldOptionDouble
-      (v_in, "Value inside the sphere");
+      (v_in, "Value inside the ball");
     options["VOut"] = new FieldOptionDouble
-      (v_out, "Value outside the sphere");
+      (v_out, "Value outside the ball");
 
     options["XCenter"] = new FieldOptionDouble
-      (xc, "X coordinate of the sphere center");
+      (xc, "X coordinate of the ball center");
     options["YCenter"] = new FieldOptionDouble
-      (yc, "Y coordinate of the sphere center");
+      (yc, "Y coordinate of the ball center");
     options["ZCenter"] = new FieldOptionDouble
-      (zc, "Z coordinate of the sphere center");
-
+      (zc, "Z coordinate of the ball center");
 
     options["Radius"] = new FieldOptionDouble
       (R,"Radius");
-
   }
   const char *getName()
   {
-    return "Sphere";
+    return "Ball";
   }
   double operator() (double x, double y, double z, GEntity *ge=0)
   {
@@ -1298,19 +1297,20 @@ class PostViewField : public Field
     if(!octree->searchTensorWithTol(x, y, z, l, 0, 0, 0.05))
       Msg::Info("No tensor element found containing point (%g,%g,%g)", x, y, z);
     if(crop_negative_values){
-      for(int i = 0; i < 9; i++){
-        if(l[i] <= 0) l[i] = MAX_LC;
+      if(l[0] <= 0 && l[1] <= 0 && l[2] <= 0 &&
+         l[3] <= 0 && l[4] <= 0 && l[5] <= 0 &&
+         l[6] <= 0 && l[7] <= 0 && l[8] <= 0){
+        for(int i = 0; i < 9; i++) l[i] = MAX_LC;
+      }
+      else{
+        for(int i = 0; i < 9; i++){
+          if(l[i] <= 0) l[i] = 0;
+        }
       }
     }
-    metr(0, 0) = l[0];
-    metr(0, 1) = l[1];
-    metr(0, 2) = l[2];
-    metr(1, 0) = l[3];
-    metr(1, 1) = l[4];
-    metr(1, 2) = l[5];
-    metr(2, 0) = l[6];
-    metr(2, 1) = l[7];
-    metr(2, 2) = l[8];
+    metr(0, 0) = l[0]; metr(0, 1) = l[1]; metr(0, 2) = l[2];
+    metr(1, 0) = l[3]; metr(1, 1) = l[4]; metr(1, 2) = l[5];
+    metr(2, 0) = l[6]; metr(2, 1) = l[7]; metr(2, 2) = l[8];
   }
   const char *getName()
   {
@@ -1539,12 +1539,13 @@ class MaxField : public Field
 class RestrictField : public Field
 {
   int iField;
-  std::list<int> edges, faces, regions;
+  std::list<int> vertices, edges, faces, regions;
  public:
   RestrictField()
   {
     iField = 1;
     options["IField"] = new FieldOptionInt(iField, "Field index");
+    options["VerticesList"] = new FieldOptionList(vertices, "Point indices");
     options["EdgesList"] = new FieldOptionList(edges, "Curve indices");
     options["FacesList"] = new FieldOptionList(faces, "Surface indices");
     options["RegionsList"] = new FieldOptionList(regions, "Volume indices");
@@ -1552,14 +1553,15 @@ class RestrictField : public Field
   std::string getDescription()
   {
     return "Restrict the application of a field to a given list of geometrical "
-      "curves, surfaces or volumes.";
+      "points, curves, surfaces or volumes.";
   }
   double operator() (double x, double y, double z, GEntity *ge=0)
   {
     Field *f = (GModel::current()->getFields()->get(iField));
     if(!f || iField == id) return MAX_LC;
     if(!ge) return (*f) (x, y, z);
-    if((ge->dim() == 0) ||
+    if((ge->dim() == 0 && std::find
+        (vertices.begin(), vertices.end(), ge->tag()) != vertices.end()) ||
        (ge->dim() == 1 && std::find
         (edges.begin(), edges.end(), ge->tag()) != edges.end()) ||
        (ge->dim() == 2 && std::find
@@ -1821,7 +1823,6 @@ class AttractorField : public Field
           it != faces_id.end(); ++it) {
 	GFace *f = GModel::current()->getFaceByTag(*it);
 	if (f){
-
 	  if (f->mesh_vertices.size()){
 	    for (unsigned int i=0;i<f->mesh_vertices.size();i++){
 	      MVertex *v = f->mesh_vertices[i];
@@ -1848,7 +1849,7 @@ class AttractorField : public Field
         ((points.size()) ? points.size() :
          n_nodes_by_edge * n_nodes_by_edge * faces_id.size());
 
-      Msg::Info("%d points found in points clouds (%d edges)", totpoints,
+      Msg::Info("%d points found in point clouds (%d edges)", totpoints,
                 (int)edges_id.size());
 
       if(totpoints){
@@ -2031,7 +2032,8 @@ void BoundaryLayerField::setupFor2d(int iF)
 	 it != ed.end() ; ++it){
       bool isIn = false;
       int iE = (*it)->tag();
-      bool found = std::find ( edges_id_saved.begin(),edges_id_saved.end(),iE ) != edges_id_saved.end();
+      bool found = std::find(edges_id_saved.begin(), edges_id_saved.end(), iE) !=
+        edges_id_saved.end();
       //      printf("edges %d found %d\n",iE,found);
       // this edge is a BL Edge
       if (found){
@@ -2041,15 +2043,17 @@ void BoundaryLayerField::setupFor2d(int iF)
 	else {
 	  // more than one face and
 	  std::list<GFace*>::iterator itf = fc.begin();
-	  bool found_this = std::find ( faces_id_saved.begin(),faces_id_saved.end(),iF ) != faces_id_saved.end();
+	  bool found_this = std::find(faces_id_saved.begin(), faces_id_saved.end(), iF) !=
+            faces_id_saved.end();
 	  if (!found_this)isIn = true;
 	  else {
 	    bool foundAll = true;
 	    for ( ; itf != fc.end() ; ++itf){
 	      int iF2 = (*itf)->tag();
-	      foundAll &= std::find ( faces_id_saved.begin(),faces_id_saved.end(),iF2 ) != faces_id_saved.end();
+	      foundAll &= std::find(faces_id_saved.begin(), faces_id_saved.end(), iF2) !=
+                faces_id_saved.end();
 	    }
-	    if (foundAll)isIn = true;
+	    if(foundAll) isIn = true;
 	  }
 	}
       }
@@ -2059,8 +2063,7 @@ void BoundaryLayerField::setupFor2d(int iF)
 	nodes_id.push_back ((*it)->getEndVertex()->tag());
       }
     }
-    //    printf("face %d %d BL Edges\n", iF, (int)edges_id.size());
-
+    // printf("face %d %d BL Edges\n", iF, (int)edges_id.size());
     removeAttractors();
   }
 }
@@ -2071,10 +2074,8 @@ void BoundaryLayerField::setupFor3d()
   removeAttractors();
 }
 
-
 double BoundaryLayerField::operator() (double x, double y, double z, GEntity *ge)
 {
-
   if (update_needed){
     for(std::list<int>::iterator it = nodes_id.begin();
 	it != nodes_id.end(); ++it) {
@@ -2110,8 +2111,8 @@ double BoundaryLayerField::operator() (double x, double y, double z, GEntity *ge
 }
 
 // assume that the closest point is one of the model vertices
-void BoundaryLayerField::computeFor1dMesh (double x, double y, double z,
-					   SMetric3 &metr)
+void BoundaryLayerField::computeFor1dMesh(double x, double y, double z,
+                                          SMetric3 &metr)
 {
   double xpk = 0., ypk = 0., zpk = 0.;
   double distk = 1.e22;
@@ -2300,7 +2301,7 @@ FieldManager::FieldManager()
 #endif
   map_type_name["Box"] = new FieldFactoryT<BoxField>();
   map_type_name["Cylinder"] = new FieldFactoryT<CylinderField>();
-  map_type_name["Sphere"] = new FieldFactoryT<SphereField>();
+  map_type_name["Ball"] = new FieldFactoryT<BallField>();
   map_type_name["Frustum"] = new FieldFactoryT<FrustumField>();
   map_type_name["LonLat"] = new FieldFactoryT<LonLatField>();
 #if defined(HAVE_POST)
@@ -2397,3 +2398,40 @@ void FieldManager::setBackgroundMesh(int iView)
   (*this)[id] = f;
   _background_field = id;
 }
+    
+
+
+
+GenericField::GenericField(){};
+
+
+GenericField::~GenericField(){};
+
+
+double GenericField::operator() (double x, double y, double z, GEntity *ge){
+  std::vector<double> sizes(cbs.size());
+  std::vector<ptrfunction>::iterator itcbs = cbs.begin();
+  std::vector<void*>::iterator itdata = user_data.begin();
+//  std::cout << "#callbacks=" << cbs.size() << std::endl;
+//  std::cout << "#user_data=" << user_data.size() << std::endl;
+  for (std::vector<double>::iterator it = sizes.begin();it!=sizes.end();it++,itdata++,itcbs++){
+    bool ok = (*itcbs)(x,y,z,(*itdata),(*it));
+    if (!ok){
+      Msg::Warning("GenericField::ERROR from callback ");
+      std::cout << "GenericField::ERROR from callback number " << std::distance(sizes.begin(),it) << std::endl;
+    }
+//    std::cout << "callback " << std::distance(sizes.begin(),it) << ": size set to " << *it << std::endl;
+  }
+//  std::cout << "   ----> min = " << (*std::min_element(sizes.begin(),sizes.end())) << std::endl;
+  return (*std::min_element(sizes.begin(),sizes.end()));
+}
+
+
+
+void GenericField::setCallbackWithData(ptrfunction fct, void *data){
+  user_data.push_back(data);
+  cbs.push_back(fct);
+}
+
+
+
