@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -36,6 +36,8 @@
 #include "simple3D.h"
 #include "yamakawa.h"
 
+#include "pointInsertion.h"
+
 #if defined(HAVE_OPTHOM)
 #include "OptHomRun.h"
 #include "OptHomElastic.h"
@@ -49,43 +51,32 @@
 template<class T>
 static void GetQualityMeasure(std::vector<T*> &ele,
                               double &gamma, double &gammaMin, double &gammaMax,
-                              double &eta, double &etaMin, double &etaMax,
+                              double &minSICN, double &minSICNMin, double &minSICNMax,
                               double &rho, double &rhoMin, double &rhoMax,
-                              double &disto, double &distoMin, double &distoMax,
-                              double &scaledJacMin, double &scaledJacMax,
-                              double quality[4][100])
+                              double quality[3][100])
 {
   for(unsigned int i = 0; i < ele.size(); i++){
     double g = ele[i]->gammaShapeMeasure();
     gamma += g;
     gammaMin = std::min(gammaMin, g);
     gammaMax = std::max(gammaMax, g);
-    double e = ele[i]->etaShapeMeasure();
-    eta += e;
-    etaMin = std::min(etaMin, e);
-    etaMax = std::max(etaMax, e);
+    double s = ele[i]->minSICNShapeMeasure();
+    minSICN += s;
+    minSICNMin = std::min(minSICNMin, s);
+    minSICNMax = std::max(minSICNMax, s);
     double r = ele[i]->rhoShapeMeasure();
     rho += r;
     rhoMin = std::min(rhoMin, r);
     rhoMax = std::max(rhoMax, r);
-    double jmin,jmax; ele[i]->scaledJacRange(jmin,jmax);
-    double d = jmin;
-    disto += d;
-    distoMin = std::min(distoMin, d);
-    distoMax = std::max(distoMax, d);
-    scaledJacMin = std::min(scaledJacMin, jmin);
-    scaledJacMax = std::max(scaledJacMax, jmax);
-
     for(int j = 0; j < 100; j++){
-      if(g > j / 100. && g <= (j + 1) / 100.) quality[0][j]++;
-      if(e > j / 100. && e <= (j + 1) / 100.) quality[1][j]++;
+      if(s > (2*j-100) / 100. && s <= (2*j-98) / 100.) quality[0][j]++;
+      if(g > j / 100. && g <= (j + 1) / 100.) quality[1][j]++;
       if(r > j / 100. && r <= (j + 1) / 100.) quality[2][j]++;
-      if(d > j / 100. && d <= (j + 1) / 100.) quality[3][j]++;
     }
   }
 }
 
-void GetStatistics(double stat[50], double quality[4][100])
+void GetStatistics(double stat[50], double quality[3][100])
 {
   for(int i = 0; i < 50; i++) stat[i] = 0.;
 
@@ -107,9 +98,11 @@ void GetStatistics(double stat[50], double quality[4][100])
     stat[4] += (*it)->mesh_vertices.size();
 
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
+    // TODO: make this an option! if((*it)->getVisibility()){
     stat[5] += (*it)->mesh_vertices.size();
     stat[7] += (*it)->triangles.size();
     stat[8] += (*it)->quadrangles.size();
+    //}
   }
 
   for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it){
@@ -128,45 +121,36 @@ void GetStatistics(double stat[50], double quality[4][100])
     for(int i = 0; i < 3; i++)
       for(int j = 0; j < 100; j++)
         quality[i][j] = 0.;
+    double minSICN = 0., minSICNMin = 1., minSICNMax = -1.;
     double gamma = 0., gammaMin = 1., gammaMax = 0.;
-    double eta = 0., etaMin = 1., etaMax = 0.;
     double rho = 0., rhoMin = 1., rhoMax = 0.;
-    double disto = 0., distoMin=1., distoMax = 0.;
-    double jmin = 1.e22, jmax = -1.e22;
 
     double N = stat[9] + stat[10] + stat[11] + stat[12];
     if(N){ // if we have 3D elements
       for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it){
         GetQualityMeasure((*it)->tetrahedra, gamma, gammaMin, gammaMax,
-                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-                          disto, distoMin, distoMax, jmin, jmax, quality);
+                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
         GetQualityMeasure((*it)->hexahedra, gamma, gammaMin, gammaMax,
-                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-                          disto, distoMin, distoMax, jmin, jmax, quality);
+                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
         GetQualityMeasure((*it)->prisms, gamma, gammaMin, gammaMax,
-                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-                          disto, distoMin, distoMax,jmin, jmax, quality);
+                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
         GetQualityMeasure((*it)->pyramids, gamma, gammaMin, gammaMax,
-                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-                          disto, distoMin, distoMax, jmin, jmax, quality);
+                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
       }
     }
     else{ // 2D elements
       N = stat[7] + stat[8];
       for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
         GetQualityMeasure((*it)->quadrangles, gamma, gammaMin, gammaMax,
-                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-                          disto, distoMin, distoMax, jmin, jmax, quality);
+                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
         GetQualityMeasure((*it)->triangles, gamma, gammaMin, gammaMax,
-                          eta, etaMin, etaMax, rho, rhoMin, rhoMax,
-                          disto, distoMin, distoMax, jmin, jmax, quality);
+                          minSICN, minSICNMin, minSICNMax, rho, rhoMin, rhoMax, quality);
       }
     }
     if(N){
-      stat[17] = gamma / N ;  stat[18] = gammaMin;  stat[19] = gammaMax;
-      stat[20] = eta / N ;    stat[21] = etaMin;    stat[22] = etaMax;
-      stat[23] = rho / N ;    stat[24] = rhoMin;    stat[25] = rhoMax;
-      stat[46] = disto / N ;  stat[47] = distoMin;  stat[48] = distoMax;
+      stat[17] = minSICN / N; stat[18] = minSICNMin; stat[19] = minSICNMax;
+      stat[20] = gamma / N;   stat[21] = gammaMin;   stat[22] = gammaMax;
+      stat[23] = rho / N;     stat[24] = rhoMin;     stat[25] = rhoMax;
     }
   }
 
@@ -235,14 +219,13 @@ static void Mesh0D(GModel *m)
   }
   for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); ++it){
     GVertex *gv = *it;
-    if (gv->meshMaster() != gv->tag()){
+    if (gv->meshMaster() != gv){
       if (gv->correspondingVertices.empty()){
-	GVertex *master = m->getVertexByTag(abs(gv->meshMaster()));
-	if(master)gv->correspondingVertices[gv->mesh_vertices[0]] = (master->mesh_vertices[0]);
+        GVertex *master = dynamic_cast<GVertex*> (gv->meshMaster());
+        if(master)gv->correspondingVertices[gv->mesh_vertices[0]] = (master->mesh_vertices[0]);
       }
     }
   }
-
 }
 
 static void Mesh1D(GModel *m)
@@ -262,8 +245,8 @@ static void Mesh1D(GModel *m)
     int nPending = 0;
     for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it){
       if ((*it)->meshStatistics.status == GEdge::PENDING){
-	mesher(*it);
-	nPending++;
+        mesher(*it);
+        nPending++;
       }
       if(!nIter) Msg::ProgressMeter(nPending, nTot, false, "Meshing 1D...");
     }
@@ -278,13 +261,19 @@ static void Mesh1D(GModel *m)
 
 static void PrintMesh2dStatistics(GModel *m)
 {
-
   FILE *statreport = 0;
   if(CTX::instance()->createAppendMeshStatReport == 1)
     statreport = Fopen(CTX::instance()->meshStatReportFileName.c_str(), "w");
   else if(CTX::instance()->createAppendMeshStatReport == 2)
     statreport = Fopen(CTX::instance()->meshStatReportFileName.c_str(), "a");
-  else return;
+  else
+    return;
+
+  if(!statreport){
+    Msg::Error("Could not open file '%s'",
+               CTX::instance()->meshStatReportFileName.c_str());
+    return;
+  }
 
   double worst = 1, best = 0, avg = 0;
   double e_long = 0, e_short = 1.e22, e_avg = 0;
@@ -310,7 +299,7 @@ static void PrintMesh2dStatistics(GModel *m)
       e_long = std::max((*it)->meshStatistics.longest_edge_length, e_long);
       e_short = std::min((*it)->meshStatistics.smallest_edge_length, e_short);
       if ((*it)->meshStatistics.status == GFace::FAILED ||
-	  (*it)->meshStatistics.status == GFace::PENDING) nUnmeshed++;
+          (*it)->meshStatistics.status == GFace::PENDING) nUnmeshed++;
       nTotT += (*it)->meshStatistics.nbTriangle;
       nTotE += (*it)->meshStatistics.nbEdge;
       nTotGoodLength += (*it)->meshStatistics.nbGoodLength;
@@ -362,15 +351,15 @@ static void Mesh2D(GModel *m)
 #pragma omp parallel for schedule (dynamic)
 #endif
       for(size_t K = 0 ; K < temp.size() ; K++){
-	if (temp[K]->meshStatistics.status == GFace::PENDING){
+        if (temp[K]->meshStatistics.status == GFace::PENDING){
           backgroundMesh::current()->unset();
-	  meshGFace mesher(true);
-	  mesher(temp[K]);
-
+          meshGFace mesher(true);
+          mesher(temp[K]);
 #if defined(HAVE_BFGS)
           if(CTX::instance()->mesh.optimizeLloyd){
             if (temp[K]->geomType()==GEntity::CompoundSurface ||
-                temp[K]->geomType()==GEntity::Plane || temp[K]->geomType()==GEntity::RuledSurface) {
+                temp[K]->geomType()==GEntity::Plane ||
+                temp[K]->geomType()==GEntity::RuledSurface) {
               if (temp[K]->meshAttributes.method != MESH_TRANSFINITE &&
                   !temp[K]->meshAttributes.extrude) {
                 smoothing smm(CTX::instance()->mesh.optimizeLloyd,6);
@@ -386,14 +375,13 @@ static void Mesh2D(GModel *m)
             }
           }
 #endif
-
 #if defined(_OPENMP)
 #pragma omp critical
 #endif
-	  {
-	    nPending++;
-	  }
-	}
+          {
+            nPending++;
+          }
+        }
         if(!nIter) Msg::ProgressMeter(nPending, nTot, false, "Meshing 2D...");
       }
 #if defined(_OPENMP)
@@ -405,11 +393,11 @@ static void Mesh2D(GModel *m)
           backgroundMesh::current()->unset();
           meshGFace mesher(true);
           mesher(*it);
-
 #if defined(HAVE_BFGS)
           if(CTX::instance()->mesh.optimizeLloyd){
             if ((*it)->geomType()==GEntity::CompoundSurface ||
-                (*it)->geomType()==GEntity::Plane || (*it)->geomType()==GEntity::RuledSurface) {
+                (*it)->geomType()==GEntity::Plane ||
+                (*it)->geomType()==GEntity::RuledSurface) {
               if ((*it)->meshAttributes.method != MESH_TRANSFINITE &&
                   !(*it)->meshAttributes.extrude) {
                 smoothing smm(CTX::instance()->mesh.optimizeLloyd,6);
@@ -425,7 +413,6 @@ static void Mesh2D(GModel *m)
             }
           }
 #endif
-
           nPending++;
         }
         if(!nIter) Msg::ProgressMeter(nPending, nTot, false, "Meshing 2D...");
@@ -460,10 +447,10 @@ static void FindConnectedRegions(std::vector<GRegion*> &delaunay,
       oneDomain.insert(r);
       std::list<GFace*> faces = r->faces();
       for (std::list<GFace*> :: iterator it = faces.begin(); it != faces.end() ; ++it){
-	GFace *gf = *it;
-	GRegion *other = gf->getRegion(0) == r ? gf->getRegion(1) : gf->getRegion(0);
-	if (other != 0 && oneDomain.find(other) == oneDomain.end())
-	  _stack.push (other);
+        GFace *gf = *it;
+        GRegion *other = gf->getRegion(0) == r ? gf->getRegion(1) : gf->getRegion(0);
+        if (other != 0 && oneDomain.find(other) == oneDomain.end())
+          _stack.push (other);
       }
     }
     std::vector<GRegion*> temp1,temp2;
@@ -476,7 +463,7 @@ static void FindConnectedRegions(std::vector<GRegion*> &delaunay,
     delaunay=temp1;
   }
   Msg::Info("Delaunay Meshing %d volumes with %d connected components",
-	    nbVolumes,connected.size());
+            nbVolumes,connected.size());
 }
 
 static void Mesh3D(GModel *m)
@@ -506,42 +493,75 @@ static void Mesh3D(GModel *m)
   FindConnectedRegions(delaunay, connected);
 
   // remove quads elements for volumes that are recombined
-  // pragma OMP ICI ??
+  // pragma OMP here ?
   for(unsigned int i = 0; i < connected.size(); i++){
-    for(unsigned j=0;j<connected[i].size();j++){
+    for(unsigned j = 0; j < connected[i].size(); j++){
       GRegion *gr = connected[i][j];
       if(CTX::instance()->mesh.recombine3DAll || gr->meshAttributes.recombine3D){
-	std::list<GFace*> f = gr->faces();
-	for (std::list<GFace*>::iterator it = f.begin();
-	     it != f.end() ; ++it) quadsToTriangles (*it,1000000);
+        std::list<GFace*> f = gr->faces();
+        for(std::list<GFace*>::iterator it = f.begin(); it != f.end() ; ++it)
+          quadsToTriangles(*it, 1000000);
       }
     }
   }
 
-  // pragma OMP ICI ??
+  double time_recombination = 0., vol_element_recombination = 0.;
+  double vol_hexa_recombination = 0.;
+  int nb_elements_recombination = 0, nb_hexa_recombination = 0;
+
+  // pragma OMP here ?
   for(unsigned int i = 0; i < connected.size(); i++){
     MeshDelaunayVolume(connected[i]);
 
-    //Additional code for hex mesh begin
-    for(unsigned j=0;j<connected[i].size();j++){
+    // additional code for experimental hex mesh
+    for(unsigned j = 0; j < connected[i].size(); j++){
       GRegion *gr = connected[i][j];
-      //R-tree
+      bool treat_region_ok = false;
       if(CTX::instance()->mesh.algo3d == ALGO_3D_RTREE){
-	Filler f;
-	f.treat_region(gr);
+        if (old_algo_hexa()){
+          Filler f;
+          f.treat_region(gr);
+          treat_region_ok = true;
+        }
+        else{
+          Filler3D f;
+          treat_region_ok = f.treat_region(gr);
+        }
       }
-      //Recombine3D into hex
-      if(CTX::instance()->mesh.recombine3DAll || gr->meshAttributes.recombine3D){
-	Recombinator rec;
-	rec.execute();
-	Supplementary sup;
-	sup.execute();
-	PostOp post;
-	post.execute(0);
+      if(treat_region_ok && (CTX::instance()->mesh.recombine3DAll ||
+                             gr->meshAttributes.recombine3D)){
+        double a = Cpu();
+        Recombinator rec;
+        rec.execute(gr);
+        Supplementary sup;
+        sup.execute(gr);
+        PostOp post;
+        post.execute(gr,0);
+        nb_elements_recombination += post.get_nb_elements();
+        nb_hexa_recombination += post.get_nb_hexahedra();
+        vol_element_recombination += post.get_vol_elements();
+        vol_hexa_recombination += post.get_vol_hexahedra();
+        // partial export
+        stringstream ss;
+        ss << "yamakawa_part_";
+        ss << gr->tag();
+        ss << ".msh";
+        export_gregion_mesh(gr, ss.str().c_str());
+        time_recombination += (Cpu() - a);
       }
     }
   }
 
+  if(CTX::instance()->mesh.recombine3DAll){
+    Msg::Info("RECOMBINATION timing:");
+    Msg::Info(" --- CUMULATIVE TIME RECOMBINATION : %g s.", time_recombination);
+    Msg::Info("RECOMBINATION CUMULATIVE STATISTICS:");
+    Msg::Info(".... Percentage of hexahedra   (#) : %g",
+              nb_hexa_recombination*100./nb_elements_recombination);
+    Msg::Info(".... Percentage of hexahedra (Vol) : %g",
+              vol_hexa_recombination*100./vol_element_recombination);
+  }
+
   // Ensure that all volume Jacobians are positive
   m->setAllVolumesPositive();
 
