@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -58,6 +58,8 @@ static bool computeEquidistantParameters(GEdge *ge, double u0, double uN, int N,
     u[i] = u[i - 1] + du;
   }
 
+  //return true;
+
   // create the tangent matrix
   const int M = N - 2;
   fullMatrix<double> J(M, M);
@@ -101,116 +103,109 @@ static bool computeEquidistantParameters(GEdge *ge, double u0, double uN, int N,
 }
 
 static bool computeEquidistantParameters(GFace *gf, double u0, double uN,
-                                         double v0, double vN, int N,
+                                         double v0, double vN, SPoint3 &p0,
+                                         SPoint3 &pN, int N, bool geodesic,
                                          double *u, double *v)
 {
-  double t[100];
-  // initialize the points by equal subdivision of geodesics
-  u[0] = u0;
-  v[0] = v0;
-  t[0] = 0;
-  for (int i = 1; i < N; i++){
-    t[i] = (double)i / (N - 1);
-    SPoint2 p = gf->geodesic(SPoint2(u0, v0), SPoint2(uN, vN), t[i]);
-    u[i] = p.x();
-    v[i] = p.y();
+  const int NI = N-1;
+  u[0] = u0; v[0] = v0;
+  u[NI] = uN; v[NI] = vN;
+  const double fact = 1./(double)NI;
+  for (int i = 1; i < NI; i++){
+    const double t = i*fact;
+    u[i] = u0 + (uN-u0)*t;
+    v[i] = v0 + (vN-v0)*t;
+    if (geodesic) {
+      SPoint3 pc(t*pN + (1.-t)*p0);
+      double guess[2] = {u[i], v[i]};
+      GPoint gp = gf->closestPoint(pc, guess);
+      if(gp.succeeded()){
+        u[i] = gp.u();
+        v[i] = gp.v();
+      }
+    }
+  }
+  return true;
+}
+
+// --------- Creation of high-order edge vertices -----------
+
+static bool getEdgeVerticesonGeo(GEdge *ge, MVertex *v0, MVertex *v1,
+                                 std::vector<MVertex*> &ve, int nPts = 1)
+{
+  static const double relaxFail = 1e-2;
+  double u0 = 0., u1 = 0., US[100];
+  bool reparamOK = reparamMeshVertexOnEdge(v0, ge, u0);
+  if(ge->periodic(0) && ge->getEndVertex()->getNumMeshVertices() > 0 &&
+     v1 == ge->getEndVertex()->mesh_vertices[0])
+    u1 = ge->parBounds(0).high();
+  else
+    reparamOK &= reparamMeshVertexOnEdge(v1, ge, u1);
+
+  if (reparamOK) {
+    double uMin = std::min(u0, u1), uMax = std::max(u0, u1);
+    bool failed = true;
+    double relax = 1.;
+    while (failed && (relax > relaxFail)) {
+      failed = !computeEquidistantParameters(ge, uMin, uMax, nPts + 2, US, relax);
+      relax *= 0.5;
+    }
+    if (failed) {
+      Msg::Warning
+        ("Failed to compute equidistant parameters (relax = %g, value = %g) "
+         "for edge %d-%d parametrized with %g %g on GEdge %d",
+         relax, US[1], v0->getNum(), v1->getNum(),u0,u1,ge->tag());
+      US[0] = uMin;
+      const double du = (uMax - uMin) / (nPts+1);
+      for (int i = 1; i <= nPts; i++) US[i] = US[i - 1] + du;
+    }
+  }
+  else
+    Msg::Error("Cannot reparam a mesh Vertex in high order meshing");
+  if (!reparamOK) return false;
+
+  for(int j = 0; j < nPts; j++) {
+    MVertex *v;
+    int count = u0<u1? j + 1 : nPts + 1  - (j + 1);
+    GPoint pc = ge->point(US[count]);
+    v = new MEdgeVertex(pc.x(), pc.y(), pc.z(), ge,US[count]);
+    // this destroys the ordering of the mesh vertices on the edge
+    ve.push_back(v);
   }
-  u[N] = uN;
-  v[N] = vN;
-  t[N] = 1.0;
 
   return true;
 }
 
-
-// --------- Creation of high-order edge vertices -----------
-
-static bool getEdgeVerticesonGeo(GEdge *ge, MVertex *v0, MVertex *v1, std::vector<MVertex*> &ve, int nPts = 1)
+static bool getEdgeVerticesonGeo(GFace *gf, MVertex *v0, MVertex *v1,
+                                 std::vector<MVertex*> &ve, int nPts = 1)
 {
-    double u0 = 0., u1 = 0., US[100];
-    bool reparamOK = reparamMeshVertexOnEdge(v0, ge, u0);
-    if(ge->periodic(0) && ge->getEndVertex()->getNumMeshVertices() > 0 &&
-       v1 == ge->getEndVertex()->mesh_vertices[0])
-      u1 = ge->parBounds(0).high();
-    else
-      reparamOK &= reparamMeshVertexOnEdge(v1, ge, u1);
-
-    if(reparamOK){
-      double relax = 1.;
-      while (1){
-        if(computeEquidistantParameters(ge, std::min(u0,u1), std::max(u0,u1),
-                                        nPts + 2, US, relax))
-          break;
-
-        relax /= 2.0;
-        if(relax < 1.e-2) break;
-      }
-      if(relax < 1.e-2){
-        Msg::Warning
-          ("Failed to compute equidistant parameters (relax = %g, value = %g) "
-           "for edge %d-%d parametrized with %g %g on GEdge %d",
-           relax, US[1], v0->getNum(), v1->getNum(),u0,u1,ge->tag());
-        reparamOK = false;
-      }
+  SPoint2 p0, p1;
+  double US[100], VS[100];
+  bool reparamOK = reparamMeshEdgeOnFace(v0, v1, gf, p0, p1);
+  if(reparamOK) {
+    SPoint3 pnt0, pnt1;
+    if (nPts >= 30)
+      computeEquidistantParameters(gf, p0[0], p1[0], p0[1], p1[1],
+                                   pnt0, pnt1, nPts + 2, false, US, VS);
+    else {
+      pnt0 = v0->point();
+      pnt1 = v1->point();
+      computeEquidistantParameters(gf, p0[0], p1[0], p0[1], p1[1],
+                                   pnt0, pnt1, nPts + 2, true, US, VS);
     }
-    else
-      Msg::Error("Cannot reparam a mesh Vertex in high order meshing");
-    if (!reparamOK) return false;
+  }
+  else {
+    Msg::Error("Cannot reparam a mesh Vertex in high order meshing");
+    return false;
+  }
 
-    for(int j = 0; j < nPts; j++) {
-      MVertex *v;
-      int count = u0<u1? j + 1 : nPts + 1  - (j + 1);
-      GPoint pc = ge->point(US[count]);
-      v = new MEdgeVertex(pc.x(), pc.y(), pc.z(), ge,US[count]);
-      // this destroys the ordering of the mesh vertices on the edge
-      ve.push_back(v);
-    }
+  for(int j = 0; j < nPts; j++){
+    GPoint pc = gf->point(US[j + 1], VS[j + 1]);
+    MVertex *v = new MFaceVertex(pc.x(), pc.y(), pc.z(), gf, US[j + 1], VS[j + 1]);
+    ve.push_back(v);
+  }
 
-    return true;
-}
-
-static bool getEdgeVerticesonGeo(GFace *gf, MVertex *v0, MVertex *v1, std::vector<MVertex*> &ve, int nPts = 1)
-{
-      SPoint2 p0, p1;
-      double US[100], VS[100];
-      bool reparamOK = reparamMeshEdgeOnFace(v0, v1, gf, p0, p1);
-      if(reparamOK) {
-        if (nPts >= 30)
-          computeEquidistantParameters(gf, p0[0], p1[0], p0[1], p1[1], nPts + 2, US, VS);
-        else {
-          US[0]      =  p0[0];
-          VS[0]      =  p0[1];
-          US[nPts+1] =  p1[0];
-          VS[nPts+1] =  p1[1];
-          for(int j = 0; j < nPts; j++){
-            const double t = (double)(j + 1) / (nPts + 1);
-            SPoint3 pc(t*v1->x()+(1.-t)*v0->x(),
-                       t*v1->y()+(1.-t)*v0->y(),
-                       t*v1->z()+(1.-t)*v0->z());
-            SPoint2 guess = p0 * (1.-t) + p1 * t;
-            GPoint gp = gf->closestPoint(pc, guess);
-            if(gp.succeeded()){
-              US[j+1] = gp.u();
-              VS[j+1] = gp.v();
-            }
-            else{
-              US[j+1] = guess.x();
-              VS[j+1] = guess.y();
-            }
-          }
-        }
-      }
-      else
-        Msg::Error("Cannot reparam a mesh Vertex in high order meshing");
-      if (!reparamOK) return false;
-
-      for(int j = 0; j < nPts; j++){
-        GPoint pc = gf->point(US[j + 1], VS[j + 1]);
-        MVertex *v = new MFaceVertex(pc.x(), pc.y(), pc.z(), gf, US[j + 1], VS[j + 1]);
-        ve.push_back(v);
-      }
-
-      return true;
+  return true;
 }
 
 static void interpVerticesInExistingEdge(GEntity *ge, const MElement *edgeEl,
@@ -253,12 +248,14 @@ static void getEdgeVertices(GEdge *ge, MElement *ele, std::vector<MVertex*> &ve,
   const bool increasing = getMinMaxVert(veOld[0], veOld[1], vMin, vMax);
   std::pair<MVertex*, MVertex*> p(vMin, vMax);
   std::vector<MVertex*> veEdge;
+  // Get vertices on geometry if asked
   bool gotVertOnGeo = linear ? false :
-                      getEdgeVerticesonGeo(ge, veOld[0], veOld[1], veEdge, nPts);       // Get vertices on geometry if asked
-  if (!gotVertOnGeo)                                                                    // If not on geometry, create from mesh interpolation
+    getEdgeVerticesonGeo(ge, veOld[0], veOld[1], veEdge, nPts);
+  // If not on geometry, create from mesh interpolation
+  if (!gotVertOnGeo)
     interpVerticesInExistingEdge(ge, ele, veEdge, nPts);
   newHOVert.insert(newHOVert.end(), veEdge.begin(), veEdge.end());
-  if(increasing)                                          // Add newly created vertices to list
+  if(increasing) // Add newly created vertices to list
     edgeVertices[p].insert(edgeVertices[p].end(), veEdge.begin(), veEdge.end());
   else
     edgeVertices[p].insert(edgeVertices[p].end(), veEdge.rbegin(), veEdge.rend());
@@ -281,21 +278,23 @@ static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
     const bool increasing = getMinMaxVert(veOld[0], veOld[1], vMin, vMax);
     std::pair<MVertex*, MVertex*> p(vMin, vMax);
     std::vector<MVertex*> veEdge;
-    if(edgeVertices.count(p)) {                                                             // Vertices already exist
+    if(edgeVertices.count(p)) { // Vertices already exist
       if(increasing)
         veEdge.assign(edgeVertices[p].begin(), edgeVertices[p].end());
       else
         veEdge.assign(edgeVertices[p].rbegin(), edgeVertices[p].rend());
     }
-    else {                                                                                  // Vertices do not exist, create them
+    else { // Vertices do not exist, create them
+      // Get vertices on geometry if asked
       bool gotVertOnGeo = linear ? false :
-                          getEdgeVerticesonGeo(gf, veOld[0], veOld[1], veEdge, nPts);       // Get vertices on geometry if asked
-      if (!gotVertOnGeo) {                                                                   // If not on geometry, create from mesh interpolation
+        getEdgeVerticesonGeo(gf, veOld[0], veOld[1], veEdge, nPts);
+      if (!gotVertOnGeo) {
+        // If not on geometry, create from mesh interpolation
         const MLineN edgeEl(veOld, ele->getPolynomialOrder());
         interpVerticesInExistingEdge(gf, &edgeEl, veEdge, nPts);
       }
       newHOVert.insert(newHOVert.end(), veEdge.begin(), veEdge.end());
-      if(increasing)                                                                        // Add newly created vertices to list
+      if(increasing) // Add newly created vertices to list
         edgeVertices[p].insert(edgeVertices[p].end(), veEdge.begin(), veEdge.end());
       else
         edgeVertices[p].insert(edgeVertices[p].end(), veEdge.rbegin(), veEdge.rend());
@@ -306,7 +305,8 @@ static void getEdgeVertices(GFace *gf, MElement *ele, std::vector<MVertex*> &ve,
 
 // Get new interior vertices for an edge in a 3D element
 static void getEdgeVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &ve,
-                            std::vector<MVertex*> &newHOVert, edgeContainer &edgeVertices,
+                            std::vector<MVertex*> &newHOVert,
+                            edgeContainer &edgeVertices,
                             bool linear, int nPts = 1)
 {
   for(int i = 0; i < ele->getNumEdges(); i++) {
@@ -316,17 +316,17 @@ static void getEdgeVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &v
     const bool increasing = getMinMaxVert(veOld[0], veOld[1], vMin, vMax);
     std::pair<MVertex*, MVertex*> p(vMin, vMax);
     std::vector<MVertex*> veEdge;
-    if(edgeVertices.count(p)) {                                                             // Vertices already exist
+    if(edgeVertices.count(p)) { // Vertices already exist
       if(increasing)
         veEdge.assign(edgeVertices[p].begin(), edgeVertices[p].end());
       else
         veEdge.assign(edgeVertices[p].rbegin(), edgeVertices[p].rend());
     }
-    else {                                                                                  // Vertices do not exist, create them
+    else { // Vertices do not exist, create them
       const MLineN edgeEl(veOld, ele->getPolynomialOrder());
       interpVerticesInExistingEdge(gr, &edgeEl, veEdge, nPts);
       newHOVert.insert(newHOVert.end(), veEdge.begin(), veEdge.end());
-      if(increasing)                                                                        // Add newly created vertices to list
+      if(increasing) // Add newly created vertices to list
         edgeVertices[p].insert(edgeVertices[p].end(), veEdge.begin(), veEdge.end());
       else
         edgeVertices[p].insert(edgeVertices[p].end(), veEdge.rbegin(), veEdge.rend());
@@ -335,7 +335,6 @@ static void getEdgeVertices(GRegion *gr, MElement *ele, std::vector<MVertex*> &v
   }
 }
 
-
 // --------- Creation of high-order face vertices -----------
 
 static void reorientTrianglePoints(std::vector<MVertex*> &vtcs, int orientation,
@@ -441,9 +440,9 @@ static void reorientQuadPoints(std::vector<MVertex*> &vtcs, int orientation,
   }
 }
 
-static int getNewFacePointsInVolume(MElement *incomplete, int nPts, fullMatrix<double> &points)
+static int getNewFacePointsInVolume(MElement *incomplete, int nPts,
+                                    fullMatrix<double> &points)
 {
-
   int startFace = 0;
 
   switch (incomplete->getType()){
@@ -589,25 +588,6 @@ static void interpVerticesInExistingFace(GEntity *ge, const MElement *faceEl,
   }
 }
 
-
-//static void interpVerticesInIncompleteFace(GRegion *gr, const MFace &face, const std::vector<MVertex*> &ve,
-//                                           std::vector<MVertex*> &veFace, int nPts)
-//{
-//  MElement *incomplete;
-//
-//  fullMatrix<double> points;
-//  int start = getNewFacePointsInFace(face.getNumVertices(), nPts, points);
-//  for (int k = start; k < points.size1(); k++) {
-//    double t1 = points(k, 0);
-//    double t2 = points(k, 1);
-//    SPoint3 pos;
-//    incomplete->pnt(t1, t2, 0, pos);
-//    MVertex* v = new MVertex(pos.x(), pos.y(), pos.z(), gr);
-//    veFace.push_back(v);
-//  }
-//  delete incomplete;
-//}
-
 // Get new interior vertices for a 2D element
 static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele,
                             std::vector<MVertex*> &vf, std::vector<MVertex*> &newHOVert,
@@ -619,9 +599,9 @@ static void getFaceVertices(GFace *gf, MElement *incomplete, MElement *ele,
 
   MFace face = ele->getFace(0);
   std::vector<MVertex*> veFace;
-  if (!linear)                                                                      // Get vertices on geometry if asked...
+  if (!linear) // Get vertices on geometry if asked...
     getFaceVerticesOnGeo(gf, incomplete, ele, veFace, nPts);
-  else                                                                              // ... otherwise, create from mesh interpolation
+  else // ... otherwise, create from mesh interpolation
     interpVerticesInExistingFace(gf, ele, veFace, nPts);
   newHOVert.insert(newHOVert.end(), veFace.begin(), veFace.end());
   faceVertices[face].insert(faceVertices[face].end(), veFace.begin(), veFace.end());
@@ -642,11 +622,12 @@ static void getFaceVertices(GRegion *gr, MElement *incomplete, MElement *ele,
     int numVert = (face.getNumVertices() == 3) ? nPts * (nPts-1) / 2 : nPts * nPts;
     std::vector<MVertex*> veFace;
     faceContainer::iterator fIter = faceVertices.find(face);
-    if(fIter != faceVertices.end()){                                                      // Vertices already exist
+    if(fIter != faceVertices.end()){ // Vertices already exist
       std::vector<MVertex*> vtcs = fIter->second;
       int orientation;
       bool swap;
-      if (fIter->first.computeCorrespondence(face, orientation, swap)) {                  // Check correspondence and apply permutation if needed
+      if (fIter->first.computeCorrespondence(face, orientation, swap)) {
+        // Check correspondence and apply permutation if needed
         if(face.getNumVertices() == 3 && nPts > 1)
           reorientTrianglePoints(vtcs, orientation, swap);
         else if(face.getNumVertices() == 4)
@@ -656,8 +637,8 @@ static void getFaceVertices(GRegion *gr, MElement *incomplete, MElement *ele,
         Msg::Error("Error in face lookup for recuperation of high order face nodes");
       veFace.assign(vtcs.begin(), vtcs.end());
     }
-    else {                                                                                // Vertices do not exist, create them by interpolation
-      if (linear) {                                                                       // Interpolate on existing mesh
+    else { // Vertices do not exist, create them by interpolation
+      if (linear) { // Interpolate on existing mesh
         std::vector<MVertex*> vfOld;
         ele->getFaceVertices(i,vfOld);
         MElement *faceEl;
@@ -669,7 +650,7 @@ static void getFaceVertices(GRegion *gr, MElement *incomplete, MElement *ele,
         delete faceEl;
       }
       else {
-        if (incomplete) {                                                                 // Interpolate in incomplete 3D element if given
+        if (incomplete) { // Interpolate in incomplete 3D element if given
           for(int k = index; k < index + numVert; k++){
             MVertex *v;
             const double t1 = points(k, 0);
@@ -681,7 +662,7 @@ static void getFaceVertices(GRegion *gr, MElement *incomplete, MElement *ele,
             veFace.push_back(v);
           }
         }
-        else {                                                                            // TODO: Interpolate in incomplete face constructed on the fly
+        else { // TODO: Interpolate in incomplete face constructed on the fly
 //          std::vector<MVertex*> &vtcs = faceVertices[face];
 //          fullMatrix<double> points;
 //          int start = getNewFacePointsInFace(face.getNumVertices(), nPts, points);
@@ -742,12 +723,10 @@ static void getFaceVertices(GRegion *gr, MElement *incomplete, MElement *ele,
   }
 }
 
-
 // --------- Creation of high-order volume vertices -----------
 
 static int getNewVolumePoints(MElement *incomplete, int nPts, fullMatrix<double> &points)
 {
-
   int startInter = 0;
 
   switch (incomplete->getType()){
@@ -767,7 +746,8 @@ static int getNewVolumePoints(MElement *incomplete, int nPts, fullMatrix<double>
       Msg::Error("getFaceAndInteriorVertices not implemented for order %i", nPts+1);
       break;
     }
-    startInter = ((nPts+2)*(nPts+3)*(nPts+4)-(nPts-2)*(nPts-1)*(nPts))/6;  // = 4+6*(p-1)+4*(p-2)*(p-1)/2 = 4*(p+1)*(p+2)/2-6*(p-1)-2*4 = 2*p*p+2
+    startInter = ((nPts+2)*(nPts+3)*(nPts+4)-(nPts-2)*(nPts-1)*(nPts))/6;
+    // = 4+6*(p-1)+4*(p-2)*(p-1)/2 = 4*(p+1)*(p+2)/2-6*(p-1)-2*4 = 2*p*p+2
     break;
   case TYPE_HEX :
     switch (nPts){
@@ -784,7 +764,8 @@ static int getNewVolumePoints(MElement *incomplete, int nPts, fullMatrix<double>
       Msg::Error("getFaceAndInteriorVertices not implemented for order %i", nPts+1);
       break;
     }
-    startInter = (nPts+2)*(nPts+2)*(nPts+2) - (nPts)*(nPts)*(nPts) ; // = 6*(p-1)*(p-1)+12*(p-1)+8 = 6*(p+1)*(p+1)-12*(p-1)-2*8 = 6*p*p+2
+    startInter = (nPts+2)*(nPts+2)*(nPts+2) - (nPts)*(nPts)*(nPts) ;
+    // = 6*(p-1)*(p-1)+12*(p-1)+8 = 6*(p+1)*(p+1)-12*(p-1)-2*8 = 6*p*p+2
     break;
   case TYPE_PRI :
     switch (nPts){
@@ -801,7 +782,9 @@ static int getNewVolumePoints(MElement *incomplete, int nPts, fullMatrix<double>
       Msg::Error("getFaceAndInteriorVertices not implemented for order %i", nPts+1);
       break;
     }
-    startInter = 4*(nPts+1)*(nPts+1)+2;  // = 4*p*p+2 = 6+9*(p-1)+2*(p-2)*(p-1)/2+3*(p-1)*(p-1) = 2*(p+1)*(p+2)/2+3*(p+1)*(p+1)-9*(p-1)-2*6
+    startInter = 4*(nPts+1)*(nPts+1)+2;
+    // = 4*p*p+2 = 6+9*(p-1)+2*(p-2)*(p-1)/2+3*(p-1)*(p-1)
+    // = 2*(p+1)*(p+2)/2+3*(p+1)*(p+1)-9*(p-1)-2*6
     break;
   case TYPE_PYR:
     switch (nPts){
@@ -829,7 +812,8 @@ static int getNewVolumePoints(MElement *incomplete, int nPts, fullMatrix<double>
 
 // Get new interior vertices for a 3D element (except pyramid)
 static void getVolumeVertices(GRegion *gr, MElement *incomplete, MElement *ele,
-                              std::vector<MVertex*> &vr,  std::vector<MVertex*> &newHOVert,
+                              std::vector<MVertex*> &vr,
+                              std::vector<MVertex*> &newHOVert,
                               bool linear, int nPts = 1)
 {
   fullMatrix<double> points;
@@ -849,8 +833,10 @@ static void getVolumeVertices(GRegion *gr, MElement *incomplete, MElement *ele,
 }
 
 // Get new interior vertices for a pyramid
-static void getVolumeVerticesPyramid(GRegion *gr, MElement *ele, const std::vector<MVertex*> &ve,
-                                     std::vector<MVertex*> &vr, std::vector<MVertex*> &newHOVert,
+static void getVolumeVerticesPyramid(GRegion *gr, MElement *ele,
+                                     const std::vector<MVertex*> &ve,
+                                     std::vector<MVertex*> &vr,
+                                     std::vector<MVertex*> &newHOVert,
                                      bool linear, int nPts = 1)
 {
   vr.reserve((nPts-1)*(nPts)*(2*(nPts-1)+1)/6);
@@ -976,16 +962,19 @@ static void setHighOrder(GEdge *ge, std::vector<MVertex*> &newHOVert,
     std::vector<MVertex*> ve;
     getEdgeVertices(ge, l, ve, newHOVert, edgeVertices, linear, nbPts);
     if(nbPts == 1)
-      lines2.push_back(new MLine3(l->getVertex(0), l->getVertex(1), ve[0], l->getPartition()));
+      lines2.push_back(new MLine3(l->getVertex(0), l->getVertex(1),
+                                  ve[0], l->getPartition()));
     else
-      lines2.push_back(new MLineN(l->getVertex(0), l->getVertex(1), ve, l->getPartition()));
+      lines2.push_back(new MLineN(l->getVertex(0), l->getVertex(1),
+                                  ve, l->getPartition()));
     delete l;
   }
   ge->lines = lines2;
   ge->deleteVertexArrays();
 }
 
-static MTriangle *setHighOrder(MTriangle *t, GFace *gf, std::vector<MVertex*> &newHOVert,
+static MTriangle *setHighOrder(MTriangle *t, GFace *gf,
+                               std::vector<MVertex*> &newHOVert,
                                edgeContainer &edgeVertices,
                                faceContainer &faceVertices,
                                bool linear, bool incomplete, int nPts)
@@ -1008,7 +997,8 @@ static MTriangle *setHighOrder(MTriangle *t, GFace *gf, std::vector<MVertex*> &n
   }
 }
 
-static MQuadrangle *setHighOrder(MQuadrangle *q, GFace *gf, std::vector<MVertex*> &newHOVert,
+static MQuadrangle *setHighOrder(MQuadrangle *q, GFace *gf,
+                                 std::vector<MVertex*> &newHOVert,
                                  edgeContainer &edgeVertices,
                                  faceContainer &faceVertices,
                                  bool linear, bool incomplete, int nPts)
