@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -234,10 +234,10 @@ bool evalSwapForOptimize(BDS_Edge *e, GFace *gf, BDS_Mesh &m)
 
   // First, evaluate what we gain in element quality if the
   // swap is performed
-  double qa1 = qmTriangle(p11, p12, p13, QMTRI_RHO);
-  double qa2 = qmTriangle(p21, p22, p23, QMTRI_RHO);
-  double qb1 = qmTriangle(p31, p32, p33, QMTRI_RHO);
-  double qb2 = qmTriangle(p41, p42, p43, QMTRI_RHO);
+  double qa1 = qmTriangle::gamma(p11, p12, p13);
+  double qa2 = qmTriangle::gamma(p21, p22, p23);
+  double qb1 = qmTriangle::gamma(p31, p32, p33);
+  double qb2 = qmTriangle::gamma(p41, p42, p43);
   double qa = std::min(qa1, qa2);
   double qb = std::min(qb1, qb2);
   double qualIndicator = qb - qa;
@@ -357,10 +357,10 @@ bool evalSwap(BDS_Edge *e, double &qa, double &qb)
 
   if(e->numfaces() != 2) return false;
   e->oppositeof (op);
-  double qa1 = qmTriangle(e->p1, e->p2, op[0], QMTRI_RHO);
-  double qa2 = qmTriangle(e->p1, e->p2, op[1], QMTRI_RHO);
-  double qb1 = qmTriangle(e->p1, op[0], op[1], QMTRI_RHO);
-  double qb2 = qmTriangle(e->p2, op[0], op[1], QMTRI_RHO);
+  double qa1 = qmTriangle::gamma(e->p1, e->p2, op[0]);
+  double qa2 = qmTriangle::gamma(e->p1, e->p2, op[1]);
+  double qb1 = qmTriangle::gamma(e->p1, op[0], op[1]);
+  double qb2 = qmTriangle::gamma(e->p2, op[0], op[1]);
   qa = std::min(qa1, qa2);
   qb = std::min(qb1, qb2);
   return true;
@@ -381,10 +381,10 @@ int edgeSwapTestQuality(BDS_Edge *e, double fact=1.1, bool force=false)
     if (!edgeSwapTestAngle(e, cos(CTX::instance()->mesh.allowSwapEdgeAngle * M_PI / 180.)))
       return -1;
 
-  double qa1 = qmTriangle(e->p1, e->p2, op[0], QMTRI_RHO);
-  double qa2 = qmTriangle(e->p1, e->p2, op[1], QMTRI_RHO);
-  double qb1 = qmTriangle(e->p1, op[0], op[1], QMTRI_RHO);
-  double qb2 = qmTriangle(e->p2, op[0], op[1], QMTRI_RHO);
+  double qa1 = qmTriangle::gamma(e->p1, e->p2, op[0]);
+  double qa2 = qmTriangle::gamma(e->p1, e->p2, op[1]);
+  double qb1 = qmTriangle::gamma(e->p1, op[0], op[1]);
+  double qb2 = qmTriangle::gamma(e->p2, op[0], op[1]);
   double qa = std::min(qa1, qa2);
   double qb = std::min(qb1, qb2);
   if(qb > fact * qa) return 1;
@@ -479,6 +479,14 @@ static void midpointsphere(GFace *gf, double u1, double v1, double u2, double v2
 }
 */
 
+bool edges_sort(std::pair<double, BDS_Edge*> a, std::pair<double, BDS_Edge*> b)
+{
+  if (a.first == b.first)
+    return ((*a.second) < (*b.second));
+  else
+    return (a.first < b.first);
+}
+
 void splitEdgePass(GFace *gf, BDS_Mesh &m, double MAXE_, int &nb_split)
 {
   std::list<BDS_Edge*>::iterator it = m.edges.begin();
@@ -494,7 +502,7 @@ void splitEdgePass(GFace *gf, BDS_Mesh &m, double MAXE_, int &nb_split)
     ++it;
   }
 
-  std::sort(edges.begin(), edges.end());
+  std::sort(edges.begin(), edges.end(), edges_sort);
 
   for (unsigned int i = 0; i < edges.size(); ++i){
     BDS_Edge *e = edges[i].second;
@@ -547,7 +555,7 @@ void collapseEdgePass(GFace *gf, BDS_Mesh &m, double MINE_, int MAXNP, int &nb_c
     ++it;
   }
 
-  std::sort(edges.begin(), edges.end());
+  std::sort(edges.begin(), edges.end(), edges_sort);
 
   for (unsigned int i = 0; i < edges.size(); i++){
     BDS_Edge *e = edges[i].second;
@@ -771,8 +779,8 @@ void invalidEdgesPeriodic(BDS_Mesh &m, std::map<BDS_Point*, MVertex*,PointLessTh
   while (it != m.edges.end()){
     BDS_Edge *e = *it;
     if (!e->deleted && e->numfaces() == 2){
-      std::map<BDS_Point*, MVertex*>::iterator itp1 = recoverMap->find(e->p1);
-      std::map<BDS_Point*, MVertex*>::iterator itp2 = recoverMap->find(e->p2);
+      std::map<BDS_Point*, MVertex*,PointLessThan>::iterator itp1 = recoverMap->find(e->p1);
+      std::map<BDS_Point*, MVertex*,PointLessThan>::iterator itp2 = recoverMap->find(e->p2);
       if (itp1 != recoverMap->end() &&
           itp2 != recoverMap->end() &&
           itp1->second == itp2->second) toSplit.insert(e);
