@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -31,11 +31,13 @@
 #include "GFaceCompound.h"
 #include "meshGRegionMMG3D.h"
 #include "simple3D.h"
-#include "Levy3D.h"
 #include "directions3D.h"
+#include "pointInsertion.h"
+#include "Levy3D.h"
 #include "discreteFace.h"
 #include "filterElements.h"
 
+
 #if defined(HAVE_ANN)
 #include "ANN/ANN.h"
 #endif
@@ -160,10 +162,9 @@ void printVoronoi(GRegion *gr,  std::set<SPoint3> &candidates)
   }
 
   //print voronoi poles
-  FILE *outfile;
   //smooth_normals *snorm = gr->model()->normals;
-  outfile = Fopen("nodes.pos", "w");
-  fprintf(outfile, "View \"Voronoi poles\" {\n");
+  FILE *outfile = Fopen("nodes.pos", "w");
+  if(outfile) fprintf(outfile, "View \"Voronoi poles\" {\n");
   std::map<MVertex*, std::set<MTetrahedron*> >::iterator itmap = node2Tet.begin();
   for(; itmap != node2Tet.end(); itmap++){
     //MVertex *v = itmap->first;
@@ -174,35 +175,36 @@ void printVoronoi(GRegion *gr,  std::set<SPoint3> &candidates)
     for(; it != allTets.end(); it++){
       double radius =  (*it)->getCircumRadius();
       if (radius > maxRadius){
-    	maxRadius = radius;
-    	poleTet = *it;
+        maxRadius = radius;
+        poleTet = *it;
       }
     }
     //if (v->onWhat()->dim() == 2 ){
-      SPoint3 pc = poleTet->circumcenter();
-      //double nx,ny,nz;
-      // SVector3 vN = snorm->get(v->x(), v->y(), v->z(), nx,ny,nz);
-      // SVector3 pcv(pc.x()-nx, pc.y()-ny,pc.z()-nz);
-      // printf("nx=%g ny=%g nz=%g dot=%g \n",  nx,ny,nz, dot(vN, pcv));
-      // if ( dot(vN, pcv) > 0.0 )
-      double x[3] = {pc.x(), pc.y(), pc.z()};
-      double uvw[3];
-      poleTet->xyz2uvw(x, uvw);
-      //bool inside = poleTet->isInside(uvw[0], uvw[1], uvw[2]);
-      //if (inside){
-	fprintf(outfile,"SP(%g,%g,%g)  {%g};\n",
-		pc.x(), pc.y(), pc.z(), maxRadius);
-	candidates.insert(pc);
-	//}
+    SPoint3 pc = poleTet->circumcenter();
+    //double nx,ny,nz;
+    // SVector3 vN = snorm->get(v->x(), v->y(), v->z(), nx,ny,nz);
+    // SVector3 pcv(pc.x()-nx, pc.y()-ny,pc.z()-nz);
+    // printf("nx=%g ny=%g nz=%g dot=%g \n",  nx,ny,nz, dot(vN, pcv));
+    // if ( dot(vN, pcv) > 0.0 )
+    double x[3] = {pc.x(), pc.y(), pc.z()};
+    double uvw[3];
+    poleTet->xyz2uvw(x, uvw);
+    //bool inside = poleTet->isInside(uvw[0], uvw[1], uvw[2]);
+    //if (inside){
+    if(outfile)
+      fprintf(outfile,"SP(%g,%g,%g)  {%g};\n", pc.x(), pc.y(), pc.z(), maxRadius);
+    candidates.insert(pc);
+    //}
     //}
   }
-  fprintf(outfile,"};\n");
-  fclose(outfile);
+  if(outfile){
+    fprintf(outfile,"};\n");
+    fclose(outfile);
+  }
 
-  //print scalar lines
-  FILE *outfile2;
-  outfile2 = Fopen("edges.pos", "w");
-  fprintf(outfile2, "View \"Voronoi edges\" {\n");
+  // print scalar lines
+  FILE *outfile2 = Fopen("edges.pos", "w");
+  if(outfile2) fprintf(outfile2, "View \"Voronoi edges\" {\n");
   std::map<MFace, std::vector<MTetrahedron*> , Less_Face >::iterator itmap2 = face2Tet.begin();
   for(; itmap2 != face2Tet.end(); itmap2++){
     std::vector<MTetrahedron*>  allTets = itmap2->second;
@@ -212,13 +214,15 @@ void printVoronoi(GRegion *gr,  std::set<SPoint3> &candidates)
     //std::set<SPoint3>::const_iterator it1 = candidates.find(pc1);
     //std::set<SPoint3>::const_iterator it2 = candidates.find(pc2);
     //if( it1 != candidates.end() || it2 != candidates.end())
+    if(outfile2)
       fprintf(outfile2,"SL(%g,%g,%g,%g,%g,%g)  {%g,%g};\n",
-	      pc1.x(), pc1.y(), pc1.z(), pc2.x(), pc2.y(), pc2.z(),
-	      allTets[0]->getCircumRadius(),allTets[1]->getCircumRadius());
-   }
-  fprintf(outfile2,"};\n");
-  fclose(outfile2);
-
+              pc1.x(), pc1.y(), pc1.z(), pc2.x(), pc2.y(), pc2.z(),
+              allTets[0]->getCircumRadius(),allTets[1]->getCircumRadius());
+  }
+  if(outfile2){
+    fprintf(outfile2,"};\n");
+    fclose(outfile2);
+  }
 }
 
 
@@ -309,7 +313,8 @@ void buildTetgenStructure(GRegion *gr, tetgenio &in, std::vector<MVertex*> &numb
 
   in.mesh_dim = 3;
   in.firstnumber = 1;
-  in.numberofpoints = allBoundingVertices.size() + Filler::get_nbr_new_vertices() +
+  int nbvertices_filler = (old_algo_hexa()) ? Filler::get_nbr_new_vertices() : Filler3D::get_nbr_new_vertices();
+  in.numberofpoints = allBoundingVertices.size() + nbvertices_filler +
     LpSmoother::get_nbr_interior_vertices();
   in.pointlist = new REAL[in.numberofpoints * 3];
   in.pointmarkerlist = NULL;
@@ -325,9 +330,12 @@ void buildTetgenStructure(GRegion *gr, tetgenio &in, std::vector<MVertex*> &numb
     ++itv;
   }
 
-  for(int i=0;i<Filler::get_nbr_new_vertices();i++){
+  for(int i=0;i<nbvertices_filler;i++){
     MVertex* v;
-    v = Filler::get_new_vertex(i);
+    if (old_algo_hexa())
+      v = Filler::get_new_vertex(i);
+    else
+      v = Filler3D::get_new_vertex(i);
     in.pointlist[(I - 1) * 3 + 0] = v->x();
     in.pointlist[(I - 1) * 3 + 1] = v->y();
     in.pointlist[(I - 1) * 3 + 2] = v->z();
@@ -904,7 +912,7 @@ static bool modifyInitialMeshForTakingIntoAccountBoundaryLayers(GRegion *gr, spl
 
   // now treat the Wedges
   // we have to know the two target GFaces that are concerned with a GEdge
-  std::set<MEdge>::iterator ite =  edges.begin();
+  std::set<MEdge,Less_Edge>::iterator ite =  edges.begin();
   while(ite != edges.end()){
     MEdge e = *ite;
     MVertex *v1 = e.getVertex(0);
@@ -972,15 +980,17 @@ static bool modifyInitialMeshForTakingIntoAccountBoundaryLayers(GRegion *gr, spl
   // ------------------------------------------------------------------------------------
   {
     FILE *ff2 = fopen ("tato3D.pos","w");
-    fprintf(ff2,"View \" \"{\n");
-    for (unsigned int i = 0; i < blPrisms.size();i++){
-      blPrisms[i]->writePOS(ff2,1,0,0,0,0,0);
+    if(ff2){
+      fprintf(ff2,"View \" \"{\n");
+      for (unsigned int i = 0; i < blPrisms.size();i++){
+        blPrisms[i]->writePOS(ff2,1,0,0,0,0,0);
+      }
+      for (unsigned int i = 0; i < blHexes.size();i++){
+        blHexes[i]->writePOS(ff2,1,0,0,0,0,0);
+      }
+      fprintf(ff2,"};\n");
+      fclose(ff2);
     }
-    for (unsigned int i = 0; i < blHexes.size();i++){
-      blHexes[i]->writePOS(ff2,1,0,0,0,0,0);
-    }
-    fprintf(ff2,"};\n");
-    fclose(ff2);
   }
 
   for (unsigned int i = 0; i < blPrisms.size();i++){
@@ -1002,14 +1012,15 @@ static bool modifyInitialMeshForTakingIntoAccountBoundaryLayers(GRegion *gr, spl
   std::map<MFace,MElement*,Less_Face>::iterator it =  bfaces.begin();
 
   FILE *ff = fopen ("toto3D.pos","w");
-  fprintf(ff,"View \" \"{\n");
+  if(ff) fprintf(ff,"View \" \"{\n");
   for (; it != bfaces.end(); ++it){
     if (it->first.getNumVertices() == 3){
       nf->triangles.push_back(new MTriangle (it->first.getVertex(0),it->first.getVertex(1),it->first.getVertex(2)));
-      fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){1,1,1};\n",
-	      it->first.getVertex(0)->x(),it->first.getVertex(0)->y(),it->first.getVertex(0)->z(),
-	      it->first.getVertex(1)->x(),it->first.getVertex(1)->y(),it->first.getVertex(1)->z(),
-	      it->first.getVertex(2)->x(),it->first.getVertex(2)->y(),it->first.getVertex(2)->z());
+      if(ff)
+        fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){1,1,1};\n",
+                it->first.getVertex(0)->x(),it->first.getVertex(0)->y(),it->first.getVertex(0)->z(),
+                it->first.getVertex(1)->x(),it->first.getVertex(1)->y(),it->first.getVertex(1)->z(),
+                it->first.getVertex(2)->x(),it->first.getVertex(2)->y(),it->first.getVertex(2)->z());
     }
     else {
 
@@ -1055,35 +1066,38 @@ static bool modifyInitialMeshForTakingIntoAccountBoundaryLayers(GRegion *gr, spl
       nf->triangles.push_back(new MTriangle (it->first.getVertex(1),it->first.getVertex(2),newv));
       nf->triangles.push_back(new MTriangle (it->first.getVertex(2),it->first.getVertex(3),newv));
       nf->triangles.push_back(new MTriangle (it->first.getVertex(3),it->first.getVertex(0),newv));
+      if(ff){
+        fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){2,2,2};\n",
+                it->first.getVertex(0)->x(),it->first.getVertex(0)->y(),it->first.getVertex(0)->z(),
+                it->first.getVertex(1)->x(),it->first.getVertex(1)->y(),it->first.getVertex(1)->z(),
+                newv->x(),newv->y(),newv->z());
 
-      fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){2,2,2};\n",
-	      it->first.getVertex(0)->x(),it->first.getVertex(0)->y(),it->first.getVertex(0)->z(),
-	      it->first.getVertex(1)->x(),it->first.getVertex(1)->y(),it->first.getVertex(1)->z(),
-	      newv->x(),newv->y(),newv->z());
+        fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){2,2,2};\n",
+                it->first.getVertex(1)->x(),it->first.getVertex(1)->y(),it->first.getVertex(1)->z(),
+                it->first.getVertex(2)->x(),it->first.getVertex(2)->y(),it->first.getVertex(2)->z(),
+                newv->x(),newv->y(),newv->z());
 
-      fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){2,2,2};\n",
-	      it->first.getVertex(1)->x(),it->first.getVertex(1)->y(),it->first.getVertex(1)->z(),
-	      it->first.getVertex(2)->x(),it->first.getVertex(2)->y(),it->first.getVertex(2)->z(),
-	      newv->x(),newv->y(),newv->z());
+        fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){2,2,2};\n",
+                it->first.getVertex(2)->x(),it->first.getVertex(2)->y(),it->first.getVertex(2)->z(),
+                it->first.getVertex(3)->x(),it->first.getVertex(3)->y(),it->first.getVertex(3)->z(),
+                newv->x(),newv->y(),newv->z());
+        fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){2,2,2};\n",
+                it->first.getVertex(3)->x(),it->first.getVertex(3)->y(),it->first.getVertex(3)->z(),
+                it->first.getVertex(0)->x(),it->first.getVertex(0)->y(),it->first.getVertex(0)->z(),
+                newv->x(),newv->y(),newv->z());
 
-      fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){2,2,2};\n",
-	      it->first.getVertex(2)->x(),it->first.getVertex(2)->y(),it->first.getVertex(2)->z(),
-	      it->first.getVertex(3)->x(),it->first.getVertex(3)->y(),it->first.getVertex(3)->z(),
-	      newv->x(),newv->y(),newv->z());
-      fprintf(ff,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){2,2,2};\n",
-	      it->first.getVertex(3)->x(),it->first.getVertex(3)->y(),it->first.getVertex(3)->z(),
-	      it->first.getVertex(0)->x(),it->first.getVertex(0)->y(),it->first.getVertex(0)->z(),
-	      newv->x(),newv->y(),newv->z());
-
-      fprintf(ff,"SQ (%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){3,3,3,3};\n",
-	      it->first.getVertex(0)->x(),it->first.getVertex(0)->y(),it->first.getVertex(0)->z(),
-	      it->first.getVertex(1)->x(),it->first.getVertex(1)->y(),it->first.getVertex(1)->z(),
-	      it->first.getVertex(2)->x(),it->first.getVertex(2)->y(),it->first.getVertex(2)->z(),
-	      it->first.getVertex(3)->x(),it->first.getVertex(3)->y(),it->first.getVertex(3)->z());
+        fprintf(ff,"SQ (%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){3,3,3,3};\n",
+                it->first.getVertex(0)->x(),it->first.getVertex(0)->y(),it->first.getVertex(0)->z(),
+                it->first.getVertex(1)->x(),it->first.getVertex(1)->y(),it->first.getVertex(1)->z(),
+                it->first.getVertex(2)->x(),it->first.getVertex(2)->y(),it->first.getVertex(2)->z(),
+                it->first.getVertex(3)->x(),it->first.getVertex(3)->y(),it->first.getVertex(3)->z());
+      }
     }
   }
-  fprintf(ff,"};\n");
-  fclose(ff);
+  if(ff){
+    fprintf(ff,"};\n");
+    fclose(ff);
+  }
 
   printf("discrete face with %d %d elems\n", (int)nf->triangles.size(),
          (int)nf->quadrangles.size());
@@ -1153,9 +1167,9 @@ bool CreateAnEmptyVolumeMesh(GRegion *gr)
   char opts[128];
   buildTetgenStructure(gr, in, numberedV, sqr);
   printf("tetgen structure created\n");
-  sprintf(opts, "-Ype%c",
-	  (Msg::GetVerbosity() < 3) ? 'Q':
-	  (Msg::GetVerbosity() > 6) ? 'V': '\0');
+  sprintf(opts, "-Ype%sT%g",
+	  (Msg::GetVerbosity() < 3) ? "Q" : (Msg::GetVerbosity() > 6) ? "V" : "",
+          CTX::instance()->mesh.toleranceInitialDelaunay);
   try{
     tetrahedralize(opts, &in, &out);
   }
@@ -1214,29 +1228,10 @@ void MeshDelaunayVolumeTetgen(std::vector<GRegion*> &regions)
     std::vector<MVertex*> numberedV;
     char opts[128];
     buildTetgenStructure(gr, in, numberedV, sqr);
-    if(CTX::instance()->mesh.algo3d == ALGO_3D_FRONTAL_DEL ||
-       CTX::instance()->mesh.algo3d == ALGO_3D_FRONTAL_HEX ||
-       CTX::instance()->mesh.algo3d == ALGO_3D_MMG3D ||
-       CTX::instance()->mesh.algo2d == ALGO_2D_FRONTAL_QUAD ||
-       CTX::instance()->mesh.algo2d == ALGO_2D_BAMG){
-      sprintf(opts, "Ype%c",  (Msg::GetVerbosity() < 3) ? 'Q':
-	      (Msg::GetVerbosity() > 6) ? 'V': '\0');
-      // removed -q because mesh sizes at new vertices are wrong
-      // sprintf(opts, "-q1.5pY%c",  (Msg::GetVerbosity() < 3) ? 'Q':
-      // 	 (Msg::GetVerbosity() > 6) ? 'V': '\0');
-    }
-    else if (CTX::instance()->mesh.algo3d == ALGO_3D_RTREE){
-       sprintf(opts, "S0Ype%c",  (Msg::GetVerbosity() < 3) ? 'Q':
-	       (Msg::GetVerbosity() > 6) ? 'V': '\0');
-    }
-    else {
-      sprintf(opts, "Ype%c",
-              (Msg::GetVerbosity() < 3) ? 'Q':
-              (Msg::GetVerbosity() > 6) ? 'V': '\0');
-      // removed -q because mesh sizes at new vertices are wrong
-      // sprintf(opts, "-q3.5Ype%c", (Msg::GetVerbosity() < 3) ? 'Q':
-      //        (Msg::GetVerbosity() > 6) ? 'V': '\0');*/
-    }
+    sprintf(opts, "%sYpe%sT%g",
+            CTX::instance()->mesh.algo3d == ALGO_3D_RTREE ? "S0" : "",
+            (Msg::GetVerbosity() < 3) ? "Q" : (Msg::GetVerbosity() > 6) ? "V" : "",
+            CTX::instance()->mesh.toleranceInitialDelaunay);
     try{
       tetrahedralize(opts, &in, &out);
     }
@@ -1297,9 +1292,11 @@ void MeshDelaunayVolumeTetgen(std::vector<GRegion*> &regions)
   else if(CTX::instance()->mesh.algo3d == ALGO_3D_MMG3D){
     refineMeshMMG(gr);
   }
-  else
-    if(!Filler::get_nbr_new_vertices() && !LpSmoother::get_nbr_interior_vertices()){
-      insertVerticesInRegion(gr,2000000000,!_BL);
+  else{
+      int nbvertices_filler = (old_algo_hexa()) ? Filler::get_nbr_new_vertices() : Filler3D::get_nbr_new_vertices();
+      if(!nbvertices_filler && !LpSmoother::get_nbr_interior_vertices()){
+        insertVerticesInRegion(gr,2000000000,!_BL);
+      }
     }
 
   //emi test frame field
@@ -1414,7 +1411,8 @@ namespace nglib {
 }
 using namespace nglib;
 
-static void getAllBoundingVertices(GRegion *gr, std::set<MVertex*> &allBoundingVertices)
+static void getAllBoundingVertices(GRegion *gr,
+                                   std::set<MVertex*, MVertexLessThanNum> &allBoundingVertices)
 {
   std::list<GFace*> faces = gr->faces();
   std::list<GFace*>::iterator it = faces.begin();
@@ -1437,10 +1435,10 @@ Ng_Mesh *buildNetgenStructure(GRegion *gr, bool importVolumeMesh,
   Ng_Init();
   Ng_Mesh *ngmesh = Ng_NewMesh();
 
-  std::set<MVertex*> allBoundingVertices;
+  std::set<MVertex*, MVertexLessThanNum> allBoundingVertices;
   getAllBoundingVertices(gr, allBoundingVertices);
 
-  std::set<MVertex*>::iterator itv = allBoundingVertices.begin();
+  std::set<MVertex*, MVertexLessThanNum>::iterator itv = allBoundingVertices.begin();
   int I = 1;
   while(itv != allBoundingVertices.end()){
     double tmp[3];
@@ -1562,10 +1560,9 @@ int intersect_line_triangle(double X[3], double Y[3], double Z[3] ,
   b[1] = P[1] - Y[0];
   b[2] = P[2] - Z[0];
 
-  if(!sys3x3_with_tol(mat, b, res, &det))
-    {
-      return 0;
-    }
+  if(!sys3x3_with_tol(mat, b, res, &det)){
+    return 0;
+  }
   //  printf("coucou %g %g %g\n",res[0],res[1],res[2]);
   if(res[0] >= eps_prec && res[0] <= 1.0 - eps_prec &&
      res[1] >= eps_prec && res[1] <= 1.0 - eps_prec &&
@@ -1597,8 +1594,13 @@ void meshNormalsPointOutOfTheRegion(GRegion *gr)
   std::list<GFace*> faces = gr->faces();
   std::list<GFace*>::iterator it = faces.begin();
 
-  //for (std::list<GFace*>::iterator itb = faces.begin(); itb != faces.end(); itb ++)
-  //  printf("face=%d size =%d\n", (*itb)->tag(), faces.size());
+  // perform intersection check in normalized coordinates
+  SBoundingBox3d bbox = gr->bounds();
+  double scaling = norm(SVector3(bbox.max(), bbox.min()));
+  if(!scaling){
+    Msg::Warning("Bad scaling in meshNormalsPointOutOfTheRegion");
+    scaling = 1.;
+  }
 
   double rrr[6];
   setRand(rrr);
@@ -1611,6 +1613,12 @@ void meshNormalsPointOutOfTheRegion(GRegion *gr)
       double X[3] = {t->getVertex(0)->x(), t->getVertex(1)->x(), t->getVertex(2)->x()};
       double Y[3] = {t->getVertex(0)->y(), t->getVertex(1)->y(), t->getVertex(2)->y()};
       double Z[3] = {t->getVertex(0)->z(), t->getVertex(1)->z(), t->getVertex(2)->z()};
+      for(int i = 0; i < 3; i++){
+        X[i] /= scaling;
+        Y[i] /= scaling;
+        Z[i] /= scaling;
+      }
+
       double P[3] = {(X[0] + X[1] + X[2]) / 3.,
                      (Y[0] + Y[1] + Y[2]) / 3.,
                      (Z[0] + Z[1] + Z[2]) / 3.};
@@ -1637,6 +1645,11 @@ void meshNormalsPointOutOfTheRegion(GRegion *gr)
                              t_b->getVertex(2)->y()};
             double Z_b[3] = {t_b->getVertex(0)->z(), t_b->getVertex(1)->z(),
                              t_b->getVertex(2)->z()};
+            for(int i = 0; i < 3; i++){
+              X_b[i] /= scaling;
+              Y_b[i] /= scaling;
+              Z_b[i] /= scaling;
+            }
             int inters = intersect_line_triangle(X_b, Y_b, Z_b, P, N, 1.e-9);
             nb_intersect += inters;
           }
@@ -1662,12 +1675,14 @@ void meshNormalsPointOutOfTheRegion(GRegion *gr)
   }
 
   // FILE *fp = Fopen("debug.pos", "w");
-  // fprintf(fp, "View \"debug\" {\n");
-  // for(std::list<GFace*>::iterator it = faces.begin(); it != faces.end(); it++)
-  //   for(unsigned int i = 0; i < (*it)->triangles.size(); i++)
-  //     (*it)->triangles[i]->writePOS(fp, 1., (*it)->tag());
-  // fprintf(fp, "};\n");
-  // fclose(fp);
+  // if(fp){
+  //   fprintf(fp, "View \"debug\" {\n");
+  //   for(std::list<GFace*>::iterator it = faces.begin(); it != faces.end(); it++)
+  //     for(unsigned int i = 0; i < (*it)->triangles.size(); i++)
+  //       (*it)->triangles[i]->writePOS(fp, 1., (*it)->tag());
+  //   fprintf(fp, "};\n");
+  //   fclose(fp);
+  // }
 }
 
 void meshGRegion::operator() (GRegion *gr)
@@ -1702,7 +1717,7 @@ void meshGRegion::operator() (GRegion *gr)
 
   // replace discreteFaces by their compounds
   {
-    std::set<GFace*> mySet;
+    std::set<GFace*, GEntityLessThan> mySet;
     std::list<GFace*>::iterator it = faces.begin();
     while(it != faces.end()){
       if((*it)->getCompound())
@@ -1778,7 +1793,7 @@ void optimizeMeshGRegionGmsh::operator() (GRegion *gr)
   if(ep && ep->mesh.ExtrudeMesh && ep->geo.Mode == EXTRUDED_ENTITY) return;
 
   Msg::Info("Optimizing volume %d", gr->tag());
-  optimizeMesh(gr, QMTET_2);
+  optimizeMesh(gr, qmTetrahedron::QMTET_GAMMA);
 }
 
 
