@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -285,7 +285,7 @@ void meshMetric::computeHessian()
       }
       b(i) = vals[vv[i]];
     }
-    ATA.gemmWithAtranspose(A,A,1.,0.);
+    ATA.gemm(A,A,1.,0., true, false);
     A.multWithATranspose(b,1.,0.,ATb);
     ATA.luSolve(ATb,coeffs);
     const double &x = ver->x(), &y = ver->y(), &z = ver->z();
