@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -35,3 +35,9 @@ class voroMetal3D{
   void correspondance(double,double,double,double,int,bool&,double,double,double);
   bool equal(double,double,double);
 };
+
+
+void microstructure(const char *filename);
+void computeBestSeeds(const char *filename);
+
+
