@@ -6,6 +6,7 @@
 // Contributor(s):
 //   Tristan Carrier Baudoin
 
+
 #include "GmshConfig.h"
 #include "surfaceFiller.h"
 #include "Field.h"
@@ -17,9 +18,7 @@
 /// Here, we aim at producing a set of points that
 /// enables to generate a nice quad mesh
 
-#if defined(HAVE_RTREE)
 #include "rtree.h"
-#endif
 
 #include "MVertex.h"
 #include "MElement.h"
@@ -27,11 +26,13 @@
 #include "BackgroundMesh.h"
 #include "intersectCurveSurface.h"
 
+#include "pointInsertionRTreeTools.h"
+
 using namespace std;
 
-static const double FACTOR = .71;
-static const int NUMDIR = 3;
-static const double DIRS [NUMDIR] = {0.0, M_PI/20.,-M_PI/20.};
+//static const double FACTOR = .71;
+//static const int NUMDIR = 3;
+//static const double DIRS [NUMDIR] = {0.0, M_PI/20.,-M_PI/20.};
 //PE MODIF
 //static const int NUMDIR = 1;
 //static const double DIRS [NUMDIR] = {0.0};
@@ -45,154 +46,153 @@ static const double DIRS [NUMDIR] = {0.0, M_PI/20.,-M_PI/20.};
 /// in a r-tree structure for generating points with the
 /// right spacing in the tangent plane
 
-#if defined(HAVE_RTREE)
 
-struct surfacePointWithExclusionRegion {
-  MVertex *_v;
-  SPoint2 _center;
-  SPoint2 _p[4][NUMDIR];
-  SPoint2 _q[4];
-  SMetric3 _meshMetric;
-  double _distanceSummed;
-  /*
-         + p3
-    p4   |
-    +----c-----+ p2
-         |
-         + p1
-
-*/
-
-  surfacePointWithExclusionRegion (MVertex *v, SPoint2 p[4][NUMDIR], SPoint2 &_mp, SMetric3 & meshMetric, surfacePointWithExclusionRegion *father = 0){
-    _v = v;
-    _meshMetric = meshMetric;
-    _center = _mp;
-    for (int i=0;i<4;i++)_q[i] = _center + (p[i][0]+p[(i+1)%4][0]-_center*2)*FACTOR;
-    for (int i=0;i<4;i++)for (int j=0;j<NUMDIR;j++)_p[i][j] = p[i][j];
-
-    if (!father){
-      fullMatrix<double> V(3,3);
-      fullVector<double> S(3);
-      meshMetric.eig(V,S);
-      double l = std::max(std::max(S(0),S(1)),S(2));
-      _distanceSummed = sqrt(1/(l*l));
-    }
-    else {
-      _distanceSummed = father->_distanceSummed + distance (father->_v,_v);
-    }
-  }
-  bool inExclusionZone (const SPoint2 &p){
-    double mat[2][2];
-    double b[2] , uv[2];
-    mat[0][0]= _q[1].x()-_q[0].x();
-    mat[0][1]= _q[2].x()-_q[0].x();
-    mat[1][0]= _q[1].y()-_q[0].y();
-    mat[1][1]= _q[2].y()-_q[0].y();
-    b[0] = p.x() - _q[0].x();
-    b[1] = p.y() - _q[0].y();
-    sys2x2(mat, b, uv);
-    //    printf("inversion 1 : %g %g \n",uv[0],uv[1]);
-    if (uv[0] >= 0 && uv[1] >= 0 && 1.-uv[0] - uv[1] >= 0)return true;
-    mat[0][0]= _q[3].x()-_q[2].x();
-    mat[0][1]= _q[0].x()-_q[2].x();
-    mat[1][0]= _q[3].y()-_q[2].y();
-    mat[1][1]= _q[0].y()-_q[2].y();
-    b[0] = p.x() - _q[2].x();
-    b[1] = p.y() - _q[2].y();
-    sys2x2(mat, b, uv);
-    //    printf("inversion 2 : %g %g \n",uv[0],uv[1]);
-    if (uv[0] >= 0 && uv[1] >= 0 && 1.-uv[0] - uv[1] >= 0)return true;
-    return false;
-  }
-  void minmax  (double _min[2], double _max[2]) const{
-    _min[0] = std::min(std::min(std::min(_q[0].x(),_q[1].x()),_q[2].x()),_q[3].x());
-    _min[1] = std::min(std::min(std::min(_q[0].y(),_q[1].y()),_q[2].y()),_q[3].y());
-    _max[0] = std::max(std::max(std::max(_q[0].x(),_q[1].x()),_q[2].x()),_q[3].x());
-    _max[1] = std::max(std::max(std::max(_q[0].y(),_q[1].y()),_q[2].y()),_q[3].y());
-  }
-  void print (FILE *f, int i){
-    fprintf(f,"SP(%g,%g,%g){%d};\n",_center.x(),_center.y(),0.0,i);
-    fprintf(f,"SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d,%d};\n",
-	    _q[0].x(),_q[0].y(),0.0,
-	    _q[1].x(),_q[1].y(),0.0,
-	    _q[2].x(),_q[2].y(),0.0,
-	    _q[3].x(),_q[3].y(),0.0,i,i,i,i);
-
-  }
-};
-
-struct my_wrapper {
-  bool _tooclose;
-  SPoint2 _p;
-  my_wrapper (SPoint2 sp) : _tooclose (false), _p(sp) {}
-};
-
-struct smoothness_point_pair{
-  double rank;
-  surfacePointWithExclusionRegion* ptr;
-};
-
-class compareSurfacePointWithExclusionRegionPtr_Smoothness
-{
-  public:
-    inline bool operator () (const smoothness_point_pair &a, const smoothness_point_pair &b)  const
-    {
-      if (a.rank == b.rank){
-        if(a.ptr->_distanceSummed > b.ptr->_distanceSummed) return false;
-        if(a.ptr->_distanceSummed < b.ptr->_distanceSummed) return true;
-        return a.ptr<b.ptr;
-      }
-      // else
-      return (a.rank < b.rank);
-    }
-};
-
-
-class compareSurfacePointWithExclusionRegionPtr
-{
- public:
-  inline bool operator () (const surfacePointWithExclusionRegion *a, const surfacePointWithExclusionRegion *b)  const
-  {
-    if(a->_distanceSummed > b->_distanceSummed) return false;
-    if(a->_distanceSummed < b->_distanceSummed) return true;
-    return a<b;
-  }
-};
-
-
-
-
-bool rtree_callback(surfacePointWithExclusionRegion *neighbour,void* point){
-  my_wrapper *w = static_cast<my_wrapper*>(point);
-
-  if (neighbour->inExclusionZone(w->_p)){
-    w->_tooclose = true;
-    return false;
-  }
-
-  return true;
-}
-
-bool inExclusionZone (SPoint2 &p,
-		      RTree<surfacePointWithExclusionRegion*,double,2,double> &rtree,
-		      std::vector<surfacePointWithExclusionRegion*> & all ){
-  // should assert that the point is inside the domain
-  if (!backgroundMesh::current()->inDomain(p.x(),p.y(),0)) return true;
-
-  my_wrapper w (p);
-  double _min[2] = {p.x()-1.e-1, p.y()-1.e-1},_max[2] = {p.x()+1.e-1,p.y()+1.e-1};
-  rtree.Search(_min,_max,rtree_callback,&w);
-
-  return w._tooclose;
-
-  for (unsigned int i=0;i<all.size();++i){
-    if (all[i]->inExclusionZone(p)){
-      //      printf("%g %g is in exclusion zone of %g %g\n",p.x(),p.y(),all[i]._center.x(),all[i]._center.y());
-      return true;
-    }
-  }
-  return false;
-}
+//struct surfacePointWithExclusionRegion {
+//  MVertex *_v;
+//  SPoint2 _center;
+//  SPoint2 _p[4][NUMDIR];
+//  SPoint2 _q[4];
+//  SMetric3 _meshMetric;
+//  double _distanceSummed;
+//  /*
+//         + p3
+//    p4   |
+//    +----c-----+ p2
+//         |
+//         + p1
+//
+//*/
+//
+//  surfacePointWithExclusionRegion (MVertex *v, SPoint2 p[4][NUMDIR], SPoint2 &_mp, SMetric3 & meshMetric, surfacePointWithExclusionRegion *father = 0){
+//    _v = v;
+//    _meshMetric = meshMetric;
+//    _center = _mp;
+//    for (int i=0;i<4;i++)_q[i] = _center + (p[i][0]+p[(i+1)%4][0]-_center*2)*FACTOR;
+//    for (int i=0;i<4;i++)for (int j=0;j<NUMDIR;j++)_p[i][j] = p[i][j];
+//
+//    if (!father){
+//      fullMatrix<double> V(3,3);
+//      fullVector<double> S(3);
+//      meshMetric.eig(V,S);
+//      double l = std::max(std::max(S(0),S(1)),S(2));
+//      _distanceSummed = sqrt(1/(l*l));
+//    }
+//    else {
+//      _distanceSummed = father->_distanceSummed + distance (father->_v,_v);
+//    }
+//  }
+//  bool inExclusionZone (const SPoint2 &p){
+//    double mat[2][2];
+//    double b[2] , uv[2];
+//    mat[0][0]= _q[1].x()-_q[0].x();
+//    mat[0][1]= _q[2].x()-_q[0].x();
+//    mat[1][0]= _q[1].y()-_q[0].y();
+//    mat[1][1]= _q[2].y()-_q[0].y();
+//    b[0] = p.x() - _q[0].x();
+//    b[1] = p.y() - _q[0].y();
+//    sys2x2(mat, b, uv);
+//    //    printf("inversion 1 : %g %g \n",uv[0],uv[1]);
+//    if (uv[0] >= 0 && uv[1] >= 0 && 1.-uv[0] - uv[1] >= 0)return true;
+//    mat[0][0]= _q[3].x()-_q[2].x();
+//    mat[0][1]= _q[0].x()-_q[2].x();
+//    mat[1][0]= _q[3].y()-_q[2].y();
+//    mat[1][1]= _q[0].y()-_q[2].y();
+//    b[0] = p.x() - _q[2].x();
+//    b[1] = p.y() - _q[2].y();
+//    sys2x2(mat, b, uv);
+//    //    printf("inversion 2 : %g %g \n",uv[0],uv[1]);
+//    if (uv[0] >= 0 && uv[1] >= 0 && 1.-uv[0] - uv[1] >= 0)return true;
+//    return false;
+//  }
+//  void minmax  (double _min[2], double _max[2]) const{
+//    _min[0] = std::min(std::min(std::min(_q[0].x(),_q[1].x()),_q[2].x()),_q[3].x());
+//    _min[1] = std::min(std::min(std::min(_q[0].y(),_q[1].y()),_q[2].y()),_q[3].y());
+//    _max[0] = std::max(std::max(std::max(_q[0].x(),_q[1].x()),_q[2].x()),_q[3].x());
+//    _max[1] = std::max(std::max(std::max(_q[0].y(),_q[1].y()),_q[2].y()),_q[3].y());
+//  }
+//  void print (FILE *f, int i){
+//    fprintf(f,"SP(%g,%g,%g){%d};\n",_center.x(),_center.y(),0.0,i);
+//    fprintf(f,"SQ(%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){%d,%d,%d,%d};\n",
+//	    _q[0].x(),_q[0].y(),0.0,
+//	    _q[1].x(),_q[1].y(),0.0,
+//	    _q[2].x(),_q[2].y(),0.0,
+//	    _q[3].x(),_q[3].y(),0.0,i,i,i,i);
+//
+//  }
+//};
+//
+//struct my_wrapper {
+//  bool _tooclose;
+//  SPoint2 _p;
+//  my_wrapper (SPoint2 sp) : _tooclose (false), _p(sp) {}
+//};
+//
+//struct smoothness_point_pair{
+//  double rank;
+//  surfacePointWithExclusionRegion* ptr;
+//};
+//
+//class compareSurfacePointWithExclusionRegionPtr_Smoothness
+//{
+//  public:
+//    inline bool operator () (const smoothness_point_pair &a, const smoothness_point_pair &b)  const
+//    {
+//      if (a.rank == b.rank){
+//        if(a.ptr->_distanceSummed > b.ptr->_distanceSummed) return false;
+//        if(a.ptr->_distanceSummed < b.ptr->_distanceSummed) return true;
+//        return a.ptr<b.ptr;
+//      }
+//      // else
+//      return (a.rank < b.rank);
+//    }
+//};
+//
+//
+//class compareSurfacePointWithExclusionRegionPtr
+//{
+// public:
+//  inline bool operator () (const surfacePointWithExclusionRegion *a, const surfacePointWithExclusionRegion *b)  const
+//  {
+//    if(a->_distanceSummed > b->_distanceSummed) return false;
+//    if(a->_distanceSummed < b->_distanceSummed) return true;
+//    return a<b;
+//  }
+//};
+//
+//
+//
+//
+//bool rtree_callback(surfacePointWithExclusionRegion *neighbour,void* point){
+//  my_wrapper *w = static_cast<my_wrapper*>(point);
+//
+//  if (neighbour->inExclusionZone(w->_p)){
+//    w->_tooclose = true;
+//    return false;
+//  }
+//
+//  return true;
+//}
+//
+//bool inExclusionZone (SPoint2 &p,
+//		      RTree<surfacePointWithExclusionRegion*,double,2,double> &rtree,
+//		      std::vector<surfacePointWithExclusionRegion*> & all ){
+//  // should assert that the point is inside the domain
+//  if (!backgroundMesh::current()->inDomain(p.x(),p.y(),0)) return true;
+//
+//  my_wrapper w (p);
+//  double _min[2] = {p.x()-1.e-1, p.y()-1.e-1},_max[2] = {p.x()+1.e-1,p.y()+1.e-1};
+//  rtree.Search(_min,_max,rtree_callback,&w);
+//
+//  return w._tooclose;
+//
+//  for (unsigned int i=0;i<all.size();++i){
+//    if (all[i]->inExclusionZone(p)){
+//      //      printf("%g %g is in exclusion zone of %g %g\n",p.x(),p.y(),all[i]._center.x(),all[i]._center.y());
+//      return true;
+//    }
+//  }
+//  return false;
+//}
 
 
 
@@ -373,7 +373,7 @@ bool compute4neighbors (GFace *gf,   // the surface
       //      printf("L = %12.5E D = %12.5E ERR = %12.5E\n",L,D,100*fabs(D-L)/(D+L));
     }
 
-    if (1 && goNonLinear){//---------------------------------------------------//
+    if (1 && goNonLinear){
       surfaceFunctorGFace ss (gf);                                        //
       SVector3 dirs[4] = {t1*(-1.0),t2*(-1.0),t1*(1.0),t2*(1.0)};                     //
       for (int i=0;i<4;i++){                                              //
@@ -418,7 +418,6 @@ bool compute4neighbors (GFace *gf,   // the surface
   return true;
 }
 
-// ---------------------------------------------------------------------------------------------
 
 // recover element around vertex v and interpolate smoothness on this element...
 double get_smoothness(MVertex *v, GFace *gf, const map<MVertex*,double> &vertices2smoothness){
@@ -470,7 +469,6 @@ double get_smoothness(MVertex *v, GFace *gf, const map<MVertex*,double> &vertice
   return res;
 }
 
-// ---------------------------------------------------------------------------------------------
 
 void print_nodal_info_int(string filename, map<MVertex*, int> &mapp){
   ofstream out(filename.c_str());
@@ -485,7 +483,6 @@ void print_nodal_info_int(string filename, map<MVertex*, int> &mapp){
   out.close();
 }
 
-// ---------------------------------------------------------------------------------------------
 
 void print_nodal_info_double(string filename, map<MVertex*, double> &mapp){
   ofstream out(filename.c_str());
@@ -500,7 +497,6 @@ void print_nodal_info_double(string filename, map<MVertex*, double> &mapp){
   out.close();
 }
 
-// ---------------------------------------------------------------------------------------------
 
 void export_point(surfacePointWithExclusionRegion *sp, int DIR, FILE *crossf, GFace *gf){
   // get the unit normal at that point
@@ -550,7 +546,6 @@ void export_point(surfacePointWithExclusionRegion *sp, int DIR, FILE *crossf, GF
   fprintf(crossf,"VP(%g,%g,%g) {%g,%g,%g};\n",sp->_v->x(),sp->_v->y(),sp->_v->z(),-t2.x()*size_2,-t2.y()*size_2,-t2.z()*size_2);
 }
 
-// ---------------------------------------------------------------------------------------------
 
 bool get_local_sizes_and_directions(const MVertex *v_center, const SPoint2 &midpoint, const int DIR, GFace* gf, double (&covar1)[2], double (&covar2)[2], double &size_param_1, double &size_param_2, double &L, SVector3 &t1, SVector3 &t2, SVector3 &n, FILE *crossf=NULL){
 //bool get_RK_stuff(const MVertex *v_center, const SPoint2 &midpoint, const int DIR, GFace* gf, double (&covar1)[2], double (&covar2)[2], double &size_param_1, double &size_param_2, double &L, SVector3 &t1, SVector3 &t2, SVector3 &n, FILE *crossf, const SVector3 &previous_t1, const SVector3 &previous_t2, bool use_previous_basis=false, bool export_cross=true){
@@ -687,14 +682,11 @@ bool get_local_sizes_and_directions(const MVertex *v_center, const SPoint2 &midp
   return true;
 }
 
-#endif
 
-// ---------------------------------------------------------------------------------------------
 
 // using fifo based on smoothness criteria
 void packingOfParallelogramsSmoothness(GFace* gf,  std::vector<MVertex*> &packed, std::vector<SMetric3> &metrics){
   cout << endl << "------------------------------------------" << endl << "   PACKINGOFPARALLELOGRAMS: NEW ALGO BASED ON SMOOTHNESS" << endl << "------------------------------------------" << endl;
-#if defined(HAVE_RTREE)
   const bool goNonLinear = true;
 
   const bool debug = false;
@@ -809,11 +801,11 @@ void packingOfParallelogramsSmoothness(GFace* gf,  std::vector<MVertex*> &packed
   std::set<MVertex*>::iterator it =  bnd_vertices.begin() ;
 
   char NAME[345]; sprintf(NAME,"crossReal%d.pos",gf->tag());
-  FILE *crossf=NULL;
+  FILE *crossf = NULL;
   if (debug){
     crossf = Fopen (NAME,"w");
   }
-  if (crossf)fprintf(crossf,"View \"\"{\n");
+  if (crossf) fprintf(crossf,"View \"\"{\n");
   for (; it !=  bnd_vertices.end() ; ++it){
     SPoint2 midpoint;
     //compute4neighbors_RK2 (gf, *it, midpoint, goNonLinear, newp, metricField,crossf);
@@ -895,9 +887,9 @@ void packingOfParallelogramsSmoothness(GFace* gf,  std::vector<MVertex*> &packed
   // surface mesh
   char ccc[256]; sprintf(ccc,"points%d.pos",gf->tag());
   FILE *f = Fopen(ccc,"w");
-  fprintf(f,"View \"\"{\n");
+  if(f) fprintf(f, "View \"\"{\n");
   for (unsigned int i=0;i<vertices.size();i++){
-    vertices[i]->print(f,i);
+    if(f) vertices[i]->print(f,i);
     if(vertices[i]->_v->onWhat() == gf) {
       packed.push_back(vertices[i]->_v);
       metrics.push_back(vertices[i]->_meshMetric);
@@ -906,13 +898,13 @@ void packingOfParallelogramsSmoothness(GFace* gf,  std::vector<MVertex*> &packed
     }
     delete  vertices[i];
   }
-  fprintf(f,"};");
-  fclose(f);
-#endif
+  if(f){
+    fprintf(f,"};");
+    fclose(f);
+  }
 }
 
 
-// ---------------------------------------------------------------------------------------------
 
 
 // fills a surface with points in order to build a nice
@@ -922,12 +914,18 @@ void packingOfParallelograms(GFace* gf,  std::vector<MVertex*> &packed, std::vec
 //  packingOfParallelogramsSmoothness(gf,packed,metrics);
 //  return;
   // END PE MODIF
-#if defined(HAVE_RTREE)
 
   const bool goNonLinear = true;
 
   //  FILE *f = Fopen ("parallelograms.pos","w");
 
+
+  // test test test
+  stringstream ssa;
+  ssa << "oldbgm_angles_" << gf->tag() << ".pos";
+  backgroundMesh::current()->print(ssa.str(),gf,1);
+  // test test test
+
   // get all the boundary vertices
   std::set<MVertex*> bnd_vertices;
   for(unsigned int i=0;i<gf->getNumMeshElements();i++){
@@ -950,7 +948,7 @@ void packingOfParallelograms(GFace* gf,  std::vector<MVertex*> &packed, std::vec
 
   char NAME[345]; sprintf(NAME,"crossReal%d.pos",gf->tag());
   FILE *crossf = Fopen (NAME,"w");
-  if (crossf)fprintf(crossf,"View \"\"{\n");
+  if (crossf) fprintf(crossf,"View \"\"{\n");
   for (; it !=  bnd_vertices.end() ; ++it){
     SPoint2 midpoint;
     compute4neighbors (gf, *it, midpoint, goNonLinear, newp, metricField,crossf);
@@ -1013,10 +1011,10 @@ void packingOfParallelograms(GFace* gf,  std::vector<MVertex*> &packed, std::vec
     // surface mesh
     char ccc[256]; sprintf(ccc,"points%d.pos",gf->tag());
     FILE *f = Fopen(ccc,"w");
-    fprintf(f,"View \"\"{\n");
+    if(f) fprintf(f,"View \"\"{\n");
     for (unsigned int i=0;i<vertices.size();i++){
       //    if(vertices[i]->_v->onWhat() != gf)
-      vertices[i]->print(f,i);
+      if(f) vertices[i]->print(f,i);
       if(vertices[i]->_v->onWhat() == gf) {
         packed.push_back(vertices[i]->_v);
         metrics.push_back(vertices[i]->_meshMetric);
@@ -1030,9 +1028,11 @@ void packingOfParallelograms(GFace* gf,  std::vector<MVertex*> &packed, std::vec
       }
       delete  vertices[i];
     }
-    fprintf(f,"};");
-    fclose(f);
+    if(f){
+      fprintf(f,"};");
+      fclose(f);
+    }
     //  printf("packed.size = %d\n",packed.size());
     //  delete rtree;
-#endif
 }
+
