@@ -1,8 +1,9 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
+#include <algorithm>
 #include "GmshConfig.h"
 #include "GmshMessage.h"
 #include "Numeric.h"
@@ -507,7 +508,6 @@ double ComputeScalarRep(int numComp, double *val)
 
 void eigenvalue2x2(double mat[2][2], double v[2])
 {
-
   double a=1;
   double b=-(mat[0][0]+mat[1][1]);
   double c= (mat[0][0]*mat[1][1])-(mat[0][1]*mat[1][0]);
@@ -516,7 +516,6 @@ void eigenvalue2x2(double mat[2][2], double v[2])
 
   v[0] = (-b+sqrt(det))/(2*a);
   v[1] = (-b-sqrt(det))/(2*a);
-
 }
 
 void eigenvalue(double mat[3][3], double v[3])
@@ -694,7 +693,7 @@ bool newton_fd(bool (*func)(fullVector<double> &, fullVector<double> &, void *),
       dx(0) = f(0) / J(0, 0);
     else
       if (!J.luSolve(f, dx))
-	return false;
+        return false;
 
     for (int i = 0; i < N; i++)
       x(i) -= relax * dx(i);
@@ -704,125 +703,10 @@ bool newton_fd(bool (*func)(fullVector<double> &, fullVector<double> &, void *),
   return false;
 }
 
-/*
-  min_a f(x+a*d);
-
-  f(x+a*d) = f(x) + f'(x) (
-
-*/
-
-void gmshLineSearch(double (*func)(fullVector<double> &, void *), void* data,
-                    fullVector<double> &x, fullVector<double> &p,
-                    fullVector<double> &g, double &f,
-                    double stpmax, int &check)
-{
-  int i;
-  double alam, alam2 = 1., alamin, f2 = 0., fold2 = 0., rhs1, rhs2, temp, tmplam = 0.;
-
-  const double ALF = 1.0e-4;
-  const double TOLX = 1.0e-9;
-
-  fullVector<double> xold(x);
-  const double fold = (*func)(xold, data);
-
-  check=0;
-  int n = x.size();
-  double norm = p.norm();
-  if (norm > stpmax) p.scale(stpmax / norm);
-  double slope=0.0;
-  for (i = 0; i < n; i++) slope += g(i)*p(i);
-  double test=0.0;
-  for (i = 0; i < n; i++) {
-    temp = fabs(p(i)) / std::max(fabs(xold(i)), 1.0);
-    if (temp > test) test = temp;
-  }
-  /*
-    for (int j=0;j<100;j++){
-    double sx = (double)j/99;
-    for (i=0;i<n;i++) x(i)=xold(i)+10*sx*p(i);
-    double jzede = (*func)(x,data);
-    }
-  */
-
-  alamin = TOLX / test;
-  alam = 1.0;
-  while(1) {
-    for (i = 0; i < n; i++) x(i) = xold(i) + alam*p(i);
-    f = (*func)(x, data);
-    //    printf("f = %g x = %g %g alam = %g p = %g %g\n",f,x(0),x(1),alam,p(0),p(1));
-    if (alam < alamin) {
-      for (i = 0; i <n; i++) x(i) = xold(i);
-      //      printf("ALERT : alam %g alamin %g\n",alam,alamin);
-      check = 1;
-      return;
-    }
-    else if (f <= fold + ALF * alam * slope) return;
-    else {
-      if (alam == 1.0)
-        tmplam = -slope / (2.0 * (f - fold - slope));
-      else {
-        rhs1 = f - fold - alam * slope;
-        rhs2 = f2 - fold2 - alam2 * slope;
-        const double a = (rhs1/(alam*alam)-rhs2/(alam2*alam2))/(alam-alam2);
-        const double b = (-alam2*rhs1/(alam*alam)+alam*rhs2/(alam2*alam2))/(alam-alam2);
-        if (a == 0.0) tmplam = -slope / (2.0 * b);
-        else {
-          const double disc = b*b-3.0*a*slope;
-          if (disc < 0.0) Msg::Error("Roundoff problem in gmshLineSearch.");
-          else tmplam = (-b+sqrt(disc))/(3.0*a);
-        }
-        if (tmplam > 0.5 * alam)
-          tmplam = 0.5 * alam;
-      }
-    }
-    alam2 = alam;
-    f2 = f;
-    fold2 = fold;
-    alam = std::max(tmplam, 0.1 * alam);
-  }
-}
-
-double minimize_grad_fd(double (*func)(fullVector<double> &, void *),
-                        fullVector<double> &x, void *data)
-{
-  const int MAXIT = 3;
-  const double EPS = 1.e-4;
-  const int N = x.size();
-
-  fullVector<double> grad(N);
-  fullVector<double> dir(N);
-  double f, feps;//, finit;
-
-  for (int iter = 0; iter < MAXIT; iter++){
-    // compute gradient of func
-    f = func(x, data);
-    //if (iter == 0) finit = f;
-    // printf("Opti iter %d x = (%g %g) f = %g\n",iter,x(0),x(1),f);
-    // printf("grad = (");
-    for (int j = 0; j < N; j++){
-      double h = EPS * fabs(x(j));
-      if(h == 0.) h = EPS;
-      x(j) += h;
-      feps = func(x, data);
-      grad(j) = (feps - f) / h;
-      // printf("%g ",grad(j));
-      dir(j) = -grad(j);
-      x(j) -= h;
-    }
-    // printf(")\n ");
-    // do a 1D line search to fine the minimum
-    // of f(x - \alpha \nabla f)
-    double f, stpmax=100000;
-    int check;
-    gmshLineSearch(func, data, x, dir, grad, f, stpmax, check);
-    // printf("Line search done x = (%g %g) f = %g\n",x(0),x(1),f);
-    if (check == 1) break;
-  }
-
-  return f;
-}
 
 /*
+  distance to triangle
+
   P(p) = p1 + t1 xi + t2 eta
 
   t1 = (p2-p1) ; t2 = (p3-p1) ;
@@ -830,7 +714,7 @@ double minimize_grad_fd(double (*func)(fullVector<double> &, void *),
   (P(p) - p) = d n
 
   (p1 + t1 xi + t2 eta - p) = d n
-  t1 xi + t2 eta + d n = p - p1
+  t1 xi + t2 eta - d n = p - p1
 
   | t1x t2x -nx | |xi  |   |px-p1x|
   | t1y t2y -ny | |eta | = |py-p1y|
@@ -849,9 +733,7 @@ double minimize_grad_fd(double (*func)(fullVector<double> &, void *),
 
 void signedDistancePointTriangle(const SPoint3 &p1,const SPoint3 &p2, const SPoint3 &p3,
 				 const SPoint3 &p, double &d, SPoint3 &closePt)
-
 {
-
   SVector3 t1 = p2 - p1;
   SVector3 t2 = p3 - p1;
   SVector3 t3 = p3 - p2;
@@ -873,9 +755,9 @@ void signedDistancePointTriangle(const SPoint3 &p1,const SPoint3 &p2, const SPoi
   v = (inv[1][0] * pp1.x() + inv[1][1] * pp1.y() + inv[1][2] * pp1.z());
   d = (inv[2][0] * pp1.x() + inv[2][1] * pp1.y() + inv[2][2] * pp1.z());
   double sign = (d > 0) ? 1. : -1.;
-  if (d == 0) sign = 1.e10;
-  if (u >= 0 && v >= 0 && 1.-u-v >= 0.0){
-    closePt = SPoint3(0.,0.,0.);//TO DO
+  if (d == 0.) sign = 1.e10;
+  if (u >= 0. && v >= 0. && 1.-u-v >= 0.0){ //P(p) inside triangle
+    closePt = p1 + (p2-p1)*u + (p3-p1)*v;
   }
   else {
     const double t12 = dot(pp1, t1) / n2t1;
@@ -883,7 +765,6 @@ void signedDistancePointTriangle(const SPoint3 &p1,const SPoint3 &p2, const SPoi
     SVector3 pp2 = p - p2;
     const double t23 = dot(pp2, t3) / n2t3;
     d = 1.e10;
-    SPoint3 closePt;
     if (t12 >= 0 && t12 <= 1.){
       d = sign * std::min(fabs(d), p.distance(p1 + (p2 - p1) * t12));
       closePt = p1 + (p2 - p1) * t12;
@@ -908,9 +789,9 @@ void signedDistancePointTriangle(const SPoint3 &p1,const SPoint3 &p2, const SPoi
       closePt = p3;
       d = sign * std::min(fabs(d), p.distance(p3));
     }
-   }
-
+  }
 }
+
 void signedDistancesPointsTriangle(std::vector<double> &distances,
                                    std::vector<SPoint3> &closePts,
                                    const std::vector<SPoint3> &pts,
@@ -997,8 +878,6 @@ void signedDistancesPointsTriangle(std::vector<double> &distances,
   }
 }
 
-
-
 void signedDistancePointLine(const SPoint3 &p1, const SPoint3 &p2, const SPoint3 &p,
                              double &d, SPoint3 &closePt)
 {
@@ -1154,9 +1033,9 @@ int computeDistanceRatio(const double &y, const double &yp, const double &x,
     }
     else{
       if (x == xp){
-	y1 = x1;
+        y1 = x1;
         y2 = x2;
-	x1 = -b;
+        x1 = -b;
         x2 = -b;
       }
       else{
@@ -1169,32 +1048,32 @@ int computeDistanceRatio(const double &y, const double &yp, const double &x,
         else{
           y1 = -(b + x1) / a;
           y2 = -(b + x2) / a;
-	}
+        }
       }
     }
     if (x1 == x2){
       propdist = (y1 - y) / (yp - y);
       if(propdist < 0.0){
-	propdist = (y2 - y) / (yp - y);
+        propdist = (y2 - y) / (yp - y);
       }
     }
     else{
       if (xp != x){
         propdist = (x1 - x) / (xp - x);
-	if (propdist < 0.0){
-	  propdist = (x2 - x) / (xp - x);
-	}
+        if (propdist < 0.0){
+          propdist = (x2 - x) / (xp - x);
+        }
       }
       else{
-	if (yp != y){
-	  propdist = (y1 - y) / (yp - y);
-	  if(propdist < 0.0){
-	    propdist = (y2 - y) / (yp - y);
-	  }
-	}
+        if (yp != y){
+          propdist = (y1 - y) / (yp - y);
+          if(propdist < 0.0){
+            propdist = (y2 - y) / (yp - y);
+          }
+        }
         else{
-	  propdist = 0.01;
-	}
+          propdist = 0.01;
+        }
       }
     }
     *distance = propdist;
@@ -1202,13 +1081,53 @@ int computeDistanceRatio(const double &y, const double &yp, const double &x,
   }
 }
 
+void signedDistancesPointsEllipsePoint(std::vector<double> &distances,
+                                      std::vector<double> &distancesE,
+                                      std::vector<int> &isInYarn,
+                                      std::vector<SPoint3> &closePts,
+                                      const std::vector<SPoint3> &pts,
+                                      const SPoint3 &p1,
+                                      const SPoint3 &p2,
+                                      const double radius)
+{
+  distances.clear();
+  distances.resize(pts.size());
+  distancesE.clear();
+  distancesE.resize(pts.size());
+  isInYarn.clear();
+  isInYarn.resize(pts.size());
+  closePts.clear();
+  closePts.resize(pts.size());
+  double d;
+  for (unsigned int i = 0; i < pts.size();i++){
+    SPoint3 closePt;
+    const SPoint3 &p = pts[i];
+    signedDistancePointLine(p1, p2, p, d, closePt);
+    closePts[i] = closePt;
+    distances[i] = d;
+    if (d <= radius){
+      isInYarn[i] = 1;
+      distancesE[i] = radius - d;
+    }
+    else{
+      isInYarn[i] = 0;
+      distancesE[i] = d - radius;
+    }
+  }
+}
+
 void signedDistancesPointsEllipseLine(std::vector<double>&distances,
                                       std::vector<double> &distancesE,
                                       std::vector<int>&isInYarn,
                                       std::vector<SPoint3>&closePts,
                                       const std::vector<SPoint3> &pts,
                                       const SPoint3 &p1,
-                                      const SPoint3 &p2)
+                                      const SPoint3 &p2,
+                                      const double maxA,
+                                      const double minA,
+                                      const double maxB,
+                                      const double minB,
+                                      const int typeLevelSet)
 {
   distances.clear();
   distances.resize(pts.size());
@@ -1229,37 +1148,127 @@ void signedDistancesPointsEllipseLine(std::vector<double>&distances,
     int direction=0;
     if (!(p.x()==closePt.x() && p.y()==closePt.y() && p.z()==closePt.z())){
       double xp,yp,x,y,otherp,other,propdist;
-      if (p1.x()==p2.x()){
-        direction=1;
-        if (fabs(closePt.x() - 0.0) < 0.00000001) isInYarn[i] = 1;
-        if (fabs(closePt.x() - 2.2) < 0.00000001) isInYarn[i] = 4;
-        if (fabs(closePt.x() - 4.4) < 0.00000001) isInYarn[i] = 2;
-        if (fabs(closePt.x() - 6.6) < 0.00000001) isInYarn[i] = 5;
-        if (fabs(closePt.x() - 8.8) < 0.00000001) isInYarn[i] = 3;
-	if (fabs(closePt.x() - 11.0) < 0.00000001) isInYarn[i] = 1;
-      }
-      else{
-        if (p1.y() == p2.y()){
-          direction = 2;
-	  if (fabs(closePt.y() - 0.0) < 0.00000001) isInYarn[i] = 6;
-	  if (fabs(closePt.y() - 2.2) < 0.00000001) isInYarn[i] = 7;
-	  if (fabs(closePt.y() - 4.4) < 0.00000001) isInYarn[i] = 8;
-	  if (fabs(closePt.y() - 6.6) < 0.00000001) isInYarn[i] = 9;
-	  if (fabs(closePt.y() - 8.8) < 0.00000001) isInYarn[i] = 10;
-	  if (fabs(closePt.y() - 11.0) < 0.00000001) isInYarn[i] = 6;
+      if (typeLevelSet==2){
+        if (p1.x()==p2.x()){
+          direction=1;
+          if (fabs(closePt.x()-0.0)<0.00000001) isInYarn[i]=1;
+          if (fabs(closePt.x()-2.2)<0.00000001) isInYarn[i]=1;
         }
         else{
-	  printf("WTF %lf %lf\n", closePt.x(), closePt.y());
+          if (p1.y()==p2.y()){
+            direction=2;
+            if (fabs(closePt.y()-0.0)<0.00000001) isInYarn[i]=6;
+            if (fabs(closePt.y()-2.2)<0.00000001) isInYarn[i]=6;
+          }
+          else{
+            printf("Error %lf %lf\n",closePt.x(),closePt.y());
+          }
+        }
+      }
+      if (typeLevelSet==1){
+        if (p1.x()==p2.x()){
+          direction=1;
+          //if (fabs(closePt.x() - 0.0) < 0.00000001) isInYarn[i] = 1;
+          if (fabs(closePt.x() - 2.2) < 0.00000001) isInYarn[i] = 4;
+          //if (fabs(closePt.x() - 4.4) < 0.00000001) isInYarn[i] = 2;
+          //if (fabs(closePt.x() - 6.6) < 0.00000001) isInYarn[i] = 5;
+          //if (fabs(closePt.x() - 8.8) < 0.00000001) isInYarn[i] = 3;
+          //if (fabs(closePt.x() - 11.0) < 0.00000001) isInYarn[i] = 1;
+        }
+        else{
+          if (p1.y() == p2.y()){
+            direction = 2;
+            //if (fabs(closePt.y() - 0.0) < 0.00000001) isInYarn[i] = 6;
+            if (fabs(closePt.y() - 2.2) < 0.00000001) isInYarn[i] = 7;
+            //if (fabs(closePt.y() - 4.4) < 0.00000001) isInYarn[i] = 8;
+            //if (fabs(closePt.y() - 6.6) < 0.00000001) isInYarn[i] = 9;
+            //if (fabs(closePt.y() - 8.8) < 0.00000001) isInYarn[i] = 10;
+            //if (fabs(closePt.y() - 11.0) < 0.00000001) isInYarn[i] = 6;
+          }
+          else{
+            printf("Error %lf %lf\n", closePt.x(), closePt.y());
+          }
+        }
+      }
+      if (typeLevelSet==4){
+        if (p1.x()==p2.x()){
+          direction=1;
+          if (fabs(closePt.x()-0.0)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=1;
+          if (fabs(closePt.x()-2.2)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=4;
+          if (fabs(closePt.x()-4.4)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=2;
+          if (fabs(closePt.x()-6.6)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=5;
+          if (fabs(closePt.x()-8.8)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=3;
+          if (fabs(closePt.x()-11.0)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=1;
+          if (fabs(closePt.x()-0.0)<0.00000001 && closePt.z()>0.35) isInYarn[i]=11;
+          if (fabs(closePt.x()-2.2)<0.00000001 && closePt.z()>0.35) isInYarn[i]=14;
+          if (fabs(closePt.x()-4.4)<0.00000001 && closePt.z()>0.35) isInYarn[i]=12;
+          if (fabs(closePt.x()-6.6)<0.00000001 && closePt.z()>0.35) isInYarn[i]=15;
+          if (fabs(closePt.x()-8.8)<0.00000001 && closePt.z()>0.35) isInYarn[i]=13;
+          if (fabs(closePt.x()-11.0)<0.00000001 && closePt.z()>0.35) isInYarn[i]=11;
+        }
+        else{
+          if (p1.y()==p2.y()){
+            direction=2;
+            if (fabs(closePt.y()-0.0)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=6;
+            if (fabs(closePt.y()-2.2)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=7;
+            if (fabs(closePt.y()-4.4)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=8;
+            if (fabs(closePt.y()-6.6)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=9;
+            if (fabs(closePt.y()-8.8)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=10;
+            if (fabs(closePt.y()-11.0)<0.00000001 && closePt.z()<=0.35) isInYarn[i]=6;
+            if (fabs(closePt.y()-0.0)<0.00000001 && closePt.z()>0.35) isInYarn[i]=16;
+            if (fabs(closePt.y()-2.2)<0.00000001 && closePt.z()>0.35) isInYarn[i]=17;
+            if (fabs(closePt.y()-4.4)<0.00000001 && closePt.z()>0.35) isInYarn[i]=18;
+            if (fabs(closePt.y()-6.6)<0.00000001 && closePt.z()>0.35) isInYarn[i]=19;
+            if (fabs(closePt.y()-8.8)<0.00000001 && closePt.z()>0.35) isInYarn[i]=20;
+            if (fabs(closePt.y()-11.0)<0.00000001 && closePt.z()>0.35) isInYarn[i]=16;
+          }
+          else{
+            printf("Error %lf %lf\n",closePt.x(),closePt.y());
+          }
+        }
+      }
+      if (typeLevelSet==3){
+        direction=3;
+        isInYarn[i]=1;
+      }
+      if (typeLevelSet==5){
+        if(p1.x()==p2.x()){
+          direction=1;
+          if (fabs(closePt.x() - 0.0) < 0.00000001) isInYarn[i] = 1;
+          if (fabs(closePt.x() - 3.225) < 0.00000001) isInYarn[i] = 2;
+          if (fabs(closePt.x() - 6.45) < 0.00000001) isInYarn[i] = 3;
+          if (fabs(closePt.x() - 9.675) < 0.00000001) isInYarn[i] = 4;
+          if (fabs(closePt.x() - 12.9) < 0.00000001) isInYarn[i] = 1;
+        }
+        else{
+          if (p1.y()==p2.y()){
+            direction=2;
+            if (fabs(closePt.y() - 0.0) < 0.00000001) isInYarn[i] = 5;
+            if (fabs(closePt.y() - 1.665) < 0.00000001) isInYarn[i] = 6;
+            if (fabs(closePt.y() - 3.33) < 0.00000001) isInYarn[i] = 7;
+            if (fabs(closePt.y() - 4.995) < 0.00000001) isInYarn[i] = 8;
+            if (fabs(closePt.y() - 6.66) < 0.00000001) isInYarn[i] = 5;
+          }
+          else{
+            printf("Error %lf %lf\n",closePt.x(),closePt.y());
+          }
         }
       }
       changeReferential(direction, p, closePt, p1, p2, &xp, &yp,
                         &otherp, &x, &y, &other);
-      int result;
+      int result = 1;
       if (fabs(other-otherp) > 0.01){
-	result = 1;
+        result = 1;
       }
       else{
-        result = computeDistanceRatio(y, yp, x, xp, &propdist, 1.1, 0.0875);
+        if (direction==1){
+          result = computeDistanceRatio(y, yp, x, xp, &propdist, maxA, minA);
+        }
+        else{
+          if (direction==2){
+            result = computeDistanceRatio(y, yp, x, xp, &propdist, maxB, minB);
+          }
+        }
       }
       if (result == 1){
         distancesE[i] = 1.e10;
@@ -1271,7 +1280,7 @@ void signedDistancesPointsEllipseLine(std::vector<double>&distances,
           distancesE[i] = (1.0 / propdist) - 1.0;
         }
         else{
-	  distancesE[i] = (1.0 - (1.0 / propdist)) / 3.0;
+          distancesE[i] = (1.0 - (1.0 / propdist));
         }
       }
     }
@@ -1282,8 +1291,8 @@ void signedDistancesPointsEllipseLine(std::vector<double>&distances,
   }
 }
 
-int intersection_segments(const SPoint3 &p1, const SPoint3 &p2,
-                          const SPoint3 &q1, const SPoint3 &q2,
+int intersection_segments(const SPoint2 &p1, const SPoint2 &p2,
+                          const SPoint2 &q1, const SPoint2 &q2,
                           double x[2])
 {
   double xp_max = std::max(p1.x(), p2.x());
@@ -1308,9 +1317,67 @@ int intersection_segments(const SPoint3 &p1, const SPoint3 &p2,
     double b[2] = {q1.x() - p1.x(), q1.y() - p1.y()};
     sys2x2(A, b, x);
     return (x[0] >= 0.0 && x[0] <= 1. &&
-	    x[1] >= 0.0 && x[1] <= 1.);
+            x[1] >= 0.0 && x[1] <= 1.);
   }
 }
+/// 3D VERSION
+int intersection_segments(const SPoint3 &p1, const SPoint3 &p2,
+                          const SPoint3 &q1, const SPoint3 &q2,
+                          double x[2])
+{
+  SVector3 v1(p2,p1),v2(q2,q1);
+  double n1 = v1.norm();
+  double n2 = v2.norm();
+  double EPS = 1.e-10*std::max(n1,n2);
+  double A[2][2];
+  A[0][0] = p2.x() - p1.x();
+  A[0][1] = q1.x() - q2.x();
+  A[1][0] = p2.y() - p1.y();
+  A[1][1] = q1.y() - q2.y();
+  double a[2] = {q1.x() - p1.x(), q1.y() - p1.y()};
+  double B[2][2];
+  B[0][0] = p2.z() - p1.z();
+  B[0][1] = q1.z() - q2.z();
+  B[1][0] = p2.y() - p1.y();
+  B[1][1] = q1.y() - q2.y();
+  double b[2] = {q1.z() - p1.z(), q1.y() - p1.y()};
+  double C[2][2];
+  C[0][0] = p2.z() - p1.z();
+  C[0][1] = q1.z() - q2.z();
+  C[1][0] = p2.x() - p1.x();
+  C[1][1] = q1.x() - q2.x();
+  double c[2] = {q1.z() - p1.z(), q1.x() - p1.x()};
+  double detA = fabs(det2x2(A));
+  double detB = fabs(det2x2(B));
+  double detC = fabs(det2x2(C));
+  //  printf("%12.5E %12.5E %12.5E\n",detA,detB,detC);
+  if (detA > detB && detA > detC)
+    sys2x2(A, a, x);
+  else if (detB > detA && detB > detC)
+    sys2x2(B, b, x);
+  else
+    sys2x2(C, c, x);
+  if(x[0] >= 0.0 && x[0] <= 1. &&
+     x[1] >= 0.0 && x[1] <= 1.) {
+
+    SPoint3 x1 (p1.x()*(1.-x[0]) + p2.x()*x[0],
+		p1.y()*(1.-x[0]) + p2.y()*x[0],
+		p1.z()*(1.-x[0]) + p2.z()*x[0]);
+    SPoint3 x2 (q1.x()*(1.-x[0]) + q2.x()*x[0],
+		q1.y()*(1.-x[0]) + q2.y()*x[0],
+		q1.z()*(1.-x[0]) + q2.z()*x[0]);
+
+    SVector3 d (x2,x1);
+    double nd = norm(d);
+    if (nd > EPS){
+      x[0] = x[1] = 1.e22;
+      return false;
+    }
+    return true;
+  }
+  return false;
+}
+
 
 void computeMeanPlaneSimple(const std::vector<SPoint3> &points, mean_plane &meanPlane)
 {
@@ -1420,7 +1487,7 @@ void projectPointsToPlane(const std::vector<SPoint3> &pts, std::vector<SPoint3>
 
 void transformPointsIntoOrthoBasis(const std::vector<SPoint3> &ptsProj,
                                    std::vector<SPoint3> &pointsUV,
-				   const SPoint3 &ptCG, const mean_plane &meanPlane)
+                                   const SPoint3 &ptCG, const mean_plane &meanPlane)
 {
   pointsUV.resize(ptsProj.size());
   SVector3 normal(meanPlane.a, meanPlane.b, meanPlane.c);
