@@ -1,12 +1,14 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
+#include <algorithm>
 #include "discreteFrechetDistance.h"
 #include "fullMatrix.h"
 
-static double distance (const SPoint3 &p1, const SPoint3 &p2){
+static double distance (const SPoint3 &p1, const SPoint3 &p2)
+{
   return p1.distance(p2);
 }
 
