@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -6,10 +6,10 @@
 #ifndef _FULL_MATRIX_H_
 #define _FULL_MATRIX_H_
 
-#include <math.h>
-#include <stdio.h>
 #include "GmshConfig.h"
 #include "GmshMessage.h"
+#include <cmath>
+#include <cstdio>
 
 template <class scalar> class fullMatrix;
 
@@ -150,6 +150,12 @@ class fullVector
     return *this;
   }
 
+  void copy(const fullVector<scalar> &v, int i0, int ni, int desti0)
+  {
+    for(int i = i0, desti = desti0; i < i0 + ni; i++, desti++)
+      (*this)(desti) = v(i);
+  }
+
   // set
   /**
      @param r A vector index between 0 and size() - 1;
@@ -495,7 +501,7 @@ class fullMatrix
     if (nbRows != -1 && nbColumns == -1)
       nbColumns = _r * _c / nbRows;
     if (nbRows*nbColumns != size1()*size2())
-      Msg::Error("Invalid reshape, total number of entries must be equal");
+      Msg::Error("Invalid reshape, total number of entries must be equal (new %i x %i != old %i x %i)", nbRows, nbColumns, size1(), size2());
     _r = nbRows;
     _c = nbColumns;
   }
