@@ -1,16 +1,15 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include <limits>
 #include <cmath>
+#include <algorithm>
 #include "nodalBasis.h"
 #include "BasisFactory.h"
 #include "pointsGenerators.h"
 
-
-
 namespace ClosureGen {
   inline double pow2(double x)
   {
@@ -40,7 +39,8 @@ namespace ClosureGen {
     }
   }
 
-  void rotateHexFull(int iFace, int iRot, int iSign, double uI, double vI, double wI, double &uO, double &vO, double &wO)
+  void rotateHexFull(int iFace, int iRot, int iSign, double uI, double vI,
+                     double wI, double &uO, double &vO, double &wO)
   {
     switch (iFace) {
       case 0: uO = uI; vO = vI; wO = wI; break;
@@ -344,7 +344,8 @@ namespace ClosureGen {
                               bool serendip, const fullMatrix<double> &points)
   {
     closure.clear();
-    const nodalBasis &fsFace = *BasisFactory::getNodalBasis(ElementType::getTag(TYPE_QUA, order, serendip));
+    const nodalBasis &fsFace = *BasisFactory::getNodalBasis
+      (ElementType::getTag(TYPE_QUA, order, serendip));
     for (int iRotate = 0; iRotate < 4; iRotate++){
       for (int iSign = 1; iSign >= -1; iSign -= 2){
         for (int iFace = 0; iFace < 6; iFace++) {
@@ -647,3 +648,21 @@ nodalBasis::nodalBasis(int tag)
   }
 
 }
+
+void nodalBasis::getReferenceNodesForBezier(fullMatrix<double> &nodes) const
+{
+  if (parentType != TYPE_PYR && !serendip) {
+    nodes = points;
+  }
+  else {
+    const bool serendipSpace = false;
+    if (parentType != TYPE_PYR) {
+      FuncSpaceData data(true, type, order, &serendipSpace);
+      gmshGeneratePoints(data, nodes);
+    }
+    else {
+      FuncSpaceData data(true, type, false, order, order, &serendipSpace);
+      gmshGeneratePoints(data, nodes);
+    }
+  }
+}
