@@ -1,9 +1,10 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include "pointsGenerators.h"
+#include "GmshDefines.h"
 #include "MTriangle.h"
 #include "MQuadrangle.h"
 #include "MTetrahedron.h"
@@ -14,6 +15,48 @@
 
 // Points Generators
 
+void gmshGeneratePoints(FuncSpaceData data, fullMatrix<double> &points)
+{
+  switch (data.elementType()) {
+    case TYPE_PNT :
+      points = gmshGeneratePointsLine(0);
+      return;
+    case TYPE_LIN :
+      points = gmshGeneratePointsLine(data.spaceOrder());
+      return;
+    case TYPE_TRI :
+      points = gmshGeneratePointsTriangle(data.spaceOrder(),
+                                          data.spaceIsSerendipity());
+      return;
+    case TYPE_QUA :
+      points = gmshGeneratePointsQuadrangle(data.spaceOrder(),
+                                            data.spaceIsSerendipity());
+      return;
+    case TYPE_TET :
+      points = gmshGeneratePointsTetrahedron(data.spaceOrder(),
+                                             data.spaceIsSerendipity());
+      return;
+    case TYPE_PRI :
+      points = gmshGeneratePointsPrism(data.spaceOrder(),
+                                       data.spaceIsSerendipity());
+      return;
+    case TYPE_HEX :
+      points = gmshGeneratePointsHexahedron(data.spaceOrder(),
+                                            data.spaceIsSerendipity());
+      return;
+    case TYPE_PYR :
+      points = gmshGeneratePointsPyramidGeneral(data.isPyramidalSpace(),
+                                                data.nij(),
+                                                data.nk(),
+                                                data.spaceIsSerendipity());
+      return;
+    default :
+      Msg::Error("Unknown element type %d (tag %d) for points generation",
+          data.elementType(), data.elementTag());
+      return;
+  }
+}
+
 fullMatrix<double> gmshGeneratePointsLine(int order)
 {
   fullMatrix<double> points = gmshGenerateMonomialsLine(order);
@@ -89,7 +132,7 @@ fullMatrix<double> gmshGeneratePointsPyramid(int order, bool serendip)
   if (order == 0) return points;
 
   for (int i = 0; i < points.size1(); ++i) {
-    points(i, 2) = points(i, 2) / order;
+    points(i, 2) = 1 - points(i, 2) / order;
     const double duv = -1. + points(i, 2);
     points(i, 0) = duv + points(i, 0) * 2. / order;
     points(i, 1) = duv + points(i, 1) * 2. / order;
@@ -97,8 +140,67 @@ fullMatrix<double> gmshGeneratePointsPyramid(int order, bool serendip)
   return points;
 }
 
+fullMatrix<double> gmshGeneratePointsPyramidGeneral(bool pyr, int nij, int nk, bool forSerendipPoints)
+{
+  fullMatrix<double> points =
+      gmshGenerateMonomialsPyramidGeneral(pyr, nij, nk, forSerendipPoints);
+
+  if (points.size1() == 1) return points;
+
+  const int div = pyr ? nk+nij : std::max(nij, nk);
+  for (int i = 0; i < points.size1(); ++i) {
+    points(i, 2) = (nk - points(i, 2)) / div;
+    const double scale = 1. - points(i, 2);
+    points(i, 0) = scale * (-1 + points(i, 0) * 2. / div);
+    points(i, 1) = scale * (-1 + points(i, 1) * 2. / div);
+  }
+  return points;
+}
+
 // Monomials Generators
 
+void gmshGenerateMonomials(FuncSpaceData data, fullMatrix<double> &monomials)
+{
+  switch (data.elementType()) {
+    case TYPE_PNT :
+      monomials = gmshGenerateMonomialsLine(0);
+      return;
+    case TYPE_LIN :
+      monomials = gmshGenerateMonomialsLine(data.spaceOrder());
+      return;
+    case TYPE_TRI :
+      monomials = gmshGenerateMonomialsTriangle(data.spaceOrder(),
+                                                data.spaceIsSerendipity());
+      return;
+    case TYPE_QUA :
+      monomials = gmshGenerateMonomialsQuadrangle(data.spaceOrder(),
+                                                  data.spaceIsSerendipity());
+      return;
+    case TYPE_TET :
+      monomials = gmshGenerateMonomialsTetrahedron(data.spaceOrder(),
+                                                   data.spaceIsSerendipity());
+      return;
+    case TYPE_PRI :
+      monomials = gmshGenerateMonomialsPrism(data.spaceOrder(),
+                                             data.spaceIsSerendipity());
+      return;
+    case TYPE_HEX :
+      monomials = gmshGenerateMonomialsHexahedron(data.spaceOrder(),
+                                                  data.spaceIsSerendipity());
+      return;
+    case TYPE_PYR :
+      monomials = gmshGenerateMonomialsPyramidGeneral(data.isPyramidalSpace(),
+                                                      data.nij(),
+                                                      data.nk(),
+                                                      data.spaceIsSerendipity());
+      return;
+    default :
+      Msg::Error("Unknown element type %d (tag %d) for monomials generation",
+          data.elementType(), data.elementTag());
+      return;
+  }
+}
+
 fullMatrix<double> gmshGenerateMonomialsLine(int order, bool serendip)
 {
   fullMatrix<double> monomials(order + 1, 1);
@@ -195,12 +297,12 @@ fullMatrix<double> gmshGenerateMonomialsQuadrangle(int order, bool forSerendipPo
 }
 
 /*
-00 10 20 30 40 ..
-01 11 21 31 41 ..
+00 10 20 30 40 ...
+01 11 21 31 41 ...
 02 12
 03 13
 04 14
-. .
+...
 */
 
 fullMatrix<double> gmshGenerateMonomialsQuadSerendipity(int order)
@@ -668,28 +770,29 @@ fullMatrix<double> gmshGenerateMonomialsPyramid(int order, bool forSerendipPoint
   int nbMonomials = forSerendipPoints ? 5 + (order-1)*8 :
                                         (order+1)*((order+1)+1)*(2*(order+1)+1)/6;
   if (forSerendipPoints && !order) nbMonomials = 1;
+
   fullMatrix<double> monomials(nbMonomials, 3);
 
   monomials(0, 0) = 0;
   monomials(0, 1) = 0;
-  monomials(0, 2) = 0;
+  monomials(0, 2) = order;
 
   if (order > 0) {
     monomials(1, 0) = order;
     monomials(1, 1) = 0;
-    monomials(1, 2) = 0;
+    monomials(1, 2) = order;
 
     monomials(2, 0) = order;
     monomials(2, 1) = order;
-    monomials(2, 2) = 0;
+    monomials(2, 2) = order;
 
     monomials(3, 0) = 0;
     monomials(3, 1) = order;
-    monomials(3, 2) = 0;
+    monomials(3, 2) = order;
 
     monomials(4, 0) = 0;
     monomials(4, 1) = 0;
-    monomials(4, 2) = order;
+    monomials(4, 2) = 0;
 
     if (order > 1) {
       int index = 5;
@@ -751,7 +854,11 @@ fullMatrix<double> gmshGenerateMonomialsPyramid(int order, bool forSerendipPoint
 
         if (order > 2) {
           fullMatrix<double> inner = gmshGenerateMonomialsPyramid(order - 3);
-          inner.add(1);
+          fullMatrix<double> prox;
+          prox.setAsProxy(inner, 0, 2);
+          prox.add(1);
+          prox.setAsProxy(inner, 2, 1);
+          prox.add(2);
           monomials.copy(inner, 0, nbMonomials - index, 0, 3, index, 0);
         }
       }
@@ -760,3 +867,144 @@ fullMatrix<double> gmshGenerateMonomialsPyramid(int order, bool forSerendipPoint
   return monomials;
 }
 
+fullMatrix<double> gmshGenerateMonomialsPyramidGeneral(
+    bool pyr, int nij, int nk, bool forSerendipPoints)
+{
+  if (nij < 0 || nk < 0) {
+    Msg::Fatal("Wrong arguments for pyramid's monomials generation ! (%d & %d)", nij, nk);
+  }
+  if (!pyr && nk > 0 && nij == 0) {
+    Msg::Error("Wrong argument association for pyramid's monomials generation ! Setting nij to 1");
+    nij = 1;
+  }
+
+  // If monomials for pyramidal space, generate them in gmsh convention.
+  if (forSerendipPoints || (pyr && nij == 0))
+    return gmshGenerateMonomialsPyramid(nk, forSerendipPoints);
+
+  // Otherwise, just put corners at first places,
+  // order of others having no importance.
+
+  int nbMonomials;
+  if (pyr) {
+    nbMonomials = 0;
+    for (int k = 0; k <= nk; ++k) {
+      nbMonomials += (k+nij+1) * (k+nij+1);
+    }
+  }
+  else nbMonomials = (nij+1) * (nij+1) * (nk+1);
+
+  fullMatrix<double> monomials(nbMonomials, 3);
+
+  monomials(0, 0) = 0;
+  monomials(0, 1) = 0;
+  monomials(0, 2) = nk;
+
+  if (nk == 0 && nij == 0) return monomials;
+
+  // Here, nij > 0
+  int nijBase = pyr ? nk+nij : nij;
+
+  // Corners
+  monomials(1, 0) = nijBase;
+  monomials(1, 1) = 0;
+  monomials(1, 2) = nk;
+
+  monomials(2, 0) = nijBase;
+  monomials(2, 1) = nijBase;
+  monomials(2, 2) = nk;
+
+  monomials(3, 0) = 0;
+  monomials(3, 1) = nijBase;
+  monomials(3, 2) = nk;
+
+  int index = 4;
+
+  if (nk > 0) {
+    monomials(4, 0) = 0;
+    monomials(4, 1) = 0;
+    monomials(4, 2) = 0;
+
+    monomials(5, 0) = nij;
+    monomials(5, 1) = 0;
+    monomials(5, 2) = 0;
+
+    monomials(6, 0) = nij;
+    monomials(6, 1) = nij;
+    monomials(6, 2) = 0;
+
+    monomials(7, 0) = 0;
+    monomials(7, 1) = nij;
+    monomials(7, 2) = 0;
+
+    index = 8;
+  }
+
+  // Base
+  if (nijBase > 1) {
+    for (int iedge = 0; iedge < 4; ++iedge) {
+      int i0 = iedge;
+      int i1 = (iedge+1) % 4;
+
+      int u0 = (monomials(i1,0)-monomials(i0,0)) / nijBase;
+      int u1 = (monomials(i1,1)-monomials(i0,1)) / nijBase;
+
+      for (int i = 1; i < nijBase; ++i, ++index) {
+        monomials(index, 0) = monomials(i0, 0) + i * u0;
+        monomials(index, 1) = monomials(i0, 1) + i * u1;
+        monomials(index, 2) = nk;
+      }
+    }
+
+    for (int i = 1; i < nijBase; ++i) {
+      for (int j = 1; j < nijBase; ++j, ++index) {
+        monomials(index, 0) = i;
+        monomials(index, 1) = j;
+        monomials(index, 2) = nk;
+      }
+    }
+  }
+
+  // Above base
+  if (nk > 0) {
+    // Top
+    if (nij > 1) {
+      for (int iedge = 0; iedge < 4; ++iedge) {
+        int i0 = 4 + iedge;
+        int i1 = 4 + (iedge+1)%4;
+
+        int u0 = (monomials(i1,0)-monomials(i0,0)) / nijBase;
+        int u1 = (monomials(i1,1)-monomials(i0,1)) / nijBase;
+
+        for (int i = 1; i < nijBase; ++i, ++index) {
+          monomials(index, 0) = monomials(i0, 0) + i * u0;
+          monomials(index, 1) = monomials(i0, 1) + i * u1;
+          monomials(index, 2) = 0;
+        }
+      }
+
+      for (int i = 1; i < nijBase; ++i) {
+        for (int j = 1; j < nijBase; ++j, ++index) {
+          monomials(index, 0) = i;
+          monomials(index, 1) = j;
+          monomials(index, 2) = 0;
+        }
+      }
+    }
+
+    // Between bottom & top
+    for (int k = nk-1; k > 0; --k) {
+      int currentnij = pyr ? k+nij : nij;
+      for (int i = 0; i <= currentnij; ++i) {
+        for (int j = 0; j <= currentnij; ++j, ++index) {
+          monomials(index, 0) = i;
+          monomials(index, 1) = j;
+          monomials(index, 2) = k;
+        }
+      }
+    }
+  }
+
+  return monomials;
+}
+
