@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -7,7 +7,7 @@
 #define POINTSGENERATORS_H
 
 #include "fullMatrix.h"
-
+#include "FuncSpaceData.h"
  /*
   * Functions to generate point distributions on
   * the references elements, for all orders.
@@ -18,6 +18,8 @@
 
 // Points
 
+void gmshGeneratePoints(FuncSpaceData, fullMatrix<double> &);
+
 fullMatrix<double> gmshGeneratePointsLine(int order);
 
 fullMatrix<double> gmshGeneratePointsTriangle(int order, bool serendip = false);
@@ -28,10 +30,13 @@ fullMatrix<double> gmshGeneratePointsHexahedron(int order, bool serendip = false
 fullMatrix<double> gmshGeneratePointsPrism(int order, bool serendip = false);
 
 fullMatrix<double> gmshGeneratePointsPyramid(int order, bool serendip = false);
+fullMatrix<double> gmshGeneratePointsPyramidGeneral(bool pyr, int nij, int nk, bool serendip = false);
 
 
 // Monomial exponents
 
+void gmshGenerateMonomials(FuncSpaceData, fullMatrix<double> &);
+
 fullMatrix<double> gmshGenerateMonomialsLine(int order, bool serendip = false);
 
 fullMatrix<double> gmshGenerateMonomialsTriangle(int order, bool serendip = false);
@@ -44,8 +49,12 @@ fullMatrix<double> gmshGenerateMonomialsHexaSerendipity(int order);
 fullMatrix<double> gmshGenerateMonomialsPrism(int order, bool forSerendipPoints = false);
 fullMatrix<double> gmshGenerateMonomialsPrismSerendipity(int order);
 
+// Generate monomials of pyramidal nodal space {X^i Y^j Z^k | i,j <= k, k <= 'order'},
 fullMatrix<double> gmshGenerateMonomialsPyramid(int order, bool forSerendipPoints = false);
-//fullMatrix<double> gmshGenerateMonomialsPyramidSerendipity(int order); //TODO
+// If 'serendip' == true, generate monomials of serendipity pyramid at order 'nk',
+// else if 'pyr' == true, generate monomials of space {X^i Y^j Z^k | i,j <= k+'nij', k <= 'nk'},
+// else if 'pyr' == false, generate monomials of space {X^i Y^j Z^k | i,j <= 'nij', k <= 'nk'}
+fullMatrix<double> gmshGenerateMonomialsPyramidGeneral(bool pyr, int nij, int nk, bool forSerendipPoints = false);
 
 
 
