@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -19,6 +19,7 @@ class simpleFunction {
   virtual ~simpleFunction(){}
   virtual bool hasDerivatives() {return _hasDerivatives;};
   virtual scalar operator () (double x, double y, double z) const { return _val; }
+  virtual void setElement(MElement *e) const {}
   virtual void gradient (double x, double y, double z,
 			 scalar & dfdx, scalar & dfdy, scalar & dfdz) const
   { dfdx = dfdy = dfdz = 0.0; }
@@ -31,14 +32,36 @@ class simpleFunction {
     dfdzx = dfdzy = dfdzz = 0.0; }
 };
 
+template <class scalar>
+class constantPerElement : public simpleFunction<scalar>
+{
+  std::map<MElement *,scalar> _data;
+  mutable MElement *_e;
+ public :
+  constantPerElement () : _e(0){}
+  void set(MElement *e, scalar v) {
+    _data[e] = v;
+  } 
+  void setElement(MElement *e) const { 
+    _e = e; }
+  virtual scalar operator () (double x, double y, double z) const 
+  { 
+    if (!_e)return 0.0;
+    typename std::map<MElement *,scalar>::const_iterator it = _data.find(_e);
+    if (it == _data.end())return 0.0;
+    return it->second;
+  }
+};
+
+
 template <class scalar>
 class simpleFunctionOnElement : public simpleFunction<scalar>
 {
-  MElement *_e;
+  mutable MElement *_e;
  public :
   simpleFunctionOnElement(scalar val=0) : simpleFunction<scalar>(val),_e(0) {}
   virtual ~simpleFunctionOnElement(){}
-  void setElement(MElement *e) { _e = e; }
+  void setElement(MElement *e) const { _e = e; }
   MElement * getElement(void) const { return _e; }
   MElement * getElement(double x, double y, double z) const
   {
