@@ -1,29 +1,20 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
 #include "AnalyseCurvedMesh.h"
-#include "GModel.h"
 #include "OS.h"
 #include "Context.h"
-
-#include <cmath>
-#include <queue>
-#include <sstream>
-
-//#include "Gmsh.h"
-//#include "GmshMessage.h"
-//#include "polynomialBasis.h"
-//#include <fstream>
-//#include "PView.h"
-
 #if defined(HAVE_OPENGL)
 #include "drawContext.h"
 #endif
-#if defined(HAVE_FLTK)
-#include "FlGui.h"
-#endif
+#include "PView.h"
+#include "GModel.h"
+#include "MElement.h"
+#include "bezierBasis.h"
+#include "MetricBasis.h"
+#include <sstream>
 
 namespace {
 
@@ -41,12 +32,12 @@ private:
   fullVector<double> _jacBez;
   double _minL, _maxL, _minB, _maxB; //Extremum of Jac at corners and of bezier values
   int _depthSub;
-  const JacobianBasis *_jfs;
+  const bezierBasis *_bfs;
 
 public:
-  BezierJacobian(fullVector<double> &, const JacobianBasis *, int depth);
+  BezierJacobian(fullVector<double>&, const bezierBasis*, int depth);
   void subDivisions(fullVector<double> &vect) const
-    {_jfs->subdivideBezierCoeff(_jacBez, vect);}
+    {_bfs->subDivisor.mult(_jacBez, vect);}
 
   bool boundsOk(double tol, double minL, double maxL) {
     return (minL <= 0 || _minB > 0) &&
@@ -64,9 +55,9 @@ public:
 } // namespace
 
 StringXNumber CurvedMeshOptions_Number[] = {
-  {GMSH_FULLRC, "Show: (0) Jacobian, (1) Metric", NULL, 1},
-  {GMSH_FULLRC, "Number of PView", NULL, 2},
-  {GMSH_FULLRC, "Hidding threshold", NULL, .1},
+  {GMSH_FULLRC, "Show: 0:J, 1:R, 2:J&&R", NULL, 1},
+  {GMSH_FULLRC, "Draw PView", NULL, 1},
+  {GMSH_FULLRC, "Hidding threshold", NULL, 10},
   {GMSH_FULLRC, "Dimension of elements", NULL, -1},
   {GMSH_FULLRC, "Recompute bounds", NULL, 0},
   {GMSH_FULLRC, "Tolerance", NULL, 1e-3}
@@ -89,30 +80,36 @@ StringXNumber *GMSH_AnalyseCurvedMeshPlugin::getOption(int iopt)
 std::string GMSH_AnalyseCurvedMeshPlugin::getHelp() const
 {
   return "Plugin(AnalyseCurvedMesh) analyse all elements of a given dimension. "
-    "It computes, min(J) where J is the scaled Jacobian determinant. Eventually, "
-    "it computes min(R) where R is the ratio between the smaller and the greater "
-    "of the eigenvalues of the metric. It creates one or more PView and hides "
+    "It computes, min(J) where J is the scaled Jacobian determinant and, if "
+    "asked, min(R) where R is the ratio between the smaller and the greater "
+    "of the eigenvalues of the metric. It creates a PView and hides "
     "elements for which min({J, R}) < 'Hidding threshold'.\n"
     "\n"
+    "J is faster to compute but gives informations only on validity while R "
+    "gives also informations on quality.\n"
+    "\n"
     "Parameters:\n"
     "\n"
-    "- Show [...] = {0, 1}: If 0, computes Jacobian and shows min(J). If 1, computes "
-    "Jacobian and metric and shows min(R).\n"
+    "- Show [...] = {0, 1, 2}: If 0, computes Jacobian and shows min(J). If 1, "
+    "computes Jacobian and metric and shows min(R). If 2, behaves like it is 1 "
+    "but draw the two min(J) and min(R) PView\n"
     "\n"
-    "- Number of PView = {0, 1, 2}: If 1, create one PView with all elements. If 2, create "
-    "two PView, one with straight-sided elements and one with curved elements.\n"
+    "- Draw PView = {0, 1}: Creates a PView of min({J, R}) if it does not "
+    "exist already. If 'Recompute' = 1, a new PView is redrawed.\n"
     "\n"
     "- Hidding threshold = [0,1]: Hides all element for which min(R) or min(J) "
     "is strictly greater than the threshold. If = 1, no effect, if = 0 hide "
     "all elements except invalid.\n"
     "\n"
-    "- Dimension = {-1, 1, 2, 3}: If = -1, analyse element of the greater dimension.\n"
+    "- Dimension = {-1, 1, 2, 3, 4}: If = -1, analyse element of the greater "
+    "dimension. If = 4, analyse 2D and 3D elements\n"
     "\n"
-    "- Recompute = {0,1}: If the mesh has changed, set to 1 to recompute the bounds.\n"
+    "- Recompute = {0,1}: If the mesh has changed, set to 1 to recompute the "
+    "bounds.\n"
     "\n"
-    "- Tolerance = ]0, 1[: Tolerance on the computation of min(R) or min(J). "
-    "It should be at most 0.01 but it can be set to 1 to just check the validity of "
-    "the mesh.";
+    "- Tolerance = ]0, 1[: Tolerance on the computation of min({R, J}). "
+    "It should be at most 0.01 but it can be set to 1 or greater to just check "
+    "the validity of the mesh.";
 }
 
 // Execution
@@ -120,115 +117,94 @@ PView *GMSH_AnalyseCurvedMeshPlugin::execute(PView *v)
 {
   _m = GModel::current();
   _computeMetric = static_cast<int>(CurvedMeshOptions_Number[0].def);
-  _numPView      = static_cast<int>(CurvedMeshOptions_Number[1].def);
+  bool drawPView = static_cast<int>(CurvedMeshOptions_Number[1].def);
   _threshold     = static_cast<double>(CurvedMeshOptions_Number[2].def);
-  _dim           = static_cast<int>(CurvedMeshOptions_Number[3].def);
-  _recompute     = static_cast<bool>(CurvedMeshOptions_Number[4].def);
+  int askedDim   = static_cast<int>(CurvedMeshOptions_Number[3].def);
+  bool recompute = static_cast<bool>(CurvedMeshOptions_Number[4].def);
   _tol           = static_cast<double>(CurvedMeshOptions_Number[5].def);
 
-  if (_dim < 0 || _dim > 3) _dim = _m->getDim();
+  if (askedDim < 0 || askedDim > 4) askedDim = _m->getDim();
 
-  if (_recompute) {
+  if (recompute) {
     _data.clear();
-    _computedJ3D = false;
-    _computedJ2D = false;
-    _computedJ1D = false;
-    _computedR3D = false;
-    _computedR2D = false;
+    if (askedDim < 4) {
+      _computedR[askedDim-1] = false;
+      _computedJ[askedDim-1] = false;
+      _PViewJ[askedDim-1] = false;
+      _PViewR[askedDim-1] = false;
+    }
+    else {
+      _computedR[1] = _computedR[2] = false;
+      _computedJ[1] = _computedJ[2] = false;
+      _PViewJ[1] = _PViewJ[2] = false;
+      _PViewR[1] = _PViewR[2] = false;
+    }
     _msgHide = true;
-    _1PViewJ = false;
-    _2PViewJ = false;
-    _1PViewR = false;
-    _2PViewR = false;
   }
 
-  if ((_dim == 1 && !_computedJ1D) ||
-      (_dim == 2 && !_computedJ2D) ||
-      (_dim == 3 && !_computedJ3D)   ) {
-    double time = Cpu();
-    Msg::StatusBar(true, "Computing Jacobian for %dD elements...", _dim);
-    _computeMinMaxJandValidity();
-    Msg::StatusBar(true, "... Done computing Jacobian (%g seconds)", Cpu()-time);
-    _printStatJacobian();
-  }
-
-  if (_computeMetric &&
-      ((_dim == 2 && !_computedR2D) ||
-       (_dim == 3 && !_computedR3D)   )) {
-    double time = Cpu();
-    Msg::StatusBar(true, "Computing metric for %dD elements...", _dim);
-    _computeMinR();
-    Msg::StatusBar(true, "... Done computing metric (%g seconds)", Cpu()-time);
-    _printStatMetric();
-  }
-
-  if (_computeMetric &&
-      (_dim == 1 ||
-       (_dim == 2 && !_computedR2D) ||
-       (_dim == 3 && !_computedR3D))) {
-    return 0;
+  // Compute J and/or R
+  for (int dim = 1; dim <= 3 && dim <= _m->getDim(); ++dim) {
+    if ((askedDim == 4 && dim > 1) || dim == askedDim) {
+      if (!_computedJ[dim-1]) {
+        double time = Cpu();
+        Msg::StatusBar(true, "Computing Jacobian for %dD elements...", dim);
+        _computeMinMaxJandValidity(dim);
+        Msg::StatusBar(true, "... Done computing Jacobian (%g seconds)", Cpu()-time);
+        _printStatJacobian();
+      }
+      if (_computeMetric && !_computedR[dim-1]) {
+        double time = Cpu();
+        Msg::StatusBar(true, "Computing metric for %dD elements...", dim);
+        _computeMinR(dim);
+        Msg::StatusBar(true, "... Done computing metric (%g seconds)", Cpu()-time);
+        _printStatMetric();
+      }
+    }
   }
 
   // Create PView
-
-  std::map<int, std::vector<double> > *dataPV1 = NULL;
-  std::map<int, std::vector<double> > *dataPV2 = NULL;
-  std::stringstream name1, name2;
-  if (_numPView == 1 &&
-      ((_computeMetric && !_1PViewR) || (!_computeMetric && !_1PViewJ))) {
-    dataPV1 = new std::map<int, std::vector<double> >();
-    if (_computeMetric) {
-      name1 << "min R";
-      _1PViewR = true;
-    }
-    else {
-      name1 << "min J";
-      _1PViewJ = true;
-    }
-    for (unsigned int i = 0; i < _data.size(); ++i) {
-      MElement *const el = _data[i].element();
-      if (el->getDim() == _dim) {
-        double val = _computeMetric ? _data[i].minR() : _data[i].minJ();
-        (*dataPV1)[el->getNum()].push_back(val);
+  if (drawPView)
+  for (int dim = 1; dim <= 3; ++dim) {
+    if ((askedDim == 4 && dim > 1) || dim == askedDim) {
+      if (!_PViewJ[dim-1] && _computeMetric != 1) {
+        _PViewJ[dim-1] = true;
+        std::map<int, std::vector<double> > dataPV;
+        for (unsigned int i = 0; i < _data.size(); ++i) {
+          MElement *const el = _data[i].element();
+          if (el->getDim() == dim) {
+            dataPV[el->getNum()].push_back(_data[i].minJ());
+          }
+        }
+        if (dataPV.size()) {
+          std::stringstream name;
+          name << "min J " << dim << "D";
+          new PView(name.str().c_str(), "ElementData", _m, dataPV);
+        }
       }
-    }
-  }
-  else if (_numPView == 2 &&
-      ((_computeMetric && !_2PViewR) || (!_computeMetric && !_2PViewJ))) {
-    dataPV1 = new std::map<int, std::vector<double> >();
-    dataPV2 = new std::map<int, std::vector<double> >();
-    if (_computeMetric) {
-      name1 << "min R straight";
-      name2 << "min R curved";
-      _2PViewR = true;
-    }
-    else {
-      name1 << "min J straight";
-      name2 << "min J curved";
-      _2PViewJ = true;
-    }
-    for (unsigned int i = 0; i < _data.size(); ++i) {
-      MElement *const el = _data[i].element();
-      if (el->getDim() == _dim) {
-        double val = _computeMetric ? _data[i].minR() : _data[i].minJ();
-        if (_data[i].maxJ() - _data[i].minJ() < _tol * 1e-2)
-          (*dataPV1)[el->getNum()].push_back(val);
-        else
-          (*dataPV2)[el->getNum()].push_back(val);
+      if (!_PViewR[dim-1] && _computeMetric) {
+        _PViewR[dim-1] = true;
+        std::map<int, std::vector<double> > dataPV;
+        for (unsigned int i = 0; i < _data.size(); ++i) {
+          MElement *const el = _data[i].element();
+          if (el->getDim() == dim)
+            dataPV[el->getNum()].push_back(_data[i].minR());
+        }
+        if (dataPV.size()) {
+          std::stringstream name;
+          name << "min R " << dim << "D";
+          new PView(name.str().c_str(), "ElementData", _m, dataPV);
+        }
       }
     }
   }
 
-  if (dataPV1) new PView(name1.str().c_str(), "ElementData", _m, *dataPV1);
-  if (dataPV2) new PView(name2.str().c_str(), "ElementData", _m, *dataPV2);
-
-  //
-
+  // Hide elements
 #if defined(HAVE_OPENGL)
-  if (_hideWithThreshold() && _msgHide) {
+  if (_hideWithThreshold(askedDim) && _msgHide) {
     _msgHide = false;
     Msg::Info("Note: Some elements have been hidden (param 'Hidding Threshold').");
-    Msg::Info("      (To revert visibility : Tools > Visibility > Interactive > Show All)");
+    Msg::Info("      (To revert: set 'Hidding Threshold' to 1 and rerun");
+    Msg::Info("                  or, Tools > Visibility > Interactive > Show All)");
   }
   CTX::instance()->mesh.changed = (ENT_ALL);
   drawContext::global()->draw();
@@ -237,11 +213,12 @@ PView *GMSH_AnalyseCurvedMeshPlugin::execute(PView *v)
   return 0;
 }
 
-void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity()
+void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity(int dim)
 {
-  switch (_dim) {
+  if (_computedJ[dim-1]) return;
+
+  switch (dim) {
     case 3 :
-      if (_computedJ3D) break;
       for (GModel::riter it = _m->firstRegion(); it != _m->lastRegion(); it++) {
         GRegion *r = *it;
         unsigned int numType[5] = {0, 0, 0, 0, 0};
@@ -252,11 +229,9 @@ void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity()
           _computeMinMaxJandValidity(el, numType[type]);
         }
       }
-      _computedJ3D = true;
       break;
 
     case 2 :
-      if (_computedJ2D) break;
       for (GModel::fiter it = _m->firstFace(); it != _m->lastFace(); it++) {
         GFace *f = *it;
         unsigned int numType[3] = {0, 0, 0};
@@ -267,24 +242,23 @@ void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity()
           _computeMinMaxJandValidity(el, numType[type]);
         }
       }
-      _computedJ2D = true;
       break;
 
     case 1 :
-      if (_computedJ1D) break;
       for (GModel::eiter it = _m->firstEdge(); it != _m->lastEdge(); it++) {
         GEdge *e = *it;
         unsigned int numElement = e->getNumMeshElements();
         MElement *const *el = e->getStartElementType(0);
         _computeMinMaxJandValidity(el, numElement);
       }
-      _computedJ1D = true;
       break;
 
     default :
-      Msg::Error("I can not analyse any element.");
+      Msg::Fatal("This should not happen.");
       return;
   }
+
+  _computedJ[dim-1] = true;
 }
 
 void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity(MElement *const*el, int numEl)
@@ -297,12 +271,15 @@ void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity(MElement *const*el
     Msg::Error("Jacobian function space not implemented for type of element %d", el[0]->getNum());
     return;
   }
+  const bezierBasis *bfs = jfs->getBezier();
+
+  _data.reserve(_data.size() + numEl);
 
   const int numSamplingPt = jfs->getNumJacNodes();
   const int numMapNodes = jfs->getNumMapNodes();
   fullVector<double> jacobian(numSamplingPt);
   fullVector<double> jacBez(numSamplingPt);
-  fullVector<double> subJacBez(jfs->getNumSubNodes());
+  fullVector<double> subJacBez(bfs->getNumSubNodes());
 
   for (int k = 0; k < numEl; ++k) {
     fullMatrix<double> nodesXYZ(numMapNodes,3);
@@ -310,7 +287,7 @@ void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity(MElement *const*el
     jfs->getScaledJacobian(nodesXYZ,jacobian);
     jfs->lag2Bez(jacobian, jacBez);
 
-    BezierJacobian *bj = new BezierJacobian(jacBez, jfs, 0);
+    BezierJacobian *bj = new BezierJacobian(jacBez, bfs, 0);
     std::vector<BezierJacobian*> heap;
     heap.push_back(bj);
 
@@ -324,17 +301,23 @@ void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity(MElement *const*el
 
       bj->subDivisions(subJacBez);
       currentDepth = bj->depth() + 1;
-      if (currentDepth > 20) Msg::Error("depth is too damn high !");
+      if (currentDepth > 20) {
+        static int a = 0;
+        if (++a == 1)
+          Msg::Error("depth is too damn high ! elem type %d",
+                     el[k]->getTypeForMSH());
+      }
 
-      for (int i = 0; i < jfs->getNumDivisions(); i++) {
+      for (int i = 0; i < bfs->getNumDivision(); i++) {
         jacBez.setAsProxy(subJacBez, i * numSamplingPt, numSamplingPt);
-        bj = new BezierJacobian(jacBez, jfs, currentDepth);
-        minL = std::min(minL, bj->minL());
-        maxL = std::max(maxL, bj->maxL());
+        BezierJacobian *newbj = new BezierJacobian(jacBez, bfs, currentDepth);
+        minL = std::min(minL, newbj->minL());
+        maxL = std::max(maxL, newbj->maxL());
 
-        heap.push_back(bj);
+        heap.push_back(newbj);
         push_heap(heap.begin(), heap.end(), lessMinVal());
       }
+      delete bj;
     }
 
     make_heap(heap.begin(), heap.end(), lessMax());
@@ -346,17 +329,23 @@ void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity(MElement *const*el
 
       bj->subDivisions(subJacBez);
       currentDepth = bj->depth() + 1;
-      if (currentDepth > 20) Msg::Error("depth is too damn high !");
+      if (currentDepth > 20) {
+        static int a = 0;
+        if (++a == 1)
+          Msg::Error("depth is too damn high ! elem type %d",
+                     el[k]->getTypeForMSH());
+      }
 
-      for (int i = 0; i < jfs->getNumDivisions(); i++) {
+      for (int i = 0; i < bfs->getNumDivision(); i++) {
         jacBez.setAsProxy(subJacBez, i * numSamplingPt, numSamplingPt);
-        bj = new BezierJacobian(jacBez, jfs, currentDepth);
-        minL = std::min(minL, bj->minL());
-        maxL = std::max(maxL, bj->maxL());
+        BezierJacobian *newbj = new BezierJacobian(jacBez, bfs, currentDepth);
+        minL = std::min(minL, newbj->minL());
+        maxL = std::max(maxL, newbj->maxL());
 
-        heap.push_back(bj);
+        heap.push_back(newbj);
         push_heap(heap.begin(), heap.end(), lessMax());
       }
+      delete bj;
     }
 
     double minB = minL;
@@ -364,52 +353,15 @@ void GMSH_AnalyseCurvedMeshPlugin::_computeMinMaxJandValidity(MElement *const*el
     for (unsigned int i = 0; i < heap.size(); ++i) {
       minB = std::min(minB, heap[i]->minB());
       maxB = std::max(maxB, heap[i]->maxB());
+      delete heap[i];
     }
-
-    // TODO REMOVE #include "BasisFactory.h"
-
-    /*if (el[k]->getNum() == 26776) {
-      Msg::Info("%g %g %g %g", minB, minL, maxL, maxB);
-
-      / *const nodalBasis* nb = BasisFactory::getNodalBasis(el[k]->getTypeForMSH());
-      int tag1 = ElementType::getTag(el[k]->getType(), 1);
-      const nodalBasis* nb1 = BasisFactory::getNodalBasis(tag1);
-
-      //double *f;
-      //f = new double[nb1->points.size1()];
-      fullMatrix<double> f;
-      nb1->f(nb->points, f);
-      Msg::Info("SIZES [%d %d] (%d, %d)", f.size1(), f.size2(), nb->points.size1(), nb1->points.size1());
-
-      for (int i = 0; i < nb->points.size1(); ++i) {
-        double X[3] = {0, 0, 0};
-        for (int j = 0; j < f.size2(); ++j) {
-          X[0] += el[k]->getVertex(j)->x() * f(i, j);
-          X[1] += el[k]->getVertex(j)->y() * f(i, j);
-          X[2] += el[k]->getVertex(j)->z() * f(i, j);
-        }
-        if (std::abs(X[0] - el[k]->getVertex(i)->x()) < 1e-12 &&
-            std::abs(X[1] - el[k]->getVertex(i)->y()) < 1e-12 &&
-            std::abs(X[2] - el[k]->getVertex(i)->z()) < 1e-12);
-        else {
-          Msg::Info("aaarg %d/%d : [%g %g %g] ([%g %g %g])", i+1, nb->points.size1(),
-              el[k]->getVertex(i)->x(), el[k]->getVertex(i)->y(), el[k]->getVertex(i)->z(),
-              X[0], X[1], X[2]);
-        }
-      }* /
-    }*/
-
     _data.push_back(CurvedMeshPluginData(el[k], minB, maxB));
   }
 }
 
-void GMSH_AnalyseCurvedMeshPlugin::_computeMinR()
+void GMSH_AnalyseCurvedMeshPlugin::_computeMinR(int dim)
 {
-  if (_dim == 1 ||
-      (_dim == 2 && _computedR2D) ||
-      (_dim == 3 && _computedR3D)) {
-    return;
-  }
+  if (_computedR[dim-1]) return;
 
   MetricBasis::setTol(_tol);
 
@@ -418,37 +370,40 @@ void GMSH_AnalyseCurvedMeshPlugin::_computeMinR()
 
   for (unsigned int i = 0; i < _data.size(); ++i) {
     MElement *const el = _data[i].element();
-    if (el->getDim() == _dim) {
+    if (el->getDim() == dim) {
       if (_data[i].minJ() <= 0)
         _data[i].setMinR(0);
       else if (_data[i].maxJ() - _data[i].minJ() < _tol * 1e-2)
-        _data[i].setMinR(MetricBasis::sampleR(el, 0));
+        _data[i].setMinR(MetricBasis::minSampledR(el, 0));
       else
         _data[i].setMinR(MetricBasis::boundMinR(el));
     }
     if (i >= nextCheck) {
       nextCheck += _data.size() / 100;
-      if (Cpu() - time > 10. && i*100/_data.size() > percentage + 4) {
-        percentage = i*100/_data.size();
-        time = Cpu();
+      double curTime = Cpu();
+      unsigned int curPercentage = i*100/_data.size();
+      if ((curTime - time > 10. && curPercentage > percentage + 4) ||
+          (curTime - time > 15. && curPercentage < 5)) {
+        percentage = curPercentage;
+        time = curTime;
         Msg::StatusBar(true, "%d%% (remaining time ~%g secondes)",
-            percentage, (Cpu()-initial) / (i+1) * (_data.size() - i-1));
+            percentage, (time-initial) / (i+1) * (_data.size() - i-1));
       }
     }
   }
 
-  if (_dim == 2) _computedR2D = true;
-  if (_dim == 3) _computedR3D = true;
+  _computedR[dim-1] = true;
 }
 
-bool GMSH_AnalyseCurvedMeshPlugin::_hideWithThreshold()
+bool GMSH_AnalyseCurvedMeshPlugin::_hideWithThreshold(int askedDim)
 {
   if (_threshold >= 1.) return false;
 
   bool ans = false;
   for (unsigned int i = 0; i < _data.size(); ++i) {
     MElement *const el = _data[i].element();
-    if (el->getDim() == _dim) {
+    const int dim = el->getDim();
+    if ((askedDim == 4 && dim > 1) || dim == askedDim) {
       if (( _computeMetric && _data[i].minR() > _threshold) ||
           (!_computeMetric && _data[i].minJ() > _threshold)   ) {
         el->setVisibility(0);
@@ -465,7 +420,7 @@ bool GMSH_AnalyseCurvedMeshPlugin::_hideWithThreshold()
 void GMSH_AnalyseCurvedMeshPlugin::_printStatMetric()
 {
   if (_data.empty()) {
-    Msg::Info("No stat to print...");
+    Msg::Info("No stat to print.");
     return;
   }
   double infminR, supminR, avgminR;
@@ -485,7 +440,7 @@ void GMSH_AnalyseCurvedMeshPlugin::_printStatMetric()
 void GMSH_AnalyseCurvedMeshPlugin::_printStatJacobian()
 {
   if (_data.empty()) {
-    Msg::Info("No stat to print...");
+    Msg::Info("No stat to print.");
     return;
   }
   double infminJ, supminJ, avgminJ;
@@ -517,15 +472,15 @@ void GMSH_AnalyseCurvedMeshPlugin::_printStatJacobian()
 }
 
 BezierJacobian::BezierJacobian(fullVector<double> &v,
-    const JacobianBasis *jfs, int depth)
+    const bezierBasis *bfs, int depth)
 {
   _jacBez = v;
   _depthSub = depth;
-  _jfs = jfs;
+  _bfs = bfs;
 
   _minL = _maxL = v(0);
   int i = 1;
-  for (; i < jfs->getNumLagCoeff(); i++) {
+  for (; i < bfs->getNumLagCoeff(); i++) {
     if (_minL > v(i)) _minL = v(i);
     if (_maxL < v(i)) _maxL = v(i);
   }
