@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -7,11 +7,8 @@
 #define _ANALYSECURVEDMESH_H_
 
 #include "Plugin.h"
-#include "JacobianBasis.h"
-#include "MetricBasis.h"
-#include "MElement.h"
-
 #include <vector>
+class MElement;
 
 extern "C"
 {
@@ -41,30 +38,28 @@ public:
 class GMSH_AnalyseCurvedMeshPlugin : public GMSH_PostPlugin
 {
 private :
-  int _dim;
   GModel *_m;
   double _threshold, _tol;
-  int _numPView, _computeMetric;
-  bool _recompute;
-  bool _computedR3D, _computedR2D;
-  bool _computedJ3D, _computedJ2D, _computedJ1D;
-  bool _1PViewJ, _2PViewJ, _1PViewR, _2PViewR;
+  int _computeMetric;
+
+  // for 1d, 2d, 3d
+  bool _computedR[3], _computedJ[3], _PViewJ[3], _PViewR[3];
   bool _msgHide;
 
   std::vector<CurvedMeshPluginData> _data;
 
 public :
   GMSH_AnalyseCurvedMeshPlugin() {
-    _computedR3D = false;
-    _computedR2D = false;
-    _computedJ3D = false;
-    _computedJ2D = false;
-    _computedJ1D = false;
+    _m = NULL;
+    _threshold = _tol = -1;
+    _computeMetric = -1;
+    for (int i = 0; i < 3; ++i) {
+      _computedR[i] = false;
+      _computedJ[i] = false;
+      _PViewJ[i] = false;
+      _PViewR[i] = false;
+    }
     _msgHide = true;
-    _1PViewJ = false;
-    _2PViewJ = false;
-    _1PViewR = false;
-    _2PViewR = false;
   }
   std::string getName() const { return "AnalyseCurvedMesh"; }
   std::string getShortHelp() const {
@@ -75,12 +70,29 @@ public :
   int getNbOptions() const;
   StringXNumber *getOption(int);
   PView *execute(PView *);
+  void setTol(double tol) {_tol = tol;}
+  void computeMinJ(MElement *const *el, int numEl, double *minJ, bool *straight) {
+    std::vector<CurvedMeshPluginData> save(_data);
+    _data.clear();
+    _computeMinMaxJandValidity(el, numEl);
+    if (minJ) {
+      for (unsigned int i = 0; i < _data.size(); ++i) {
+        minJ[i] = _data[i].minJ();
+      }
+    }
+    if (straight) {
+      for (unsigned int i = 0; i < _data.size(); ++i) {
+        straight[i] = _data[i].maxJ() - _data[i].minJ() < _tol * 1e-1;
+      }
+    }
+    _data = save;
+  }
 
 private :
-  void _computeMinMaxJandValidity();
+  void _computeMinMaxJandValidity(int dim);
   void _computeMinMaxJandValidity(MElement *const *, int numEl);
-  void _computeMinR();
-  bool _hideWithThreshold();
+  void _computeMinR(int dim);
+  bool _hideWithThreshold(int askedDim);
   void _printStatMetric();
   void _printStatJacobian();
 };
