@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -15,6 +15,7 @@
 #include "MElementCut.h"
 #include "Numeric.h"
 #include "GmshMessage.h"
+#include "pyramidalBasis.h"
 
 PViewDataGModel::PViewDataGModel(DataType type)
   : PViewData(), _min(VAL_INF), _max(-VAL_INF), _type(type)
@@ -143,12 +144,18 @@ bool PViewDataGModel::finalize(bool computeMinMax, const std::string &interpolat
           // the mesh is curved
           MElement *e = _getOneElementOfGivenType(model, it->first);
           if(e && e->getPolynomialOrder() > 1 && e->getFunctionSpace()){
-            const polynomialBasis *fs = (polynomialBasis*) e->getFunctionSpace();
-            setInterpolationMatrices(it->first, *(it->second[0]), *(it->second[1]),
+            if (it->first == TYPE_PYR) { // KH 18/9/2014 very nasty fix since pyramids /= polynomial
+              const pyramidalBasis  *fs = (pyramidalBasis*) e->getFunctionSpace();
+              setInterpolationMatrices(it->first, *(it->second[0]), *(it->second[1]),
                                      fs->coefficients, fs->monomials);
+            }
+            else {
+              const polynomialBasis *fs = (polynomialBasis*) e->getFunctionSpace();
+              setInterpolationMatrices(it->first, *(it->second[0]), *(it->second[1]),
+                                       fs->coefficients, fs->monomials);
+            }
           }
-          else
-            setInterpolationMatrices(it->first, *(it->second[0]), *(it->second[1]));
+          else setInterpolationMatrices(it->first, *(it->second[0]), *(it->second[1]));
         }
         else if(it->second.size() == 4){
           // use provided matrices for field and geometry
@@ -168,38 +175,62 @@ bool PViewDataGModel::finalize(bool computeMinMax, const std::string &interpolat
     // type, assume isoparametric elements (except for ElementData,
     // for which we know the interpolation: it's constant)
     int types[] = {TYPE_PNT, TYPE_LIN, TYPE_TRI, TYPE_QUA, TYPE_TET, TYPE_HEX,
-                   TYPE_PRI, /*TYPE_PYR - Not polynomial!,*/ TYPE_POLYG, TYPE_POLYH};
+                   TYPE_PRI,TYPE_PYR, TYPE_POLYG, TYPE_POLYH};
     for(unsigned int i = 0; i < sizeof(types) / sizeof(types[0]); i++){
       if(!haveInterpolationMatrices(types[i])){
         MElement *e = _getOneElementOfGivenType(model, types[i]);
         if(e){
-          const polynomialBasis *fs = (polynomialBasis*) e->getFunctionSpace();
+          
+          const polynomialBasis *fs = dynamic_cast<const polynomialBasis*> (e->getFunctionSpace());
           if(fs){
             if(e->getPolynomialOrder() > 1){
               if(_type == ElementData){
                 // data is constant per element: force the interpolation matrix
                 fullMatrix<double> coef(1, 1);
-                coef(0, 0) = 1.;
-                fullMatrix<double> mono(1, 3);
-                mono(0, 0) = 0.;
-                mono(0, 1) = 0.;
-                mono(0, 2) = 0.;
-                setInterpolationMatrices(types[i], coef, mono,
+                  coef(0, 0) = 1.;
+                  fullMatrix<double> mono(1, 3);
+                  mono(0, 0) = 0.;
+                  mono(0, 1) = 0.;
+                  mono(0, 2) = 0.;
+                  setInterpolationMatrices(types[i], coef, mono,
+                                           fs->coefficients, fs->monomials);
+                }
+              else
+                setInterpolationMatrices(types[i],
+                                         fs->coefficients, fs->monomials,
                                          fs->coefficients, fs->monomials);
               }
-              else
-                setInterpolationMatrices(types[i], fs->coefficients, fs->monomials,
-                                         fs->coefficients, fs->monomials);
-            }
             else
               setInterpolationMatrices(types[i], fs->coefficients, fs->monomials);
           }
+          else {
+            const pyramidalBasis *fs = dynamic_cast<const pyramidalBasis*> (e->getFunctionSpace());
+            if(fs){
+              if(e->getPolynomialOrder() > 1){
+                if(_type == ElementData){
+                  // data is constant per element: force the interpolation matrix
+                  fullMatrix<double> coef(1, 1);
+                  coef(0, 0) = 1.;
+                  fullMatrix<double> mono(1, 3);
+                  mono(0, 0) = 0.;
+                  mono(0, 1) = 0.;
+                  mono(0, 2) = 0.;
+                  setInterpolationMatrices(types[i], coef, mono,
+                                           fs->coefficients, fs->monomials);
+                }
+                else
+                  setInterpolationMatrices(types[i],
+                                           fs->coefficients, fs->monomials,
+                                           fs->coefficients, fs->monomials);
+              }
+              else
+                setInterpolationMatrices(types[i], fs->coefficients, fs->monomials);
+            }
+          }
         }
       }
     }
-
   }
-
   return PViewData::finalize();
 }
 
@@ -592,6 +623,14 @@ void PViewDataGModel::getValue(int step, int ent, int ele, int nod, int comp, do
     break;
   case ElementNodeData:
   case GaussPointData:
+    if(_steps[step]->getMult(e->getNum()) < nod + 1){
+      nod = 0;
+      static bool first = true;
+      if(first){
+        Msg::Warning("Some elements in ElementNodeData have less values than number of nodes");
+        first = false;
+      }
+    }
     val = _steps[step]->getData(e->getNum())[_steps[step]->getNumComponents() * nod + comp];
     break;
   case ElementData:
@@ -613,6 +652,14 @@ void PViewDataGModel::setValue(int step, int ent, int ele, int nod, int comp, do
     break;
   case ElementNodeData:
   case GaussPointData:
+    if(_steps[step]->getMult(e->getNum()) < nod + 1){
+      nod = 0;
+      static bool first = true;
+      if(first){
+        Msg::Warning("Some elements in ElementNodeData have less values than number of nodes");
+        first = false;
+      }
+    }
     _steps[step]->getData(e->getNum())[_steps[step]->getNumComponents() * nod + comp] = val;
     break;
   case ElementData:
@@ -647,6 +694,9 @@ void PViewDataGModel::smooth()
     _steps2.push_back(new stepData<double>(m, numComp, _steps[step]->getFileName(),
                                            _steps[step]->getFileIndex(),
                                            _steps[step]->getTime()));
+    _steps2.back()->fillEntities();
+    _steps2.back()->computeBoundingBox();
+
     std::map<int, int> nodeConnect;
     for(int ent = 0; ent < getNumEntities(step); ent++){
       for(int ele = 0; ele < getNumElements(step, ent); ele++){
