@@ -180,7 +180,8 @@ bool PViewData::writePOS(const std::string &fileName, bool binary, bool parsed,
 
 bool PViewData::writeMSH(const std::string &fileName, double version, bool binary,
                          bool saveMesh, bool multipleView, int partitionNum,
-                         bool saveInterpolationMatrices, bool forceNodeData)
+                         bool saveInterpolationMatrices, bool forceNodeData,
+                         bool forceElementData)
 {
   Msg::Error("MSH export not implemented for this view type");
   return false;
