@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -6,6 +6,7 @@
 #include <math.h>
 #include <list>
 #include <set>
+#include <algorithm>
 #include "adaptiveData.h"
 #include "Plugin.h"
 #include "OS.h"
@@ -20,6 +21,7 @@ std::set<adaptiveVertex> adaptiveQuadrangle::allVertices;
 std::set<adaptiveVertex> adaptiveTetrahedron::allVertices;
 std::set<adaptiveVertex> adaptiveHexahedron::allVertices;
 std::set<adaptiveVertex> adaptivePrism::allVertices;
+std::set<adaptiveVertex> adaptivePyramid::allVertices;
 
 std::list<adaptivePoint*> adaptivePoint::all;
 std::list<adaptiveLine*> adaptiveLine::all;
@@ -28,6 +30,7 @@ std::list<adaptiveQuadrangle*> adaptiveQuadrangle::all;
 std::list<adaptiveTetrahedron*> adaptiveTetrahedron::all;
 std::list<adaptiveHexahedron*> adaptiveHexahedron::all;
 std::list<adaptivePrism*> adaptivePrism::all;
+std::list<adaptivePyramid*> adaptivePyramid::all;
 
 int adaptivePoint::numNodes = 1;
 int adaptiveLine::numNodes = 2;
@@ -36,6 +39,7 @@ int adaptiveQuadrangle::numNodes = 4;
 int adaptivePrism::numNodes = 6;
 int adaptiveTetrahedron::numNodes = 4;
 int adaptiveHexahedron::numNodes = 8;
+int adaptivePyramid::numNodes = 5;
 
 int adaptivePoint::numEdges = 0;
 int adaptiveLine::numEdges = 1;
@@ -44,10 +48,11 @@ int adaptiveQuadrangle::numEdges = 4;
 int adaptivePrism::numEdges = 9;
 int adaptiveTetrahedron::numEdges = 6;
 int adaptiveHexahedron::numEdges = 12;
+int adaptivePyramid::numEdges = 8;
 
 template <class T>
 static void cleanElement()
-{  
+{
   for(typename std::list<T*>::iterator it = T::all.begin(); it != T::all.end(); ++it)
     delete *it;
   T::all.clear();
@@ -66,6 +71,40 @@ static void computeShapeFunctions(fullMatrix<double> *coeffs, fullMatrix<double>
   coeffs->mult(*tmp, *sf);
 }
 
+/*! Bergot space is characterised by polynomials
+  \f$ \mathcal B_{ijk} =
+  \mathcal P_i \left(\frac{\xi }{1-\zeta}\right)
+  \mathcal P_j \left(\frac{\eta}{1-\zeta}\right)
+  \left(1-\zeta\right)^{max(i,j)}
+  \mathcal P^{2 max(i,j),0}_k \left(2 \zeta -1\right)~|~i,j \leq p, k \leq p - max(i,j) \f$
+  and hence by the "monomials"
+  \f$ \mu_{ijk} =
+  \left(\frac{\xi }{\1-\zeta}\right)^i
+  \left(\frac{\eta}{\1-\zeta}\right)^j
+  \left(1-\zeta\right)^{max(i,j)} \zeta^k~|~i,j \leq p~,~k \leq p-max(i,j)
+  \f$
+*/
+static void computeShapeFunctionsPyramid(fullMatrix<double> *coeffs,
+                                         fullMatrix<double> *eexps,
+                                         double u, double v, double w,
+                                         fullVector<double> *sf,
+                                         fullVector<double> *tmp)
+{
+
+  double oneMinW = (w==1) ? 1e-12:1-w;
+  for(int l = 0; l < eexps->size1(); l++) {
+    int i = (*eexps)(l,0);
+    int j = (*eexps)(l,1);
+    int k = (*eexps)(l,2);
+    int m = std::max(i,j);
+    (*tmp)(l)  = pow(u,i);
+    (*tmp)(l) *= pow(v,j);
+    (*tmp)(l) *= pow(w,k);
+    (*tmp)(l) *= pow(oneMinW,m-i-j);
+  }
+  coeffs->mult(*tmp, *sf);
+}
+
 adaptiveVertex *adaptiveVertex::add(double x, double y, double z,
                                   std::set<adaptiveVertex> &allVertices)
 {
@@ -123,7 +162,7 @@ void adaptiveLine::recurCreate(adaptiveLine *e, int maxlevel, int level)
   adaptiveVertex *p1 = e->p[0];
   adaptiveVertex *p2 = e->p[1];
   adaptiveVertex *p12 = adaptiveVertex::add
-    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5, 
+    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5,
      allVertices);
   adaptiveLine *e1 = new adaptiveLine(p1, p12);
   recurCreate(e1, maxlevel, level);
@@ -201,13 +240,13 @@ void adaptiveTriangle::recurCreate(adaptiveTriangle *t, int maxlevel, int level)
   adaptiveVertex *p2 = t->p[1];
   adaptiveVertex *p3 = t->p[2];
   adaptiveVertex *p12 = adaptiveVertex::add
-    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5, 
+    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5,
      allVertices);
   adaptiveVertex *p13 = adaptiveVertex::add
-    ((p1->x + p3->x) * 0.5, (p1->y + p3->y) * 0.5, (p1->z + p3->z) * 0.5, 
+    ((p1->x + p3->x) * 0.5, (p1->y + p3->y) * 0.5, (p1->z + p3->z) * 0.5,
      allVertices);
   adaptiveVertex *p23 = adaptiveVertex::add
-    ((p3->x + p2->x) * 0.5, (p3->y + p2->y) * 0.5, (p3->z + p2->z) * 0.5, 
+    ((p3->x + p2->x) * 0.5, (p3->y + p2->y) * 0.5, (p3->z + p2->z) * 0.5,
      allVertices);
   adaptiveTriangle *t1 = new adaptiveTriangle(p1, p12, p13);
   recurCreate(t1, maxlevel, level);
@@ -315,19 +354,19 @@ void adaptiveQuadrangle::recurCreate(adaptiveQuadrangle *q, int maxlevel, int le
   adaptiveVertex *p3 = q->p[2];
   adaptiveVertex *p4 = q->p[3];
   adaptiveVertex *p12 = adaptiveVertex::add
-    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5, 
+    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5,
      allVertices);
   adaptiveVertex *p23 = adaptiveVertex::add
-    ((p2->x + p3->x) * 0.5, (p2->y + p3->y) * 0.5, (p2->z + p3->z) * 0.5, 
+    ((p2->x + p3->x) * 0.5, (p2->y + p3->y) * 0.5, (p2->z + p3->z) * 0.5,
      allVertices);
   adaptiveVertex *p34 = adaptiveVertex::add
-    ((p3->x + p4->x) * 0.5, (p3->y + p4->y) * 0.5, (p3->z + p4->z) * 0.5, 
+    ((p3->x + p4->x) * 0.5, (p3->y + p4->y) * 0.5, (p3->z + p4->z) * 0.5,
      allVertices);
   adaptiveVertex *p14 = adaptiveVertex::add
-    ((p1->x + p4->x) * 0.5, (p1->y + p4->y) * 0.5, (p1->z + p4->z) * 0.5, 
+    ((p1->x + p4->x) * 0.5, (p1->y + p4->y) * 0.5, (p1->z + p4->z) * 0.5,
      allVertices);
   adaptiveVertex *pc = adaptiveVertex::add
-    ((p1->x + p2->x + p3->x + p4->x) * 0.25, (p1->y + p2->y + p3->y + p4->y) * 0.25, 
+    ((p1->x + p2->x + p3->x + p4->x) * 0.25, (p1->y + p2->y + p3->y + p4->y) * 0.25,
      (p1->z + p2->z + p3->z + p4->z) * 0.25, allVertices);
   adaptiveQuadrangle *q1 = new adaptiveQuadrangle(p1, p12, pc, p14);
   recurCreate(q1, maxlevel, level);
@@ -441,7 +480,7 @@ void adaptiveTetrahedron::recurCreate(adaptiveTetrahedron *t, int maxlevel, int
     ((p0->x + p3->x) * 0.5, (p0->y + p3->y) * 0.5, (p0->z + p3->z) * 0.5,
      allVertices);
   adaptiveVertex *pe3 = adaptiveVertex::add
-    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5, 
+    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5,
      allVertices);
   adaptiveVertex *pe4 = adaptiveVertex::add
     ((p1->x + p3->x) * 0.5, (p1->y + p3->y) * 0.5, (p1->z + p3->z) * 0.5,
@@ -578,7 +617,7 @@ void adaptiveHexahedron::recurCreate(adaptiveHexahedron *h, int maxlevel, int le
   adaptiveVertex *p6 = h->p[6];
   adaptiveVertex *p7 = h->p[7];
   adaptiveVertex *p01 = adaptiveVertex::add
-    ((p0->x + p1->x) * 0.5, (p0->y + p1->y) * 0.5, (p0->z + p1->z) * 0.5, 
+    ((p0->x + p1->x) * 0.5, (p0->y + p1->y) * 0.5, (p0->z + p1->z) * 0.5,
      allVertices);
   adaptiveVertex *p12 = adaptiveVertex::add
     ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5,
@@ -614,19 +653,19 @@ void adaptiveHexahedron::recurCreate(adaptiveHexahedron *h, int maxlevel, int le
     ((p7->x + p3->x) * 0.5, (p7->y + p3->y) * 0.5, (p7->z + p3->z) * 0.5,
      allVertices);
   adaptiveVertex *p0145 = adaptiveVertex::add
-    ((p45->x + p01->x) * 0.5, (p45->y + p01->y) * 0.5,(p45->z + p01->z) * 0.5, 
+    ((p45->x + p01->x) * 0.5, (p45->y + p01->y) * 0.5,(p45->z + p01->z) * 0.5,
      allVertices);
   adaptiveVertex *p1256 = adaptiveVertex::add
-    ((p12->x + p56->x) * 0.5, (p12->y + p56->y) * 0.5, (p12->z + p56->z) * 0.5, 
+    ((p12->x + p56->x) * 0.5, (p12->y + p56->y) * 0.5, (p12->z + p56->z) * 0.5,
      allVertices);
   adaptiveVertex *p2367 = adaptiveVertex::add
-    ((p23->x + p67->x) * 0.5, (p23->y + p67->y) * 0.5, (p23->z + p67->z) * 0.5, 
+    ((p23->x + p67->x) * 0.5, (p23->y + p67->y) * 0.5, (p23->z + p67->z) * 0.5,
      allVertices);
   adaptiveVertex *p0347 = adaptiveVertex::add
     ((p03->x + p47->x) * 0.5, (p03->y + p47->y) * 0.5, (p03->z + p47->z) * 0.5,
      allVertices);
   adaptiveVertex *p4756 = adaptiveVertex::add
-    ((p47->x + p56->x) * 0.5, (p47->y + p56->y) * 0.5, (p47->z + p56->z) * 0.5, 
+    ((p47->x + p56->x) * 0.5, (p47->y + p56->y) * 0.5, (p47->z + p56->z) * 0.5,
      allVertices);
   adaptiveVertex *p0312 = adaptiveVertex::add
     ((p03->x + p12->x) * 0.5, (p03->y + p12->y) * 0.5, (p03->z + p12->z) * 0.5,
@@ -773,40 +812,40 @@ void adaptivePrism::recurCreate(adaptivePrism *p, int maxlevel, int level)
   adaptiveVertex *p5 = p->p[4];
   adaptiveVertex *p6 = p->p[5];
   adaptiveVertex *p14 = adaptiveVertex::add
-    ((p1->x + p4->x) * 0.5, (p1->y + p4->y) * 0.5, (p1->z + p4->z) * 0.5, 
+    ((p1->x + p4->x) * 0.5, (p1->y + p4->y) * 0.5, (p1->z + p4->z) * 0.5,
      allVertices);
   adaptiveVertex *p25 = adaptiveVertex::add
-    ((p2->x + p5->x) * 0.5, (p2->y + p5->y) * 0.5, (p2->z + p5->z) * 0.5, 
+    ((p2->x + p5->x) * 0.5, (p2->y + p5->y) * 0.5, (p2->z + p5->z) * 0.5,
      allVertices);
   adaptiveVertex *p36 = adaptiveVertex::add
-    ((p3->x + p6->x) * 0.5, (p3->y + p6->y) * 0.5, (p3->z + p6->z) * 0.5, 
+    ((p3->x + p6->x) * 0.5, (p3->y + p6->y) * 0.5, (p3->z + p6->z) * 0.5,
      allVertices);
   adaptiveVertex *p12 = adaptiveVertex::add
-    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5, 
+    ((p1->x + p2->x) * 0.5, (p1->y + p2->y) * 0.5, (p1->z + p2->z) * 0.5,
      allVertices);
   adaptiveVertex *p23 = adaptiveVertex::add
-    ((p2->x + p3->x) * 0.5, (p2->y + p3->y) * 0.5, (p2->z + p3->z) * 0.5, 
+    ((p2->x + p3->x) * 0.5, (p2->y + p3->y) * 0.5, (p2->z + p3->z) * 0.5,
      allVertices);
   adaptiveVertex *p31 = adaptiveVertex::add
-    ((p3->x + p1->x) * 0.5, (p3->y + p1->y) * 0.5, (p3->z + p1->z) * 0.5, 
+    ((p3->x + p1->x) * 0.5, (p3->y + p1->y) * 0.5, (p3->z + p1->z) * 0.5,
      allVertices);
   adaptiveVertex *p1425 = adaptiveVertex::add
-    ((p14->x + p25->x) * 0.5, (p14->y + p25->y) * 0.5, (p14->z + p25->z) * 0.5, 
+    ((p14->x + p25->x) * 0.5, (p14->y + p25->y) * 0.5, (p14->z + p25->z) * 0.5,
      allVertices);
   adaptiveVertex *p2536 = adaptiveVertex::add
-    ((p25->x + p36->x) * 0.5, (p25->y + p36->y) * 0.5, (p25->z + p36->z) * 0.5, 
+    ((p25->x + p36->x) * 0.5, (p25->y + p36->y) * 0.5, (p25->z + p36->z) * 0.5,
      allVertices);
   adaptiveVertex *p3614 = adaptiveVertex::add
-    ((p36->x + p14->x) * 0.5, (p36->y + p14->y) * 0.5, (p36->z + p14->z) * 0.5, 
+    ((p36->x + p14->x) * 0.5, (p36->y + p14->y) * 0.5, (p36->z + p14->z) * 0.5,
      allVertices);
   adaptiveVertex *p45 = adaptiveVertex::add
-    ((p4->x + p5->x) * 0.5, (p4->y + p5->y) * 0.5, (p4->z + p5->z) * 0.5, 
+    ((p4->x + p5->x) * 0.5, (p4->y + p5->y) * 0.5, (p4->z + p5->z) * 0.5,
      allVertices);
   adaptiveVertex *p56 = adaptiveVertex::add
-    ((p5->x + p6->x) * 0.5, (p5->y + p6->y) * 0.5, (p5->z + p6->z) * 0.5, 
+    ((p5->x + p6->x) * 0.5, (p5->y + p6->y) * 0.5, (p5->z + p6->z) * 0.5,
      allVertices);
   adaptiveVertex *p64 = adaptiveVertex::add
-    ((p6->x + p4->x) * 0.5, (p6->y + p4->y) * 0.5, (p6->z + p4->z) * 0.5, 
+    ((p6->x + p4->x) * 0.5, (p6->y + p4->y) * 0.5, (p6->z + p4->z) * 0.5,
      allVertices);
   p->e[0] = new adaptivePrism(p1, p12, p31, p14, p1425, p3614);
   recurCreate(p->e[0], maxlevel, level);
@@ -883,6 +922,163 @@ void adaptivePrism::recurError(adaptivePrism *p, double AVG, double tol)
   }
 }
 
+void adaptivePyramid::create(int maxlevel)
+{
+  cleanElement<adaptivePyramid>();
+  adaptiveVertex *p1 = adaptiveVertex::add(-1, -1, 0, allVertices);
+  adaptiveVertex *p2 = adaptiveVertex::add(1, -1, 0, allVertices);
+  adaptiveVertex *p3 = adaptiveVertex::add(1, 1, 0, allVertices);
+  adaptiveVertex *p4 = adaptiveVertex::add(-1, 1, 0, allVertices);
+  adaptiveVertex *p5 = adaptiveVertex::add(0, 0, 1, allVertices);
+  adaptivePyramid *p = new adaptivePyramid(p1, p2, p3, p4, p5);
+  recurCreate(p, maxlevel, 0);
+}
+
+void adaptivePyramid::recurCreate(adaptivePyramid *p, int maxlevel, int level)
+{
+  all.push_back(p);
+  if(level++ >= maxlevel) return;
+
+  // quad points
+  adaptiveVertex *p1 = p->p[0];
+  adaptiveVertex *p2 = p->p[1];
+  adaptiveVertex *p3 = p->p[2];
+  adaptiveVertex *p4 = p->p[3];
+
+  // apex
+  adaptiveVertex *p5 = p->p[4];
+
+  // center of the quad
+
+  adaptiveVertex *p1234 = adaptiveVertex::add
+    ((p1->x + p2->x + p3->x + p4->x)*0.25,
+     (p1->y + p2->y + p3->y + p4->y)*0.25,
+     (p1->z + p2->z + p3->z + p4->z)*0.25,allVertices);
+
+  // quad edge points
+
+  adaptiveVertex *p12 = adaptiveVertex::add
+    ((p1->x + p2->x)*0.5,
+     (p1->y + p2->y)*0.5,
+     (p1->z + p2->z)*0.5,allVertices);
+
+  adaptiveVertex *p23 = adaptiveVertex::add
+    ((p2->x + p3->x)*0.5,
+     (p2->y + p3->y)*0.5,
+     (p2->z + p3->z)*0.5,allVertices);
+
+  adaptiveVertex *p34 = adaptiveVertex::add
+    ((p3->x + p4->x)*0.5,
+     (p3->y + p4->y)*0.5,
+     (p3->z + p4->z)*0.5,allVertices);
+
+  adaptiveVertex *p41 = adaptiveVertex::add
+    ((p4->x + p1->x)*0.5,
+     (p4->y + p1->y)*0.5,
+     (p4->z + p1->z)*0.5,allVertices);
+
+  // quad vertex to apex edge points
+
+  adaptiveVertex *p15 = adaptiveVertex::add
+    ((p1->x + p5->x)*0.5,
+     (p1->y + p5->y)*0.5,
+     (p1->z + p5->z)*0.5,allVertices);
+
+  adaptiveVertex *p25 = adaptiveVertex::add
+    ((p2->x + p5->x)*0.5,
+     (p2->y + p5->y)*0.5,
+     (p2->z + p5->z)*0.5,allVertices);
+
+  adaptiveVertex *p35 = adaptiveVertex::add
+    ((p3->x + p5->x)*0.5,
+     (p3->y + p5->y)*0.5,
+     (p3->z + p5->z)*0.5,allVertices);
+
+  adaptiveVertex *p45 = adaptiveVertex::add
+    ((p4->x + p5->x)*0.5,
+     (p4->y + p5->y)*0.5,
+     (p4->z + p5->z)*0.5,allVertices);
+
+  // four base pyramids on the quad base
+
+  p->e[0] = new adaptivePyramid(p1, p12, p1234, p41, p15);
+  recurCreate(p->e[0], maxlevel, level);
+  p->e[1] = new adaptivePyramid(p2, p23, p1234, p12, p25);
+  recurCreate(p->e[1], maxlevel, level);
+  p->e[2] = new adaptivePyramid(p3, p34, p1234, p23, p35);
+  recurCreate(p->e[2], maxlevel, level);
+  p->e[3] = new adaptivePyramid(p4, p41, p1234, p34, p45);
+  recurCreate(p->e[3], maxlevel, level);
+
+  // top pyramids
+
+  p->e[4] = new adaptivePyramid(p15,p25,p35,p45,p5);
+  recurCreate(p->e[4], maxlevel, level);
+  p->e[5] = new adaptivePyramid(p15,p45,p35,p25,p1234);
+  recurCreate(p->e[5], maxlevel, level);
+
+  // degenerated pyramids to replace the remaining tetrahedral holes
+  // degenerated quad in the interior of the element, apices on the quad edges
+
+  p->e[6] = new adaptivePyramid(p1234,p25,p15,p1234,p12);
+  recurCreate(p->e[6], maxlevel, level);
+  p->e[7] = new adaptivePyramid(p1234,p35,p25,p1234,p23);
+  recurCreate(p->e[7], maxlevel, level);
+  p->e[8] = new adaptivePyramid(p1234,p45,p35,p1234,p34);
+  recurCreate(p->e[8], maxlevel, level);
+  p->e[9] = new adaptivePyramid(p1234,p15,p45,p1234,p41);
+  recurCreate(p->e[9], maxlevel, level);
+}
+
+void adaptivePyramid::error(double AVG, double tol)
+{
+  adaptivePyramid *p = *all.begin();
+  recurError(p, AVG, tol);
+}
+
+void adaptivePyramid::recurError(adaptivePyramid *p, double AVG, double tol)
+{
+  if(!p->e[0])
+    p->visible = true;
+  else {
+    double vi[10];
+    for (int i = 0; i < 10; i++) vi[i] = p->e[i]->V();
+    double vr = 0;
+    for (int i = 0; i < 6 ; i++) vr += vi[i];     // pyramids   have volume V/8
+    for (int i = 6; i < 10; i++) vr += vi[i]*0.5; // tetrahedra have volume V/16
+    vr /= 8.;
+    const double v = p->V();
+    if(!p->e[0]->e[0]) {
+      if(fabs(v - vr) > AVG * tol){
+        p->visible = false;
+        for (int i = 0; i < 10; i++) recurError(p->e[i],AVG,tol);
+      }
+      else
+        p->visible = true;
+    }
+    else {
+      bool err = false;
+      for(int i = 0; i < 10; i++){
+        double vj[10];
+        for (int j = 0; j < 10; j++) vj[j] = p->e[i]->e[j]->V();
+        double vri = 0;
+        for (int j = 0; j < 6 ; j++) vri += vj[j];
+        for (int j = 6; j < 10; j++) vri += vj[j]*0.5;
+        vri /= 8.;
+        err |= (fabs((vi[i] - vri)) > AVG * tol);
+      }
+      err |= (fabs((v - vr)) > AVG * tol);
+      if(err) {
+        p->visible = false;
+        for(int i = 0; i < 10; i++)
+          recurError(p->e[i], AVG, tol);
+      }
+      else
+        p->visible = true;
+    }
+  }
+}
+
 template <class T>
 adaptiveElements<T>::adaptiveElements(std::vector<fullMatrix<double>*> &p)
   : _coeffsVal(0), _eexpsVal(0), _interpolVal(0),
@@ -916,24 +1112,24 @@ void adaptiveElements<T>::init(int level)
   T::create(level);
   int numVals = _coeffsVal ? _coeffsVal->size1() : T::numNodes;
   int numNodes = _coeffsGeom ? _coeffsGeom->size1() : T::numNodes;
-  
+
   if(_interpolVal) delete _interpolVal;
   _interpolVal = new fullMatrix<double>(T::allVertices.size(), numVals);
-  
+
   if(_interpolGeom) delete _interpolGeom;
   _interpolGeom = new fullMatrix<double>(T::allVertices.size(), numNodes);
-  
+
   fullVector<double> sfv(numVals), *tmpv = 0;
   fullVector<double> sfg(numNodes), *tmpg = 0;
   if(_eexpsVal) tmpv = new fullVector<double>(_eexpsVal->size1());
   if(_eexpsGeom) tmpg = new fullVector<double>(_eexpsGeom->size1());
 
   int i = 0;
-  for(std::set<adaptiveVertex>::iterator it = T::allVertices.begin(); 
+  for(std::set<adaptiveVertex>::iterator it = T::allVertices.begin();
       it != T::allVertices.end(); ++it) {
 
     if(_coeffsVal && _eexpsVal)
-      computeShapeFunctions(_coeffsVal, _eexpsVal, 
+      computeShapeFunctions(_coeffsVal, _eexpsVal,
                             it->x, it->y, it->z, &sfv, tmpv);
     else
       T::GSF(it->x, it->y, it->z, sfv);
@@ -941,7 +1137,7 @@ void adaptiveElements<T>::init(int level)
       (*_interpolVal)(i, j) = sfv(j);
 
     if(_coeffsGeom && _eexpsGeom)
-      computeShapeFunctions(_coeffsGeom, _eexpsGeom, 
+      computeShapeFunctions(_coeffsGeom, _eexpsGeom,
                             it->x, it->y, it->z, &sfg, tmpg);
     else
       T::GSF(it->x, it->y, it->z, sfg);
@@ -960,11 +1156,66 @@ void adaptiveElements<T>::init(int level)
 #endif
 }
 
+template <>
+void adaptiveElements<adaptivePyramid>::init(int level)
+{
+#ifdef TIMER
+  double t1 = GetTimeInSeconds();
+#endif
+
+  adaptivePyramid::create(level);
+  int numVals  = _coeffsVal  ? _coeffsVal->size1()  : adaptivePyramid::numNodes;
+  int numNodes = _coeffsGeom ? _coeffsGeom->size1() : adaptivePyramid::numNodes;
+
+  if(_interpolVal) delete _interpolVal;
+  _interpolVal = new fullMatrix<double>(adaptivePyramid::allVertices.size(), numVals);
+
+  if(_interpolGeom) delete _interpolGeom;
+  _interpolGeom = new fullMatrix<double>(adaptivePyramid::allVertices.size(), numNodes);
+
+  fullVector<double> sfv(numVals), *tmpv = 0;
+  fullVector<double> sfg(numNodes), *tmpg = 0;
+  if(_eexpsVal) tmpv = new fullVector<double>(_eexpsVal->size1());
+  if(_eexpsGeom) tmpg = new fullVector<double>(_eexpsGeom->size1());
+
+  int i = 0;
+  for(std::set<adaptiveVertex>::iterator it = adaptivePyramid::allVertices.begin();
+      it != adaptivePyramid::allVertices.end(); ++it) {
+
+    if(_coeffsVal && _eexpsVal)
+      computeShapeFunctionsPyramid(_coeffsVal, _eexpsVal,
+                            it->x, it->y, it->z, &sfv, tmpv);
+    else
+      adaptivePyramid::GSF(it->x, it->y, it->z, sfv);
+
+    for(int j = 0; j < numVals; j++)
+      (*_interpolVal)(i, j) = sfv(j);
+
+    if(_coeffsGeom && _eexpsGeom)
+      computeShapeFunctionsPyramid(_coeffsGeom, _eexpsGeom,
+                            it->x, it->y, it->z, &sfg, tmpg);
+    else
+      adaptivePyramid::GSF(it->x, it->y, it->z, sfg);
+    for(int j = 0; j < numNodes; j++)
+      (*_interpolGeom)(i, j) = sfg(j);
+
+    i++;
+  }
+
+  if(tmpv) delete tmpv;
+  if(tmpg) delete tmpg;
+
+#ifdef TIMER
+  adaptiveData::timerInit += GetTimeInSeconds() - t1;
+  return;
+#endif
+}
+
 template <class T>
-void adaptiveElements<T>::adapt(double tol, int numComp, 
+void adaptiveElements<T>::adapt(double tol, int numComp,
                                 std::vector<PCoords> &coords,
-                                std::vector<PValues> &values, 
-                                double &minVal, double &maxVal, 
+                                std::vector<PValues> &values,
+                                double &minVal, double &maxVal,
                                 GMSH_PostPlugin *plug,
                                 bool onlyComputeMinMax)
 {
@@ -972,17 +1223,17 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
     Msg::Error("Can only adapt scalar or vector data");
     return;
   }
-  
+
   int numVertices = T::allVertices.size();
 
   if(!numVertices){
     Msg::Error("No adapted vertices to interpolate");
     return;
   }
-  
+
   int numVals = _coeffsVal ? _coeffsVal->size1() : T::numNodes;
   if(numVals != (int)values.size()){
-    Msg::Error("Wrong number of values in adaptation %d != %i", 
+    Msg::Error("Wrong number of values in adaptation %d != %i",
                numVals, values.size());
     return;
   }
@@ -990,7 +1241,7 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
 #ifdef TIMER
   double t1 = GetTimeInSeconds();
 #endif
-  
+
   fullVector<double> val(numVals), res(numVertices);
   if(numComp == 1){
     for(int i = 0; i < numVals; i++)
@@ -1003,14 +1254,14 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
   }
   _interpolVal->mult(val, res);
 
-  //minVal = VAL_INF;  
-  //maxVal = -VAL_INF;  
+  //minVal = VAL_INF;
+  //maxVal = -VAL_INF;
   for(int i = 0; i < numVertices; i++){
     minVal = std::min(minVal, res(i));
     maxVal = std::max(maxVal, res(i));
   }
   if(onlyComputeMinMax) return;
-  
+
   fullMatrix<double> *resxyz = 0;
   if(numComp == 3){
     fullMatrix<double> valxyz(numVals, 3);
@@ -1022,14 +1273,15 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
     }
     _interpolVal->mult(valxyz, *resxyz);
   }
-  
+
   int numNodes = _coeffsGeom ? _coeffsGeom->size1() : T::numNodes;
   if(numNodes != (int)coords.size()){
-    Msg::Error("Wrong number of nodes in adaptation %d != %i", 
+    Msg::Error("Wrong number of nodes in adaptation %d != %i",
                numNodes, coords.size());
+    if(resxyz) delete resxyz;
     return;
   }
-  
+
   fullMatrix<double> xyz(numNodes, 3), XYZ(numVertices, 3);
   for(int i = 0; i < numNodes; i++){
     xyz(i, 0) = coords[i].c[0];
@@ -1050,7 +1302,7 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
     adaptiveVertex *p = (adaptiveVertex*)&(*it);
     p->val = res(i);
     if(resxyz){
-      p->valx = (*resxyz)(i, 0);
+      p->val  = (*resxyz)(i, 0);
       p->valy = (*resxyz)(i, 1);
       p->valz = (*resxyz)(i, 2);
     }
@@ -1059,22 +1311,22 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
     p->Z = XYZ(i, 2);
     i++;
   }
-  
+
   if(resxyz) delete resxyz;
-  
-  for(typename std::list<T*>::iterator it = T::all.begin(); 
+
+  for(typename std::list<T*>::iterator it = T::all.begin();
       it != T::all.end(); it++)
     (*it)->visible = false;
-  
+
   if(!plug || tol != 0.){
     double avg = fabs(maxVal - minVal);
     if(tol < 0) avg = 1.; // force visibility to the smallest subdivision
     T::error(avg, tol);
   }
-  
+
   if(plug)
     plug->assignSpecificVisibility();
-  
+
   coords.clear();
   values.clear();
   for(typename std::list<T*>::iterator it = T::all.begin();
@@ -1086,30 +1338,30 @@ void adaptiveElements<T>::adapt(double tol, int numComp,
         if(numComp == 1)
           values.push_back(PValues(p[i]->val));
         else
-          values.push_back(PValues(p[i]->valx, p[i]->valy, p[i]->valz));
+          values.push_back(PValues(p[i]->val, p[i]->valy, p[i]->valz));
       }
     }
   }
 }
 
 template <class T>
-void adaptiveElements<T>::addInView(double tol, int step, 
-                                    PViewData *in, PViewDataList *out, 
+void adaptiveElements<T>::addInView(double tol, int step,
+                                    PViewData *in, PViewDataList *out,
                                     GMSH_PostPlugin *plug)
 {
   int numComp = in->getNumComponents(0, 0, 0);
   if(numComp != 1 && numComp != 3) return;
-  
+
   int numEle = 0, *outNb = 0;
   std::vector<double> *outList = 0;
   switch(T::numEdges){
-  case 0: 
-    numEle = in->getNumPoints(); 
+  case 0:
+    numEle = in->getNumPoints();
     outNb = (numComp == 1) ? &out->NbSP : &out->NbVP;
     outList = (numComp == 1) ? &out->SP : &out->VP;
     break;
-  case 1: 
-    numEle = in->getNumLines(); 
+  case 1:
+    numEle = in->getNumLines();
     outNb = (numComp == 1) ? &out->NbSL : &out->NbVL;
     outList = (numComp == 1) ? &out->SL : &out->VL;
     break;
@@ -1128,22 +1380,27 @@ void adaptiveElements<T>::addInView(double tol, int step,
     outNb = (numComp == 1) ? &out->NbSS : &out->NbVS;
     outList = (numComp == 1) ? &out->SS : &out->VS;
     break;
-  case 9: 
+  case 9:
     numEle = in->getNumPrisms();
     outNb = (numComp == 1) ? &out->NbSI : &out->NbVI;
     outList = (numComp == 1) ? &out->SI : &out->VI;
     break;
-  case 12: 
+  case 8:
+    numEle = in->getNumPyramids();
+    outNb = (numComp == 1) ? &out->NbSY : &out->NbVY;
+    outList = (numComp == 1) ? &out->SY : &out->VY;
+    break;
+  case 12:
     numEle = in->getNumHexahedra();
     outNb = (numComp == 1) ? &out->NbSH : &out->NbVH;
     outList = (numComp == 1) ? &out->SH : &out->VH;
     break;
   }
   if(!numEle) return;
-  
+
   outList->clear();
   *outNb = 0;
-  
+
   for(int ent = 0; ent < in->getNumEntities(step); ent++){
     for(int ele = 0; ele < in->getNumElements(step, ent); ele++){
       if(in->skipElement(step, ent, ele) ||
@@ -1167,20 +1424,20 @@ void adaptiveElements<T>::addInView(double tol, int step,
       else if(numComp == 3){
         for(int i = 0; i < numVal / 3; i++){
           double vx, vy, vz;
-          in->getValue(step, ent, ele, 3 * i, vx); 
-          in->getValue(step, ent, ele, 3 * i + 1, vy); 
-          in->getValue(step, ent, ele, 3 * i + 2, vz); 
+          in->getValue(step, ent, ele, 3 * i, vx);
+          in->getValue(step, ent, ele, 3 * i + 1, vy);
+          in->getValue(step, ent, ele, 3 * i + 2, vz);
           values.push_back(PValues(vx, vy, vz));
         }
       }
       adapt(tol, numComp, coords, values, out->Min, out->Max, plug);
       *outNb += coords.size() / T::numNodes;
       for(unsigned int i = 0; i < coords.size() / T::numNodes; i++){
-        for(int k = 0; k < T::numNodes; ++k) 
+        for(int k = 0; k < T::numNodes; ++k)
           outList->push_back(coords[T::numNodes * i + k].c[0]);
-        for(int k = 0; k < T::numNodes; ++k) 
+        for(int k = 0; k < T::numNodes; ++k)
           outList->push_back(coords[T::numNodes * i + k].c[1]);
-        for(int k = 0; k < T::numNodes; ++k) 
+        for(int k = 0; k < T::numNodes; ++k)
           outList->push_back(coords[T::numNodes * i + k].c[2]);
         for(int k = 0; k < T::numNodes; ++k)
           for(int l = 0; l < numComp; ++l)
@@ -1191,9 +1448,9 @@ void adaptiveElements<T>::addInView(double tol, int step,
 }
 
 adaptiveData::adaptiveData(PViewData *data)
-  : _step(-1), _level(-1), _tol(-1.), _inData(data), 
-    _points(0), _lines(0), _triangles(0), _quadrangles(0), 
-    _tetrahedra(0), _hexahedra(0), _prisms(0)
+  : _step(-1), _level(-1), _tol(-1.), _inData(data),
+    _points(0), _lines(0), _triangles(0), _quadrangles(0),
+    _tetrahedra(0), _hexahedra(0), _prisms(0),_pyramids(0)
 {
   _outData = new PViewDataList(true);
   _outData->setName(data->getName() + "_adapted");
@@ -1226,6 +1483,10 @@ adaptiveData::adaptiveData(PViewData *data)
     _inData->getInterpolationMatrices(TYPE_HEX, p);
     _hexahedra = new adaptiveElements<adaptiveHexahedron>(p);
   }
+  if(_inData->getNumPyramids()){
+    _inData->getInterpolationMatrices(TYPE_PYR, p);
+    _pyramids = new adaptiveElements<adaptivePyramid>(p);
+  }
 }
 
 adaptiveData::~adaptiveData()
@@ -1237,13 +1498,14 @@ adaptiveData::~adaptiveData()
   if(_tetrahedra) delete _tetrahedra;
   if(_prisms) delete _prisms;
   if(_hexahedra) delete _hexahedra;
+  if(_pyramids) delete _pyramids;
   delete _outData;
 }
 
 double adaptiveData::timerInit = 0.;
 double adaptiveData::timerAdapt = 0.;
 
-void adaptiveData::changeResolution(int step, int level, double tol, 
+void adaptiveData::changeResolution(int step, int level, double tol,
                                     GMSH_PostPlugin *plug)
 {
   timerInit = timerAdapt = 0.;
@@ -1256,6 +1518,7 @@ void adaptiveData::changeResolution(int step, int level, double tol,
     if(_tetrahedra) _tetrahedra->init(level);
     if(_prisms) _prisms->init(level);
     if(_hexahedra) _hexahedra->init(level);
+    if(_pyramids)  _pyramids->init(level);
   }
   if(plug || _step != step || _level != level || _tol != tol){
     _outData->setDirty(true);
@@ -1266,12 +1529,13 @@ void adaptiveData::changeResolution(int step, int level, double tol,
     if(_tetrahedra) _tetrahedra->addInView(tol, step, _inData, _outData, plug);
     if(_prisms) _prisms->addInView(tol, step, _inData, _outData, plug);
     if(_hexahedra) _hexahedra->addInView(tol, step, _inData, _outData, plug);
+    if(_pyramids) _pyramids->addInView(tol, step, _inData, _outData, plug);
     _outData->finalize();
   }
   _step = step;
   _level = level;
   _tol = tol;
-  
+
 #ifdef TIMER
   printf("init time = %g\n", timerInit);
   printf("adapt time = %g\n", timerAdapt);
