@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -912,7 +912,7 @@ public:
   }
   void getGradShapeFunction(int num, double u, double v, double w, double s[3])
   {
-    if(w == 1. && num != 4) {
+    if(w == 1.) {
       switch(num) {
       case 0 : s[0] = -0.25 ;
                s[1] = -0.25 ;
