@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -102,6 +102,9 @@ class STensor33 {
     return *this;
   }*/
   void print(const char *) const;
+
+  const double* data() const{return _val;}
+  double* data() {return _val;}
 };
 
 // tensor product
