@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -78,6 +78,11 @@ class STensor43 {
     for (int i = 0; i < 81; i++) _val[i] += other._val[i];
     return *this;
   }
+  STensor43& operator -= (const STensor43 &other)
+  {
+    for (int i = 0; i < 81; i++) _val[i] -= other._val[i];
+    return *this;
+  }
   STensor43& operator *= (const double &other)
   {
     for (int i = 0; i < 81; i++) _val[i] *= other;
@@ -148,6 +153,8 @@ class STensor43 {
     return *this;
   }*/
   void print(const char *) const;
+  const double* data() const {return _val;}
+  double* data() {return _val;}
 };
 
 // tensor product
