@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -69,6 +69,9 @@ class FunctionSpaceBase
   virtual ~FunctionSpaceBase(){}
   virtual int getNumKeys(MElement *ele) = 0; // if one needs the number of dofs
   virtual void getKeys(MElement *ele, std::vector<Dof> &keys) = 0;
+  virtual void getKeysOnVertex(MElement* ele, MVertex* v, const std::vector<int>& comp, std::vector<Dof>& keys){
+    Msg::Warning("this function is defined to get Dofs of vertex %d on element %d",v->getNum(),ele->getNum());
+  }
   virtual FunctionSpaceBase* clone(const std::vector<int>& comp) const {return NULL;}
 };
 
@@ -115,7 +118,8 @@ class ScalarLagrangeFunctionSpaceOfElement : public FunctionSpace<double>
   virtual void f(MElement *ele, double u, double v, double w, std::vector<ValType> &vals)
   {
     if(ele->getParent()) {
-      if(ele->getTypeForMSH() == MSH_LIN_B || ele->getTypeForMSH() == MSH_TRI_B) { //FIXME MPolygonBorders...
+      if(ele->getTypeForMSH() == MSH_LIN_B || ele->getTypeForMSH() == MSH_TRI_B ||
+         ele->getTypeForMSH() == MSH_POLYG_B) { //FIXME MPolygonBorders...
         ele->movePointFromParentSpaceToElementSpace(u, v, w);
       }
     }
@@ -128,7 +132,8 @@ class ScalarLagrangeFunctionSpaceOfElement : public FunctionSpace<double>
   virtual void gradf(MElement *ele, double u, double v, double w, std::vector<GradType> &grads)
   {
     if(ele->getParent()) {
-      if(ele->getTypeForMSH() == MSH_LIN_B || ele->getTypeForMSH() == MSH_TRI_B) { //FIXME MPolygonBorders...
+      if(ele->getTypeForMSH() == MSH_LIN_B || ele->getTypeForMSH() == MSH_TRI_B ||
+         ele->getTypeForMSH() == MSH_POLYG_B) { //FIXME MPolygonBorders...
         ele->movePointFromParentSpaceToElementSpace(u, v, w);
       }
     }
@@ -150,7 +155,8 @@ class ScalarLagrangeFunctionSpaceOfElement : public FunctionSpace<double>
   virtual void hessfuvw(MElement *ele, double u, double v, double w, std::vector<HessType> &hess)
   {
     if(ele->getParent()) {
-      if(ele->getTypeForMSH() == MSH_LIN_B || ele->getTypeForMSH() == MSH_TRI_B) { //FIXME MPolygonBorders...
+      if(ele->getTypeForMSH() == MSH_LIN_B || ele->getTypeForMSH() == MSH_TRI_B ||
+         ele->getTypeForMSH() == MSH_POLYG_B) { //FIXME MPolygonBorders...
         ele->movePointFromParentSpaceToElementSpace(u, v, w);
       }
     }
@@ -169,7 +175,8 @@ class ScalarLagrangeFunctionSpaceOfElement : public FunctionSpace<double>
   virtual void gradfuvw(MElement *ele, double u, double v, double w, std::vector<GradType> &grads)
   {
     if(ele->getParent()) {
-      if(ele->getTypeForMSH() == MSH_LIN_B || ele->getTypeForMSH() == MSH_TRI_B) { //FIXME MPolygonBorders...
+      if(ele->getTypeForMSH() == MSH_LIN_B || ele->getTypeForMSH() == MSH_TRI_B ||
+         ele->getTypeForMSH() == MSH_POLYG_B) { //FIXME MPolygonBorders...
         ele->movePointFromParentSpaceToElementSpace(u, v, w);
       }
     }
