@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -24,6 +24,9 @@ class linearSystemBase {
   virtual void zeroRightHandSide() = 0;
   virtual void zeroSolution() = 0;
   virtual int systemSolve() = 0;
+  // x = A*b
+  virtual int matMult() { return 0; }
+
   void setParameter (std::string key, std::string value);
   std::string getParameter(std::string key) const;
   virtual void insertInSparsityPattern(int _row, int _col){};
