@@ -27,6 +27,8 @@
 //
 // Contributors: Thomas Toulorge, Jonathan Lambrechts
 
+static int NEVAL = 0;
+
 #include <iostream>
 #include <algorithm>
 #include "OptHomMesh.h"
@@ -34,6 +36,11 @@
 #include "OptHomRun.h"
 #include "GmshMessage.h"
 #include "GmshConfig.h"
+#include "OptHomCADDist.h"
+#include "MElement.h"
+#include "MLine.h"
+#include "MTriangle.h"
+#include "GModel.h"
 
 #if defined(HAVE_BFGS)
 
@@ -106,22 +113,16 @@ bool OptHOM::addJacObjGrad(double &Obj, alglib::real_1d_array &gradObj)
   return true;
 }
 
-bool OptHOM::addBndObjGrad(double factor, double &Obj, alglib::real_1d_array &gradObj)
-{
-  maxDistCAD = 0.0;
 
+bool OptHOM::addApproximationErrorObjGrad(double factor, double &Obj, alglib::real_1d_array &gradObj, simpleFunction<double>& fct)
+{
   std::vector<double> gradF;
-  double DISTANCE = 0.0;
   for (int iEl = 0; iEl < mesh.nEl(); iEl++) {
     double f;
-    if (mesh.bndDistAndGradients(iEl, f, gradF, geomTol)) {
-      maxDistCAD = std::max(maxDistCAD,f);
-      DISTANCE += f;
-      Obj += f * factor;
-      for (size_t i = 0; i < mesh.nPCEl(iEl); ++i){
-        gradObj[mesh.indPCEl(iEl, i)] += gradF[i] * factor;
-	//	printf("gradf[%d] = %12.5E\n",i,gradF[i]*factor);
-      }
+    mesh.approximationErrorAndGradients(iEl, f, gradF, 1.e-6, fct);
+    Obj += f * factor;
+    for (size_t i = 0; i < mesh.nPCEl(iEl); ++i){
+      gradObj[mesh.indPCEl(iEl, i)] += gradF[i] * factor;
     }
   }
   //  printf("DIST = %12.5E\n",DISTANCE);
@@ -129,6 +130,233 @@ bool OptHOM::addBndObjGrad(double factor, double &Obj, alglib::real_1d_array &gr
 
 }
 
+static void computeGradSFAtNodes (MElement *e, std::vector<std::vector<SVector3> > &gsf)
+{
+  const nodalBasis &elbasis = *e->getFunctionSpace();
+  double s[256][3];
+  for (int j=0;j<e->getNumVertices();j++){
+    std::vector<SVector3> g(e->getNumVertices());
+    double u_mesh = elbasis.points(j,0);
+    double v_mesh = elbasis.points(j,1);
+    e->getGradShapeFunctions ( u_mesh , v_mesh , 0,  s);
+    for (int k=0;k<e->getNumVertices();k++)
+      g[k] = SVector3(s[k][0],s[k][1],s[k][2]);
+    gsf.push_back(g);
+  }
+}
+
+
+bool OptHOM::addBndObjGrad(double factor, double &Obj, alglib::real_1d_array &gradObj)
+{
+  // set the mesh to its present position
+  std::vector<SPoint3> xyz,uvw;
+  mesh.getGEntityPositions(xyz,uvw);
+  mesh.updateGEntityPositions();
+
+  //could be better (e.g. store the model in the Mesh:: datastrucure)
+
+  GModel *gm = GModel::current();
+
+  // for all model edges, compute the error between the geometry and the mesh
+
+  maxDistCAD = 0.0;
+  double distCAD = 0.0;
+
+  for (GModel::eiter it = gm->firstEdge(); it != gm->lastEdge(); ++it){
+    // do not do straight lines
+    if ((*it)->geomType() == GEntity::Line)continue;
+    // look at all mesh lines
+
+    std::vector<bool> doWeCompute((*it)->lines.size());
+    for (unsigned int i=0;i<(*it)->lines.size(); i++){
+      doWeCompute[i] = false;
+      for (unsigned int j=0;j<(*it)->lines[i]->getNumVertices(); j++){
+        int index = mesh.getFreeVertexStartIndex((*it)->lines[i]->getVertex(j));
+        if (index >=0){
+          doWeCompute[i] = true;
+          continue;
+        }
+      }
+    }
+
+    std::vector<double> dist((*it)->lines.size());
+    for (unsigned int i=0;i<(*it)->lines.size(); i++){
+      if (doWeCompute[i]){
+        // compute the distance from the geometry to the mesh
+        dist[i] = MLineGEdgeDistance ( (*it)->lines[i] , *it );
+        maxDistCAD = std::max(maxDistCAD,dist[i]);
+        distCAD += dist [i] * factor;
+      }
+    }
+    // be clever to compute the derivative : iterate on all
+    // Distance = \sum_{lines} Distance (line, GEdge)
+    // For a high order vertex, we compute the derivative only by
+    // recomputing the distance to one only line
+    const double eps = 1.e-6;
+    for (unsigned int i=0;i<(*it)->lines.size(); i++){
+      if (doWeCompute[i]){
+        for (int j=2 ; j<(*it)->lines[i]->getNumVertices()  ; j++){
+          MVertex *v = (*it)->lines[i]->getVertex(j);
+          int index = mesh.getFreeVertexStartIndex(v);
+          //	printf("%d %d (%d %d)\n",v->getNum(),index,v->onWhat()->tag(),v->onWhat()->dim());
+          if (index >= 0){
+            double t;
+            v->getParameter(0,t);
+            SPoint3 pp (v->x(),v->y(),v->z());
+            GPoint gp = (*it)->point(t+eps);
+            v->setParameter(0,t+eps);
+            v->setXYZ(gp.x(),gp.y(),gp.z());
+            double dist2 = MLineGEdgeDistance ( (*it)->lines[i] , *it );
+            double deriv = (dist2 - dist[i])/eps;
+            v->setXYZ(pp.x(),pp.y(),pp.z());
+            v->setParameter(0,t);
+            //	  printf("%g %g %g\n",dist[i],dist2, MLineGEdgeDistance ( (*it)->lines[i] , *it ));
+            // get the index of the vertex
+            gradObj[index] += deriv * factor;
+          }
+        }
+      }
+      //    printf("done\n");
+      // For a low order vertex classified on the GEdge, we recompute
+    // two distances for the two MLines connected to the vertex
+      for (unsigned int i=0;i<(*it)->lines.size()-1; i++){
+        MVertex *v =  (*it)->lines[i]->getVertex(1);
+        int index = mesh.getFreeVertexStartIndex(v);
+        if (index >= 0){
+          double t;
+          v->getParameter(0,t);
+          SPoint3 pp (v->x(),v->y(),v->z());
+          GPoint gp = (*it)->point(t+eps);
+          v->setParameter(0,t+eps);
+          v->setXYZ(gp.x(),gp.y(),gp.z());
+          MLine *l1 = (*it)->lines[i];
+          MLine *l2 = (*it)->lines[i+1];
+          //	printf("%d %d -- %d %d\n",l1->getVertex(0)->getNum(),l1->getVertex(1)->getNum(),l2->getVertex(0)->getNum(),l2->getVertex(1)->getNum());
+          double deriv =
+            (MLineGEdgeDistance ( l1 , *it ) - dist[i])  /eps +
+            (MLineGEdgeDistance ( l2 , *it ) - dist[i+1])/eps;
+          v->setXYZ(pp.x(),pp.y(),pp.z());
+          v->setParameter(0,t);
+          gradObj[index] += deriv * factor;
+        }
+      }
+    }
+  }
+  //  printf("computing distance : 1D part %12.5E\n",distCAD);
+
+  // now the 3D part !
+
+  std::vector<std::vector<SVector3> > gsfT;
+  computeGradSFAtNodes ( (*gm->firstFace())->triangles[0],gsfT);
+
+  std::map<MVertex*,SVector3> normalsToCAD;
+
+
+  for(GModel::fiter it = gm->firstFace(); it != gm->lastFace(); ++it){
+    // do not do plane surfaces
+    if ((*it)->geomType() == GEntity::Plane)continue;
+    std::map<MTriangle*,double> dist;
+
+    std::vector<bool> doWeCompute((*it)->triangles.size());
+    for (unsigned int i=0;i<(*it)->triangles.size(); i++){
+      doWeCompute[i] = false;
+      for (unsigned int j=0;j<(*it)->triangles[i]->getNumVertices(); j++){
+        int index = mesh.getFreeVertexStartIndex((*it)->triangles[i]->getVertex(j));
+        if (index >=0){
+          doWeCompute[i] = true;
+        }
+      }
+      if (doWeCompute[i]){
+        for (unsigned int j=0;j<(*it)->triangles[i]->getNumVertices(); j++){
+          MVertex *v = (*it)->triangles[i]->getVertex(j);
+          if (normalsToCAD.find(v) == normalsToCAD.end()){
+            SPoint2 p_cad;
+            reparamMeshVertexOnFace(v, *it, p_cad);
+            SVector3 tg_cad = (*it)->normal(p_cad);
+            tg_cad.normalize();
+            normalsToCAD[v] = tg_cad;
+          }
+        }
+      }
+    }
+
+    for (unsigned int i=0;i<(*it)->triangles.size(); i++){
+      // compute the distance from the geometry to the mesh
+      if(doWeCompute[i]){
+        const double d = MFaceGFaceDistanceOld((*it)->triangles[i], *it, &gsfT, &normalsToCAD);
+        dist[(*it)->triangles[i]] = d;
+        maxDistCAD = std::max(maxDistCAD,d);
+        distCAD += d * factor;
+      }
+    }
+
+    // be clever again to compute the derivatives
+    const double eps = 1.e-6;
+    for (unsigned int i=0;i<(*it)->triangles.size(); i++){
+      if(doWeCompute[i]){
+        for (unsigned int j=0;j<(*it)->triangles[i]->getNumVertices(); j++){
+          //    for (; itm !=v2t.end(); ++itm){
+          MVertex   *v = (*it)->triangles[i]->getVertex(j);
+          if(v->onWhat()->dim() == 1){
+            int index = mesh.getFreeVertexStartIndex(v);
+            if (index >= 0){
+              MTriangle *t = (*it)->triangles[i];
+              GEdge *ge = v->onWhat()->cast2Edge();
+              double t_;
+              v->getParameter(0,t_);
+              SPoint3 pp (v->x(),v->y(),v->z());
+              GPoint gp = ge->point(t_+eps);
+              v->setParameter(0,t_+eps);
+              v->setXYZ(gp.x(),gp.y(),gp.z());
+              const double distT = dist[t];
+              double deriv =  (MFaceGFaceDistanceOld(t, *it, &gsfT, &normalsToCAD) - distT) /eps;
+              v->setXYZ(pp.x(),pp.y(),pp.z());
+              v->setParameter(0,t_);
+              gradObj[index] += deriv * factor;
+            }
+          }
+
+          if(v->onWhat() == *it){
+            int index = mesh.getFreeVertexStartIndex(v);
+            if (index >= 0){
+              MTriangle *t = (*it)->triangles[i];
+              double uu,vv;
+              v->getParameter(0,uu);
+              v->getParameter(1,vv);
+              SPoint3 pp (v->x(),v->y(),v->z());
+
+              const double distT = dist[t];
+
+              GPoint gp = (*it)->point(uu+eps,vv);
+              v->setParameter(0,uu+eps);
+              v->setXYZ(gp.x(),gp.y(),gp.z());
+              double deriv = (MFaceGFaceDistanceOld(t, *it, &gsfT, &normalsToCAD) - distT) /eps;
+              v->setXYZ(pp.x(),pp.y(),pp.z());
+              v->setParameter(0,uu);
+              gradObj[index] += deriv * factor;
+
+              gp = (*it)->point(uu,vv+eps);
+              v->setParameter(1,vv+eps);
+              v->setXYZ(gp.x(),gp.y(),gp.z());
+              deriv = (MFaceGFaceDistanceOld(t, *it, &gsfT, &normalsToCAD) - distT) /eps;
+              v->setXYZ(pp.x(),pp.y(),pp.z());
+              v->setParameter(1,vv);
+              gradObj[index+1] += deriv * factor;
+            }
+          }
+        }
+      }
+    }
+  }
+  mesh.updateGEntityPositions(xyz,uvw);
+  Obj +=distCAD;
+  //  printf("computing distance : 2D part %12.5E\n",distCAD);
+  //  printf("%22.15E\n",distCAD);
+  return true;
+}
+
+
+
 bool OptHOM::addMetricMinObjGrad(double &Obj, alglib::real_1d_array &gradObj)
 {
 
@@ -156,7 +384,7 @@ bool OptHOM::addMetricMinObjGrad(double &Obj, alglib::real_1d_array &gradObj)
 
 // Contribution of the vertex distance to the objective function value and
 // gradients
-bool OptHOM::addDistObjGrad(double Fact, double Fact2, double &Obj,
+bool OptHOM::addDistObjGrad(double Fact, double &Obj,
                             alglib::real_1d_array &gradObj)
 {
   maxDist = 0;
@@ -164,37 +392,60 @@ bool OptHOM::addDistObjGrad(double Fact, double Fact2, double &Obj,
   int nbBnd = 0;
 
   for (int iFV = 0; iFV < mesh.nFV(); iFV++) {
-    const double Factor = invLengthScaleSq*(mesh.forced(iFV) ? Fact : Fact2);
     const double dSq = mesh.distSq(iFV), dist = sqrt(dSq);
-    Obj += Factor * dSq;
+    Obj += Fact * dSq;
     std::vector<double> gDSq(mesh.nPCFV(iFV));
     mesh.gradDistSq(iFV,gDSq);
     for (int iPC = 0; iPC < mesh.nPCFV(iFV); iPC++)
-      gradObj[mesh.indPCFV(iFV,iPC)] += Factor*gDSq[iPC];
+      gradObj[mesh.indPCFV(iFV,iPC)] += Fact*gDSq[iPC];
     maxDist = std::max(maxDist, dist);
     avgDist += dist;
     nbBnd++;
   }
   if (nbBnd != 0) avgDist /= nbBnd;
 
+  //  printf("DISTANCE %22.15E\n",avgDist);
+
   return true;
 }
 
+// FIXME TEST
+// Assume a unit square centered on 0,0
+// fct is
+//class toto : public simpleFunction<double>
+//{
+//public :
+//  double operator () (double x, double y, double z) const{
+//    const double r = sqrt(x*x+y*y);
+//    const double r0 = .3;
+//  const double f = atan(20*(x));
+//    return f;
+//  }
+//};
+
+
 void OptHOM::evalObjGrad(const alglib::real_1d_array &x, double &Obj,
                          alglib::real_1d_array &gradObj)
 {
+  NEVAL++;
   mesh.updateMesh(x.getcontent());
 
   Obj = 0.;
   for (int i = 0; i < gradObj.length(); i++) gradObj[i] = 0.;
 
+  //  printf("Computing Obj : ");
+  /// control Jacobians
   addJacObjGrad(Obj, gradObj);
-  addDistObjGrad(lambda, lambda2, Obj, gradObj);
+  /// Control distance to the straight sided mesh
+  addDistObjGrad(lambda, Obj, gradObj);
+
   if(_optimizeMetricMin)
     addMetricMinObjGrad(Obj, gradObj);
   if(_optimizeCAD)
     addBndObjGrad(lambda3, Obj, gradObj);
-  //  printf("maxDistCAD = %12.5E distMax = %12.5E Obj %12.5E\n",maxDistCAD,distance_max,Obj);
+
+  //  printf("Obj = %12.5E\n",Obj);
+
   if ((minJac > barrier_min) && (maxJac < barrier_max || !_optimizeBarrierMax) && (maxDistCAD < distance_max|| !_optimizeCAD) ) {
     Msg::Info("Reached %s (%g %g) requirements, setting null gradient",
               _optimizeMetricMin ? "svd" : "jacobian", minJac, maxJac);
@@ -209,6 +460,7 @@ void evalObjGradFunc(const alglib::real_1d_array &x, double &Obj,
   (static_cast<OptHOM*>(HOInst))->evalObjGrad(x, Obj, gradObj);
 }
 
+
 void OptHOM::recalcJacDist()
 {
   maxDist = 0;
@@ -269,37 +521,17 @@ void OptHOM::calcScale(alglib::real_1d_array &scale)
     for (int iPC = 0; iPC < mesh.nPCFV(iFV); iPC++)
       scale[mesh.indPCFV(iFV,iPC)] = scaleFV[iPC];
   }
-
-//  std::vector<double> inSize(mesh.nEl());
-//  mesh.elInSize(inSize);
-//  for (int iEl = 0; iEl < mesh.nEl(); iEl++) {
-////    std::cout << "DBGTT: inSize[" << iEl << "] = " << inSize[iEl] << std::endl;
-//    for (int iPCEl = 0; iPCEl < mesh.nPCEl(iEl); iPCEl++)
-//      scale[mesh.indPCEl(iEl,iPCEl)] *= inSize[iEl];
-//  }
-
-//  for (int iEl = 0; iEl < mesh.nEl(); iEl++) {
-//    std::vector<double> sJ(mesh.nBezEl(iEl)), gSJ(mesh.nBezEl(iEl)*mesh.nPCEl(iEl));
-//    mesh.scaledJacAndGradients(iEl,sJ,gSJ);
-//    for (int iPC = 0; iPC < mesh.nPCEl(iEl); iPC++) {
-//      double grad = 0.;
-//      for (int l = 0; l < mesh.nBezEl(iEl); l++) grad = std::max(grad,fabs(gSJ[mesh.indGSJ(iEl,l,iPC)]));
-//      scale[mesh.indPCEl(iEl,iPC)] *= mesh.nBezEl(iEl)/grad;
-////      std::cout << "DBGTT: scale[" << mesh.indPCEl(iEl,iPC) << "] = " << scale[mesh.indPCEl(iEl,iPC)] << std::endl;
-//    }
-//  }
-
 }
 
-void OptHOM::OptimPass(alglib::real_1d_array &x,
-                       const alglib::real_1d_array &initGradObj, int itMax)
+
+void OptHOM::OptimPass(alglib::real_1d_array &x, int itMax)
 {
 
   static const double EPSG = 0.;
   static const double EPSF = 0.;
   static const double EPSX = 0.;
   static int OPTMETHOD = 1;
-  
+
   Msg::Info("--- Optimization pass with initial jac. range (%g, %g), jacBar = %g",
             minJac, maxJac, jacBar);
 
@@ -359,7 +591,8 @@ void OptHOM::OptimPass(alglib::real_1d_array &x,
   }
 }
 
-int OptHOM::optimize(double weightFixed, double weightFree, double weightCAD, double b_min,
+
+int OptHOM::optimize(double weight, double weightCAD, double b_min,
                      double b_max, bool optimizeMetricMin, int pInt,
                      int itMax, int optPassMax, int optCAD, double distanceMax, double tolerance)
 {
@@ -373,8 +606,7 @@ int OptHOM::optimize(double weightFixed, double weightFree, double weightCAD, do
   _optimizeMetricMin = optimizeMetricMin;
   _optimizeCAD = optCAD;
   // Set weights & length scale for non-dimensionalization
-  lambda = weightFixed;
-  lambda2 = weightFree;
+  lambda = weight;
   lambda3 = weightCAD;
   geomTol = tolerance;
   std::vector<double> dSq(mesh.nEl());
@@ -415,12 +647,15 @@ int OptHOM::optimize(double weightFixed, double weightFree, double weightCAD, do
 
   int ITER = 0;
   bool minJacOK = true;
+
   while (minJac < barrier_min || (maxDistCAD > distance_max && _optimizeCAD)) {
     const double startMinJac = minJac;
-    OptimPass(x, gradObj, itMax);
+    NEVAL = 0;
+    OptimPass(x, itMax);
+    printf("######  NEVAL = %d\n",NEVAL);
     recalcJacDist();
     jacBar = (minJac > 0.) ? 0.9*minJac : 1.1*minJac;
-    if (_optimizeCAD)   jacBar = std::min(jacBar,barrier_min); 
+    if (_optimizeCAD)   jacBar = std::min(jacBar,barrier_min);
     if (ITER ++ > optPassMax) {
       minJacOK = (minJac > barrier_min && (maxDistCAD < distance_max || !_optimizeCAD));
       break;
@@ -438,7 +673,7 @@ int OptHOM::optimize(double weightFixed, double weightFree, double weightCAD, do
     jacBar =  1.1 * maxJac;
     while (maxJac > barrier_max ) {
       const double startMaxJac = maxJac;
-      OptimPass(x, gradObj, itMax);
+      OptimPass(x, itMax);
       recalcJacDist();
       jacBar =  1.1 * maxJac;
       if (ITER ++ > optPassMax) break;
