@@ -34,6 +34,7 @@
 #include <math.h>
 #include "GmshConfig.h"
 #include "OptHomMesh.h"
+#include "simpleFunction.h"
 
 #if defined(HAVE_BFGS)
 
@@ -50,8 +51,9 @@ public:
   // are in the range; returns 0 if the mesh is valid (all jacobians positive,
   // JMIN > 0) but JMIN < barrier_min || JMAX > barrier_max; returns -1 if the
   // mesh is invalid : some jacobians cannot be made positive
-  int optimize(double lambda, double lambda2, double lambda3, double barrier_min, double barrier_max,
-               bool optimizeMetricMin, int pInt, int itMax, int optPassMax, int optimizeCAD, double optCADDistMax, double tolerance);
+  int optimize(double lambda, double lambda3, double barrier_min, double barrier_max,
+               bool optimizeMetricMin, int pInt, int itMax, int optPassMax,
+               int optimizeCAD, double optCADDistMax, double tolerance);
   void recalcJacDist();
   inline void getJacDist(double &minJ, double &maxJ, double &maxD, double &avgD);
   void updateMesh(const alglib::real_1d_array &x);
@@ -60,9 +62,8 @@ public:
   void printProgress(const alglib::real_1d_array &x, double Obj);
 
   double barrier_min, barrier_max, distance_max, geomTol;
-
  private:
-  double lambda, lambda2, lambda3, jacBar, invLengthScaleSq;
+  double lambda, lambda3, jacBar, invLengthScaleSq;
   int iter, progressInterv; // Current iteration, interval of iterations for reporting
   bool _optimizeMetricMin;
   double initObj, initMaxDist, initAvgDist; // Values for reporting
@@ -71,14 +72,13 @@ public:
                             // true : fixed barrier min + moving barrier max
   bool _optimizeCAD; // false : do not minimize the distance between mesh and CAD
                      // true : minimize the distance between mesh and CAD
+  bool addApproximationErrorObjGrad(double Fact, double &Obj, alglib::real_1d_array &gradObj, simpleFunction<double>& fct);
   bool addJacObjGrad(double &Obj, alglib::real_1d_array &gradObj);
-  bool addBndObjGrad(double Fact, double &Obj, alglib::real_1d_array &gradObj);
+  bool addBndObjGrad (double Fact, double &Obj, alglib::real_1d_array &gradObj);
   bool addMetricMinObjGrad(double &Obj, alglib::real_1d_array &gradObj);
-  bool addDistObjGrad(double Fact, double Fact2, double &Obj,
-                      alglib::real_1d_array &gradObj);
+  bool addDistObjGrad(double Fact, double &Obj, alglib::real_1d_array &gradObj);
   void calcScale(alglib::real_1d_array &scale);
-  void OptimPass(alglib::real_1d_array &x, const alglib::real_1d_array &initGradObj,
-                 int itMax);
+  void OptimPass(alglib::real_1d_array &x, int itMax);
 };
 
 void OptHOM::getJacDist(double &minJ, double &maxJ, double &maxD, double &avgD)
@@ -86,6 +86,10 @@ void OptHOM::getJacDist(double &minJ, double &maxJ, double &maxD, double &avgD)
   minJ = minJac; maxJ = maxJac; maxD = maxDist; avgD = avgDist;
 }
 
+double distanceToGeometry(GModel *gm);
+void distanceFromElementsToGeometry(GModel *gm, int dim, std::map<MElement*,double> &distances);
+
+
 #endif
 
 #endif
