@@ -35,10 +35,12 @@ public class ParameterNumber extends Parameter {
     private EditText _edittext;
     private Stepper _stepper;
 
-    public ParameterNumber(Context context, Gmsh gmsh, String name){
+    public ParameterNumber(Context context, Gmsh gmsh, String name)
+    {
         super(context, gmsh, name);
     }
-    public ParameterNumber(Context context, Gmsh gmsh, String name,  double value, double min, double max, double step)
+    public ParameterNumber(Context context, Gmsh gmsh, String name,
+                           double value, double min, double max, double step)
     {
         this(context, gmsh, name);
         _value = value;
@@ -46,7 +48,8 @@ public class ParameterNumber extends Parameter {
         _max = max;
         _step = step;
     }
-    public ParameterNumber(Context context, Gmsh gmsh, String name, boolean readOnly, double value, double min, double max, double step)
+    public ParameterNumber(Context context, Gmsh gmsh, String name, boolean readOnly,
+                           double value, double min, double max, double step)
     {
         this(context, gmsh, name, value, min, max, step);
         _readOnly = readOnly;
@@ -61,13 +64,13 @@ public class ParameterNumber extends Parameter {
     {
         super.update();
         if(_bar != null) {
-            _title.setText(getShortName() + " (" + formatDouble(_value)+ ")");
+            _title.setText(getShortName() + " (" + formatDouble(_value) + ")");
             _bar.setMax(100);
             _bar.setProgress((int)(100*(_value-_min)/(_max-_min)));
             _bar.setEnabled(!this.isReadOnly());
         }
         else if(_spinner != null) {
-            for(int i=0;i<_choices.size();i++)
+            for(int i = 0; i < _choices.size(); i++)
                 if(_values.get(i) == _value)
                     _spinner.setSelection(i, true);
         }
@@ -76,7 +79,7 @@ public class ParameterNumber extends Parameter {
             _checkbox.setChecked((_value == 0)? false : true);
         }
         else if(_edittext != null) {
-            _edittext.setText(""+formatDouble(_value));
+            _edittext.setText("" + formatDouble(_value));
         }
         else if(_stepper != null) {
             _stepper.setMaximum((int)Math.round(_max));
@@ -86,19 +89,15 @@ public class ParameterNumber extends Parameter {
     }
     public void setValue(double value)
     {
-        if(value < _min || value > _max) {
-            //Log.w("ParameterNumber", "Incorect value "+value+" (max="+_max+" min="+_min+")");
-            return;
-        }
         if(value == _value) return;
         _value = value;
         _changed = true;
         _gmsh.setParam(getType(), getName(), String.valueOf(value));
         if(mListener != null) mListener.OnParameterChanged();
     }
-    public void setMin(double min) {_min = min;this.update();}
-    public void setMax(double max) {_max = max;this.update();}
-    public void setStep(double step) {_step = step;this.update();}
+    public void setMin(double min) { _min = min; this.update(); }
+    public void setMax(double max) { _max = max; this.update(); }
+    public void setStep(double step) { _step = step;this.update(); }
     public void addChoice(double choice, String value)
     {
         if(_values == null) {
@@ -108,7 +107,9 @@ public class ParameterNumber extends Parameter {
             _choices.add(value);
             if(_spinner == null) {
                 _spinner = new Spinner(_context);
-                _adapter = new ArrayAdapter<String>(_context, android.R.layout.simple_spinner_dropdown_item, _choices);
+                _adapter = new ArrayAdapter<String>(_context,
+                                                    android.R.layout.simple_spinner_dropdown_item,
+                                                    _choices);
                 _adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
                 _spinner.setAdapter(_adapter);
             }
@@ -119,20 +120,19 @@ public class ParameterNumber extends Parameter {
                     _choices.set(i, value);
                     return;
                 }
-                else if(_values.get(i).equals(choice))
-                    {
-                        _choices.add(value);
-                        return;
-                    }
+                else if(_values.get(i).equals(choice)){
+                    _choices.add(value);
+                    return;
+                }
             }
             _values.add(choice);
             _choices.add(value);
         }
     }
-    public double getValue() {return _value;}
-    public double getMax() {return _max;}
-    public double getMin() {return _min;}
-    public double getStep() {return _step;}
+    public double getValue() { return _value; }
+    public double getMax() { return _max; }
+    public double getMin() { return _min; }
+    public double getStep() { return _step; }
     public int fromString(String s)
     {
         int pos = super.fromString(s);
@@ -149,7 +149,7 @@ public class ParameterNumber extends Parameter {
         pos++;// index
         int nChoix = Integer.parseInt(infos[pos++]); // choices' size
         double choices[] = new double[nChoix];
-        for(int i=0; i<nChoix; i++)
+        for(int i = 0; i < nChoix; i++)
             choices[i] = Double.parseDouble(infos[pos++]); // choice
         int nLabels = Integer.parseInt(infos[pos++]); // labels' size
         if(nChoix == 2 && choices[0] == 0 && choices[1] == 1 && nLabels == 0) {
@@ -159,7 +159,7 @@ public class ParameterNumber extends Parameter {
         }
         if(_choices != null)_choices.clear();
         if(_values != null) _values.clear();
-        for(int i=0; i<nLabels && nChoix == nLabels; i++){
+        for(int i = 0; i < nLabels && nChoix == nLabels; i++){
             double val = Double.parseDouble(infos[pos++]); // choice
             this.addChoice(val, infos[pos++]); // label
         }
@@ -181,43 +181,48 @@ public class ParameterNumber extends Parameter {
         paramLayout.addView(_title);
         if(!_readOnly) paramLayout.setOnLongClickListener(new View.OnLongClickListener(){
                 @Override
-                public boolean onLongClick(View v){
-			AlertDialog.Builder builder = new AlertDialog.Builder(_context);
-			LinearLayout layout = new LinearLayout(_context);
-			layout.setOrientation(LinearLayout.VERTICAL);
-			TextView label = new TextView(_context);
-			label.setText("Edit value of \n" + _name);
-			EditText edit = new EditText(_context);
-			edit.setText(String.valueOf(_value));
-			edit.addTextChangedListener(new TextWatcher() {
-				public void onTextChanged(CharSequence s, int start, int before, int count) {
-					try {
-						if(s.length() < 1)  _tmpValue = 1;
-						_tmpValue = Double.parseDouble(s.toString());
-					} catch(NumberFormatException e) {
-	                                	_tmpValue = 1;
-					}
-				}
-		                    public void beforeTextChanged(CharSequence s, int start, int count, int after) {} // UNUSED Auto-generated method stub
-		                    public void afterTextChanged(Editable s) {} // UNUSED Auto-generated method stub
-		        	});
-			edit.requestFocus();
-			//_context.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
-			layout.addView(label);
-			layout.addView(edit);
-			builder.setView(layout)
-				.setPositiveButton("OK", new DialogInterface.OnClickListener() {
-					public void onClick(DialogInterface dialog, int id) {
-						setValue(_tmpValue);
-					}
-				})
-				.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
-					public void onClick(DialogInterface dialog, int id) {
-						// User cancelled the dialog
-					}
-				});
-			builder.create().show();
-                	return true;
+                public boolean onLongClick(View v)
+                {
+                    AlertDialog.Builder builder = new AlertDialog.Builder(_context);
+                    LinearLayout layout = new LinearLayout(_context);
+                    layout.setOrientation(LinearLayout.VERTICAL);
+                    TextView label = new TextView(_context);
+                    label.setText("Edit value of \n" + _name);
+                    EditText edit = new EditText(_context);
+                    edit.setText(String.valueOf(_value));
+                    edit.addTextChangedListener(new TextWatcher() {
+                            public void onTextChanged(CharSequence s, int start, int before, int count) {
+                                try {
+                                    if(s.length() < 1)  _tmpValue = 1;
+                                    _tmpValue = Double.parseDouble(s.toString());
+                                }
+                                catch(NumberFormatException e) {
+                                    _tmpValue = 1;
+                                }
+                            }
+                            // UNUSED Auto-generated method stub
+                            public void beforeTextChanged(CharSequence s, int start, int count, int after) {}
+                            // UNUSED Auto-generated method stub
+                            public void afterTextChanged(Editable s) {}
+                        });
+                    edit.requestFocus();
+                    //_context.getWindow().setSoftInputMode
+                    //    (WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
+                    layout.addView(label);
+                    layout.addView(edit);
+                    builder.setView(layout)
+                        .setPositiveButton("OK", new DialogInterface.OnClickListener() {
+                                public void onClick(DialogInterface dialog, int id) {
+                                    setValue(_tmpValue);
+                                }
+                            })
+                        .setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
+                                public void onClick(DialogInterface dialog, int id) {
+                                    // User cancelled the dialog
+                                }
+                            });
+                    builder.create().show();
+                    return true;
                 }
             });
         if(_spinner != null) {
@@ -237,9 +242,7 @@ public class ParameterNumber extends Parameter {
                     public void onStopTrackingTouch(SeekBar seekBar) {
                         setValue(getMin() + (getMax() - getMin())*seekBar.getProgress()/100);
                     }
-
                     public void onStartTrackingTouch(SeekBar seekBar) {}
-
                     public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {}
                 });
         }
@@ -248,7 +251,6 @@ public class ParameterNumber extends Parameter {
             paramLayout.addView(_checkbox);
             _checkbox.setEnabled(!_readOnly);
             _checkbox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
-
                     public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                         setValue((isChecked)? 1 : 0);
                     }
@@ -261,35 +263,37 @@ public class ParameterNumber extends Parameter {
             _edittext.setOnKeyListener(new View.OnKeyListener() {
                     public boolean onKey(View v, int keyCode, KeyEvent event) {
                         if(keyCode == KeyEvent.KEYCODE_ENTER){ // hide the keyboard
-                            InputMethodManager imm = (InputMethodManager)_context.getSystemService(
-                                                                                                   Context.INPUT_METHOD_SERVICE);
+                            InputMethodManager imm = (InputMethodManager)_context.getSystemService
+                                (Context.INPUT_METHOD_SERVICE);
                             imm.hideSoftInputFromWindow(_edittext.getWindowToken(), 0);
                             setValue(_value);
                             _edittext.clearFocus();
                             return true;
                         }
-                        if(keyCode > KeyEvent.KEYCODE_9 && keyCode != KeyEvent.KEYCODE_NUMPAD_DOT && (keyCode <KeyEvent.KEYCODE_NUMPAD_0 || keyCode >KeyEvent.KEYCODE_NUMPAD_9) && keyCode != KeyEvent.KEYCODE_DEL)
+                        if(keyCode > KeyEvent.KEYCODE_9 &&
+                           keyCode != KeyEvent.KEYCODE_NUMPAD_DOT &&
+                           (keyCode <KeyEvent.KEYCODE_NUMPAD_0 ||
+                            keyCode >KeyEvent.KEYCODE_NUMPAD_9) &&
+                           keyCode != KeyEvent.KEYCODE_DEL)
                             return true;
                         return false;
                     }
                 });
             _edittext.addTextChangedListener(new TextWatcher() {
-
                     public void onTextChanged(CharSequence s, int start, int before, int count) {
                         try {
                             if(s.length() < 1) _value = 1;
                             else _value = Double.parseDouble(s.toString());
                         }
-                        catch(NumberFormatException e)
-                            {
-                                _value = 1;
-                                //_edittext.setText("");
-                            }
+                        catch(NumberFormatException e){
+                            _value = 1;
+                            //_edittext.setText("");
+                        }
                     }
-
-                    public void beforeTextChanged(CharSequence s, int start, int count, int after) {} // UNUSED Auto-generated method stub
-                    public void afterTextChanged(Editable s) {} // UNUSED Auto-generated method stub
-
+                    // UNUSED Auto-generated method stub
+                    public void beforeTextChanged(CharSequence s, int start, int count, int after) {}
+                    // UNUSED Auto-generated method stub
+                    public void afterTextChanged(Editable s) {}
                 });
         }
         else if(_stepper != null) {
@@ -303,7 +307,10 @@ public class ParameterNumber extends Parameter {
         return paramLayout;
     }
     private OnParameterChangedListener mListener;
-    public void setOnParameterChangedListener(OnParameterChangedListener listener) { mListener = listener;}
+    public void setOnParameterChangedListener(OnParameterChangedListener listener)
+    {
+        mListener = listener;
+    }
     public interface OnParameterChangedListener
     {
         void OnParameterChanged();
