@@ -16,14 +16,15 @@ import android.widget.Spinner;
 
 public class ParameterString extends Parameter{
 
-    private  String _kind;
+    private String _kind;
     private int _index;
     private ArrayList<String> _choices;
     private ArrayAdapter<String> _adapter;
     private Spinner _spinner;
     private EditText _edittext;
 
-    public ParameterString(Context context, Gmsh gmsh, String name) {
+    public ParameterString(Context context, Gmsh gmsh, String name)
+    {
         super(context, gmsh, name);
         _choices = new ArrayList<String>();
         _choices.add("-"); // Default choice
@@ -33,12 +34,15 @@ public class ParameterString extends Parameter{
     {
         if(_spinner != null) return;
         _spinner = new Spinner(_context);
-        _adapter = new ArrayAdapter<String>(_context, android.R.layout.simple_spinner_dropdown_item, _choices);
+        _adapter = new ArrayAdapter<String>(_context,
+                                            android.R.layout.simple_spinner_dropdown_item,
+                                            _choices);
         _adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
         _spinner.setAdapter(_adapter);
     }
 
-    protected void update(){
+    protected void update()
+    {
         super.update();
         if(_spinner != null)
             _spinner.setSelection(_index);
@@ -46,14 +50,17 @@ public class ParameterString extends Parameter{
             _edittext.setText(_choices.get(0));
     }
 
-    public void setValue(int index) {
+    public void setValue(int index)
+    {
         if(index == _index) return;
         _changed = true;
         _index = index;
         _gmsh.setParam(getType(), getName(), _choices.get(_index));
         if(mListener != null) mListener.OnParameterChanged();
     }
-    public void setValue(String value) {
+
+    public void setValue(String value)
+    {
         int index = _choices.indexOf(value);
         if(index < 0) { // the value is not in the list, add it
             this.addChoices(value);
@@ -65,8 +72,10 @@ public class ParameterString extends Parameter{
         _gmsh.setParam(getType(), getName(), value);
         if(mListener != null) mListener.OnParameterChanged();
     }
-    public void setKind(String kind) {_kind = kind;}
-    public void addChoices(String choice) {
+
+    public void setKind(String kind) { _kind = kind; }
+    public void addChoices(String choice)
+    {
         if(_edittext == null && _spinner == null) createSpinner();
         for(String c : _choices) // do not add a duplicate value
             if(c.equals(choice))return;
@@ -75,11 +84,16 @@ public class ParameterString extends Parameter{
         _choices.add(choice);
         this.update();
     }
-    public String getValue() {if( _index < 0) return "";return _choices.get(_index);}
-    public String getKind() {return _kind;}
-    public int getIndex() {return _index;}
-    public ArrayList<String> getChoices() {return _choices;}
-    public int fromString(String s){
+    public String getValue() {
+        if( _index < 0)
+            return "";
+        return _choices.get(_index);
+    }
+    public String getKind() { return _kind; }
+    public int getIndex() { return _index; }
+    public ArrayList<String> getChoices() { return _choices; }
+    public int fromString(String s)
+    {
         int pos = super.fromString(s);
         if(pos <= 0) return -1; // error
         String[] infos = s.split(Character.toString((char)0x03));
@@ -97,8 +111,9 @@ public class ParameterString extends Parameter{
         this.update();
         return pos;
     }
-    public String getType(){return "ParameterString";}
-    public LinearLayout getView() {
+    public String getType(){ return "ParameterString"; }
+    public LinearLayout getView()
+    {
         LinearLayout paramLayout = new LinearLayout(_context);
         paramLayout.setOrientation(LinearLayout.VERTICAL);
         paramLayout.addView(_title);
@@ -106,24 +121,20 @@ public class ParameterString extends Parameter{
             paramLayout.addView(_spinner);
             _spinner.setEnabled(!_readOnly);
             _spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
-
                     public void onNothingSelected(AdapterView<?> arg0) {}
-
                     public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {
                         setValue(pos);
                     }
-
                 });
         }
         else if(_edittext != null){
             paramLayout.addView(_edittext);
             _edittext.setEnabled(!_readOnly);
             _edittext.setOnKeyListener(new View.OnKeyListener() {
-
                     public boolean onKey(View v, int keyCode, KeyEvent event) {
                         if(keyCode == KeyEvent.KEYCODE_ENTER){ // hide the keyboard
-                            InputMethodManager imm = (InputMethodManager)_context.getSystemService(
-                                                                                                   Context.INPUT_METHOD_SERVICE);
+                            InputMethodManager imm = (InputMethodManager)_context.getSystemService
+                                (Context.INPUT_METHOD_SERVICE);
                             imm.hideSoftInputFromWindow(_edittext.getWindowToken(), 0);
                             _edittext.clearFocus();
                             return true;
@@ -132,13 +143,11 @@ public class ParameterString extends Parameter{
                     }
                 });
             _edittext.addTextChangedListener(new TextWatcher() {
-
                     public void onTextChanged(CharSequence s, int start, int before, int count) {
                         _choices.clear(); _choices.add(s.toString());
                     }
-
-                    public void beforeTextChanged(CharSequence s, int start, int count, int after) {} // UNUSED Auto-generated method stub
-
+                    // UNUSED Auto-generated method stub
+                    public void beforeTextChanged(CharSequence s, int start, int count, int after) {}
                     public void afterTextChanged(Editable s) {
                         _gmsh.setParam(getType(), getName(), _choices.get(0));
                     }
@@ -147,8 +156,12 @@ public class ParameterString extends Parameter{
         return paramLayout;
     }
     private OnParameterChangedListener mListener;
-    public void setOnParameterChangedListener(OnParameterChangedListener listener) { mListener = listener;}
-    public interface OnParameterChangedListener {
+    public void setOnParameterChangedListener(OnParameterChangedListener listener)
+    {
+        mListener = listener;
+    }
+    public interface OnParameterChangedListener
+    {
         void OnParameterChanged();
     }
 
