@@ -46,8 +46,8 @@ drawContext::drawContext(float fontFactor, bool retina)
 
 static void checkGlError(const char* op)
 {
-  for (GLint error = glGetError(); error; error = glGetError())
-    Msg::Error("%s: glError (0x%x)",op,error);
+  //for (GLint error = glGetError(); error; error = glGetError())
+  //  Msg::Error("%s: glError (0x%x)",op,error);
 }
 
 void drawContext::load(std::string filename)
@@ -363,8 +363,12 @@ void drawContext::drawPView(PView *p)
   glLineWidth((GLfloat)opt->lineWidth);
 
   drawArray(p->va_points, GL_POINTS, true);
+
   drawArray(p->va_lines, GL_LINES, true);
+
+  glEnable(GL_LIGHTING);
   drawArray(p->va_triangles, GL_TRIANGLES, true, true);
+  glDisable(GL_LIGHTING);
 
   glLineWidth(1);
   glPointSize(1);
@@ -382,7 +386,7 @@ void drawContext::drawScale()
   double height = size / 10.;
   double dh = height / 5;
 
-	// Draw the scale bar
+  // Draw the scale bar
   int nPview = 0;
   for(int i=0; i<PView::list.size();i++){
     PView *p = PView::list[i];
@@ -578,12 +582,77 @@ void drawContext::drawAxes()
   glPopMatrix();
 }
 
+int drawContext::fix2dCoordinates(double *x, double *y)
+{
+  int ret = (*x > 99999 && *y > 99999) ? 3 : (*y > 99999) ? 2 : (*x > 99999) ? 1 : 0;
+
+  if(*x < 0) // measure from right border
+    *x = _right + *x;
+  else if(*x > 99999) // by convention, x-centered
+    *x = _right / 2;
+
+  if(*y < 0) // measure from bottom border
+    *y = -(*y);
+  else if(*y > 99999) // by convention, y-centered
+    *y = _top / 2.;
+  else
+    *y = _top - *y;
+  return ret;
+}
+
+void drawContext::drawText2d()
+{
+  glPushMatrix();
+  glLoadIdentity();
+
+  for(unsigned int i = 0; i < PView::list.size(); i++){
+    PViewData *data = PView::list[i]->getData();
+    PViewOptions *opt = PView::list[i]->getOptions();
+    if(opt->visible && opt->drawStrings){
+      for(int j = 0; j < data->getNumStrings2D(); j++){
+        double x, y, style;
+        std::string str;
+        data->getString2D(j, opt->timeStep, str, x, y, style);
+        //fix2dCoordinates(&x, &y);
+        GLfloat colors[] = {0., 0, 0, 1.};
+        drawString s(str.c_str(), 20 * _fontFactor, colors);
+        // FIXME:
+        s.draw(_left + (_right - _left) / 2.,
+               _bottom + 0.8 * (_top - _bottom), 0,
+               _width/(_right-_left), _height/(_top-_bottom), true);
+      }
+    }
+  }
+
+  glPopMatrix();
+}
+
+void drawGraph2d()
+{
+  glPushMatrix();
+  glLoadIdentity();
+
+  std::vector<PView*> graphs;
+  for(unsigned int i = 0; i < PView::list.size(); i++){
+    PViewData *data = PView::list[i]->getData();
+    PViewOptions *opt = PView::list[i]->getOptions();
+    if(!data->getDirty() && opt->visible && opt->type != PViewOptions::Plot3D)
+      graphs.push_back(PView::list[i]);
+  }
+  if(graphs.empty()) return;
+
+  // FIXME: draw 2d graph(s)
+
+  glPopMatrix();
+}
+
 void drawContext::drawView()
 {
   OrthofFromGModel();
 
   glMatrixMode(GL_MODELVIEW);
-  // fill the background
+
+  // draw the background
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   if(CTX::instance()->bgGradient){
     glPushMatrix();
@@ -611,10 +680,73 @@ void drawContext::drawView()
   }
   checkGlError("Draw background");
 
+  // init lights
+  glPushMatrix();
   glLoadIdentity();
   glScalef(_scale[0], _scale[1], _scale[2]);
   glTranslatef(_translate[0], _translate[1], _translate[2]);
+  for(int i = 0; i < 6; i++) {
+    if(CTX::instance()->light[i]) {
+      GLfloat position[4] = {
+        (GLfloat)CTX::instance()->lightPosition[i][0],
+        (GLfloat)CTX::instance()->lightPosition[i][1],
+        (GLfloat)CTX::instance()->lightPosition[i][2],
+        (GLfloat)CTX::instance()->lightPosition[i][3]
+      };
+      glLightfv((GLenum)(GL_LIGHT0 + i), GL_POSITION, position);
+      GLfloat r = (GLfloat)(CTX::instance()->unpackRed
+                            (CTX::instance()->color.ambientLight[i]) / 255.);
+      GLfloat g = (GLfloat)(CTX::instance()->unpackGreen
+                            (CTX::instance()->color.ambientLight[i]) / 255.);
+      GLfloat b = (GLfloat)(CTX::instance()->unpackBlue
+                            (CTX::instance()->color.ambientLight[i]) / 255.);
+      GLfloat ambient[4] = {r, g, b, 1.0F};
+      glLightfv((GLenum)(GL_LIGHT0 + i), GL_AMBIENT, ambient);
+      r = (GLfloat)(CTX::instance()->unpackRed
+                    (CTX::instance()->color.diffuseLight[i]) / 255.);
+      g = (GLfloat)(CTX::instance()->unpackGreen
+                    (CTX::instance()->color.diffuseLight[i]) / 255.);
+      b = (GLfloat)(CTX::instance()->unpackBlue
+                    (CTX::instance()->color.diffuseLight[i]) / 255.);
+      GLfloat diffuse[4] = {r, g, b, 1.0F};
+      glLightfv((GLenum)(GL_LIGHT0 + i), GL_DIFFUSE, diffuse);
+      r = (GLfloat)(CTX::instance()->unpackRed
+                    (CTX::instance()->color.specularLight[i]) / 255.);
+      g = (GLfloat)(CTX::instance()->unpackGreen
+                    (CTX::instance()->color.specularLight[i]) / 255.);
+      b = (GLfloat)(CTX::instance()->unpackBlue
+                    (CTX::instance()->color.specularLight[i]) / 255.);
+      GLfloat specular[4] = {r, g, b, 1.0F};
+      glLightfv((GLenum)(GL_LIGHT0 + i), GL_SPECULAR, specular);
+      glEnable((GLenum)(GL_LIGHT0 + i));
+    }
+    else{
+      glDisable((GLenum)(GL_LIGHT0 + i));
+    }
+  }
+  glPopMatrix();
+  // ambient and diffuse material colors track glColor automatically
+  //glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
+  glEnable(GL_COLOR_MATERIAL);
+  // "white"-only specular material reflection color
+  GLfloat spec[4] = {(GLfloat)CTX::instance()->shine,
+                     (GLfloat)CTX::instance()->shine,
+                     (GLfloat)CTX::instance()->shine, 1.0F};
+  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, spec);
+  // specular exponent in [0,128] (larger means more "focused"
+  // reflection)
+  glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS,
+              (GLfloat)CTX::instance()->shineExponent);
+  glShadeModel(GL_SMOOTH);
+  glEnable(GL_RESCALE_NORMAL);
+  glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.);
+  glDisable(GL_LIGHTING);
+  checkGlError("Initialize lights");
 
+  // init position
+  glLoadIdentity();
+  glScalef(_scale[0], _scale[1], _scale[2]);
+  glTranslatef(_translate[0], _translate[1], _translate[2]);
   if(CTX::instance()->rotationCenterCg)
     glTranslatef(CTX::instance()->cg[0],
                  CTX::instance()->cg[1],
@@ -623,10 +755,8 @@ void drawContext::drawView()
     glTranslatef(CTX::instance()->rotationCenter[0],
                  CTX::instance()->rotationCenter[1],
                  CTX::instance()->rotationCenter[2]);
-
   buildRotationMatrix();
   glMultMatrixf(_rotatef);
-
   if(CTX::instance()->rotationCenterCg)
     glTranslatef(-CTX::instance()->cg[0],
                  -CTX::instance()->cg[1],
@@ -635,22 +765,18 @@ void drawContext::drawView()
     glTranslatef(-CTX::instance()->rotationCenter[0],
                  -CTX::instance()->rotationCenter[1],
                  -CTX::instance()->rotationCenter[2]);
-
   checkGlError("Initialize position");
 
+  // draw everything
   glEnable(GL_DEPTH_TEST);
-
-  drawMesh();
-  checkGlError("Draw mesh");
-  drawGeom();
-  checkGlError("Draw geometry");
-  drawPost();
-  checkGlError("Draw post-pro");
+  drawMesh(); checkGlError("Draw mesh");
+  drawGeom(); checkGlError("Draw geometry");
+  drawPost(); checkGlError("Draw post-pro");
   glDisable(GL_DEPTH_TEST);
-  drawScale();
-  checkGlError("Draw scales");
-  drawAxes();
-  checkGlError("Draw axes");
+  drawScale(); checkGlError("Draw scales");
+  drawAxes(); checkGlError("Draw axes");
+  drawText2d(); checkGlError("Draw text2d");
+  drawGraph2d(); checkGlError("Draw graph2d");
 }
 
 std::vector<std::string> commandToVector(const std::string cmd)
@@ -730,6 +856,7 @@ int onelab_cb(std::string action)
   } while(action == "compute" && !onelabStop && (onelabUtils::incrementLoop("3") ||
                                                  onelabUtils::incrementLoop("2") ||
                                                  onelabUtils::incrementLoop("1")));
+  onelabStop = false;
   locked = false;
   return onelab::server::instance()->getChanged();
 }
