@@ -1,11 +1,3 @@
-//
-//  parameter.h
-//  Onelab
-//
-//  Created by Maxime Graulich on 07/08/13.
-//  Copyright (c) 2013 Maxime Graulich. All rights reserved.
-//
-
 #import <Foundation/Foundation.h>
 #import <Gmsh/onelab.h>
 
@@ -13,9 +5,9 @@
 
 @interface Parameter : NSObject
 {
-    @protected
-    NSString *name;
-    UILabel *label;
+@protected
+  NSString *name;
+  UILabel *label;
 }
 -(id)init;
 -(void)setFrame:(CGRect)frame;
@@ -27,38 +19,46 @@
 -(void)refresh;
 @end
 
-@interface ParameterStringList : Parameter <UIPickerViewDataSource, UIPickerViewDelegate>
+@interface ParameterStringList : Parameter <UIActionSheetDelegate>
 {
-    @protected
-    UIPickerView *picker;
+@protected
+  UIButton *button;
 }
 -(id)initWithString:(onelab::string)string;
--(UIPickerView *)getList;
+-(UIPickerView *)getUIView;
 @end
 
-
-@interface ParameterNumberList : Parameter <UIPickerViewDataSource, UIPickerViewDelegate, UITextFieldDelegate>
+@interface ParameterNumberList : Parameter <UIActionSheetDelegate>
 {
-    @protected
-    UIPickerView *picker;
+@protected
+  UIButton *button;
 }
 -(id)initWithNumber:(onelab::number)number;
--(UIPickerView *)getList;
+-(UITextField *)getUIView;
 @end
 
 @interface ParameterNumberCheckbox : Parameter
 {
-    @protected
-    UISwitch *checkbox;
+@protected
+  UISwitch *checkbox;
 }
 -(id)initWithNumber:(onelab::number)number;
 -(UISwitch *)getCheckbox;
 @end
 
+@interface ParameterNumberStepper : Parameter
+{
+@protected
+  UIStepper *stepper;
+}
+-(id)initWithNumber:(onelab::number)number;
+-(UIStepper *)getStepper;
+@end
+
 @interface ParameterNumberRange : Parameter
 {
-    @protected
-    UISlider *slider;
+@protected
+  UISlider *slider;
 }
 -(id)initWithNumber:(onelab::number)number;
 -(UISlider *)getSlider;
@@ -67,7 +67,7 @@
 @interface ParameterNumberTextbox : Parameter <UITextFieldDelegate>
 {
 @protected
-    UITextField *textbox;
+  UITextField *textbox;
 }
 -(id)initWithNumber:(onelab::number)number;
 -(UITextField *)getTextbox;
