@@ -10,6 +10,18 @@
 #define PCLOSE pclose
 #endif
 
+#if defined(WIN32)
+static char dirSep='\\';
+static std::string cmdSep(" & ");
+static std::string removeCmd("del ");
+static std::string lsCmd("dir ");
+#else
+static char dirSep='/';
+static std::string cmdSep(" ; ");
+static std::string removeCmd("rm -rf ");
+static std::string lsCmd("ls ");
+#endif
+
 class onelabMetaModelServer : public GmshServer{
  private:
   localNetworkSolverClient *_client;
@@ -18,33 +30,34 @@ class onelabMetaModelServer : public GmshServer{
     : GmshServer(), _client(client) {}
   ~onelabMetaModelServer(){}
 
-  int NonBlockingSystemCall(const char *str){ return SystemCall(str); }
-  int NonBlockingWait(int socket, double waitint, double timeout)
+  int NonBlockingSystemCall(const std::string &exe, const std::string &args){
+    std::cout << "Calling now : " << exe << " " << args << std::endl;
+    return SystemCallExe(exe, args);
+  }
+  int NonBlockingWait(double waitint, double timeout, int socket)
   {
     double start = GetTimeInSeconds();
     while(1){
       if(timeout > 0 && GetTimeInSeconds() - start > timeout)
         return 2; // timeout
-      // if(_client->getPid() < 0 || (_client->getCommandLine().empty() &&
-      //                              !CTX::instance()->solver.listen))
-      if(_client->getPid() < 0 || (_client->getCommandLine().empty()))
-        return 1; // process has been killed or we stopped listening
+
+      if(_client->getPid() < 0 /*|| (_client->getCommandLine().empty())*/)
+        return 1; // process has been killed
+
       // check if there is data (call select with a zero timeout to
       // return immediately, i.e., do polling)
       int ret = Select(0, 0, socket);
+
       if(ret == 0){ // nothing available
         // wait at most waitint seconds and respond to FLTK events
-	//FlGui::instance()->wait(waitint);
 	void (*waitFct)(double) = OLMsg::GetGuiWaitFunction();
 	if(waitFct) waitFct(waitint);
       }
       else if(ret > 0){
         return 0; // data is there!
       }
-      else{
-        // ret<0 an error happened
+      else{ // ret<0 an error happened
         _client->setPid(-1);
-        _client->setGmshServer(0);
         return 1;
       }
     }
@@ -83,14 +96,223 @@ std::string localNetworkSolverClient::appendArguments(){
   return command;
 }
 
+bool localNetworkSolverClient::receiveMessage(){
+  double timer = GetTimeInSeconds();
+
+  if(!getGmshServer()){
+    OLMsg::Error("Abnormal server termination (no valid server)");
+    return false;
+  }
+
+  int type, length, swap;
+  if(!getGmshServer()->ReceiveHeader(&type, &length, &swap)){
+    OLMsg::Error("Abnormal server termination (did not receive message header)");
+    return false;
+  }
+  else if(_socketMsg)
+    OLMsg::Info("Received header <%d>",type);
+
+  std::string message(length, ' ');
+  if(!getGmshServer()->ReceiveMessage(length, &message[0])){
+    OLMsg::Error("Abnormal server termination (did not receive message body)r");
+    return false;
+  }
+  else if(_socketMsg)
+    OLMsg::Info("Received message <%s>",message.c_str());
+
+  switch (type) {
+  case GmshSocket::GMSH_START:
+    setPid(atoi(message.c_str()));
+    break;
+  case GmshSocket::GMSH_STOP:
+    setPid(-1);
+    break;
+  case GmshSocket::GMSH_PARAMETER:
+    {
+      std::string version, type, name;
+      onelab::parameter::getInfoFromChar(message, version, type, name);
+      if(onelab::parameter::version() != version){
+	OLMsg::Error("OneLab version mismatch for %s (server: %s / client: %s)",
+		     message.c_str(), onelab::parameter::version().c_str(), version.c_str());
+      }
+      if(type == "number"){
+	onelab::number p; p.fromChar(message); set(p);
+      }
+      else if(type == "string"){
+	onelab::string p; p.fromChar(message); set(p);
+      }
+      else if(type == "region"){
+	onelab::region p; p.fromChar(message); set(p);
+      }
+      else if(type == "function"){
+	onelab::function p; p.fromChar(message); set(p);
+      }
+      else
+	OLMsg::Error("FIXME query not done for this parameter type: <%s>",
+		     message.c_str());
+    }
+    break;
+  case GmshSocket::GMSH_PARAMETER_QUERY:
+    {
+      std::string version, type, name, reply;
+      onelab::parameter::getInfoFromChar(message, version, type, name);
+      if(onelab::parameter::version() != version){
+	OLMsg::Error("OneLab version mismatch for %s (server: %s / client: %s)",
+		     message.c_str(), onelab::parameter::version().c_str(), version.c_str());
+      }
+      else if(type == "number"){
+	std::vector<onelab::number> par; get(par, name);
+	if(par.size() == 1) reply = par[0].toChar();
+      }
+      else if(type == "string"){
+	std::vector<onelab::string> par; get(par, name);
+	if(par.size() == 1) reply = par[0].toChar();
+      }
+      else if(type == "region"){
+	std::vector<onelab::region> par; get(par, name);
+	if(par.size() == 1) par[0].toChar();
+      }
+      else if(type == "function"){
+	std::vector<onelab::function> par; get(par, name);
+	if(par.size() == 1) reply = par[0].toChar();
+      }
+      else
+	OLMsg::Error("Unknown OneLab parameter type in query: %s",
+		     type.c_str());
+
+      if(reply.size()){
+	getGmshServer()->SendMessage(GmshSocket::GMSH_PARAMETER,
+			    reply.size(), &reply[0]);
+      }
+      else{
+	reply = "OneLab parameter '" + name + "' not found";
+	getGmshServer()->SendMessage(GmshSocket::GMSH_INFO, reply.size(), &reply[0]);
+      }
+    }
+    break;
+  case GmshSocket::GMSH_PARAMETER_QUERY_ALL:
+    {
+      std::string version, type, name, reply;
+      std::vector<std::string> replies;
+      onelab::parameter::getInfoFromChar(message, version, type, name);
+      if(onelab::parameter::version() != version){
+	OLMsg::Error("OneLab version mismatch for %s (server: %s / client: %s)",
+		     message.c_str(), onelab::parameter::version().c_str(), version.c_str());
+      }
+      else if(type == "number"){
+	std::vector<onelab::number> numbers; get(numbers);
+	for(std::vector<onelab::number>::iterator it = numbers.begin();
+	    it != numbers.end(); it++) replies.push_back((*it).toChar());
+      }
+      else if(type == "string"){
+	std::vector<onelab::string> strings; get(strings);
+	for(std::vector<onelab::string>::iterator it = strings.begin();
+	    it != strings.end(); it++) replies.push_back((*it).toChar());
+      }
+      else if(type == "region"){
+	std::vector<onelab::region> regions; get(regions);
+	for(std::vector<onelab::region>::iterator it = regions.begin();
+	    it != regions.end(); it++) replies.push_back((*it).toChar());
+      }
+      else if(type == "function"){
+	std::vector<onelab::function> functions; get(functions);
+	for(std::vector<onelab::function>::iterator it = functions.begin();
+	    it != functions.end(); it++) replies.push_back((*it).toChar());
+      }
+      else
+	OLMsg::Error("Unknown OneLab parameter type in query: %s",
+		     type.c_str());
+
+      for(unsigned int i = 0; i < replies.size(); i++)
+	getGmshServer()->SendMessage(GmshSocket::GMSH_PARAMETER_QUERY_ALL,
+				     replies[i].size(), &replies[i][0]);
+      reply = "Sent all OneLab " + type + "s";
+      getGmshServer()->SendMessage(GmshSocket::GMSH_PARAMETER_QUERY_END,
+				   reply.size(), &reply[0]);
+    }
+    break;
+  case GmshSocket::GMSH_PROGRESS:
+    OLMsg::StatusBar(2, false, "%s %s", _name.c_str(), message.c_str());
+    break;
+  case GmshSocket::GMSH_INFO:
+    OLMsg::Direct("%-8.8s: %s", _name.c_str(), message.c_str());
+    break;
+  case GmshSocket::GMSH_WARNING:
+    OLMsg::Direct(2, "%-8.8s: %s", _name.c_str(), message.c_str());
+    break;
+  case GmshSocket::GMSH_ERROR:
+    OLMsg::Error("%-8.8s: %s", _name.c_str(), message.c_str());
+    break;
+  case GmshSocket::GMSH_MERGE_FILE:
+    // if(CTX::instance()->solver.autoMergeFile){
+    //   unsigned int n = PView::list.size();
+    //   MergePostProcessingFile(message, CTX::instance()->solver.autoShowLastStep,
+    //                           CTX::instance()->solver.autoHideNewViews, true);
+    //   drawContext::global()->draw();
+    //   if(FlGui::available() && n != PView::list.size()){
+    //     FlGui::instance()->rebuildTree();
+    //     FlGui::instance()->openModule("Post-processing");
+    //   }
+    // }
+    break;
+  case GmshSocket::GMSH_PARSE_STRING:
+    // ParseString(message);
+    // drawContext::global()->draw();
+    break;
+ case GmshSocket::GMSH_SPEED_TEST:
+    OLMsg::Info("got %d Mb message in %g seconds",
+              length / 1024 / 1024, GetTimeInSeconds() - timer);
+    break;
+  case GmshSocket::GMSH_VERTEX_ARRAY:
+    {
+      // int n = PView::list.size();
+      // PView::fillVertexArray(this, length, &message[0], swap);
+      // if(FlGui::available())
+      //   FlGui::instance()->updateViews(n != (int)PView::list.size());
+      // drawContext::global()->draw();
+    }
+    break;
+  case GmshSocket::GMSH_CONNECT:
+    {
+      // const std::string clientName = message;
+      // std::cout << "client name =" << clientName << std::endl;
+      // onelab::localNetworkClient *subClient = findClient(clientName);
+      // if (!subClient) {
+      //   subClient = new gmshLocalNetworkClient(clientName, "");
+      // }
+      // else
+      // 	std::cout << "Gmsh has " << addClient(subClient) << " clients\n";
+      // subClient->getGmshServer()->LaunchClient();
+    }
+    break;
+  case GmshSocket::GMSH_OLPARSE:
+    {
+#if defined(HAVE_ONELAB_METAMODEL)
+      localSolverClient *c = new InterfacedClient("OLParser","","");
+      std::string ofileName = message ;
+      std::ofstream outfile(ofileName.c_str());
+      if (outfile.is_open())
+        c->convert_onefile(ofileName + ".ol",outfile);
+      else
+        OLMsg::Error("The file <%s> cannot be opened",ofileName.c_str());
+      outfile.close();
+      delete c;
+#endif
+    }
+    break;
+  default:
+    OLMsg::Warning("Received unknown message type (%d)", type);
+    break;
+  }
+  return true;
+}
+
 bool localNetworkSolverClient::run()
 {
- new_connection:
-  _pid = 0;
-  _gmshServer = 0;
-
-  onelabMetaModelServer *server = new onelabMetaModelServer(this);
+  //new_connection:
+  setPid(0);
 
+  // Choose socket type (Unix or TCP/IP)
 #if defined(WIN32)
   std::string socketName = ":";
 #else
@@ -103,221 +325,93 @@ bool localNetworkSolverClient::run()
 
   std::string sockname;
   std::ostringstream tmp;
-  if(!strstr(socketName.c_str(), ":")){
-    // Unix socket
+  if(!strstr(socketName.c_str(), ":")){ // Unix socket
     tmp << socketName << getId();
     sockname = FixWindowsPath(tmp.str());
   }
-  else{
-    // TCP/IP socket
+  else{                                 // TCP/IP socket
     if(socketName.size() && socketName[0] == ':')
-      // prepend hostname if only the port number is given
-      tmp << GetHostName();
+      tmp << GetHostName(); // prepend hostname if only the port number is given
     tmp << socketName ;
     sockname = tmp.str();
   }
 
-  _socketMsg = OLMsg::GetOnelabNumber("SHOWSOCKETMESSAGES")?true:false;
-  std::string command = buildCommandLine();
-  if(command.size()) command.append(appendArguments());
+  // Build the commande line
+  std::string exe = buildCommandLine();
+  std::string command(exe);
+  if(command.size())
+    command.append(appendArguments());
+  else
+    return false;
+
+  // Create socket connection and launch client
+  onelabMetaModelServer *socketConnection = new onelabMetaModelServer(this);
+
+  std::cout << "commandline = " << command << std::endl;
 
   int sock;
   try{
-    sock = server->Start(command.c_str(), sockname.c_str(), 10);
+    sock = socketConnection->Start(exe.c_str(), command.c_str(), sockname.c_str(), 10);
   }
   catch(const char *err){
     OLMsg::Error("%s (on socket '%s')", err, sockname.c_str());
     sock = -1;
   }
 
-  if(sock < 0){
-    server->Shutdown();
-    delete server;
+  if(sock < 0){ //could not establish connection: aborting
+    socketConnection->Shutdown();
+    delete socketConnection;
+    OLMsg::Error("Connection failed on socket <%s>", sockname.c_str());
     return false;
   }
 
-  //OLMsg::StatusBar(2, true, "Now running client <%s>", _name.c_str());
+  OLMsg::StatusBar(2, true, "Now running client <%s> on socket <%d>", _name.c_str(), sock);
+  _socketMsg = OLMsg::GetOnelabNumber("SHOWSOCKETMESSAGES")?true:false;
+
+  setGmshServer(socketConnection);
 
   while(1) {
-    if(_pid < 0) break;
+    bool stop = false, haveData = false;
 
-    int stop = server->NonBlockingWait(sock, 0.0001, 0.);
-    if(stop || _pid < 0) {
-      OLMsg::Info("Stop=%d _pid=%d",stop, _pid);
-      break;
+    if(getPid() < 0) { stop = true; break; }
+    if(!getGmshServer()){
+      OLMsg::Error("Abnormal server termination (no valid server)");
+      stop = true;
     }
-    int type, length, swap;
-    if(!server->ReceiveHeader(&type, &length, &swap)){
-      OLMsg::Error("Did not receive message header: stopping server");
-      break;
+    else if(!getGmshServer()->NonBlockingWait(0.001, 0., sock)){
+      haveData = true;
     }
-    else if(_socketMsg)
-      OLMsg::Info("Received header <%d>",type);
-
-    std::string message(length, ' ');
-    if(!server->ReceiveMessage(length, &message[0])){
-      OLMsg::Error("Did not receive message body: stopping server");
-      break;
-    }
-    else if(_socketMsg)
-      OLMsg::Info("Received message <%s>",message.c_str());
-
-    switch (type) {
-    case GmshSocket::GMSH_START:
-      _pid = atoi(message.c_str());
-      _gmshServer = server;
-      break;
-    case GmshSocket::GMSH_STOP:
-      _pid = -1;
-      _gmshServer = 0;
-      break;
-    case GmshSocket::GMSH_PARAMETER:
-      {
-        std::string version, type, name;
-        onelab::parameter::getInfoFromChar(message, version, type, name);
-        if(onelab::parameter::version() != version){
-          OLMsg::Error("OneLab version mismatch for %s (server: %s / client: %s)",
-	   message.c_str(), onelab::parameter::version().c_str(), version.c_str());
-        }
-        if(type == "number"){
-          onelab::number p; p.fromChar(message); set(p);
-        }
-        else if(type == "string"){
-          onelab::string p; p.fromChar(message); set(p);
-        }
-	else if(type == "region"){
-          onelab::region p; p.fromChar(message); set(p);
-        }
-        else if(type == "function"){
-          onelab::function p; p.fromChar(message); set(p);
-        }
-        else
-          OLMsg::Error("FIXME query not done for this parameter type: <%s>",
-		     message.c_str());
-      }
-      break;
-    case GmshSocket::GMSH_PARAMETER_QUERY:
-      {
-        std::string version, type, name, reply;
-        onelab::parameter::getInfoFromChar(message, version, type, name);
-        if(onelab::parameter::version() != version){
-          OLMsg::Error("OneLab version mismatch for %s (server: %s / client: %s)",
-	   message.c_str(), onelab::parameter::version().c_str(), version.c_str());
-        }
-        else if(type == "number"){
-          std::vector<onelab::number> par; get(par, name);
-          if(par.size() == 1) reply = par[0].toChar();
-	}
-        else if(type == "string"){
-          std::vector<onelab::string> par; get(par, name);
-          if(par.size() == 1) reply = par[0].toChar();
-        }
-        else if(type == "region"){
-          std::vector<onelab::region> par; get(par, name);
-          if(par.size() == 1) par[0].toChar();
-        }
-        else if(type == "function"){
-          std::vector<onelab::function> par; get(par, name);
-          if(par.size() == 1) reply = par[0].toChar();
-        }
-        else
-          OLMsg::Error("Unknown OneLab parameter type in query: %s",
-		     type.c_str());
-
-        if(reply.size()){
-          server->SendMessage(GmshSocket::GMSH_PARAMETER,
-			      reply.size(), &reply[0]);
-        }
-        else{
-          reply = "OneLab parameter '" + name + "' not found";
-          server->SendMessage(GmshSocket::GMSH_INFO, reply.size(), &reply[0]);
-        }
-      }
-      break;
-    case GmshSocket::GMSH_PARAM_QUERY_ALL:
-      {
-        std::string version, type, name, reply;
-        std::vector<std::string> replies;
-        onelab::parameter::getInfoFromChar(message, version, type, name);
-	if(onelab::parameter::version() != version){
-          OLMsg::Error("OneLab version mismatch for %s (server: %s / client: %s)",
-           message.c_str(), onelab::parameter::version().c_str(), version.c_str());
-        }
-	else if(type == "number"){
-	  std::vector<onelab::number> numbers; get(numbers);
-	  for(std::vector<onelab::number>::iterator it = numbers.begin();
-	      it != numbers.end(); it++) replies.push_back((*it).toChar());
-	}
-	else if(type == "string"){
-	  std::vector<onelab::string> strings; get(strings);
-	  for(std::vector<onelab::string>::iterator it = strings.begin();
-	      it != strings.end(); it++) replies.push_back((*it).toChar());
-	}
-        else if(type == "region"){
-	  std::vector<onelab::region> regions; get(regions);
-	  for(std::vector<onelab::region>::iterator it = regions.begin();
-              it != regions.end(); it++) replies.push_back((*it).toChar());
-        }
-        else if(type == "function"){
-	  std::vector<onelab::function> functions; get(functions);
-	  for(std::vector<onelab::function>::iterator it = functions.begin();
-              it != functions.end(); it++) replies.push_back((*it).toChar());
-        }
-        else
-          OLMsg::Error("Unknown OneLab parameter type in query: %s",
-		     type.c_str());
-
-        for(unsigned int i = 0; i < replies.size(); i++)
-          server->SendMessage(GmshSocket::GMSH_PARAM_QUERY_ALL,
-	     replies[i].size(), &replies[i][0]);
-        reply = "Sent all OneLab " + type + "s";
-        server->SendMessage(GmshSocket::GMSH_PARAM_QUERY_END,
-			    reply.size(), &reply[0]);
-      }
-      break;
-    case GmshSocket::GMSH_PROGRESS:
-      OLMsg::StatusBar(2, false, "%s %s", _name.c_str(), message.c_str());
-      break;
-    case GmshSocket::GMSH_INFO:
-      OLMsg::Direct("%-8.8s: %s", _name.c_str(), message.c_str());
-      break;
-    case GmshSocket::GMSH_WARNING:
-      OLMsg::Direct(2, "%-8.8s: %s", _name.c_str(), message.c_str());
-      break;
-    case GmshSocket::GMSH_ERROR:
-      //OLMsg::Direct(1, "%-8.8s: %s", _name.c_str(), message.c_str());
-      OLMsg::Error("%-8.8s: %s", _name.c_str(), message.c_str());
-      break;
-    case GmshSocket::GMSH_MERGE_FILE:
-      break;
-    default:
-      OLMsg::Warning("Received unknown message type (%d)", type);
-      break;
+    else{// an error occurred
+      stop = true;
     }
+    if(stop) break;
+    if(haveData && !receiveMessage()) break;
+    if(getPid() < 0) break;
   }
 
-  server->Shutdown();
-  delete server;
+  socketConnection->Shutdown();
+  delete socketConnection;
+  setGmshServer(0);
   OLMsg::StatusBar(2, true, "Done running '%s'", _name.c_str());
 
-  if(command.empty()){
-    OLMsg::Info("Client disconnected: starting new connection");
-    goto new_connection;
-  }
+  // if(command.empty()){
+  //   OLMsg::Info("Client disconnected: starting new connection");
+  //   goto new_connection;
+  // }
 
   return true;
 }
 
 bool localNetworkSolverClient::kill()
 {
-  if(_pid > 0) {
-    if(KillProcess(_pid)){
-      OLMsg::Info("Killed '%s' (pid %d)", _name.c_str(), _pid);
-      _pid = -1;
+  if(getPid() > 0) {
+    if(KillProcess(getPid())){
+      OLMsg::Info("Killed '%s' (pid %d)", _name.c_str(), getPid());
+      setPid(-1);
       return true;
     }
   }
-  _pid = -1;
+  setPid(-1);
   return false;
 }
 
@@ -365,7 +459,7 @@ void localSolverClient::FixExecPath(const std::string &in)
     setCommandLine(cmd);
   }
 
-#if not defined(WIN32)
+#if !defined(WIN32)
   //if(split[0].find("elmerfem") != std::string::npos){
   if(!split[1].compare("ElmerSolver") && split[2].empty() && split[0].size()){
     std::string fileName = getWorkingDir() + getName() + ".sh";
@@ -374,7 +468,7 @@ void localSolverClient::FixExecPath(const std::string &in)
     if (outfile.is_open()){
       outfile << "#!/bin/bash" << std::endl;
       outfile << "export ELMER_HOME=\"" << elmerHome << "\"" << std::endl;
-      outfile << "export DYLD_LIBRARY_PATH=\".:$ELMER_HOME/lib:$DYLD_LIBRARY_PATH\"" << std::endl; 
+      outfile << "export DYLD_LIBRARY_PATH=\".:$ELMER_HOME/lib:$DYLD_LIBRARY_PATH\"" << std::endl;
       outfile << in << std::endl;
     }
     else
@@ -388,7 +482,6 @@ void localSolverClient::FixExecPath(const std::string &in)
 
 bool localSolverClient::checkCommandLine(){
   bool success=true;
-  if(!isActive()) return success;
   OLMsg::Info("Check command line <%s> for client <%s>",
 	      getCommandLine().c_str(), getName().c_str());
 
@@ -412,9 +505,7 @@ bool localSolverClient::checkCommandLine(){
     if(isNative()){ // native clients checked by initializing
       setAction("initialize");
       if(!run()){ // initializes native clients, false otherwise
-	OLMsg::Error("Invalid commandline <%s> for client <%s>",
-		     getCommandLine().c_str(), getName().c_str());
-	success=false;
+	success = false;
       }
     }
     else{
@@ -422,10 +513,10 @@ bool localSolverClient::checkCommandLine(){
       commandLine.assign(QuoteExecPath(getCommandLine()));
 
       // check whether the executable exists as a file
-      success=checkIfPresent(getCommandLine());
+      success = checkIfPresent(getCommandLine());
 
       // resolve a possible linux link
-#if not defined(WIN32)
+#if !defined(WIN32)
       if(!success){
 	char cbuf [1024];
 	FILE *fp;
@@ -454,7 +545,7 @@ bool localSolverClient::checkCommandLine(){
     OLMsg::Info("Command line ok");
   }
   else{
-    OLMsg::Error("Invalid command line <%s> for client <%s>",
+    OLMsg::Error("Invalid commandline <%s> for client <%s>",
 		 getCommandLine().c_str(), getName().c_str());
     //setCommandLine("");
     OLMsg::SetOnelabString(getName() + "/CommandLine", getCommandLine(), true);
@@ -486,13 +577,10 @@ void localSolverClient::addNumberChoice(std::string name, double val, bool readO
     ps[0].setValue(val);
     choices.push_back(val);
     ps[0].setChoices(choices);
-    //ps[0].setAttribute("Highlight","Coral");
     set(ps[0]);
   }
   else{
     OLMsg::Error("The parameter <%s> does not exist", name.c_str());
-    // ps.resize(1);
-    // ps[0].setName(name);
   }
 }
 
@@ -540,7 +628,7 @@ bool remoteClient::checkCommandLine(const std::string &commandLine){
   success=checkIfPresentRemote(commandLine);
 
   // resolve a possible linux link
-#if not defined(WIN32)
+#if !defined(WIN32)
   if(!success){
     char cbuf [1024];
     FILE *fp;
@@ -618,7 +706,7 @@ bool remoteClient::syncInputFile(const std::string &wdir, const std::string &fil
     }
     else { //should be found remote
       if(!checkIfPresentRemote(split[1])){
-	OLMsg::Error("The remote input file <%s> is not present", 
+	OLMsg::Error("The remote input file <%s> is not present",
 		     split[1].c_str());
 	return false;
       }
@@ -639,7 +727,7 @@ bool remoteClient::syncOutputFile(const std::string &wdir, const std::string &fi
 
   if(checkIfPresentRemote(split[1])){
     if(split[0].size()){ // the file must be copied back on local host
-      cmd.assign("rsync -e ssh -auv " + _remoteHost + ":"); 
+      cmd.assign("rsync -e ssh -auv " + _remoteHost + ":");
       if(_remoteDir.size())
 	cmd.append(_remoteDir);
       cmd.append(split[1]);
@@ -671,7 +759,7 @@ void MetaModel::analyze() {
   OLMsg::Info("===== ANALYZING");
   std::string fileName = getWorkingDir() + genericNameFromArgs + onelabExtension;
   openOnelabBlock();
-  OLMsg::Info("Parse file <%s> %s", fileName.c_str(), 
+  OLMsg::Info("Parse file <%s> %s", fileName.c_str(),
 	      parse_onefile(fileName)?"done":"failed");
   closeOnelabBlock();
 }
@@ -680,7 +768,7 @@ void MetaModel::compute() {
   OLMsg::Info("===== COMPUTING");
   std::string fileName = getWorkingDir() + genericNameFromArgs + onelabExtension;
   openOnelabBlock();
-  OLMsg::Info("Parse file <%s> %s", fileName.c_str(), 
+  OLMsg::Info("Parse file <%s> %s", fileName.c_str(),
 	      parse_onefile(fileName)?"done":"failed");
   closeOnelabBlock();
   onelab::server::instance()->setChanged(false);
@@ -689,10 +777,10 @@ void MetaModel::compute() {
 void MetaModel::registerClient(const std::string &name, const std::string &type, const std::string &cmdl, const std::string &host, const std::string &rdir) {
   localSolverClient *c;
 
-  // Clients are assigned by default the same (local) working dir 
+  // Clients are assigned by default the same (local) working dir
   // as the MetaModel, i.e. the working dir from args
   // A working (local relative) subdir (useful to organize submodels)
-  // can be defined with the command: client.workingSubdir(subdir) 
+  // can be defined with the command: client.workingSubdir(subdir)
   if(host.empty() || !host.compare("localhost")){ //local client
     if(!type.compare(0,6,"interf"))
       c= new InterfacedClient(name,cmdl,getWorkingDir());
@@ -747,7 +835,7 @@ void InterfacedClient::analyze() {
       std::string fileName = getWorkingDir() + split[1] + split[2];
       if(!checkIfPresent(fileName))
 	OLMsg::Error("The file <%s> is not present",fileName.c_str());
-      OLMsg::Info("Parse file <%s> %s", fileName.c_str(), 
+      OLMsg::Info("Parse file <%s> %s", fileName.c_str(),
 		  parse_onefile(fileName)?"done":"failed");
     }
   }
@@ -880,7 +968,7 @@ void EncapsulatedClient::analyze() {
       std::string fileName = getWorkingDir() + split[1] + split[2];
       if(!checkIfPresent(fileName))
 	OLMsg::Error("The file <%s> is not present",fileName.c_str());
-      OLMsg::Info("Parse file <%s> %s", fileName.c_str(), 
+      OLMsg::Info("Parse file <%s> %s", fileName.c_str(),
 		  parse_onefile(fileName)?"done":"failed");
     }
   }
@@ -963,7 +1051,6 @@ void EncapsulatedClient::compute(){
 
 bool RemoteInterfacedClient::checkCommandLine(){
   bool success;
-  if(!isActive()) return true;
   OLMsg::Info("Check command line <%s> for client <%s>",
 	      getCommandLine().c_str(), getName().c_str());
   if(getCommandLine().empty())
@@ -1039,7 +1126,6 @@ std::string RemoteNativeClient::buildCommandLine(){
 
 bool RemoteNativeClient::checkCommandLine(){
   bool success;
-  if(!isActive()) return true;
   OLMsg::Info("Check command line <%s> for client <%s>",
 	      getCommandLine().c_str(), getName().c_str());
   if(getCommandLine().empty())
@@ -1114,7 +1200,7 @@ void RemoteNativeClient::compute(){
     for(unsigned int i = 0; i < choices.size(); i++)
       if(syncOutputFile(getWorkingDir(), choices[i]))
 	OLMsg::Info("ok");
-  } 
+  }
 }
 
 
@@ -1126,7 +1212,6 @@ void RemoteNativeClient::compute(){
 
 bool RemoteEncapsulatedClient::checkCommandLine(){
   bool success;
-  if(!isActive()) return true;
   OLMsg::Info("Check command line <%s> for client <%s>",
 	      getCommandLine().c_str(), getName().c_str());
   if(getCommandLine().empty())
@@ -1227,6 +1312,10 @@ int mySystem(std::string commandLine){
 
 #include <sys/stat.h>
 #include <ctime>
+#if defined(WIN32)
+#include <io.h>
+#endif
+
 bool checkIfPresent(std::string fileName){
   struct stat buf;
   if (!stat(fileName.c_str(), &buf))
@@ -1238,7 +1327,11 @@ bool chmod(std::string fileName){
   struct stat info;
   stat(fileName.c_str(), &info);
   // chmod u=rwx
+#if defined(WIN32)
+  if (_chmod(fileName.c_str(), _S_IREAD | _S_IWRITE) != 0){
+#else
   if (chmod(fileName.c_str(), S_IRWXU) != 0){
+#endif
     perror("chmod() error");
     return false;
   }
@@ -1247,7 +1340,7 @@ bool chmod(std::string fileName){
 
 #include <sys/types.h>
 
-#if not defined WIN32
+#if !defined(WIN32)
 #include <unistd.h>
 #include <pwd.h>
 std::string getUserHomedir(){
@@ -1314,8 +1407,8 @@ std::string QuoteExecPath(const std::string &in)
 
 std::string removeBlanks(const std::string &in)
 {
-  size_t pos0=in.find_first_not_of(" ");
-  size_t pos=in.find_last_not_of(" ");
+  size_t pos0=in.find_first_not_of(" \t");
+  size_t pos=in.find_last_not_of(" \t");
   if( (pos0 != std::string::npos) && (pos != std::string::npos))
     return in.substr(pos0, pos-pos0+1);
   else
@@ -1346,7 +1439,7 @@ std::vector<std::string> SplitOLHostName(const std::string &in)
   std::vector<std::string> s(2);
   size_t pos = in.find(":");
   if(pos == std::string::npos){
-    s[0] = in; 
+    s[0] = in;
     s[1] = "";
   }
   else{
