@@ -4,7 +4,6 @@
 
 #include "OnelabClients.h"
 
-
 // reserved keywords for the onelab parser
 
 namespace olkey{
@@ -354,8 +353,8 @@ std::string localSolverClient::resolveGetVal(std::string line) {
 	  else if(!action.compare("expand")) {
 	    std::string pattern;
 	    pattern.assign(extractExpandPattern(args[0]));
-	    OLMsg::Info("Expand parameter <%s> with pattern <%s>",
-		      paramName.c_str(),pattern.c_str());
+	    // OLMsg::Info("Expand parameter <%s> with pattern <%s>",
+	    // 	      paramName.c_str(),pattern.c_str());
 	    buff.assign(1,pattern[0]);
 	    for(std::vector<double>::iterator it = choices.begin();
 		it != choices.end(); it++){
@@ -1436,10 +1435,39 @@ void localSolverClient::convert_oneline(std::string line, std::ifstream &infile,
   }
 }
 
+void preProcess(const std::string &client, const std::string &fullName){
+  std::vector<std::string> split = SplitOLFileName(fullName);
+  std::string ifileName = split[1] + split[2] ; // remove heading "_" if any
+  std::string ofileName = split[0] + split[1] ; // remove trailing ".ol"
+
+  std::string workDir = SplitFileName(split[1]) [0];
+  localSolverClient *c = new InterfacedClient(client, "", workDir);
+
+  std::ifstream  infile(ifileName.c_str());
+  if( infile.is_open()) {
+    std::ofstream outfile(ofileName.c_str());
+    if(outfile.is_open()) {
+      OLMsg::Info("Preprocess file <%s> into <%s>",ifileName.c_str(), ofileName.c_str());
+      while ( infile.good() ) {
+	std::string line;
+	getline (infile,line);
+	c->convert_oneline(line,infile,outfile);
+      }
+      outfile.close();
+    }
+    else
+      OLMsg::Error("The file <%s> cannot be opened",ofileName.c_str());
+    infile.close();
+  }
+  else
+    OLMsg::Error("The file <%s> cannot be opened",ifileName.c_str());
+  delete c;
+}
+
 void localSolverClient::convert_onefile(std::string fileName, std::ofstream &outfile) {
   std::ifstream infile(fileName.c_str());
   if (infile.is_open()){
-    //OLMsg::Info("Convert file <%s>",fileName.c_str());
+    OLMsg::Info("Convert file <%s>",fileName.c_str());
     while ( infile.good() ) {
       std::string line;
       getline (infile,line);
@@ -1559,18 +1587,6 @@ void MetaModel::client_sentence(const std::string &name,
       }
     }
   }
-/* else if(!action.compare("commandLine")){
-    if(isTodo(REGISTER)){
-      if(arguments[0].size() >= 1)
-	OLMsg::SetOnelabString(name + "/CommandLine", arguments[0], false);
-      else
-	OLMsg::Error("No pathname given for client <%s>", name.c_str());
-      if(arguments[0].size() >= 2)
-	OLMsg::SetOnelabString(name + "/HostName", arguments[1], false);
-      if(arguments[0].size() >= 3)
-	OLMsg::SetOnelabString(name + "/RemoteDir",arguments[2], false);
-    }
-    }*/
   else if(!action.compare("workingSubdir")){
     localSolverClient *c;
     if((c=findClientByName(name)))
