@@ -27,27 +27,20 @@ Please report all bugs and problems to the public mailing list
 <gmsh@geuz.org>.
 """
 
-import socket, struct, os, sys, subprocess
+import socket, struct, os, sys, subprocess, time
 _VERSION = '1.1'
 
-def file_exist(filename):
-  try:
-    with open(filename) as f:
-      return True
-  except IOError:
-    return False
-
-def path(ref,inp=''):
-  # ref is reference directory name
-  # inp is an optional file or directory name
-  dirname = os.path.dirname(ref)
+def path(dirname, inp):
+  # dirname is a directory, can be empty
+  # inp is an optional file or subdirectory name
+  # returns the path to 'inp' in the same directory as 'ref' 
+  dirname = os.path.dirname(dirname + os.sep)
   if not inp: 
-    if dirname:
-      return dirname
-    else :
-      return '.'
+    return dirname
   if inp[0] == '/' or inp[0] == '\\' or (len(inp) > 2 and inp[1] == '\:'):
-     return inp # do nothing, inp is an absolute path
+    return inp # do nothing, inp is an absolute path
+  if inp[0] == '.' :
+    inp = inp[2:] # cut off heading './' or '.\'
   if dirname: 
     return dirname + os.sep + inp # append inp to the path of the reference file
   else:
@@ -311,7 +304,7 @@ class client :
     if not self.socket or not filename :
       return
     self._send(self._GMSH_MERGE_FILE, filename)
-
+        
   def openProject(self, filename) :
     if not self.socket or not filename :
       return
@@ -345,13 +338,13 @@ class client :
       if msg == "true" :
           return True
     return False
-
+  
   def isChanged(self, name) :
     if not self.socket :
       return
     msg = ["get", name]
     self._send(self._GMSH_CLIENT_CHANGED, '\0'.join(msg))
-    (t, msg) = self._receive() 
+    (t, msg) = self._receive()
     if t == self._GMSH_CLIENT_CHANGED :
       if msg == "true" :
           return True
@@ -391,10 +384,11 @@ class client :
   def run(self, name, command, arguments=''):
     self.runSubClient(name, command, arguments)
 
-  def __init__(self):
+  def __init__(self, ref=''):
     self.socket = None
     self.name = ""
     self.addr = ""
+    self.wdir = os.path.dirname(ref)
     self._numSubClients = 0
     for i, v in enumerate(sys.argv) :
       if v == '-onelab':
@@ -405,7 +399,6 @@ class client :
     self.action = "compute" # default (subclients have no client.Action defined)
     self.action = self.getString(self.name + '/Action', False)
     self.setNumber('IsPyMetamodel',value=1,visible=0)
-    #self.defineNumber('0Metamodel/Loop',value=0,visible=0)
     self.loop = self.getNumber('0Metamodel/Loop', warn_if_not_found=False)
     self.batch = self.getNumber('0Metamodel/Batch', warn_if_not_found=False)
     self.sendInfo("Performing OneLab '" + self.action + "'")
@@ -425,7 +418,7 @@ class client :
   def __del__(self):
     self.finalize()
 
-  def call(self, name, cmdline, remote='', rundir='', logfile=''):
+  def call(self, name, cmdline, remote='', rundir='', logfile='', poll=0):
     cwd = None
     if not remote :
       argv = cmdline.rsplit(' ')
@@ -434,26 +427,47 @@ class client :
     else :
       argv=['ssh', remote , "cd %s ; %s" %(rundir,cmdline) ]
 
+    if poll and not logfile :
+        logfile = os.devnull
     if logfile:
       call = subprocess.Popen(argv, bufsize=1, cwd=cwd,
                               stdout=open(logfile,"w"),
-                              stderr=subprocess.PIPE)
+                              stderr=subprocess.STDOUT)
+      while call.poll() == None :
+        self.action = self.getString(self.name + '/Action', False)
+        if self.action == 'stop':
+          call.terminate()
+          self._send(self._GMSH_WARNING, 'client killed')
+          sys.exit(1)
+        time.sleep(1) # check every second for the stop signal from Gmsh
     else:
       call = subprocess.Popen(argv, bufsize=1, cwd=cwd,
                               stdout=subprocess.PIPE,
-                              stderr=subprocess.PIPE)
+                              stderr=subprocess.STDOUT)
+      count = 0
       for line in iter(call.stdout.readline, b''):
         print(line.rstrip())
+        count += 1
+        if count == 50 : # check every 50 lines for the stop signal from Gmsh
+          count = 0
+          self.action = self.getString(self.name + '/Action', False)
+          if self.action == 'stop':
+            call.terminate()
+            self._send(self._GMSH_WARNING, 'client killed')
+            sys.exit(1)
+      
     result = call.wait()
     if result == 0 :
-      self._send(self._GMSH_INFO, 'call \"' + ' '.join(argv) + '\"')
+      self._send(self._GMSH_INFO, 'done \"' + ' '.join(argv) + '\"')
       if self.action == 'compute':
         self.setChanged(name, False)
     else :
-      for line in iter(call.stderr.readline, b''):
-        self._send(self._GMSH_ERROR, line.rstrip().encode('utf-8'))
+      self._send(self._GMSH_ERROR, 'error \"' + ' '.join(argv) + '\"')
       sys.exit(1)
-      
+  
+  def copy(self, here, there):
+    os.system('cp '+ here + ' ' + there)
+    
   def upload(self, here, there, remote='') :
     if not here or not there :
       return
@@ -483,28 +497,72 @@ class client :
       self._send(self._GMSH_INFO, 'download: ' + ' '.join(argv))
     else :
       print(call.stderr.read())
-      
-  def solutionFiles(self, list) :
+
+  def getPath(self, inp='') :
+    return path(self.wdir, inp)
+
+  def fileExists(self, p) :
+    return os.path.exists(p)
+
+  def checkFile(self, p) :
+    if not self.fileExists(p):
+      self.sendError('path error: %s' %(p))
+      exit(0) 
+    return True
+
+  def checkPath(self, inp='') :
+    p = path(self.wdir,inp)
+    self.checkFile(p)
+    return p
+
+  def solutionFiles(self, client, list) :
     self.defineNumber('0Metamodel/9Use restored solution', value=0, choices=[0,1])
     self.defineString('0Metamodel/9Tag', value='')
     if list :
       if self.getNumber('0Metamodel/9Use restored solution') :
-        return self.getStringChoices('0Metamodel/9Solution files')  
+        solFiles = self.getStringChoices('0Metamodel/9Solution files')
+        for i in solFiles:
+          self.checkFile(i)
       else :
-        self.setString('0Metamodel/9Solution files', value=list[0],
-                       choices=list, readOnly=1)      
-    return list
+        solFiles = list
+        self.setString('0Metamodel/9Solution files', value=solFiles[0],
+                       choices=solFiles, readOnly=1)
+        for i in solFiles:
+          if not self.fileExists(i) :
+            self.setChanged(client, 'true')
+    return solFiles
 
-  def restoreSolution(self) :
-    return self.getNumber('0Metamodel/9Use restored solution')
+  def needsCompute(self, client) :
+    # Avoid recomputation when a database with restored solutions has just been loaded
+    # Otherwise, stop using restored solutions when the client elmer has changed
+    if self.useRestoredSolution() == 2: # we have just loaded a database
+      self.setRestoredSolution(1)
+      self.setChanged(client, 'false') # do not compute
+    else :
+      if self.isChanged(client):
+        self.setRestoredSolution(0)
+        return True
+    return False
+
+  def willCompute(self, client) : # Inform user about the value of needsCompute
+    if self.useRestoredSolution() == 2: # special value
+      return False
+    else :
+      return True if self.isChanged(client) else False
+
+  def useRestoredSolution(self) :
+    return self.getNumber('0Metamodel/9Use restored solution') 
+
+  def setRestoredSolution(self, val) :
+    self.setNumber('0Metamodel/9Use restored solution', value=val)
 
   def outputFiles(self, list) :
     if list :
-      self.setString(self.name+'/9Output files', value=list[0],
-                     choices=list, visible=0)
+      self.setString('0Metamodel/9Output files', value=list[0],
+                     choices=list, visible=1)
 
 # tool to extract the (i, j)th element in an array file
-from rlcompleter import readline
+# from rlcompleter import readline
 def extract(filename,i,j):
     input = open(filename,'r')
     all_lines = input.readlines()
